@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Helper.cpp 465 2008-08-12 21:13:31Z vadim $
+  $Id: Helper.cpp 2646 2010-03-04 01:19:15Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -45,7 +45,8 @@ using namespace std;
 
 static unsigned long calculateDimension(FWObject* obj)
 {
-    if (Group::cast(obj)!=NULL) {
+    if (Group::cast(obj)!=NULL)
+    {
         unsigned long res=0;
 	for (FWObject::iterator i1=obj->begin(); i1!=obj->end(); ++i1) 
         {
@@ -67,26 +68,10 @@ static unsigned long calculateDimension(FWObject* obj)
     return 0;
 }
 
-void Helper::expand_group_recursive_no_cache(FWObject *o,list<FWObject*> &ol)
-{
-    if (Group::cast( o )!=NULL) {
-        for (FWObject::iterator i2=o->begin(); i2!=o->end(); ++i2) 
-        {
-            FWObject *o1= *i2;
-            if (FWReference::cast(o1)!=NULL) o1=FWReference::cast(o1)->getPointer();
-            assert(o1);
-
-            expand_group_recursive_no_cache(o1,ol);
-        }
-    } else {
-        ol.push_back( o );
-    }
-}
-
-
 void Helper::expand_group_recursive(FWObject *o,list<FWObject*> &ol)
 {
-    if (Group::cast( o )!=NULL) {
+    if (Group::cast( o )!=NULL)
+    {
         for (FWObject::iterator i2=o->begin(); i2!=o->end(); ++i2) 
         {
             FWObject *o1= *i2;
@@ -105,15 +90,27 @@ int  Helper::findInterfaceByAddress(Address *obj)
     return findInterfaceByAddress( obj->getAddressPtr() );
 }
 
+/*
+ * ticket #1293
+ * Weird corner case: user made a mistake setting netmask of an
+ * intrface to 0.0.0.0, which made this interface match any address.
+ * At the same time, this interface was marked as "unprotected".  So,
+ * if we get an interface from helper.findInterfaceByNetzoneOrAll()
+ * but this interface is unprotected, issue a warning and use all
+ * interfaces instead.
+ */
+
 int  Helper::findInterfaceByAddress(const InetAddr *addr)
 {
     if (addr==NULL) return -1;
 
     Firewall *fw=compiler->fw;
-    list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
+    list<FWObject*> l2=fw->getByTypeDeep(Interface::TYPENAME);
     for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i)
     {
 	Interface *iface=Interface::cast(*i);
+        if (iface->isDedicatedFailover()) continue;
+        if (iface->isUnprotected()) continue;
         FWObjectTypedChildIterator j =
             iface->findByType((addr->isV4())?IPv4::TYPENAME:IPv6::TYPENAME);
         for (; j!=j.end(); ++j)
@@ -132,40 +129,49 @@ int  Helper::findInterfaceByNetzone(Address *obj)
 
 int  Helper::findInterfaceByNetzone(const InetAddr *addr) throw(string)
 {
-    Firewall *fw=compiler->fw;
+    Firewall *fw = compiler->fw;
     map<int,FWObject*> zones;
-    FWObjectTypedChildIterator i=fw->findByType(Interface::TYPENAME);
-    for ( ; i!=i.end(); ++i)
+    list<FWObject*> l2 = fw->getByTypeDeep(Interface::TYPENAME);
+    for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i)
     {
+        Interface *iface = Interface::cast(*i);
+        if (iface->isDedicatedFailover()) continue;
+        if (iface->isUnprotected()) continue;
+
         // NOTE: "network_zone" is globally unique string ID
         int netzone_id =
-            FWObjectDatabase::getIntId((*i)->getStr("network_zone"));
+            FWObjectDatabase::getIntId(iface->getStr("network_zone"));
 
-        FWObject *netzone = fw->getRoot()->findInIndex(netzone_id);
 #if 0
+        FWObject *netzone = fw->getRoot()->findInIndex(netzone_id);
         cerr << "netzone_id=" << netzone_id
-             << "  " << (*i)->getStr("network_zone")
+             << "  " << iface->getStr("network_zone")
              << "  " << netzone->getName()
              << endl;
 #endif
+
         if (netzone_id != -1)
         {
             FWObject *netzone = fw->getRoot()->findInIndex(netzone_id);
-            for (list<FWObject*>::iterator j=netzone->begin();
-                 j!=netzone->end(); ++j)
+            list<FWObject*> nz;
+            expand_group_recursive(netzone, nz);
+
+            for (list<FWObject*>::iterator j=nz.begin(); j!=nz.end(); ++j)
             {
-                assert(Address::cast(*j)!=NULL);
+                if (Address::cast(*j) == NULL) continue;
 
                 // if addr==NULL, return id of the interfacce that has
                 // net_zone=="any"
                 if (addr==NULL)
                 {
                     if ((*j)->getId()==FWObjectDatabase::ANY_ADDRESS_ID)
-                        return (*i)->getId(); // id of the interface
+                        return iface->getId(); // id of the interface
                 } else
                 {
                     if (Address::cast(*j)->belongs(*addr))
-                        zones[(*i)->getId()] = netzone;
+                    {
+                        zones[iface->getId()] = netzone;
+                    }
                 }
             }
         }
@@ -176,7 +182,7 @@ int  Helper::findInterfaceByNetzone(const InetAddr *addr) throw(string)
  * pick the one with smallest dimension
  */    
     int  res_id = -1;
-    unsigned long res_dim=LONG_MAX;
+    unsigned long res_dim = LONG_MAX;
     for (map<int,FWObject*>::iterator i=zones.begin(); i!=zones.end(); ++i) 
     {
         int iface_id = (*i).first;
@@ -185,8 +191,8 @@ int  Helper::findInterfaceByNetzone(const InetAddr *addr) throw(string)
 
         if (dim<=res_dim) 
         {
-            res_id=iface_id;
-            res_dim=dim;
+            res_id = iface_id;
+            res_dim = dim;
         }
     }
 
@@ -199,6 +205,7 @@ int  Helper::findInterfaceByNetzone(const InetAddr *addr) throw(string)
     if (res_id == -1)
         throw(string("Can not find interface with network zone that includes "
                      "address ") + string((addr)?addr->toString():"NULL"));
+
     return res_id;
 }
 
@@ -233,14 +240,17 @@ list<int> Helper::findInterfaceByNetzoneOrAll(RuleElement *re)
         {
             Rule *rule = Rule::cast(re->getParent());
             compiler->abort(
+                re->getParent(), 
                 string("findInterfaceByNetzoneOrAll failed to retrieve first "
                        "object from the rule element; is argument not of "
-                       "the type RuleElementSrc or RuleElementDst ? Rule ") +
-                rule->getLabel());
+                       "the type RuleElementSrc or RuleElementDst ?"));
+            return intf_id_list;
         }
+
         try
         {
-            intf_id_list.push_back( findInterfaceByNetzone( a ) );
+            int intf_id = findInterfaceByNetzone(a);
+            intf_id_list.push_back(intf_id);
         } catch(string err)
         {
             // could not find interface with netzone to match address 'a'
@@ -253,7 +263,8 @@ list<int> Helper::findInterfaceByNetzoneOrAll(RuleElement *re)
                 Resources::getTargetCapabilityBool(
                     compiler->fw->getStr("platform"), "network_zones");
 
-            if (supports_network_zones) compiler->warning(err);
+            if (supports_network_zones)
+                compiler->warning(err);
 
             FWObjectTypedChildIterator i = compiler->fw->findByType(
                 Interface::TYPENAME);
@@ -270,10 +281,17 @@ list<int> Helper::findInterfaceByNetzoneOrAll(RuleElement *re)
 string triplet::hash()
 {
     ostringstream ostr;
-    ostr <<  src->getAddressPtr()->toString()
-         << "."
-         << dst->getAddressPtr()->toString() 
-         <<"."
-         << srv->getId();
+    string dst_str;
+    string src_str;
+
+    Interface *intf = Interface::cast(src);
+    if (intf && intf->isDyn()) src_str = intf->getId();
+    else src_str = src->getAddressPtr()->toString();
+
+    intf = Interface::cast(dst);
+    if (intf && intf->isDyn()) dst_str = intf->getId();
+    else dst_str = dst->getAddressPtr()->toString();
+
+    ostr << src_str << "." << dst_str <<"." << srv->getId();
     return ostr.str();
 }
