@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: ActionsDialog.cpp 952 2009-05-20 21:54:45Z vadim $
+  $Id: ActionsDialog.cpp 2647 2010-03-04 02:06:35Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -32,6 +32,8 @@
 #include "ActionsDialog.h"
 #include "FWWindow.h"
 #include "FWObjectDropArea.h"
+#include "DialogFactory.h"
+#include "FWCmdRule.h"
 
 #include "fwbuilder/Library.h"
 #include "fwbuilder/Interface.h"
@@ -40,6 +42,8 @@
 #include "fwbuilder/RuleSet.h"
 #include "fwbuilder/Firewall.h"
 
+#include <memory>
+
 #include <qlineedit.h>
 #include <qspinbox.h>
 #include <qcheckbox.h>
@@ -53,6 +57,7 @@
 #include <QStackedWidget>
 #include <qcursor.h>
 #include <qregexp.h>
+#include <QtDebug>
 
 #include <iostream>
 #include <stdlib.h>
@@ -60,14 +65,14 @@
 using namespace std;
 using namespace libfwbuilder;
 
-ActionsDialog::ActionsDialog(QWidget *parent) : QWidget(parent)
+ActionsDialog::ActionsDialog(QWidget *parent) : BaseObjectDialog(parent)
 {
     m_dialog = new Ui::ActionsDialog_q;
     m_dialog->setupUi(this);
 
     m_dialog->iptBranchDropArea->addAcceptedTypes("Policy");
-    m_dialog->iptBranchDropArea->addAcceptedTypes("NAT");
-    m_dialog->iptBranchDropArea->addAcceptedTypes("Routing");
+    //m_dialog->iptBranchDropArea->addAcceptedTypes("NAT");
+    //m_dialog->iptBranchDropArea->addAcceptedTypes("Routing");
     m_dialog->iptBranchDropArea->setHelperText("Drop rule set object here");
     connect (m_dialog->iptBranchDropArea,
              SIGNAL(objectDeleted()),this,SLOT(changed()));
@@ -75,14 +80,21 @@ ActionsDialog::ActionsDialog(QWidget *parent) : QWidget(parent)
              SIGNAL(objectInserted()),this,SLOT(changed()));
 
     m_dialog->pfBranchDropArea->addAcceptedTypes("Policy");
-    m_dialog->pfBranchDropArea->addAcceptedTypes("NAT");
-    m_dialog->pfBranchDropArea->addAcceptedTypes("Routing");
+    //m_dialog->pfBranchDropArea->addAcceptedTypes("NAT");
+    //m_dialog->pfBranchDropArea->addAcceptedTypes("Routing");
     m_dialog->pfBranchDropArea->setHelperText("Drop rule set object here");
     connect (m_dialog->pfBranchDropArea,
              SIGNAL(objectDeleted()),this,SLOT(changed()));
     connect (m_dialog->pfBranchDropArea,
              SIGNAL(objectInserted()),this,SLOT(changed()));
 
+    m_dialog->natBranchDropArea->addAcceptedTypes("NAT");
+    m_dialog->natBranchDropArea->setHelperText("Drop NAT rule set object here");
+    connect (m_dialog->natBranchDropArea,
+             SIGNAL(objectDeleted()),this,SLOT(changed()));
+    connect (m_dialog->natBranchDropArea,
+             SIGNAL(objectInserted()),this,SLOT(changed()));
+
     m_dialog->pfTagDropArea->addAcceptedTypes("TagService");
     connect (m_dialog->pfTagDropArea,
              SIGNAL(objectDeleted()),this,SLOT(changed()));
@@ -103,7 +115,8 @@ ActionsDialog::~ActionsDialog()
 
 void ActionsDialog::loadFWObject(FWObject *o)
 {
-    setRule(PolicyRule::cast(o));
+    obj = o;
+    setRule(Rule::cast(o));
 }
 
 void ActionsDialog::getHelpName(QString *str)
@@ -111,12 +124,6 @@ void ActionsDialog::getHelpName(QString *str)
     *str = help_name;
 }
 
-void ActionsDialog::changed()
-{
-    //apply->setEnabled( true );
-    emit changed_sign();
-}
-
 void ActionsDialog::validate(bool *res)
 {
     FWOptions *ruleopt =rule->getOptionsObject();
@@ -147,14 +154,14 @@ void ActionsDialog::validate(bool *res)
     }
 }
 
-void ActionsDialog::isChanged(bool*)
-{
-    //*res=apply->isEnabled();
-}
-
-
 void ActionsDialog::applyChanges()
 {
+    std::auto_ptr<FWCmdRuleChange> cmd( new FWCmdRuleChangeAction(m_project, obj));
+
+    // new_state  is a copy of the rule object
+    FWObject* new_state = cmd->getNewState();
+    FWOptions* new_rule_options = Rule::cast(new_state)->getOptionsObject();
+
     if (platform=="iptables" && editor=="AccountingStr")
     {
         QString rn = m_dialog->accountingvalue_str->text();
@@ -173,23 +180,23 @@ void ActionsDialog::applyChanges()
         }
     }
 
+    data.saveAll(new_rule_options);
 
-    data.saveAll();
-
-    FWOptions *ropt = rule->getOptionsObject();
+    Rule *rule = Rule::cast(new_state);
+    PolicyRule *policy_rule = PolicyRule::cast(new_state);
 
     if (editor=="TagInt")
     {
         FWObject *tag_object = m_dialog->iptTagDropArea->getObject();
         // if tag_object==NULL, setTagObject clears setting in the rule
-        rule->setTagObject(tag_object);
+        policy_rule->setTagObject(tag_object);
     }
 
     if (editor=="TagStr")
     {
         FWObject *tag_object = m_dialog->pfTagDropArea->getObject();
         // if tag_object==NULL, setTagObject clears setting in the rule
-        rule->setTagObject(tag_object);
+        policy_rule->setTagObject(tag_object);
     }
 
     if (editor=="BranchChain")
@@ -206,16 +213,20 @@ void ActionsDialog::applyChanges()
         rule->setBranch(ruleset);
     }
 
+    if (editor=="NATBranch")
+    {
+        RuleSet *ruleset = RuleSet::cast(m_dialog->natBranchDropArea->getObject());
+        // if ruleset==NULL, setBranch clears setting in the rule
+        rule->setBranch(ruleset);
+    }
+
     if (m_dialog->useDummyNetPipe->isChecked())
-        ropt->setInt("ipfw_classify_method",DUMMYNETPIPE);
+        new_rule_options->setInt("ipfw_classify_method", DUMMYNETPIPE);
     else
-        ropt->setInt("ipfw_classify_method",DUMMYNETQUEUE);
+        new_rule_options->setInt("ipfw_classify_method", DUMMYNETQUEUE);
 
-    mw->updateLastModifiedTimestampForAllFirewalls(rule);
-}
-
-void ActionsDialog::discardChanges()
-{
+    if (!cmd->getOldState()->cmp(new_state, true))
+        m_project->undoStack->push(cmd.release());
 }
 
 void ActionsDialog::tagvalueChanged(int)
@@ -236,44 +247,44 @@ void ActionsDialog::iptRouteContinueToggled()
     m_dialog->ipt_tee->setEnabled( ! m_dialog->ipt_continue->isChecked() );
 }
 
-void ActionsDialog::setRule(PolicyRule *r )
+void ActionsDialog::setRule(Rule *r)
 {
-    rule=r;
-    FWObject *o = rule;
-    while (o!=NULL && Firewall::cast(o)==NULL) o=o->getParent();
-    assert(o!=NULL);
+    rule = r;
+
+    PolicyRule *policy_rule = PolicyRule::cast(r);
+
+    FWObject *o = r;
+    while (o!=NULL && Firewall::cast(o)==NULL) o = o->getParent();
+
+    Firewall *f = Firewall::cast(o);
+    firewall = f;
 
     FWOptions *ropt = rule->getOptionsObject();
 
-    Firewall *f=Firewall::cast(o);
-    firewall=f;
+    string act = getRuleAction(rule).toStdString();
 
-    platform=f->getStr("platform");
+    if (firewall)
+    {
+        // firewall can be NULL if rule set is in Deleted Objects library
+        fillInterfaces(m_dialog->ipt_iif);
+        fillInterfaces(m_dialog->ipt_oif);
+        fillInterfaces(m_dialog->ipf_route_opt_if);
+        fillInterfaces(m_dialog->pf_route_opt_if);
 
-    QString icn = ":/Icons/" ;
-    icn+=r->getActionAsString().c_str();
-    m_dialog->icon->setPixmap(QIcon(icn).pixmap(25,25));
+        platform = firewall->getStr("platform");
+        help_name = string(platform + "_" + act).c_str();
+        editor = DialogFactory::getActionDialogPageName(firewall, r);
+    }
 
-    QString title=QString("%3 %1 / %2")
-        .arg(QString::fromUtf8(f->getName().c_str()))
-        .arg(rule->getPosition())
-        .arg(rule->getActionAsString().c_str());
-    m_dialog->action->setText(title);
+    if (fwbdebug)
+        qDebug() << "ActionsDialog::setRule"
+                 << "Action: " << getRuleAction(rule)
+                 << "editor: " << editor.c_str();
 
-    string act = rule->getActionAsString();
-    help_name = string(platform + "_" + act).c_str();
-
-    QStringList actionsOnReject=getActionsOnReject( platform.c_str() );
+    QStringList actionsOnReject = getActionsOnReject( platform.c_str() );
     m_dialog->rejectvalue->clear();
     m_dialog->rejectvalue->addItems( getScreenNames( actionsOnReject ) );
 
-    fillInterfaces(m_dialog->ipt_iif);
-    fillInterfaces(m_dialog->ipt_oif);
-    fillInterfaces(m_dialog->ipf_route_opt_if);
-    fillInterfaces(m_dialog->pf_route_opt_if);
-
-    editor = Resources::getActionEditor(platform, act);
-
     branchNameInput = NULL;
 
     if (ropt->getInt("ipfw_classify_method") == DUMMYNETPIPE)
@@ -354,7 +365,7 @@ void ActionsDialog::setRule(PolicyRule *r )
     // REJECT action:
     data.registerOption(m_dialog->rejectvalue, ropt, "action_on_reject");
 
-    QWidget *w=m_dialog->NonePage;
+    QWidget *w = m_dialog->NonePage;
     if (editor=="Reject")
     {
         w=m_dialog->RejectPage;
@@ -362,14 +373,14 @@ void ActionsDialog::setRule(PolicyRule *r )
     else if (editor=="TagInt")
     {
         w=m_dialog->TagIntPage;
-        FWObject *o = rule->getTagObject();
+        FWObject *o = policy_rule->getTagObject();
         m_dialog->iptTagDropArea->setObject(o);
         m_dialog->iptTagDropArea->update();
     }
     else if (editor=="TagStr")
     {
         w=m_dialog->TagStrPage;
-        FWObject *o = rule->getTagObject();
+        FWObject *o = policy_rule->getTagObject();
         m_dialog->pfTagDropArea->setObject(o);
         m_dialog->pfTagDropArea->update();
     }
@@ -395,7 +406,7 @@ void ActionsDialog::setRule(PolicyRule *r )
     }
     else if (editor=="BranchChain")
     {
-        w=m_dialog->BranchChainPage;
+        w = m_dialog->BranchChainPage;
         RuleSet *ruleset = r->getBranch();
         m_dialog->iptBranchDropArea->setObject(ruleset);
 
@@ -408,6 +419,12 @@ void ActionsDialog::setRule(PolicyRule *r )
         RuleSet *ruleset = r->getBranch();
         m_dialog->iptBranchDropArea->setObject(ruleset);
     }
+    else if (editor=="NATBranch")
+    {
+        w = m_dialog->NATBranchPage;
+        RuleSet *ruleset = r->getBranch();
+        m_dialog->natBranchDropArea->setObject(ruleset);
+    }
     else if (editor=="RouteIPT")
     {
         w=m_dialog->RouteIPTPage;
@@ -442,11 +459,4 @@ void ActionsDialog::fillInterfaces(QComboBox* cb)
 
 }
 
-void ActionsDialog::closeEvent(QCloseEvent *e)
-{
-    if (fwbdebug)
-        qDebug("ActionsDialog::closeEvent  got close event: %p",e);
-    emit close_sign(e);
-    //hide();
-}
 
