@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: AddressRangeDialog.cpp 952 2009-05-20 21:54:45Z vadim $
+  $Id: AddressRangeDialog.cpp 2512 2010-02-10 23:46:54Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,7 +27,8 @@
 #include "global.h"
 #include "utils.h"
 #include "ProjectPanel.h"
-
+#include "FWCmdChange.h"
+#include "FWBSettings.h"
 #include "FWBTree.h"
 #include "AddressRangeDialog.h"
 
@@ -42,14 +43,16 @@
 #include <qcombobox.h>
 #include <qmessagebox.h>
 #include <qpushbutton.h>
-#include "FWBSettings.h"
+#include <QApplication>
+#include <QUndoStack>
+
+#include <memory>
 
-#include "FWWindow.h"
 using namespace std;
 using namespace libfwbuilder;
 
-AddressRangeDialog::AddressRangeDialog(ProjectPanel *project, QWidget *parent):
-        QWidget(parent), m_project(project)
+AddressRangeDialog::AddressRangeDialog(QWidget *parent):
+        BaseObjectDialog(parent)
 {
     m_dialog = new Ui::AddressRangeDialog_q;
     m_dialog->setupUi(this);
@@ -97,17 +100,10 @@ void AddressRangeDialog::loadFWObject(FWObject *o)
     init=false;
 }
 
-void AddressRangeDialog::changed()
-{
-    //apply->setEnabled( true );
-    emit changed_sign();
-}
-
 void AddressRangeDialog::validate(bool *res)
 {
     *res=true;
 
-    if (!isTreeReadWrite(this,obj)) { *res=false; return; }
     if (!validateName(this,obj,m_dialog->obj_name->text())) { *res=false; return; }
 
     AddressRange *s = dynamic_cast<AddressRange*>(obj);
@@ -117,44 +113,50 @@ void AddressRangeDialog::validate(bool *res)
         InetAddr(m_dialog->rangeStart->text().toLatin1().constData());
     } catch (FWException &ex)
     {
-        *res=false;
-        QMessageBox::critical(this, "Firewall Builder",
-                              tr("Illegal IP address '%1'").arg(m_dialog->rangeStart->text()),
-                              tr("&Continue"), 0, 0,
-                              0 );
+        *res = false;
+        // show warning dialog only if app has focus
+        if (QApplication::focusWidget() != NULL)
+        {
+            blockSignals(true);
+            QMessageBox::critical(this, "Firewall Builder",
+                                  tr("Illegal IP address '%1'").arg(m_dialog->rangeStart->text()),
+                                  tr("&Continue"), 0, 0,
+                                  0 );
+            blockSignals(false);
+        }
     }
     try
     {
         InetAddr(m_dialog->rangeEnd->text().toLatin1().constData());
     } catch (FWException &ex)
     {
-        *res=false;
-        QMessageBox::critical(this, "Firewall Builder",
-                              tr("Illegal IP address '%1'").arg(m_dialog->rangeEnd->text()),
-                              tr("&Continue"), 0, 0,
-                              0 );
+        *res = false;
+        if (QApplication::focusWidget() != NULL)
+        {
+            blockSignals(true);
+            QMessageBox::critical(this, "Firewall Builder",
+                                  tr("Illegal IP address '%1'").arg(m_dialog->rangeEnd->text()),
+                                  tr("&Continue"), 0, 0,
+                                  0 );
+            blockSignals(false);
+        }
     }
 }
 
-void AddressRangeDialog::isChanged(bool *)
-{
-    //*res=(!init && apply->isEnabled());
-}
 
-void AddressRangeDialog::libChanged()
-{
-    changed();
-}
 
 
 void AddressRangeDialog::applyChanges()
 {
-    AddressRange *s = dynamic_cast<AddressRange*>(obj);
+    std::auto_ptr<FWCmdChange> cmd(new FWCmdChange(m_project, obj));
+    FWObject* new_state = cmd->getNewState();
+
+    AddressRange *s = dynamic_cast<AddressRange*>(new_state);
     assert(s!=NULL);
 
-    string oldname=obj->getName();
-    obj->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
-    obj->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
+    string oldname = obj->getName();
+    new_state->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
+    new_state->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
     try
     {
         s->setRangeStart( InetAddr(m_dialog->rangeStart->text().toLatin1().constData()) );
@@ -163,26 +165,15 @@ void AddressRangeDialog::applyChanges()
     {
 
     }
-    mw->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
 
-    //apply->setEnabled( false );
-    mw->updateLastModifiedTimestampForAllFirewalls(obj);
-}
-
-void AddressRangeDialog::discardChanges()
-{
-    loadFWObject(obj);
+    if (!cmd->getOldState()->cmp(new_state, true))
+    {
+        if (obj->isReadOnly()) return;
+        m_project->undoStack->push(cmd.release());
+    }
+    
 }
 
 
 
-/* ObjectEditor class connects its slot to this signal and does all
- * the verification for us, then accepts (or not) the event. So we do
- * nothing here and defer all the processing to ObjectEditor
- */
-void AddressRangeDialog::closeEvent(QCloseEvent *e)
-{
-    emit close_sign(e);
-
-}
 
