@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: AddressTableDialog.cpp 952 2009-05-20 21:54:45Z vadim $
+  $Id: AddressTableDialog.cpp 1932 2009-11-26 03:26:24Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,17 +27,19 @@
 #include "global.h"
 #include "utils.h"
 #include "ProjectPanel.h"
-
 #include "AddressTableDialog.h"
 #include "SimpleTextView.h"
 #include "FWBSettings.h"
 #include "FWWindow.h"
+#include "FWCmdChange.h"
 
 #include "fwbuilder/Library.h"
 #include "fwbuilder/AddressTable.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/FWException.h"
 
+#include <memory>
+
 #include <qlineedit.h>
 #include <qspinbox.h>
 #include <qcheckbox.h>
@@ -53,7 +55,7 @@
 #include <qfile.h>
 #include <qtextstream.h>
 #include <qdir.h>
-#include "FWBSettings.h"
+#include <QUndoStack>
 
 
 #include <iostream>
@@ -61,7 +63,7 @@
 using namespace std;
 using namespace libfwbuilder;
 
-AddressTableDialog::AddressTableDialog(ProjectPanel *project, QWidget *parent) : QWidget(parent), m_project(project)
+AddressTableDialog::AddressTableDialog(QWidget *parent) : BaseObjectDialog(parent)
 {
     m_dialog = new Ui::AddressTableDialog_q;
     m_dialog->setupUi(this);
@@ -110,63 +112,38 @@ void AddressTableDialog::loadFWObject(FWObject *o)
     init=false;
 }
 
-void AddressTableDialog::changed()
-{
-    //BrowseButton->setEnabled(r_compiletime->isChecked() );
-    //apply->setEnabled( true );
-    emit changed_sign();
-
-}
-
 void AddressTableDialog::validate(bool *res)
 {
     *res=true;
     AddressTable *s = dynamic_cast<AddressTable*>(obj);
     assert(s!=NULL);
 
-    if (!isTreeReadWrite(this,obj)) { *res=false; return; }
     if (!validateName(this,obj,m_dialog->obj_name->text())) { *res=false; return; }
 }
 
-void AddressTableDialog::isChanged(bool *)
-{
-    //*res=(!init && apply->isEnabled());
-}
 
-void AddressTableDialog::libChanged()
-{
-    changed();
-}
 
 void AddressTableDialog::applyChanges()
 {
-    AddressTable *s = dynamic_cast<AddressTable*>(obj);
+    std::auto_ptr<FWCmdChange> cmd( new FWCmdChange(m_project, obj));
+    FWObject* new_state = cmd->getNewState();
+
+    AddressTable *s = dynamic_cast<AddressTable*>(new_state);
     assert(s!=NULL);
 
-    string oldname=obj->getName();
-    obj->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
-    obj->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
-    QByteArray cs=m_dialog->filename->text().toLocal8Bit();
+    string oldname = obj->getName();
+    new_state->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
+    new_state->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
+    QByteArray cs = m_dialog->filename->text().toLocal8Bit();
     s->setSourceName( (const char *)cs );
     s->setRunTime(m_dialog->r_runtime->isChecked() );
 
-    mw->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
-
-    //apply->setEnabled( false );
-    mw->updateLastModifiedTimestampForAllFirewalls(obj);
-}
-
-void AddressTableDialog::discardChanges()
-{
-    loadFWObject(obj);
-}
-
-
-void AddressTableDialog::closeEvent(QCloseEvent *e)
-{
-    if (fwbdebug)
-        qDebug("AddressTableDialog::closeEvent  got close event: %p",e);
-    emit close_sign(e);
+    if (!cmd->getOldState()->cmp(new_state, true))
+    {
+        if (obj->isReadOnly()) return;
+        m_project->undoStack->push(cmd.release());
+    }
+   
 }
 
 void AddressTableDialog::browse()
