@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: DialogData.cpp 792 2009-03-25 04:43:16Z vadim $
+  $Id: DialogData.cpp 2639 2010-03-01 04:39:27Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -39,6 +39,7 @@
 #include <qcheckbox.h>
 #include <qradiobutton.h>
 #include <qtextedit.h>
+#include <QtDebug>
 
 #include <iostream>
 
@@ -187,17 +188,22 @@ void DialogData::loadAll()
         loadToWidget( *i );
 }
 
-void DialogData::saveAll()
+void DialogData::saveAll(FWObject *new_obj)
 {
     for (list<DialogOption>::iterator i=options.begin(); i!=options.end(); ++i)
     {
+        FWObject *use_obj = (new_obj!=NULL) ? new_obj : i->obj;
 
         if (dynamic_cast<QComboBox*>(i->w)!=NULL)
         {
             QComboBox *cbx = dynamic_cast<QComboBox*>(i->w);
-            QString      s = cbx->currentText();
+            QString s = cbx->currentText();
             if (fwbdebug)
-                qDebug(QString("DialogData::saveAll() QComboBox %1 (i->mapping.empty()=%2) s=%3").arg(i->w->objectName()).arg(i->mapping.empty()).arg(s).toAscii().constData());
+                qDebug() << 
+                    QString("DialogData::saveAll() QComboBox %1 (i->mapping.empty()=%2) s=%3")
+                    .arg(i->w->objectName())
+                    .arg(i->mapping.empty())
+                    .arg(s);
 
             if ( !i->mapping.empty() && !s.isNull() )
             {
@@ -220,42 +226,44 @@ void DialogData::saveAll()
                 QStringList::iterator i1 = i->mapping.begin();
                 QStringList::iterator i2 = i->mapping.begin();
                 ++i2;
-                while (i2!=i->mapping.end())
+                while (i1!=i->mapping.end())
                 {
-                    if (fwbdebug) qDebug(QString("   (*i1)=%1").arg(*i1).toAscii().constData());
+                    if (fwbdebug)
+                        qDebug() << "   (*i1)=" << *i1;
 
                     if (s== (*i1)) { s= *i2; break; }
                     i1++; i1++;
                     i2++; i2++;
                 }
+
             }
             if (s.isEmpty()) s="";
-            i->obj->setStr(i->attr.toLatin1().constData(), s.toLatin1().constData());
+            use_obj->setStr(i->attr.toLatin1().constData(), s.toLatin1().constData());
         }
         if (dynamic_cast<QCheckBox*>(i->w)!=NULL)
         {
             QCheckBox *cbx=dynamic_cast<QCheckBox*>(i->w);
-            i->obj->setBool(i->attr.toLatin1().constData(), cbx->isChecked() );
+            use_obj->setBool(i->attr.toLatin1().constData(), cbx->isChecked() );
         }
         if (dynamic_cast<QLineEdit*>(i->w)!=NULL)
         {
             QLineEdit *edit=dynamic_cast<QLineEdit*>(i->w);
-            i->obj->setStr(i->attr.toLatin1().constData(), edit->text().toLatin1().constData() );
+            use_obj->setStr(i->attr.toLatin1().constData(), edit->text().toLatin1().constData() );
         }
         if (dynamic_cast<QTextEdit*>(i->w)!=NULL)
         {
             QTextEdit *edit=dynamic_cast<QTextEdit*>(i->w);
-            i->obj->setStr(i->attr.toLatin1().constData(), edit->toPlainText().toLatin1().constData() );
+            use_obj->setStr(i->attr.toLatin1().constData(), edit->toPlainText().toLatin1().constData() );
         }
         if (dynamic_cast<QRadioButton*>(i->w)!=NULL)
         {
             QRadioButton *rbtn=dynamic_cast<QRadioButton*>(i->w);
-            i->obj->setBool(i->attr.toLatin1().constData(), rbtn->isChecked() );
+            use_obj->setBool(i->attr.toLatin1().constData(), rbtn->isChecked() );
         }
         if (dynamic_cast<QSpinBox*>(i->w)!=NULL)
         {
             QSpinBox *sbx = dynamic_cast<QSpinBox*>(i->w);
-            i->obj->setInt( i->attr.toLatin1().constData(), sbx->value() );
+            use_obj->setInt( i->attr.toLatin1().constData(), sbx->value() );
         }
 
     }
