@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: DialogData.h 353 2008-07-13 01:26:39Z vadim $
+  $Id: DialogData.h 1797 2009-11-18 00:44:02Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -41,38 +41,36 @@ namespace libfwbuilder {
 
 class DialogData;
 
-class DialogOption {
+class DialogOption
+{
     friend class DialogData;
 
  public:
     enum DataType { Unknown, String, Int, Bool };
 
  private:
-    QWidget                *w;
+    QWidget *w;
     libfwbuilder::FWObject *obj;
-    QString                 attr;
-    QStringList             mapping;
-    QString                 override_str_val;
-    int                     override_int_val;
-    DataType                dtype;
-    
- public:
+    QString attr;
+    QStringList mapping;
+    QString override_str_val;
+    int override_int_val;
+    DataType dtype;
+
     DialogOption(QWidget *widget, libfwbuilder::FWObject *obj, const char* attr);
     DialogOption(QWidget *widget, libfwbuilder::FWObject *obj, const char* attr, QStringList mapping);
 
     void overrideValue(const QString &val) { override_str_val=val; }
-    void overrideValue(int            val)  { override_int_val=val; }
+    void overrideValue(int val)  { override_int_val=val; }
 
     DataType type() { return dtype; }
-    
 };
 
 
-class DialogData {
-
+class DialogData
+{
     std::list<DialogOption>  options;
 
-
     void loadToWidget( DialogOption &dopt , bool override=false );
     
  public:
@@ -92,7 +90,6 @@ class DialogData {
                         libfwbuilder::FWObject *obj,
                         const char *attr);
 
-
 /**
  * like the method above, plus adds a maping between option value
  * rendered in the widget and value stored in the object
@@ -127,8 +124,7 @@ class DialogData {
 /**
  * saves all options from their widgets to object attributes
  */
-    void saveAll();
-        
+    void saveAll(libfwbuilder::FWObject *new_obj=NULL);
 };
 
 #endif
