@@ -7,7 +7,7 @@
   Author:  Vadim Kurland     vadim@fwbuilder.org
            Illiya Yalovoy    <yalovoy@gmail.com>
 
-  $Id: DiscoveryDruid.cpp 870 2009-05-02 19:21:01Z vadim $
+  $Id: DiscoveryDruid.cpp 2441 2010-01-26 19:28:41Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,6 +27,7 @@
 #include "../../config.h"
 #include "global.h"
 #include "utils.h"
+#include "events.h"
 
 #include <qradiobutton.h>
 #include <qlineedit.h>
@@ -50,13 +51,19 @@
 #include <qhostinfo.h>
 #include <qgroupbox.h>
 #include <qmessagebox.h> 
+#include <QtDebug>
 
 #include "DiscoveryDruid.h"
+#include "ProjectPanel.h"
+
+#include "interfaceProperties.h"
+#include "interfacePropertiesObjectFactory.h"
 
 #include <iostream>
 #include <map>
 #include <vector>
 #include <set>
+#include <memory>
 
 #include "fwbuilder/HostsFile.h"
 #include "fwbuilder/IPv4.h"
@@ -68,6 +75,7 @@
 #include "fwbuilder/InetAddrMask.h"
 #include "fwbuilder/Inet6AddrMask.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/Policy.h"
 
 #include "fwbuilder/dns.h"
 #include "fwbuilder/snmp.h"
@@ -84,7 +92,7 @@ using namespace std;
 using namespace libfwbuilder;
 
 DiscoveryDruid::DiscoveryDruid(QWidget *parent, bool start_with_import) :
-    QDialog(parent, Qt::CustomizeWindowHint|Qt::WindowSystemMenuHint)
+    QDialog(parent)
 {
     m_dialog = new Ui::DiscoveryDruid_q;
     m_dialog->setupUi(this);
@@ -135,8 +143,10 @@ DiscoveryDruid::DiscoveryDruid(QWidget *parent, bool start_with_import) :
     flt_net     = new Filter();
     flt_net_d   = new FilterDialog(this);
     flt_net_d->setFilter(flt_net);
+    
+    assert(mw->activeProject()->db());
 
-    fillLibraries(m_dialog->libs,mw->db());
+    fillLibraries(m_dialog->libs, mw->activeProject()->db());
     m_dialog->libs->setEditable(true);
     m_dialog->libs->lineEdit()->setText(mw->getCurrentLib()->getName().c_str());
 
@@ -230,7 +240,7 @@ const char * DISCOVERY_DRUID_SNMPINADDR     ="SNMPInAddr";
 const char * DISCOVERY_DRUID_SNMPINMASK     ="SNMPInMask";
 const char * DISCOVERY_DRUID_SNMPRECURSIVE  ="SNMPRecursive";
 const char * DISCOVERY_DRUID_SNMPFOLLOWP2P  ="SNMPFollowP2P";
-const char * DISCOVERY_DRUID_SNMPINCLUDEVIRT="SNMPIncludeVirt";
+const char * DISCOVERY_DRUID_SNMPINCLUDEUNNUMBERED="SnmpIncludeUnnumbered";
 const char * DISCOVERY_DRUID_SNMPDODNS      ="SNMPDoDNS";
 const char * DISCOVERY_DRUID_SNMPCOMMUNITY  ="SNMPCommunity";
 const char * DISCOVERY_DRUID_SNMPRETRIES    ="SNMPRetries";
@@ -273,8 +283,8 @@ void DiscoveryDruid::restore()
                 QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPRECURSIVE));
     m_dialog->snmpfollowp2p->setChecked(st->getBool(
                 QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPFOLLOWP2P));
-    m_dialog->snmpincludevirt->setChecked(st->getBool(
-                QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPINCLUDEVIRT));
+    m_dialog->snmpincludeunnumbered->setChecked(st->getBool(
+                QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPINCLUDEUNNUMBERED));
     m_dialog->snmpdodns->setChecked(st->getBool(
                 QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPDODNS));
     s=st->getStr(QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPCOMMUNITY);
@@ -337,8 +347,8 @@ void DiscoveryDruid::save()
             QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPFOLLOWP2P,
             m_dialog->snmpfollowp2p->isChecked());
     st->setBool(
-            QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPINCLUDEVIRT,
-            m_dialog->snmpincludevirt->isChecked());
+            QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPINCLUDEUNNUMBERED,
+            m_dialog->snmpincludeunnumbered->isChecked());
     st->setBool(
             QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPDODNS,
             m_dialog->snmpdodns->isChecked());
@@ -812,9 +822,9 @@ void DiscoveryDruid::startDNSScan()
 InetAddr DiscoveryDruid::getSeedHostAddress()
 {
     if (fwbdebug)
-        qDebug(
+        qDebug() << 
             QString("DiscoveryDruid::getSeedHostAddress(): Seed host name %1").
-            arg(m_dialog->seedhostname->text()).toLatin1().constData());
+            arg(m_dialog->seedhostname->text());
 
     libfwbuilder::InetAddr   seed_host_addr;
     if (!m_dialog->seedhostname->text().isEmpty())
@@ -823,9 +833,9 @@ InetAddr DiscoveryDruid::getSeedHostAddress()
         {
             QString a = getAddrByName( m_dialog->seedhostname->text(), AF_INET);
             if (fwbdebug)
-                qDebug(
+                qDebug() <<
                     QString("DiscoveryDruid::getSeedHostAddress() address: %1").
-                       arg(a).toLatin1().constData());
+                    arg(a);
 
             return InetAddr( a.toLatin1().constData() );
         } catch(const FWException &ex) 
@@ -1305,8 +1315,24 @@ void DiscoveryDruid::loadDataFromImporter()
     if (imp!=NULL)
     {
         Firewall *fw = imp->finalize();
+        ProjectPanel *pp = mw->activeProject();
+        QString filename = pp->getFileName();
+        //pp->m_panel->om->reload();
+        //pp->m_panel->om->autoRenameChildren(fw, "");
+
+        QCoreApplication::postEvent(mw, new reloadObjectTreeEvent(filename));
+        if (mw->isEditorVisible())
+            QCoreApplication::postEvent(
+                mw, new openObjectInEditorEvent(filename, fw->getId()));
+        QCoreApplication::postEvent(
+            mw, new showObjectInTreeEvent(filename, fw->getId()));
+
+        // Open first created Policy ruleset object
+        FWObject *first_policy = fw->getFirstByType(Policy::TYPENAME);
+        if (first_policy)
+            QCoreApplication::postEvent(
+                mw, new openRulesetEvent(filename, first_policy->getId()));
 
-        mw->loadDataFromFw(fw);
     }
 }
 
@@ -1375,7 +1401,6 @@ void DiscoveryDruid::startSNMPScan()
     q->init(getSeedHostAddress(),
             m_dialog->snmpcommunity->text().toLatin1().constData(),
             m_dialog->snmprecursive->isChecked(),
-            ! m_dialog->snmpincludevirt->isChecked(),
             false,
             m_dialog->snmpfollowp2p->isChecked(),
             0,
@@ -1421,8 +1446,7 @@ void DiscoveryDruid::loadDataFromCrawler()
     set<InetAddrMask*> discovered_networks = q->getNetworks();
 
     if (fwbdebug)
-        qDebug(QString("got %1 networks").arg(
-                   discovered_networks.size()).toAscii().constData());
+        qDebug() << QString("got %1 networks").arg(discovered_networks.size());
 
     for (m=discovered_networks.begin(); m!=discovered_networks.end(); ++m)
     {
@@ -1430,8 +1454,7 @@ void DiscoveryDruid::loadDataFromCrawler()
         InetAddrMask *net = *m;
 
         if (fwbdebug)
-            qDebug(QString("network %1").arg(
-                       net->toString().c_str()).toAscii().constData());
+            qDebug() << QString("network %1").arg(net->toString().c_str());
 
         // if address in *m is ipv6, recreate it as Inet6AddrMask and 
         // use type NetworkIPv6
@@ -1455,7 +1478,7 @@ void DiscoveryDruid::loadDataFromCrawler()
     map<InetAddr, CrawlerFind>  t = q->getAllIPs();
 
     if (fwbdebug)
-        qDebug(QString("got %1 addresses").arg(t.size()).toAscii().constData());
+        qDebug() << QString("got %1 addresses").arg(t.size());
 
     m_dialog->discoveryprogress->setMaximum( t.size() );
     m_dialog->discoveryprogress->setValue(0);
@@ -1502,11 +1525,6 @@ void DiscoveryDruid::loadDataFromCrawler()
         }
     }
 #endif
-/*
-    (arg==0) ? 
-    _("Network scan completed, click 'Next' to continue") : 
-    _("There has been an error running the network scan. You can continue but data gathered by the scanner may be incomplete") 
-*/
 }
 
 
@@ -2024,6 +2042,128 @@ void DiscoveryDruid::typeFirewall()
     changeTargetObject(Firewall::TYPENAME);
 }
 
+/*
+ * Guess OS from the sysDescr string returned by the host. Returned OS
+ * name is always lower case one word, such as "linux", "ios"
+ *
+ * Examples of sysDescr strings:
+ *
+ * IOS (tm) 3600 Software (C3620-IK9O3S-M), Version 12.2(13), RELEASE SOFTWARE (fc1)
+ * Linux guardian 2.4.20 #2 Wed Nov 17 11:49:43 CET 2004 mips
+ * Linux crash 2.6.24-22-server #1 SMP Mon Nov 24 20:06:28 UTC 2008 x86_64
+ * Apple AirPort - Apple Computer, 2006.  All rights Reserved
+ * Cisco Secure FWSM Firewall Version 2.3(4)
+ * Cisco PIX Firewall Version 6.2(1) 
+ * Cisco Adaptive Security Appliance Version 8.2(0)227
+ */
+QString DiscoveryDruid::guessOS(const string &sysDescr)
+{
+    QStringList elements = QString(sysDescr.c_str()).split(" ");
+    QString first = elements[0].toLower();
+    if (first == "cisco")
+    {
+        if (elements[1].toLower() == "adaptive" &&
+            elements[2].toLower() == "security" &&
+            elements[3].toLower() == "appliance") return "pix";
+        if (elements[1].toLower() == "pix") return "pix";
+        if (elements[1].toLower() == "secure" &&
+            elements[2].toLower() == "fwsm") return "pix";
+    }
+    if (first == "darwin") return "macosx";
+    if (first == "apple") return "macosx";
+
+    return first;
+}
+
+
+FWObject* DiscoveryDruid::addInterface(FWObject *parent, InterfaceData *in,
+                                       bool skip_ip_address_check)
+{
+    ObjectManipulator *om = mw->activeProject()->m_panel->om;
+
+    if (!m_dialog->snmpincludeunnumbered->isChecked() && !skip_ip_address_check)
+    {
+        if (in->addr_mask.size()==0) return NULL;
+        if (in->addr_mask.front()->getAddressPtr()->isAny())
+            return NULL;
+    }
+
+    Interface *itf = NULL;
+    itf = Interface::cast(
+        mw->createObject(parent,
+                         QString(Interface::TYPENAME),
+                         QString(in->name.c_str())));
+
+    QString iname = om->getStandardName(itf, physAddress::TYPENAME, "mac");
+    iname = om->makeNameUnique(itf, iname, physAddress::TYPENAME);
+
+    physAddress *paddr = physAddress::cast(
+        mw->createObject(itf, physAddress::TYPENAME, iname)
+    );
+    paddr->setPhysAddress(in->mac_addr);
+
+    itf->setLabel(in->label);
+    itf->setSecurityLevel(in->securityLevel);
+
+    if (!in->interface_type.empty())
+    {
+        itf->getOptionsObject()->setStr("type", in->interface_type);
+        if (in->interface_type == "8021q")
+            itf->getOptionsObject()->setInt("vlan_id", in->vlan_id);
+    }
+
+    if (in->addr_mask.size()==0 ||
+        in->addr_mask.front()->getAddressPtr()->isAny())
+    {
+        itf->setUnnumbered(true);
+    } else
+    {
+        list<InetAddrMask*>::iterator n;
+        for (n=in->addr_mask.begin(); n!=in->addr_mask.end(); ++n)
+        {
+            const InetAddr *addr = (*n)->getAddressPtr();
+            const InetAddr *netm = (*n)->getNetmaskPtr();
+
+            if (addr->isV4())
+            {
+                try
+                {
+                    QString iname = om->getStandardName(itf, IPv4::TYPENAME, "ip");
+                    iname = om->makeNameUnique(itf, iname, IPv4::TYPENAME);
+
+                    IPv4 *ipv4= IPv4::cast(
+                        om->createObject(itf, IPv4::TYPENAME, iname)
+                    );
+                    ipv4->setAddress(*addr);
+                    ipv4->setNetmask(*netm);
+                } catch (FWException &ex)
+                {
+                    cerr << "FWException: " << ex.toString() << endl;
+                }
+            }
+
+            if (addr->isV6())
+            {
+                try
+                {
+                    QString iname = om->getStandardName(itf, IPv6::TYPENAME, "ip");
+                    iname = om->makeNameUnique(itf, iname, IPv6::TYPENAME);
+
+                    IPv6 *ipv6 = IPv6::cast(
+                        om->createObject(itf, IPv6::TYPENAME, iname)
+                    );
+                    ipv6->setAddress(*addr);
+                    ipv6->setNetmask(*netm);
+                } catch (FWException &ex)
+                {
+                    cerr << "FWException: " << ex.toString() << endl;
+                }
+            }
+        }
+    }
+    return itf;
+}
+
 void DiscoveryDruid::createRealObjects()
 {
     ObjectDescriptor od;
@@ -2061,6 +2201,8 @@ void DiscoveryDruid::createRealObjects()
         
         name = od.sysname;
 
+        QString os = guessOS(od.descr);
+
         a = od.addr.toString();
 
         if (od.isSelected)
@@ -2072,6 +2214,50 @@ void DiscoveryDruid::createRealObjects()
                 o = mw->createObject(type.c_str(), name.c_str());
                 o->setName(name);
 
+                if (type==Firewall::TYPENAME)
+                {
+                    if (os == "linux")
+                    {
+                        o->setStr("platform", "iptables");
+                        o->setStr("host_OS", "linux24");
+                    }
+                    if (os == "freebsd")
+                    {
+                        o->setStr("platform", "pf");
+                        o->setStr("host_OS", "freebsd");
+                    }
+                    if (os == "openbsd")
+                    {
+                        o->setStr("platform", "pf");
+                        o->setStr("host_OS", "openbsd");
+                    }
+                    if (os == "ios")
+                    {
+                        o->setStr("platform", "iosacl");
+                        o->setStr("host_OS", "ios");
+                    }
+                    if (os == "pix" || os == "fwsm")
+                    {
+                        o->setStr("platform", "pix");
+                        o->setStr("host_OS", "pix_os");
+                    }
+                    if (os == "apple")
+                    {
+                        o->setStr("platform", "ipfw");
+                        o->setStr("host_OS", "macosx");
+                    }
+                    if (os == "solaris")
+                    {
+                        o->setStr("platform", "ipf");
+                        o->setStr("host_OS", "solaris");
+                    }
+
+                    Resources::setDefaultTargetOptions( o->getStr("platform"),
+                                                        Firewall::cast(o) );
+                    Resources::setDefaultTargetOptions( o->getStr("host_OS"),
+                                                        Firewall::cast(o) );
+                }
+
                 if (od.interfaces.size()==0) 
                 {
                     Interface *itf= Interface::cast(
@@ -2095,57 +2281,77 @@ void DiscoveryDruid::createRealObjects()
                         ipv6->setAddress(od.addr);
                         ipv6->setNetmask(InetAddr());
                     }
-
+ 
                 } else
                 {
-                    map<int,InterfaceData>::const_iterator i;
-                    for (i=od.interfaces.begin(); i!=od.interfaces.end(); ++i)
+                    if (fwbdebug)
                     {
-                        InterfaceData in = i->second;
-                        if (in.addr_mask.size()==0) continue;
-                        if (in.addr_mask.front()->getAddressPtr()->isAny())
-                            continue;
-
-                        Interface *itf = Interface::cast(
-                            mw->createObject(o,
-                                             QString(Interface::TYPENAME),
-                                             QString(i->second.name.c_str())));
-
-                        itf->setPhysicalAddress(in.mac_addr);
-                        itf->setLabel(in.label);
-                        itf->setExt(in.ext);
-                        itf->setSecurityLevel(in.securityLevel);
-
-                        list<InetAddrMask*>::iterator n;
-                        for (n=in.addr_mask.begin(); n!=in.addr_mask.end(); ++n)
+                        map<int,InterfaceData>::iterator i;
+                        for (i=od.interfaces.begin(); i!=od.interfaces.end(); ++i)
                         {
-                            const InetAddr *addr = (*n)->getAddressPtr();
-                            const InetAddr *netm = (*n)->getNetmaskPtr();
-                            if (addr->isV4())
-                            {
-                                IPv4 *ipv4= IPv4::cast(
-                                    mw->createObject(itf, IPv4::TYPENAME,
-                                                     addr->toString().c_str())
-                                );
-                                ipv4->setAddress(*addr);
-                                ipv4->setNetmask(*netm);
-                                mw->autorename(itf, IPv4::TYPENAME, "ip");
-                            }
+                            InterfaceData *intf = &(i->second);
+                            QString str("Discovered interface %1: %2");
+                            qDebug() <<
+                                str.arg(intf->name.c_str()).arg(intf->mac_addr.c_str());
 
-                            if (addr->isV6())
-                            {
-                                IPv6 *ipv6 = IPv6::cast(
-                                    mw->createObject(itf, IPv6::TYPENAME,
-                                                     addr->toString().c_str())
-                                );
-                                ipv6->setAddress(*addr);
-                                ipv6->setNetmask(*netm);
-                                mw->autorename(itf, IPv6::TYPENAME, "ip6");
-                            }
                         }
-                        mw->autorename(itf, physAddress::TYPENAME, "mac");
+                    }
+
+                    list<InterfaceData*> interface_tree;
+                    std::auto_ptr<interfaceProperties> int_prop(
+                        interfacePropertiesObjectFactory::getInterfacePropertiesObject(o));
+                    int_prop->rearrangeInterfaces(od.interfaces, interface_tree);
+
+                    if (interface_tree.size() != od.interfaces.size())
+                    {
+                        // Some interfaces have been converted to subinterfaces
+                        // Show warning
+
+                        QMessageBox::warning(
+                            this, "Firewall Builder",
+                            tr(
+"Some discovered interfaces have been rearranged in "
+"fwbuilder objects and recreated as subinterfaces to "
+"reflect VLANs, bonding and bridging configurations. "
+"The algorithm used to guess correct relationship "
+"between interfaces and subinterfaces is imperfect "
+"because of the limited information provided by SNMP "
+"daemon. Pelase review created objects to make sure "
+"generated configuration is accurate. "
+"\n"
+"\n"
+"The program expects MAC addresses of bonding, bridge "
+"and vlan interfaces to be the same. It is especially "
+"important to review and fix generated objects if you "
+"use MAC address spoofing."
+),
+                            tr("&Continue"), 0, 0,
+                            0 );
+                        
+
+                    }
+
+                    list<InterfaceData*>::iterator it;
+                    for (it=interface_tree.begin(); it!=interface_tree.end(); ++it)
+                    {
+                        InterfaceData *in = *it;
+                        // if this interface has subinterfaces, add even if it
+                        // has no ip address (last arg)
+
+                        FWObject *intf = addInterface(
+                            o, in, in->subinterfaces.size()!=0);
+                        if (intf == NULL) continue;
+
+                        list<InterfaceData*>::iterator sit;
+                        for (sit=in->subinterfaces.begin();
+                             sit!=in->subinterfaces.end(); ++sit)
+                        {
+                            InterfaceData *subint = *sit;
+                            addInterface(intf, subint, true);
+                        }                        
                     }
                 }
+
                 if (!od.descr.empty())
                 {
                     FWOptions* opt=(dynamic_cast<Host*>(o))->getOptionsObject();
@@ -2153,22 +2359,10 @@ void DiscoveryDruid::createRealObjects()
                     opt->setStr("snmp_location",   od.location);
                     opt->setStr("snmp_contact",    od.contact);
                 }
+
                 mw->moveObject(m_dialog->libs->currentText(), o);
-                if (type==Firewall::TYPENAME)
-                {
-                    map<string,string> platforms = Resources::getPlatforms();
-                    map<string,string>::iterator i;
-                    for (i=platforms.begin(); i!=platforms.end(); i++)
-                        Resources::setDefaultTargetOptions( i->first,
-                                                            Firewall::cast(o) );
 
-                    map<string,string> OSs = Resources::getOS();
-                    for (i=OSs.begin(); i!=OSs.end(); i++)
-                        Resources::setDefaultTargetOptions( i->first,
-                                                            Firewall::cast(o) );
-
-                }
-            }else if (type==Network::TYPENAME)
+            } else if (type==Network::TYPENAME)
             {
                 Network *net=dynamic_cast<Network*>(
                     mw->createObject(type.c_str(),name.c_str())
@@ -2178,7 +2372,7 @@ void DiscoveryDruid::createRealObjects()
                 net->setAddress(InetAddr(a));
                 net->setNetmask(InetAddr(InetAddr(a)));
                 mw->moveObject(m_dialog->libs->currentText(), net);
-            }else if (type==IPv4::TYPENAME)
+            } else if (type==IPv4::TYPENAME)
             {
                 IPv4 *obj=dynamic_cast<IPv4*>(
                     mw->createObject(type.c_str(),name.c_str())
@@ -2194,30 +2388,12 @@ void DiscoveryDruid::createRealObjects()
         qApp->processEvents();
     }
     m_dialog->lastprogress->setValue(Objects.size());
-}
 
-void DiscoveryDruid::autorename(FWObject *obj,
-                                const string &objtype,
-                                const string &namesuffix)
-{
-    FWObject      *hst = obj->getParent();
-    list<FWObject*> ol = obj->getByType(objtype);
-    int           sfxn = 1;
+    ProjectPanel *pp = mw->activeProject();
+    QString filename = pp->getFileName();
 
-    for (list<FWObject*>::iterator j=ol.begin(); j!=ol.end(); ++j,++sfxn)
-    {
-        QString sfx;
-        if (ol.size()==1) sfx="";
-        else              sfx.setNum(sfxn);
-        QString nn = QString("%1:%2:%3%4")
-            .arg(QString::fromUtf8(hst->getName().c_str()))
-            .arg(QString::fromUtf8(obj->getName().c_str()))
-            .arg(namesuffix.c_str())
-            .arg(sfx);
+    QCoreApplication::postEvent(mw, new reloadObjectTreeEvent(filename));
 
-        (*j)->setName(string(nn.toUtf8().constData()));
-    }
-    ol.clear();
 }
 
 void DiscoveryDruid::importPlatformChanged(int cp)
