@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWBSettings.cpp 971 2009-05-27 20:40:51Z vadim $
+  $Id: FWBSettings.cpp 2672 2010-03-07 06:28:49Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -38,6 +38,8 @@
 #include <QDir>
 #include <QDesktopWidget>
 #include <QUuid>
+#include <QRegExp>
+#include <QtDebug>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -77,6 +79,7 @@ const char* groupModeSetpath = SETTINGS_PATH_PREFIX "/UI/GroupViewMode";
 const char* groupColsSetpath = SETTINGS_PATH_PREFIX "/UI/GroupViewColumns";
 const char* objTooltips = SETTINGS_PATH_PREFIX "/UI/objTooltips";
 const char* tooltipDelay = SETTINGS_PATH_PREFIX "/UI/tooltipDelay";
+const char* showUndoPanel = SETTINGS_PATH_PREFIX "/UI/showUndoPanel";
 
 const char* emptyRCSLog = SETTINGS_PATH_PREFIX "/RCS/emptyLog";
 const char* rcsFilePreviewStyle = SETTINGS_PATH_PREFIX "/RCS/FilePreviewStyle";
@@ -94,19 +97,28 @@ const char* iconsInRulesSize = SETTINGS_PATH_PREFIX "/UI/Icons/IconsInRulesSize"
 const char* rulesFont = SETTINGS_PATH_PREFIX "/UI/Fonts/RulesFont";
 const char* treeFont = SETTINGS_PATH_PREFIX "/UI/Fonts/TreeFont";
 const char* uiFont = SETTINGS_PATH_PREFIX "/UI/Fonts/UiFont";
+const char* compilerOutputFont = SETTINGS_PATH_PREFIX "/UI/Fonts/CompilerOutputFont";
 
 const char* clipComment = SETTINGS_PATH_PREFIX "/UI/ClipComment";
 
 const char* checkUpdates = SETTINGS_PATH_PREFIX "/UI/CheckUpdates";
+const char* updateAvailableWarningLastTime = SETTINGS_PATH_PREFIX "/UI/updateAvailableWarningLastTime";
 const char* checkUpdatesProxy = SETTINGS_PATH_PREFIX "/UI/CheckUpdatesProxy";
 
+const char* newFirewallPlatform = SETTINGS_PATH_PREFIX "/Objects/NewFireallPlatform";
+const char* newClusterFailoverProtocol = SETTINGS_PATH_PREFIX "/Objects/newClusterFailoverProtocol";
 const char* appGUID = SETTINGS_PATH_PREFIX "/ApplicationGUID";
 
+const char* targetStatus = SETTINGS_PATH_PREFIX "/TargetStatus/";
+
+
+/**
+ * Settings path defined here should match Windows registry paths used
+ * in the Windows installer/uninstaller scripts.
+ */
 FWBSettings::FWBSettings() :
-    QSettings(QSettings::UserScope, "netcitadel.com", "Firewall Builder")
+    QSettings(QSettings::UserScope, "netcitadel.com", getApplicationNameForSettings())
 {
-//    writeEntry( DTDSetpath, librespath.c_str() );
-//    writeEntry( ResSetpath, respath.c_str() );
 }
 
 /**
@@ -149,7 +161,7 @@ void FWBSettings::init()
         QDir wdir(wd);
         if (!wdir.exists() && !wdir.mkdir(wd))
         {
-            QString err= QString(QObject::tr("Working directory %1 does not exist and could not be created.\nIgnoring this setting.")).arg(wd.toLatin1().constData());
+            QString err = QString(QObject::tr("Working directory %1 does not exist and could not be created.\nIgnoring this setting.")).arg(wd);;
 
             if (app != NULL)
             {
@@ -157,7 +169,7 @@ void FWBSettings::init()
                                        "&Continue", 0, 0, 0 );
             } else
             {
-                qDebug( err.toAscii().constData() );
+                qDebug() << err;
             }
             setWDir("");
         }
@@ -194,15 +206,22 @@ void FWBSettings::init()
     ok = contains(uiFont);
     if (!ok) setUiFont(QApplication::font());
 
+    ok = contains(compilerOutputFont);
+    if (!ok) setCompilerOutputFont(QApplication::font());
+
     if (fwbdebug)
-        qDebug("Default application font: %s",
-               QApplication::font().toString().toLatin1().constData());
+        qDebug() << "Default application font:"
+                 << QApplication::font();
 
     ok = contains(clipComment);
     if (!ok) setClipComment(true);
 
     ok = contains(checkUpdates);
-    if (!ok) setCheckUpdates(true);
+    if (!ok)
+    {
+        setCheckUpdates(true);
+        setTimeOfLastUpdateAvailableWarning(0);
+    }
 
     ok = contains(compression);
     if (!ok) setCompression(false);
@@ -211,6 +230,24 @@ void FWBSettings::init()
     if (getSSHPath().isEmpty())  setSSHPath("ssh");
     if (getSCPPath().isEmpty())  setSCPPath("scp");
 #endif
+
+    // Note: hasKey calls QSettings::contains using path given as
+    // argument, prepended with SETTINGS_PATH_PREFIX
+    if (!hasKey("Window/maximized")) setInt("Window/maximized", 1);
+
+    if (!hasKey("Objects/DNSName/useCompileTimeForNewObjects"))
+        setBool("Objects/DNSName/useCompileTimeForNewObjects", true);
+
+    if (!hasKey("Objects/DNSName/useNameForDNSRecord"))
+        setBool("Objects/DNSName/useNameForDNSRecord", false);
+
+    if (!hasKey("Objects/AddressTable/useCompileTimeForNewObjects"))
+        setBool("Objects/AddressTable/useCompileTimeForNewObjects", true);
+}
+
+bool FWBSettings::hasKey(const QString &attribute)
+{
+    return QSettings::contains(SETTINGS_PATH_PREFIX "/" + attribute);
 }
 
 QString FWBSettings::getAppGUID()
@@ -220,50 +257,50 @@ QString FWBSettings::getAppGUID()
 
 QString FWBSettings::getStr(const QString &attribute)
 {
-    QString path=SETTINGS_PATH_PREFIX "/"+attribute;
+    QString path = SETTINGS_PATH_PREFIX "/" + attribute;
     return value(path).toString();
 }
 
 void FWBSettings::setStr(const QString &attribute,
                             const QString &val)
 {
-    QString path=SETTINGS_PATH_PREFIX "/"+attribute;
+    QString path = SETTINGS_PATH_PREFIX "/" + attribute;
     setValue(path,val);
 }
 
 bool FWBSettings::getBool(const QString &attribute)
 {
-    QString path=SETTINGS_PATH_PREFIX "/"+attribute;
+    QString path = SETTINGS_PATH_PREFIX "/" + attribute;
     return value(path).toBool();
 }
 
 void FWBSettings::setBool(const QString &attribute, bool val )
 {
-    QString path=SETTINGS_PATH_PREFIX "/"+attribute;
+    QString path = SETTINGS_PATH_PREFIX "/" + attribute;
     setValue(path,val);
 }
 
 int FWBSettings::getInt(const QString &attribute)
 {
-    QString path=SETTINGS_PATH_PREFIX "/"+attribute;
+    QString path = SETTINGS_PATH_PREFIX "/" + attribute;
     return value(path).toInt();
 }
 
 void FWBSettings::setInt(const QString &attribute, int val )
 {
-    QString path=SETTINGS_PATH_PREFIX "/"+attribute;
+    QString path = SETTINGS_PATH_PREFIX "/" + attribute;
     setValue(path,val);
 }
 
 QStringList FWBSettings::getList(const QString &attribute)
 {
-    QString path=SETTINGS_PATH_PREFIX "/"+attribute;
+    QString path = SETTINGS_PATH_PREFIX "/" + attribute;
     return value(path).toStringList();
 }
 
 void FWBSettings::setList(const QString &attribute, QStringList &list)
 {
-    QString path=SETTINGS_PATH_PREFIX "/"+attribute;
+    QString path = SETTINGS_PATH_PREFIX "/" + attribute;
     setValue(path, list);
 }
 
@@ -322,8 +359,6 @@ void FWBSettings::setSaveFileDir( const QString &d )
 
 void FWBSettings::save()
 {
-    if (mw->db()!=NULL)
-        setLastEdited( mw->db()->getFileName().c_str() );
 }
 
 bool FWBSettings::getRCSLogState() { return value( emptyRCSLog ).toBool(); }
@@ -358,6 +393,9 @@ void FWBSettings::setCompression(bool f) { setValue(compression, f); }
 bool FWBSettings::getDontSaveStdLib() {return value(dontSaveStdLib).toBool();}
 void FWBSettings::setDontSaveStdLib( bool f) { setValue(dontSaveStdLib,f);}
 
+bool FWBSettings::getShowUndoPanel() {return value(showUndoPanel).toBool();}
+void FWBSettings::setShowUndoPanel(bool f) {setValue(showUndoPanel, f);}
+
 bool FWBSettings::haveScreenPosition(const QString &wname)
 {
     QString val = value(QString(screenPositionSetpath)+wname ).toString();
@@ -365,8 +403,8 @@ bool FWBSettings::haveScreenPosition(const QString &wname)
 
     if (fwbdebug)
     {
-	qDebug("FWBSettings::haveScreenPosition wname '%s' ret=%d",
-		wname.toLatin1().constData(), res);
+    qDebug("FWBSettings::haveScreenPosition wname '%s' ret=%d",
+        wname.toLatin1().constData(), res);
     }
 
     return res;
@@ -391,8 +429,8 @@ QPoint FWBSettings::getScreenPosition(const QString &wname)
 
     if (fwbdebug)
     {
-	qDebug("FWBSettings::getScreenPosition wname '%s' x=%d y=%d",
-		wname.toLatin1().constData(), x,y );
+    qDebug("FWBSettings::getScreenPosition wname '%s' x=%d y=%d",
+        wname.toLatin1().constData(), x,y );
     }
 
     return QPoint(x,y);
@@ -409,8 +447,8 @@ void FWBSettings::saveScreenPosition(const QString &wname, const QPoint &p)
 
     if (fwbdebug)
     {
-	qDebug("FWBSettings::saveScreenPosition wname '%s' x=%d y=%d",
-		wname.toLatin1().constData(), x,y );
+    qDebug("FWBSettings::saveScreenPosition wname '%s' x=%d y=%d",
+        wname.toLatin1().constData(), x,y );
     }
 
     setValue(QString(screenPositionSetpath)+wname, val );
@@ -443,7 +481,7 @@ void FWBSettings::restoreGeometry(QWidget *w)
 
     if (fwbdebug)
     {
-	qDebug("FWBSettings::restoreGeometry  widget '%s' vis=%d x=%d y=%d",
+    qDebug("FWBSettings::restoreGeometry  widget '%s' vis=%d x=%d y=%d",
                name.toAscii().constData(), w->isVisible(), x,y);
     }
 
@@ -473,7 +511,7 @@ void FWBSettings::restoreGeometry(QWidget *w, const QRect &dg)
 
     if (fwbdebug)
     {
-	qDebug("FWBSettings::restoreGeometry  widget '%s' vis=%d x=%d y=%d",
+    qDebug("FWBSettings::restoreGeometry  widget '%s' vis=%d x=%d y=%d",
                name.toAscii().constData(), w->isVisible(), x,y);
     }
 
@@ -501,7 +539,7 @@ void FWBSettings::saveGeometry(QWidget *w)
 
     if (fwbdebug)
     {
-	qDebug("FWBSettings::saveGeometry  widget '%s' vis=%d val=%s",
+    qDebug("FWBSettings::saveGeometry  widget '%s' vis=%d val=%s",
                name.toAscii().constData(), w->isVisible(), val.toAscii().constData());
     }
 
@@ -568,14 +606,17 @@ void FWBSettings::getPrinterOptions(QPrinter *printer,
                                     int &pageWidth,
                                     int &pageHeight)
 {
-    printer->setPrinterName(
-        getStr("PrintSetup/printerName"));
+    QString name = getStr("PrintSetup/printerName");
+    if (!name.isEmpty())
+        printer->setPrinterName(
+            getStr("PrintSetup/printerName"));
 #ifndef _WIN32
     printer->setPrinterSelectionOption(
         getStr("PrintSetup/printerSelectionOption"));
 #endif
-    printer->setOutputFileName(
-        getStr("PrintSetup/outputFileName"));
+#ifndef Q_OS_MAC
+    printer->setOutputFileName(getStr("PrintSetup/outputFileName"));
+#endif
     printer->setOrientation(
         QPrinter::Orientation(getInt("PrintSetup/orientation")));
     printer->setPageSize(
@@ -655,7 +696,6 @@ void FWBSettings::setShowDirectionText(bool showText)
     setValue(showDirectionText, showText);
 }
 
-
 QFont FWBSettings::getRulesFont()
 {
     return getFontByType(rulesFont);
@@ -686,6 +726,16 @@ void FWBSettings::setUiFont(const QFont &font)
     setValue(uiFont, font.toString());
 }
 
+QFont FWBSettings::getCompilerOutputFont()
+{
+    return getFontByType(compilerOutputFont);
+}
+
+void FWBSettings::setCompilerOutputFont(const QFont &font)
+{
+    setValue(compilerOutputFont, font.toString());
+}
+
 QFont FWBSettings::getFontByType(const char *type)
 {
     QFont font = QFont();
@@ -715,6 +765,17 @@ void FWBSettings::setCheckUpdates(bool f)
     setValue(checkUpdates, f);
 }
 
+uint FWBSettings::getTimeOfLastUpdateAvailableWarning()
+{
+    return value(updateAvailableWarningLastTime).toUInt();
+}
+
+void FWBSettings::setTimeOfLastUpdateAvailableWarning(uint v)
+{
+    setValue(updateAvailableWarningLastTime, v);
+}
+
+
 
 QString FWBSettings::getCheckUpdatesProxy()
 {
@@ -752,6 +813,28 @@ void FWBSettings::setExpandedObjectIds(const QString &filename,
         strl.join(","));
 }
 
+int FWBSettings::getTreeSectionSize(const QString &filename,
+                                    const QString &lib,
+                                    int section_index)
+{
+    int v = value(
+        QString(SETTINGS_PATH_PREFIX "/Window/%1/%2/TreeSection/%3")
+        .arg(filename).arg(lib).arg(section_index)).toInt();
+    if (v <= 0) v = 300;
+    return v;
+}
+
+void FWBSettings::setTreeSectionSize(const QString &filename,
+                                     const QString &lib,
+                                     int section_index,
+                                     int size)
+{
+    setValue(
+        QString(SETTINGS_PATH_PREFIX "/Window/%1/%2/TreeSection/%3")
+        .arg(filename).arg(lib).arg(section_index),
+        size);
+}
+
 int FWBSettings::getVisibleRuleSetId(const QString &filename,
                                      const QString &lib)
 {
@@ -775,10 +858,29 @@ void FWBSettings::getCollapsedRuleGroups(const QString &filename,
                                          const QString &ruleset,
                                          QStringList &collapsed_groups)
 {
-    QString strl = value(QString(SETTINGS_PATH_PREFIX "/") +
-                         "Window/" + filename + "/" + firewall + "/" +
-                         ruleset + "/CollapsedRuleGroups").toString();
-    collapsed_groups = strl.split(",");
+    QString key = "Window/" + filename + "/" + firewall + "/" +
+        ruleset + "/CollapsedRuleGroups";
+    QString strl = value(QString(SETTINGS_PATH_PREFIX "/") + key).toString();
+    // QT regexp does not support negative lookbehind that we need to
+    // find all "," if they are not preceded by a "\". Will split the
+    // string on all commas, then find elements of the array that end
+    // with "\" and combine them with the following ones.
+    QStringList lst = strl.split(",");
+
+    QStringListIterator i(lst);
+    while (i.hasNext())
+    {
+        QString group_name = i.next();
+        while (group_name.endsWith("\\") && i.hasNext())
+            group_name += QString(",") + i.next();
+        group_name.replace("\\,", ",");
+        collapsed_groups.push_back(group_name);
+    }
+
+    if (fwbdebug)
+        qDebug() << "FWBSettings::getCollapsedRuleGroups"
+                 << key
+                 << collapsed_groups.join(" ||| ");
 }
 
 void FWBSettings::setCollapsedRuleGroups(const QString &filename,
@@ -786,10 +888,20 @@ void FWBSettings::setCollapsedRuleGroups(const QString &filename,
                                          const QString &ruleset,
                                          const QStringList &collapsed_groups)
 {
-    setValue(QString(SETTINGS_PATH_PREFIX "/") +
-             "Window/" + filename + "/" + firewall + "/" + ruleset +
-             "/CollapsedRuleGroups",
-             collapsed_groups.join(","));
+    QStringList lst;
+    foreach(QString group_name, collapsed_groups)
+    {
+        group_name.replace(",", "\\,");
+        lst.push_back(group_name);
+    }
+    QString key = "Window/" + filename + "/" + firewall + "/" + ruleset +
+        "/CollapsedRuleGroups";
+    QString val = lst.join(",");
+    if (fwbdebug)
+        qDebug() << "FWBSettings::setCollapsedRuleGroups"
+                 << key
+                 << val;
+    setValue(QString(SETTINGS_PATH_PREFIX "/") + key, val);
 }
 
 QStringList FWBSettings::getRecentFiles()
@@ -801,3 +913,41 @@ void FWBSettings::setRecentFiles(QStringList &list)
 {
     return setList("recentFiles", list);
 }
+
+QString FWBSettings::getNewFirewallPlatform()
+{
+    return value(newFirewallPlatform).toString();
+}
+
+void FWBSettings::setNewFirewallPlatform(const QString &platform)
+{
+    setValue(newFirewallPlatform, platform);
+}
+
+QString FWBSettings::getNewClusterFailoverProtocol()
+{
+    return value(newClusterFailoverProtocol).toString();
+}
+
+void FWBSettings::setNewClusterFailoverProtocol(const QString &platform)
+{
+    setValue(newClusterFailoverProtocol, platform);
+}
+
+QString FWBSettings::getTargetStatus(const QString &platform,
+                                     const QString &default_stat)
+{
+    QString var_path = targetStatus + platform;
+    bool ok = contains(var_path);
+    if (!ok) return default_stat;
+    return value(var_path).toString();
+}
+
+void FWBSettings::setTargetStatus(const QString &platform, const QString &status)
+{
+    QString var_path = targetStatus + platform;
+    setValue(var_path, status);
+}
+
+
+
