@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: FWObjectClipboard.h 472 2008-08-18 01:28:00Z vadim $
+  $Id: FWObjectClipboard.h 1972 2009-11-29 17:53:31Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -54,6 +54,10 @@ public:
      */
     libfwbuilder::FWObject* getObject();
 
+    /**
+     * removes object from the clipboard if it is there
+     */
+    void remove(libfwbuilder::FWObject*);
     /**
      * clear the clipboard
      */
