@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWObjectPropertiesFactory.cpp 759 2009-02-11 03:36:12Z vadim $
+  $Id: FWObjectPropertiesFactory.cpp 2639 2010-03-01 04:39:27Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -33,9 +33,11 @@
 #include <qtextstream.h>
 #include <qdatetime.h>
 #include <qregexp.h>
+#include <QtDebug>
 
 #include "FWObjectPropertiesFactory.h"
 #include "platforms.h"
+#include "DialogFactory.h"
 
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/AddressTable.h"
@@ -61,11 +63,15 @@
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/Rule.h"
+#include "fwbuilder/RuleSet.h"
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/TagService.h"
 #include "fwbuilder/UDPService.h"
 #include "fwbuilder/UserService.h"
 #include "fwbuilder/physAddress.h"
+#include "fwbuilder/StateSyncClusterGroup.h"
+#include "fwbuilder/FailoverClusterGroup.h"
+#include "fwbuilder/Cluster.h"
 
 #include <sstream>
 #include <iostream>
@@ -76,10 +82,198 @@ using namespace std;
 using namespace libfwbuilder;
 
 /*
- * API methods return STL string, so it is easier to use STL class
- * ostringstream to assemble text and then convert it to QString
- * rather than convert piece by piece.
+ * This method returns brief summary of properties, guaranteed to be 1
+ * line of text
  */
+QString FWObjectPropertiesFactory::getObjectPropertiesBrief(FWObject *obj)
+{
+    QString res;
+    QTextStream str(&res, QIODevice::WriteOnly);
+    FWObject *parent_obj = obj->getParent();
+
+    try
+    {
+        if (Library::isA(obj))
+        {
+            if (obj->isReadOnly()) str << "(read only)";
+
+        } else if (IPv4::isA(obj))
+        {
+            str <<  IPv4::cast(obj)->getAddressPtr()->toString().c_str();
+            if (parent_obj && Interface::isA(parent_obj))
+            {
+                str << "/";
+                str << IPv4::cast(obj)->getNetmaskPtr()->toString().c_str();
+            }
+        } else if (IPv6::isA(obj))
+        {
+            str <<  IPv6::cast(obj)->getAddressPtr()->toString().c_str();
+            if (parent_obj && Interface::isA(parent_obj))
+            {
+                str << "/";
+                str << QString("%1").arg(IPv6::cast(obj)->getNetmaskPtr()->getLength());
+            }
+        } else if (physAddress::isA(obj))
+        {
+            str <<  physAddress::cast(obj)->getPhysAddress().c_str();
+        } else if (DNSName::isA(obj))
+        {
+            str << DNSName::cast(obj)->getSourceName().c_str();
+        } else if (AddressTable::isA(obj))  
+        {
+            str << AddressTable::cast(obj)->getSourceName().c_str();
+        } else if (AddressRange::isA(obj))
+        {
+            AddressRange *ar=AddressRange::cast(obj);
+            str << ar->getRangeStart().toString().c_str();
+            str << " - ";
+            str << ar->getRangeEnd().toString().c_str();
+        } else if (Firewall::cast(obj))
+        {
+            if (Firewall::cast(obj)->needsInstall()) str << " * ";
+            QString platform = obj->getStr("platform").c_str();
+            QString version  = obj->getStr("version").c_str();
+            QString readableVersion = getVersionString(platform,version);
+            QString hostOS = obj->getStr("host_OS").c_str();
+            str <<  platform << "(" << readableVersion << ") / " << hostOS;
+
+        } else if (Host::isA(obj))
+        {
+            const InetAddr *addr = Address::cast(obj)->getAddressPtr();
+            if (addr)
+                str <<  addr->toString().c_str();
+            else
+                str << "(no ip address)";
+        } else if (Network::isA(obj))
+        {
+            Network *n=Network::cast(obj);
+            str << n->getAddressPtr()->toString().c_str();
+            str << "/";
+            str << n->getNetmaskPtr()->toString().c_str();
+
+        } else if (NetworkIPv6::isA(obj))
+        {
+            NetworkIPv6 *n=NetworkIPv6::cast(obj);
+            str << n->getAddressPtr()->toString().c_str();
+            str << "/";
+            str << QString("%1").arg(n->getNetmaskPtr()->getLength());
+
+        } else if (ClusterGroup::cast(obj)!=NULL)
+        {
+            ClusterGroup *g = ClusterGroup::cast(obj);
+            str << QObject::tr("type: ") << g->getStr("type").c_str();
+        } else if (Group::cast(obj)!=NULL)   // just any group
+        {
+            Group *g=Group::cast(obj);
+            str << g->size() << " " << QObject::tr(" objects");
+
+        } else if (Interface::isA(obj))
+        {
+            Interface *intf = Interface::cast(obj);
+            // trigger late initialization of options object
+            // if its read-only or part of the read-only tree, I can't help it.
+            if (!obj->isReadOnly()) intf->getOptionsObject();
+            str << intf->getLabel().c_str() << " ";
+
+            FWObject *parent = intf->getParentHost();
+
+            bool supports_security_levels = false;
+            bool supports_network_zones   = false;
+            try
+            {
+                supports_security_levels =
+                    (!parent->getStr("platform").empty() &&
+                     Resources::getTargetCapabilityBool(
+                         parent->getStr("platform"), "security_levels"));
+                supports_network_zones =
+                    (!parent->getStr("platform").empty() &&
+                     Resources::getTargetCapabilityBool(
+                         parent->getStr("platform"), "network_zones"));
+            } catch (FWException &ex)  { }
+
+            QStringList q;
+            if (supports_security_levels)
+            {
+                QString str;
+                str.setNum(intf->getSecurityLevel());
+                q.push_back(QString("sec level: %1").arg(str));
+            }
+            if (supports_network_zones)
+            {
+                int id = FWObjectDatabase::getIntId(intf->getStr("network_zone"));
+                if (id > 0)
+                {
+                    FWObject *nz_obj = obj->getRoot()->findInIndex(id);
+                    if (nz_obj)
+                        q.push_back(
+                            QString("network zone: %1")
+                            .arg(nz_obj->getName().c_str()));
+                    else
+                        q.push_back(QString("network zone: not configured"));
+
+                }
+            }
+            if (intf->isDyn())         q.push_back("dyn");
+            if (intf->isUnnumbered())  q.push_back("unnum");
+            if (intf->isDedicatedFailover())  q.push_back("failover");
+            if (intf->isBridgePort())  q.push_back("bridge port");
+            if (intf->isSlave())       q.push_back("slave");
+            if (intf->isUnprotected()) q.push_back("unp");
+            if (!q.empty()) str << q.join(",");
+
+        } else if (IPService::isA(obj))
+        {
+            str << QObject::tr("protocol: %1").arg(obj->getStr("protocol_num").c_str());
+
+        } else if (ICMPService::isA(obj) || ICMP6Service::isA(obj))
+        {
+            str << QObject::tr("type: %1").arg(obj->getStr("type").c_str())
+                << "  "
+                << QObject::tr("code: %1").arg(obj->getStr("code").c_str());
+
+        } else if (TCPService::isA(obj) || UDPService::isA(obj))
+        {
+            int sps,spe,dps,dpe;
+
+            sps=TCPUDPService::cast(obj)->getSrcRangeStart();
+            spe=TCPUDPService::cast(obj)->getSrcRangeEnd();
+            dps=TCPUDPService::cast(obj)->getDstRangeStart();
+            dpe=TCPUDPService::cast(obj)->getDstRangeEnd();
+
+            str << sps << ":" << spe << " / ";
+            str << dps << ":" << dpe;
+        } else if (TagService::isA(obj)) 
+        {
+            str << "Pattern: \"" << obj->getStr("tagcode").c_str() << "\"" ;
+        } else if (UserService::isA(obj)) 
+        {
+            const UserService* user_srv = UserService::constcast(obj);
+            str << "User id: \"" << user_srv->getUserId().c_str() << "\"" ;
+        } else if (RuleSet::cast(obj) != NULL)
+        {
+            QStringList attrs;
+            RuleSet *rs = RuleSet::cast(obj);
+            if (rs->isTop()) attrs.push_back("top ruleset");
+            if (rs->isDual()) attrs.push_back("ipv4/ipv6");
+            else
+            {
+                if (rs->isV4()) attrs.push_back("ipv4");
+                if (rs->isV6()) attrs.push_back("ipv6");
+            }
+            str << attrs.join(" ");
+        } else if (Interval::isA(obj))
+        {
+
+        }
+    } catch (FWException &ex)
+    {
+        cerr << ex.toString() << endl;
+    }
+
+    return QString::fromUtf8(res.toStdString().c_str());
+}
+
+
 QString FWObjectPropertiesFactory::getObjectProperties(FWObject *obj)
 {
     QString res;
@@ -121,7 +315,7 @@ QString FWObjectPropertiesFactory::getObjectProperties(FWObject *obj)
             str << ar->getRangeStart().toString().c_str();
             str << " - ";
             str << ar->getRangeEnd().toString().c_str();
-        } else if (Firewall::isA(obj))
+        } else if (Firewall::cast(obj))
         {
             QString platform = obj->getStr("platform").c_str();
             QString version  = obj->getStr("version").c_str();
@@ -172,27 +366,48 @@ QString FWObjectPropertiesFactory::getObjectProperties(FWObject *obj)
             str << "/";
             str << QString("%1").arg(n->getNetmaskPtr()->getLength());
 
+        } else if (ClusterGroup::cast(obj)!=NULL)
+        {
+            ClusterGroup *g = ClusterGroup::cast(obj);
+            str << QObject::tr("type: ") << g->getStr("type").c_str() << "<br>";
+            FWObjectTypedChildIterator j = obj->findByType(FWObjectReference::TYPENAME);
+            for ( ; j!=j.end(); ++j)
+            {
+                FWObject *obj = FWReference::getObject(*j);
+                if (Interface::cast(obj))
+                {
+                    FWObject *fw = obj->getParent();
+                    str << QObject::tr("Group member")
+                        << " " << fw->getName().c_str() 
+                        << ":" << obj->getName().c_str()
+                        << "<br>";
+                }
+            }
         } else if (Group::cast(obj)!=NULL)   // just any group
         {
             Group *g=Group::cast(obj);
             str << g->size() << " " << QObject::tr(" objects");
 
-        } else if (Firewall::isA(obj))
+        } else if (Firewall::cast(obj))
         {
 
         } else if (Interface::isA(obj))
         {
             Interface *intf = Interface::cast(obj);
+            str << "Label: "
+                << QString::fromUtf8(intf->getLabel().c_str()) << "<br>";
             FWObjectTypedChildIterator j = obj->findByType(IPv4::TYPENAME);
             for ( ; j!=j.end(); ++j)
             {
                 str << getObjectProperties(*j);
                 str << "<br>";
             }
-            str << " MAC: ";
             physAddress *paddr = intf->getPhysicalAddress();
             if (paddr!=NULL) 
-                str <<  paddr->getPhysAddress().c_str();
+                str << " MAC: " << paddr->getPhysAddress().c_str() << "<br>";
+            string intf_type = intf->getOptionsObject()->getStr("type");
+            if (!intf_type.empty())
+                str << " Interface Type: " << intf_type.c_str();
 
         } else if (IPService::isA(obj))
         {
@@ -361,10 +576,26 @@ QString FWObjectPropertiesFactory::getObjectPropertiesDetailed(FWObject *obj,
             str += n->getAddressPtr()->toString().c_str();
             str += "/";
             str += QString("%1").arg(n->getNetmaskPtr()->getLength());
+        } else if (ClusterGroup::cast(obj)!=NULL)
+        {
+            if (showPath && !tooltip) str += "<b>Path: </b>" + path + "<br>\n";
+            ClusterGroup *g = ClusterGroup::cast(obj);
+            str += QObject::tr("type: %1<br>").arg(g->getStr("type").c_str());
+            FWObjectTypedChildIterator j = obj->findByType(FWObjectReference::TYPENAME);
+            for ( ; j!=j.end(); ++j)
+            {
+                FWObject *obj = FWReference::getObject(*j);
+                if (Interface::cast(obj))
+                {
+                    FWObject *fw = obj->getParent();
+                    str += QObject::tr("Group member %1:%2<br>").
+                        arg(fw->getName().c_str()).arg(obj->getName().c_str());
+                }
+            }
         } else if (Group::cast(obj)!=NULL)   // just any group
         {
             if (showPath && !tooltip) str += "<b>Path: </b>" + path + "<br>\n";
-            Group *g=Group::cast(obj);
+            Group *g = Group::cast(obj);
             str += QObject::tr("%1 objects<br>\n").arg(g->size());
             int n = 0;
             list<FWObject*> ll = *g;
@@ -385,8 +616,9 @@ QString FWObjectPropertiesFactory::getObjectPropertiesDetailed(FWObject *obj,
                         + "  <b>" + QString::fromUtf8(o1->getName().c_str()) + "</b><br>\n";
                 }
             }
-        } else if (Firewall::isA(obj))
+        } else if (Firewall::cast(obj)) 
         {
+            // Note: Firewall::cast(obj) matched Firewall and Cluster
             QString platform = obj->getStr("platform").c_str();
             QString version  = obj->getStr("version").c_str();
             QString readableVersion = getVersionString(platform,version);
@@ -430,7 +662,23 @@ QString FWObjectPropertiesFactory::getObjectPropertiesDetailed(FWObject *obj,
             str += "</table>";
         } else if (Interface::isA(obj))
         {
-            str += QObject::tr("<b>Path:</b> ")+ path +"<br>\n";
+            FWObject *parent_host = obj;
+            QStringList short_path;
+            //short_path.push_front(QString::fromUtf8(obj->getName().c_str()));
+            do
+            {
+                parent_host = parent_host->getParent();
+                if (parent_host == NULL) break;
+                short_path.push_front(QString::fromUtf8(parent_host->getName().c_str()));
+            }
+            while (Host::cast(parent_host) == NULL);
+
+            str += QString("<b>Parent: </b>%1<br>\n").arg(short_path.join("/"));
+
+            Interface *intf = Interface::cast(obj);
+            str += "<b>Label: </b>";
+            str += QString::fromUtf8(intf->getLabel().c_str());
+            str += "<br>";
 
             FWObjectTypedChildIterator j = obj->findByType(IPv4::TYPENAME);
             for ( ; j!=j.end(); ++j)
@@ -438,7 +686,21 @@ QString FWObjectPropertiesFactory::getObjectPropertiesDetailed(FWObject *obj,
                 str += getObjectProperties(*j);
                 str += "<br>";
             }
-            physAddress *paddr=(Interface::cast(obj))->getPhysicalAddress();
+
+            string intf_type = intf->getOptionsObject()->getStr("type");
+            if (!intf_type.empty())
+            {
+                str += "<b>Interface Type: </b>";
+                str += intf_type.c_str();
+                if (intf_type == "8021q")
+                {
+                    int vlan_id = intf->getOptionsObject()->getInt("vlan_id");
+                    str += QString(" VLAN ID=%1").arg(vlan_id);
+                }
+                str += "<br>";
+            }
+
+            physAddress *paddr = intf->getPhysicalAddress();
             if (paddr!=NULL) 
             {
                 str += "MAC: ";
@@ -447,22 +709,19 @@ QString FWObjectPropertiesFactory::getObjectPropertiesDetailed(FWObject *obj,
             }
 
             QString q;
-            if (Interface::constcast(obj)->isDyn())        q=" dyn";
-            if (Interface::constcast(obj)->isUnnumbered()) q=" unnum";
-            if (Interface::constcast(obj)->isBridgePort()) q=" bridge port";
+            if (intf->isDyn())        q=" dyn";
+            if (intf->isUnnumbered()) q=" unnum";
+            if (intf->isBridgePort()) q=" bridge port";
             
             FWObject *p=obj;
-            while (p!=NULL && !Firewall::isA(p)) p=p->getParent();
+            while (p!=NULL && !Firewall::cast(p)) p=p->getParent();
             if (p!=NULL && (p->getStr("platform")=="pix" || p->getStr("platform")=="fwsm"))
             {
-                int sl=Interface::constcast(obj)->getSecurityLevel();
+                int sl = intf->getSecurityLevel();
                 q=q+QString("sec.level %1").arg(sl);
-            } else
-            {
-                if (Interface::constcast(obj)->isExt())        q=q+" ext";
             }
 
-            if (Interface::constcast(obj)->isUnprotected())    q=q+" unp";
+            if (intf->isUnprotected())    q=q+" unp";
             
             if (q!="") str += " (" + q + ")";
             str += "<br>\n";
@@ -546,13 +805,13 @@ QString FWObjectPropertiesFactory::getObjectPropertiesDetailed(FWObject *obj,
  * Do not translate literals 'pipe', 'queue', 'divert' below, these refer
  * to actual ipfw parameters and should not be localized.
  */
-QString FWObjectPropertiesFactory::getRuleActionProperties(PolicyRule *rule)
+QString FWObjectPropertiesFactory::getRuleActionProperties(Rule *rule)
 {
     QString par = "";
 
     if (rule!=NULL)
     {
-        string act = rule->getActionAsString();
+        QString act = getRuleAction(rule);
         
         FWObject *o = rule;
         while (o!=NULL && Firewall::cast(o)==NULL) o=o->getParent();
@@ -562,22 +821,24 @@ QString FWObjectPropertiesFactory::getRuleActionProperties(PolicyRule *rule)
         string platform=f->getStr("platform");
         
         FWOptions *ropt = rule->getOptionsObject();
-        string editor=Resources::getActionEditor(platform,act);
+        string editor = DialogFactory::getActionDialogPageName(f, rule);
 
-        if (editor!="None")
+        if (editor == "None") return "";
+
+        if (PolicyRule::isA(rule))
         {
-            switch (rule->getAction())
+            switch (PolicyRule::cast(rule)->getAction())
             {
             case PolicyRule::Reject:
                 par = ropt->getStr("action_on_reject").c_str();
                 break;
             case PolicyRule::Tag:
             {
-                FWObject *tag_object = rule->getTagObject();
+                FWObject *tag_object = PolicyRule::cast(rule)->getTagObject();
                 if (tag_object)
-                    par = tag_object->getName().c_str();
+                    par = QString::fromUtf8(tag_object->getName().c_str());
                 else
-                    par = rule->getTagValue().c_str();
+                    par = QString::fromUtf8(PolicyRule::cast(rule)->getTagValue().c_str());
                 break;
             }
             case PolicyRule::Accounting :
@@ -658,25 +919,38 @@ QString FWObjectPropertiesFactory::getRuleActionProperties(PolicyRule *rule)
             default : {}
             }
         }
-        
+
+        if (NATRule::isA(rule))
+        {
+            switch (NATRule::cast(rule)->getAction())
+            {
+            case NATRule::Translate:
+                break;
+
+            case NATRule::Branch:
+                FWObject *branch_ruleset = rule->getBranch();
+                if (branch_ruleset)
+                    par = branch_ruleset->getName().c_str();
+                break;
+            }
+        }        
     }
     
     return par;
 }
 
-QString FWObjectPropertiesFactory::getRuleActionPropertiesRich(PolicyRule *rule)
+QString FWObjectPropertiesFactory::getRuleActionPropertiesRich(Rule *rule)
 {
     FWObject *p=rule;
-    while (p!=NULL && !Firewall::isA(p)) p=p->getParent();
+    while (p!=NULL && !Firewall::cast(p)) p=p->getParent();
     assert(p!=NULL);
     string platform=p->getStr("platform"); 
-    QString act = getActionNameForPlatform(rule->getAction(),platform.c_str());
-
+    QString act = getActionNameForPlatform(Firewall::cast(p), rule);
     QString par = getRuleActionProperties(rule);
-    QString res = QObject::tr("<b>Action   :</b> ")+act+"<br>\n";
+    QString res = QObject::tr("<b>Action   :</b> %1<br>").arg(act);
     if (!par.isEmpty())
     {
-        res+=QObject::tr("<b>Parameter:</b> ")+par;
+        res += QObject::tr("<b>Parameter:</b> ")+par;
     }
     return res;
 }
@@ -718,36 +992,41 @@ QString FWObjectPropertiesFactory::getPolicyRuleOptions(Rule *rule)
             
             if (ropt->getInt("limit_value")>0)
             {
-                res+=QObject::tr("<b>Limit Value   :</b> ");
-                res+=QString(ropt->getStr("limit_value").c_str())+"<br>\n";
+                res += QObject::tr("<b>Limit value   :</b> ");
+                if (ropt->getBool("limit_value_not")) res += " ! ";
+                res += QString(ropt->getStr("limit_value").c_str())+" ";
+           
+                if (!ropt->getStr("limit_suffix").empty())
+                {
+                    res+=getScreenName(ropt->getStr("limit_suffix").c_str(),
+                                       getLimitSuffixes(platform.c_str()))+"<br>\n";
+                }
             }
-            
-            if (!ropt->getStr("limit_suffix").empty())
-            {
-                res+=QObject::tr("<b>Limit suffix  :</b> ");
-                res+=getScreenName(ropt->getStr("limit_suffix").c_str(),
-                        getLimitSuffixes(platform.c_str()))+"<br>\n";
-            }
-            
+
             if (ropt->getInt("limit_burst")>0)
             {
                 res+=QObject::tr("<b>Limit burst   :</b> ");
                 res+=QString(ropt->getStr("limit_burst").c_str())+"<br>\n";
             }
+
+            if (ropt->getInt("connlimit_value")>0)
+            {
+                res+=QObject::tr("<b>connlimit value   :</b> ");
+                if (ropt->getBool("connlimit_above_not")) res += " ! ";
+                res+=QString(ropt->getStr("connlimit_value").c_str())+"<br>\n";
+            }
             
-            res+="<ul>";
             if (ropt->getBool("firewall_is_part_of_any_and_networks"))
             {
-                res+=QObject::tr("<li><b>Part of Any</b></li>");
+                res+=QObject::tr("<b>Part of Any</b>");
                 res+="<br>\n";
             }
 
             if (ropt->getBool("stateless"))
             {
-                res+=QObject::tr("<li><b>Stateless</b></li> ");
+                res+=QObject::tr("<b>Stateless</b>");
                 res+="<br>\n";
             }
-            res+="</ul>";
             
         }else if (platform=="ipf") 
         {
@@ -804,13 +1083,33 @@ QString FWObjectPropertiesFactory::getPolicyRuleOptions(Rule *rule)
             if (ropt->getBool("stateless"))
             {
                 res+=QObject::tr("<li><b>Stateless</b></li> ");
-                res+="<br>\n";
+                //res+="<br>\n";
+            }
+            if (ropt->getBool("pf_keep_state"))
+            {
+                res+=QObject::tr("<li><b>Force 'keep-state'</b></li> ");
+                //res+="<br>\n";
+            }
+            if (ropt->getBool("pf_no_sync"))
+            {
+                res+=QObject::tr("<li><b>no-sync</b></li> ");
+                //res+="<br>\n";
+            }
+            if (ropt->getBool("pf_pflow"))
+            {
+                res+=QObject::tr("<li><b>pflow</b></li> ");
+                //res+="<br>\n";
+            }
+            if (ropt->getBool("pf_sloppy_tracker"))
+            {
+                res+=QObject::tr("<li><b>sloppy-tracker</b></li> ");
+                //res+="<br>\n";
             }
             
             if (ropt->getBool("pf_source_tracking"))
             {
                 res+=QObject::tr("<li><b>Source tracking</b></li> ");
-                res+="<br>\n";
+                //res+="<br>\n";
                 
                 res+=QObject::tr("<b>Max src nodes :</b> ");
                 res+=QString(ropt->getStr("pf_max_src_nodes").c_str())+"<br>\n";
@@ -830,6 +1129,16 @@ QString FWObjectPropertiesFactory::getPolicyRuleOptions(Rule *rule)
                 res+="<br>\n";
             }
             res+="</ul>";
+
+        }else if (platform=="iosacl")
+        {
+            res+="<ul>";
+            if (ropt->getBool("iosacl_add_mirror_rule"))
+            {
+                res+=QObject::tr("<li><b>Add mirrored rule</b></li> ");
+                res+="<br>\n";
+            }
+            res+="</ul>";
             
         }else if (platform=="pix" || platform=="fwsm")
         {
@@ -884,16 +1193,24 @@ QString FWObjectPropertiesFactory::getNATRuleOptions(Rule *rule)
         FWOptions *ropt = rule->getOptionsObject();
 
         if (fwbdebug)
-            qDebug(QString("getNATRuleOptions: platform: %2").arg(platform.c_str()).toAscii().constData());
+            qDebug() << "getNATRuleOptions: platform: " << platform.c_str();
+
+        if (platform=="iptables")
+        {
+            if (ropt->getBool("ipt_use_snat_instead_of_masq"))
+                res += QObject::tr("use SNAT instead of MASQ<br>");
+            if (ropt->getBool("ipt_nat_random"))      res += QObject::tr("random<br>");
+            if (ropt->getBool("ipt_nat_persistent"))  res += QObject::tr("persistent<br>");
+        }
             
         if (platform=="pf")
         {
-            if (ropt->getBool("pf_bitmask"))      res+=QObject::tr("bitmask");
-            if (ropt->getBool("pf_random"))       res+=QObject::tr("random");
-            if (ropt->getBool("pf_source_hash"))  res+=QObject::tr("source-hash");
-            if (ropt->getBool("pf_round_robin"))  res+=QObject::tr("round-robin");
+            if (ropt->getBool("pf_bitmask"))      res+=QObject::tr("bitmask<br>");
+            if (ropt->getBool("pf_random"))       res+=QObject::tr("random<br>");
+            if (ropt->getBool("pf_source_hash"))  res+=QObject::tr("source-hash<br>");
+            if (ropt->getBool("pf_round_robin"))  res+=QObject::tr("round-robin<br>");
             if (!res.isEmpty()) res += ",";
-            if (ropt->getBool("pf_static_port"))  res+=QObject::tr("static-port");
+            if (ropt->getBool("pf_static_port"))  res+=QObject::tr("static-port<br>");
         }
     }
     
