@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindow.h 973 2009-05-29 16:15:31Z vadim $
+  $Id: FWWindow.h 2686 2010-03-08 01:13:19Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,29 +27,22 @@
 #ifndef  __FWWINDOW_H_
 #define  __FWWINDOW_H_
 
-#include <ui_FWBMainWindow_q.h>
-#include <ui_pagesetupdialog_q.h>
+//#include <ui_FWBMainWindow_q.h>
+//#include <ui_pagesetupdialog_q.h>
 
 #include "RCS.h"
-#include "ObjectEditor.h"
 #include "HttpGet.h"
 #include "printerStream.h"
+#include "ObjectEditor.h"
 
-#include <qstring.h>
+#include <QString>
 #include <QShowEvent>
 #include <QHideEvent>
+#include <QMainWindow>
 
 #include <vector>
 #include <list>
-
-namespace libfwbuilder {
-    class FWObjectDatabase;
-    class Firewall;
-    class PolicyRule;
-    class RuleSet;
-    class Rule;
-    class FWObject;
-};
+#include <set>
 
 class ObjectManipulator;
 class ObjectTreeView;
@@ -62,23 +55,53 @@ class QTabWidget;
 class QTextEdit;
 class QTimer;
 class QTreeWidgetItem;
+class QDockWidget;
+class QUndoGroup;
+
 class RuleSetView;
 class findDialog;
+class FindObjectWidget;
+class FindWhereUsedWidget;
+class CompilerOutputPanel;
+
+namespace Ui {
+    class FWBMainWindow_q;
+    class pageSetupDialog_q;
+};
+
+namespace libfwbuilder {
+    class FWObjectDatabase;
+    class Firewall;
+    class PolicyRule;
+    class RuleSet;
+    class Rule;
+    class FWObject;
+    class FWReference;
+};
+
 
 #define MAXRECENTFILES 5
+#define EDITOR_PANEL_EDITOR_TAB 2
+#define EDITOR_PANEL_SEARCH_TAB 1
+#define EDITOR_PANEL_OUTPUT_TAB 0
+
 
 class FWWindow : public QMainWindow {
 
-    Q_OBJECT
+    Q_OBJECT;
 
     Ui::pageSetupDialog_q *psd;
         
     QMdiArea *m_space;
+    QMdiSubWindow *previous_subwindow;
     QWidget *instd;
     HttpGet *current_version_http_getter;
     
     QTimer *instDialogOnScreenTimer;
     QString noFirewalls;
+
+    QWidget *editorOwner;
+
     QPrinter *printer;
     libfwbuilder::FWObject *searchObject;
     libfwbuilder::FWObject *replaceObject;
@@ -87,41 +110,63 @@ class FWWindow : public QMainWindow {
     bool auto_load_from_rcs_head_revision;
     QAction *recentFileActs[MAXRECENTFILES];
     QAction *openRecentSeparatorAct;
-
+    QAction *undoAction;
+    QAction *redoAction;
+    
     void clearFirewallTabs();
     ProjectPanel *newProjectPanel();
     void showSub(ProjectPanel *projectW);
-    
+    void attachEditorToProjectPanel(ProjectPanel *pp);
+
+    QList<QAction*> ruleStaticActions;
+
 public:
     QVector <QString> windowsTitles;
     QVector <QMdiSubWindow*> windowsPainters;
 
+    // Passwords are cached to this map if user enables remember password.
+    // Key is QPair<FWObject::getId(), User Name>
+    // and value is QPair<Password, Enable Password>
+    QMap<QPair<int, QString>, QPair<QString, QString> > passwords;
+
+    ObjectEditor *oe;
+    FindObjectWidget *findObjectWidget;
+    FindWhereUsedWidget *findWhereUsedWidget;
+    QUndoGroup *undoGroup;
+
     ProjectPanel* activeProject();
     void updateWindowTitle();
     void updateTreeFont();
     void updateRecentFileActions();
     void updateOpenRecentMenu(const QString &fileName);
 
+    void enableBackAction();
+    
 public slots:
     void selectActiveSubWindow (/*const QString & text*/);
+    void subWindowActivated(QMdiSubWindow*);
 
+    void editorPanelTabChanged(int);
+    
     void minimize();
     void maximize();
-    void changeActiveSubwindow();
     virtual void search();
 
-    virtual void reopenFirewall();
-    virtual void redrawRuleSets();
-    virtual void changeInfoStyle();
     virtual void restoreRuleSetTab();
 
     virtual void editFind();
-    virtual void editRedo();
-    virtual void editUndo();
+
+    virtual void toggleViewObjectTree();
+    virtual void toggleViewEditor();
+    virtual void toggleViewUndo();
+
     virtual void helpContents();
     virtual void helpContentsAction();
     virtual void helpIndex();
     virtual void help();
+    virtual void showSummary();
+    virtual void showTutorial(QString tutorial = QString());
+    virtual void showReleaseNotes();
 
     virtual void fileNew();
     virtual void fileOpen();
@@ -155,6 +200,8 @@ public slots:
     virtual void compile();
     virtual void install(std::set<libfwbuilder::Firewall * > vf);
     virtual void install();
+    virtual void transferfw(std::set<libfwbuilder::Firewall * > vf);
+    virtual void transferfw();
 
     virtual void insertRule();
     virtual void addRuleAfterCurrent();
@@ -174,15 +221,16 @@ public slots:
     virtual void unlockObject();
 
     virtual void prepareEditMenu();
+    virtual void prepareViewMenu();
     virtual void prepareObjectMenu();
     virtual void prepareFileMenu();
+    virtual void prepareToolsMenu();
     virtual void prepareWindowsMenu();
     virtual void prepareFileOpenRecentMenu();
+    virtual void prepareRulesMenu();
+    virtual void cleanRulesMenu();
 
     virtual void toolsDiscoveryDruid();
-    virtual void closeAuxiliaryPanel();
-    virtual void closeEditorPanel();
-    virtual void openEditorPanel();
 
     virtual void killInstDialog();
 
@@ -191,6 +239,12 @@ public slots:
     virtual void projectWindowClosed();
 
     void tableResolutionSettingChanged(int );
+
+    void activateRule(ProjectPanel*, QString, QString, int);
+
+    void undoViewVisibilityChanged(bool);
+
+
     
  public:
     Ui::FWBMainWindow_q *m_mainWindow;
@@ -199,46 +253,27 @@ public slots:
     ~FWWindow();
 
 
-    QMdiArea* getMdiArea () {return m_space;}
     RCS * getRCS(); 
 
-    libfwbuilder::FWObject* getVisibleFirewalls();
-
     void registerAutoOpenDocFile(const QString &file_name,
                                  bool load_from_rcs_head);
     void load(QWidget *dialogs_parent,RCS *rcs);
     void load(QWidget *dialogs_parent);
     void loadLibrary(const std::string &libfpath);
-    void loadFile(const QString &filename, bool load_rcs_head);
+    bool loadFile(const QString &filename, bool load_rcs_head);
     void save();
     bool checkin(bool unlock);
     int  findFirewallInList(libfwbuilder::FWObject *f);
-    void updateTreeViewItemOrder();
-
-    void reloadAllWindowsWithFile(ProjectPanel *pp);
-    void closeRuleSetInAllWindowsWhereOpen(libfwbuilder::RuleSet *rs);
-    void closeObjectInAllWindowsWhereOpen(libfwbuilder::FWObject *obj);
+    QStringList getListOfOpenedFiles();
+    QMdiSubWindow* alreadyOpened(const QString &file_path);
     
     bool editingLibrary();
 
-    void ensureObjectVisibleInRules(libfwbuilder::FWReference *obj);
-    void ensureRuleIsVisible(libfwbuilder::Rule *rule, int col=0);
-
     QString chooseNewFileName(const QString &fname, const QString &title);
     void setFileName(const QString &fname);
     
     bool saveIfModified();
     
-    void updateFirewallName();
-    void updateRuleSetView();
-    void updateRuleOptions();
-    void updateRuleSetViewSelection();
-    
-    /**
-     * unselects whatever is selected in policy
-     */
-    void unselectRules();
-
     /**
      * selects whatever is current in rules
      */
@@ -249,22 +284,11 @@ public slots:
 
     QString getCurrentFileName();
     
-    void info(libfwbuilder::FWObject *o, bool forced = false);
-
     void setupAutoSave();
-    void findObject(libfwbuilder::FWObject *);
 
-    void findWhereUsed(libfwbuilder::FWObject *);
+    void findObject(libfwbuilder::FWObject *);
+    void findWhereUsed(libfwbuilder::FWObject *obj, ProjectPanel *pp);
     
-    /**
-     * panel that wants to open an object in the editor
-     * uses this method to request permission to do so and
-     * to register itself as an owner of the editor
-     */
-    bool requestEditorOwnership(QWidget *w,
-                                libfwbuilder::FWObject *o,
-                                ObjectEditor::OptType   otype,
-                                bool validate = true);
     bool exportLibraryTest(std::list<libfwbuilder::FWObject*> &selectedLibs);
     void exportLibraryTo(QString fname,std::list<libfwbuilder::FWObject*> &selectedLibs, bool rof);
 
@@ -275,27 +299,8 @@ public slots:
     void setSafeMode(bool f);
     void setStartupFileName(const QString &fn);
 
-    void scheduleRuleSetRedraw();
-
-    // semi-intelligent way to guess most appropriate
-    // destination directory for various file save or file open
-    // operations. If working directory is configured in
-    // preferences, then getDestDir returns that. If it is not
-    // configured and file name is given on the command line,
-    // directory where that file is located is returned. If
-    // parameter filename is empty, then current directory
-    // is returned (however on windows and mac userDataDir is returned)
-
-    QString getDestDir(const QString &filename);
-    
-    //wrapers for some ObjectManipulator functions
-    libfwbuilder::FWObject* getOpened();
-    
     libfwbuilder::FWObject*  getCurrentLib();
 
-    void insertObjectInTree(libfwbuilder::FWObject *parent,
-                            libfwbuilder::FWObject *obj);
-
     libfwbuilder::FWObject* createObject(const QString &objType,
                                           const QString &objName,
                                           libfwbuilder::FWObject *copyFrom=NULL);
@@ -311,53 +316,30 @@ public slots:
     void moveObject(const QString &targetLibName,
                     libfwbuilder::FWObject *obj);
 
-    void autorename(libfwbuilder::FWObject *obj,
-                    const std::string &objtype,
-                    const std::string &namesuffix);
-
-    void updateLibColor(libfwbuilder::FWObject *lib);
-    void updateLibName(libfwbuilder::FWObject *lib);
-
-    void updateObjName(libfwbuilder::FWObject *obj,
-                       const QString &oldName,
-                       bool  askForAutorename=true);
-    void updateObjName(libfwbuilder::FWObject *obj,
-                       const QString &oldName,
-                       const QString &oldLabel,
-                       bool  askForAutorename=true);
-
-    void updateLastModifiedTimestampForOneFirewall(libfwbuilder::FWObject *o);
-    void updateLastModifiedTimestampForAllFirewalls(libfwbuilder::FWObject *o);   
-    void updateLastInstalledTimestamp(libfwbuilder::FWObject *o);
-    void updateLastCompiledTimestamp(libfwbuilder::FWObject *o);
-       
-    void loadDataFromFw(libfwbuilder::Firewall *fw);
-
-    libfwbuilder::FWObject* pasteTo(libfwbuilder::FWObject *target,
-                                    libfwbuilder::FWObject *obj);
-    void delObj(libfwbuilder::FWObject *obj,bool openobj=true);
     ObjectTreeView* getCurrentObjectTree();
-    void openObject(QTreeWidgetItem *otvi);
-    void openObject(libfwbuilder::FWObject *obj);
-    bool editObject(libfwbuilder::FWObject *obj);
+
     void findAllFirewalls (std::list<libfwbuilder::Firewall *> &fws);
 
-    libfwbuilder::FWObject* duplicateObject(libfwbuilder::FWObject *target,
-                                            libfwbuilder::FWObject *obj,
-                                            const QString &name = QString::null,
-                                            bool  askForAutorename=true);
     void showDeletedObjects(bool f);
-    void select();
-    void unselect();
-    void info();
 
+//    void select();
+//    void unselect();
     
-    void setManipulatorFocus();
-    void clearManipulatorFocus();
-    
+    QPrinter* getPrinter();
+    libfwbuilder::FWObjectDatabase* db();
+    QString printHeader();
+
+    static void printFirewallFromFile(QString fileName,
+                                      QString firewallName,
+                                      QString outputFileName);
+
+    /**
+     * show text string on status bar for 1 sec
+     */
+    void showStatusBarMessage(const QString &txt);
+
     //wrapers for some Object Editor functions
     bool isEditorVisible();
-    bool isEditorModified();
     
     void showEditor();
     void hideEditor();
@@ -370,30 +352,31 @@ public slots:
     libfwbuilder::FWObject* getOpenedEditor();
     ObjectEditor::OptType getOpenedOptEditor();
     
-    void selectObjectInEditor(libfwbuilder::FWObject *o);
+    virtual void closeEditorPanel();
+    virtual void openEditorPanel();
     
-    void actionChangedEditor(libfwbuilder::FWObject *o);
-    bool validateAndSaveEditor();
-    //find dialog functions wrapers
-    void setFDObject(libfwbuilder::FWObject *o);
+    bool requestEditorOwnership(QWidget *w,
+                                libfwbuilder::FWObject *o,
+                                ObjectEditor::OptType   otype,
+                                bool validate = true);
+    void singleRuleCompile(libfwbuilder::Rule *rule);
+
+    void buildEditorTitleAndIcon(libfwbuilder::FWObject *obj,
+                                 ObjectEditor::OptType t,
+                                 QString *title_txt,
+                                 QPixmap *pm,
+                                 bool include_file_name);
     
-    QPrinter* getPrinter();
-    libfwbuilder::FWObjectDatabase* db();
-    QString printHeader();
-
-    static void printFirewallFromFile(QString fileName,
-                                      QString firewallName,
-                                      QString outputFileName);
-
-
-    bool isSystem(libfwbuilder::FWObject *obj);
-
  protected:
 
-    virtual void showEvent( QShowEvent *ev);
-    virtual void hideEvent( QHideEvent *ev);
-    virtual void closeEvent( QCloseEvent * );
+    virtual void showEvent(QShowEvent *ev);
+    virtual void hideEvent(QHideEvent *ev);
+    virtual void closeEvent(QCloseEvent *ev);
+    virtual bool event(QEvent *event);
 
+ protected slots:
+    void activatePreviousSubWindow();
+ 
 };
 
 #endif
