@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindowPrint.cpp 928 2009-05-12 22:08:42Z vadim $
+  $Id: FWWindowPrint.cpp 2658 2010-03-04 20:26:28Z a2k $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -29,6 +29,8 @@
 #include "utils.h"
 #include "platforms.h"
 
+#include <ui_FWBMainWindow_q.h>
+#include <ui_pagesetupdialog_q.h>
 #include "FWWindow.h"
 #include "FWBSettings.h"
 #include "PrintingProgressDialog.h"
@@ -46,6 +48,7 @@
 #include <QStatusBar>
 #include <QMdiArea>
 #include <QMdiSubWindow>
+#include <QtDebug>
 
 using namespace libfwbuilder;
 using namespace std;
@@ -55,7 +58,7 @@ void FWWindow::filePrint()
 {
     if (!activeProject())
     {
-        if (fwbdebug) qDebug("There isn't any selected subwindow");
+        if (fwbdebug) qDebug() << "There isn't any selected subwindow";
         return;
     }
 
@@ -82,7 +85,7 @@ void FWWindow::filePrint()
     }
 
     // Need error dialog
-    if (!Firewall::isA(firewall_to_print)) return;
+    if (!Firewall::cast(firewall_to_print)) return;
 
 
     if (!st->getStr("PrintSetup/newPageForSection").isEmpty())
@@ -143,20 +146,28 @@ void FWWindow::filePrint()
         //printer->setResolution(resolution);
         printer->setFullPage(fullPage);
 
+        if (fwbdebug) qDebug() << "Running QPrintDialog";
+
         QPrintDialog pdialog(printer, this);
 
+        pdialog.setWindowTitle(tr("Print configuration of %1")
+                               .arg(firewall_to_print->getName().c_str()));
+#ifndef Q_OS_MACX
         pdialog.addEnabledOption(QAbstractPrintDialog::PrintPageRange);
         pdialog.setMinMax(1,9999);
         pdialog.setPrintRange(QAbstractPrintDialog::AllPages);
+#endif
 
-        if (pdialog.exec())
+        if (pdialog.exec() == QDialog::Accepted)
         {
+            if (fwbdebug) qDebug() << "QPrintDialog finished";
+
             int fromPage = printer->fromPage();
             int toPage = printer->toPage();
-            if (fromPage==0) fromPage=1;
-            if (toPage==0) toPage=9999;
+            if (fromPage==0) fromPage = 1;
+            if (toPage==0) toPage = 9999;
 
-            statusBar()->showMessage( "Printing..." );
+            showStatusBarMessage(tr("Printing..." ));
 
             PrintingProgressDialog *ppd = 
                 new PrintingProgressDialog(this, printer, 0, false);
@@ -174,7 +185,7 @@ void FWWindow::filePrint()
             pr.setFromTo(fromPage, toPage);
 
             if (fwbdebug)
-                qDebug("Printer resolution: %d dpi", printer->resolution());
+                qDebug() << "Printer resolution (dpi):" << printer->resolution();
 
             if ( !pr.begin())
             {
@@ -197,18 +208,19 @@ void FWWindow::filePrint()
 
             if (printer->printerState() == QPrinter::Aborted)
             {
-                statusBar()->showMessage( tr("Printing aborted"), 2000 );
+                showStatusBarMessage(tr("Printing aborted"));
                 QMessageBox::information(
                     this,"Firewall Builder",
                     tr("Printing aborted"),
                     tr("&Continue"), QString::null,QString::null,
                     0, 1 );
             } else
-                statusBar()->showMessage( tr("Printing completed"), 2000 );
+                showStatusBarMessage(tr("Printing completed"));
 
         } else
         {
-            statusBar()->showMessage( tr("Printing <ed"), 2000 );
+            if (fwbdebug) qDebug() << "QPrintDialog cancelled";
+            showStatusBarMessage(tr("Printing cancelled"));
         }
 
         st->setPrinterOptions(printer,pageWidth,pageHeight);
@@ -305,9 +317,7 @@ void FWWindow::printFirewallFromFile(QString fileName,
     }
     else
     {   
-        QString errmes = "Error: can't find firewall ";
-        errmes += firewallName ;
-        qDebug (errmes.toAscii().data());
+        qDebug() << "Error: can't find firewall " << firewallName;
     }
 }
 
