@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: GroupObjectDialog.cpp 952 2009-05-20 21:54:45Z vadim $
+  $Id: GroupObjectDialog.cpp 2219 2009-12-20 03:55:26Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,9 +27,9 @@
 #include "../../config.h"
 #include "global.h"
 #include "utils.h"
-#include "ProjectPanel.h"
+#include "events.h"
 
-#include "FWWindow.h"
+#include "ProjectPanel.h"
 #include "FWBTree.h"
 #include "FWBSettings.h"
 #include "FWObjectPropertiesFactory.h"
@@ -37,12 +37,19 @@
 #include "FWObjectDrag.h"
 #include "FWObjectClipboard.h"
 #include "ObjectTreeView.h"
+#include "FWWindow.h"
+#include "FWCmdChange.h"
 
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/Library.h"
 #include "fwbuilder/Group.h"
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/ServiceGroup.h"
+#include "fwbuilder/FWObjectReference.h"
+#include "fwbuilder/FWServiceReference.h"
+#include "fwbuilder/FWIntervalReference.h"
+
+#include <memory>
 
 #include <qlineedit.h>
 #include <qtextedit.h>
@@ -58,28 +65,34 @@
 #include <qtooltip.h>
 #include <qscrollarea.h>
 #include <qpixmapcache.h>
+#include <QCoreApplication>
+#include <QUndoStack>
+#include <QtDebug>
+#include <QHeaderView>
 
 #include <iostream>
 #include <algorithm>
-#include "FWBSettings.h"
+
 
 using namespace std;
 using namespace libfwbuilder;
 
+
 enum GroupObjectDialog::viewType GroupObjectDialog::vt = GroupObjectDialog::Icon;
 
 #define LIST_VIEW_MODE "list"
 #define ICON_VIEW_MODE "icon"
 
 
-GroupObjectDialog::GroupObjectDialog(ProjectPanel *project, QWidget *parent) :
-        QWidget(parent), m_project(project)
+GroupObjectDialog::GroupObjectDialog(QWidget *parent) :
+        BaseObjectDialog(parent)
 {
     m_dialog = new Ui::GroupObjectDialog_q;
     m_dialog->setupUi(this);
 
-    obj=NULL;
-    selectedObject=NULL;
+    obj = NULL;
+    selectedObject = NULL;
+    new_object_menu = NULL;
 
     listView = new ObjectListView( m_dialog->objectViewsStack, "listView" );
     QStringList sl;
@@ -94,8 +107,11 @@ GroupObjectDialog::GroupObjectDialog(ProjectPanel *project, QWidget *parent) :
 
     m_dialog->objectViewsStack->addWidget(iconView);
     m_dialog->objectViewsStack->addWidget(listView);
-    m_dialog->objectViewsStack->setCurrentWidget(iconView);
 
+    if (st->getGroupViewMode() == ICON_VIEW_MODE)
+        m_dialog->objectViewsStack->setCurrentWidget(iconView);
+    else
+        m_dialog->objectViewsStack->setCurrentWidget(listView);
 
     setTabOrder( m_dialog->obj_name, iconView );
     setTabOrder( iconView, listView );
@@ -114,11 +130,6 @@ GroupObjectDialog::GroupObjectDialog(ProjectPanel *project, QWidget *parent) :
     m_dialog->iconViewBtn->setAutoRaise(false);
     m_dialog->listViewBtn->setAutoRaise(false);
 
-    //apply->setEnabled( false );
-
-    connect( iconView, SIGNAL( itemActivated(QListWidgetItem*) ),
-             this,     SLOT( openObject(QListWidgetItem*) ) );
-
     connect( iconView, SIGNAL( currentItemChanged(QListWidgetItem*,QListWidgetItem*) ),
              this,     SLOT( iconViewCurrentChanged(QListWidgetItem*) ) );
 
@@ -134,9 +145,10 @@ GroupObjectDialog::GroupObjectDialog(ProjectPanel *project, QWidget *parent) :
     connect( iconView, SIGNAL( delObject_sign() ),
              this,     SLOT( deleteObj() ) );
 
+    connect( iconView, SIGNAL(itemDoubleClicked(QListWidgetItem*)),
+             this, SLOT(itemDoubleClicked(QListWidgetItem*)));
+
 
-    connect( listView, SIGNAL( itemActivated(QTreeWidgetItem*,int) ),
-             this,     SLOT( openObject(QTreeWidgetItem*) ) );
     connect( listView, SIGNAL( currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*) ),
              this,     SLOT( listViewCurrentChanged(QTreeWidgetItem*) ) );
     connect( listView, SIGNAL (itemSelectionChanged()),
@@ -149,6 +161,9 @@ GroupObjectDialog::GroupObjectDialog(ProjectPanel *project, QWidget *parent) :
     connect( listView, SIGNAL( delObject_sign() ),
              this,     SLOT( deleteObj() ) );
 
+    connect( listView, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)),
+             this, SLOT(itemDoubleClicked(QTreeWidgetItem*, int)));
+
     QString s    = st->getGroupViewColumns();
     int     col0 = s.section(',',0,0).toInt();
     int     col1 = s.section(',',1,1).toInt();
@@ -159,9 +174,9 @@ GroupObjectDialog::GroupObjectDialog(ProjectPanel *project, QWidget *parent) :
     listView->setColumnWidth(0,col0);
     listView->setColumnWidth(1,col1);
 
-    QString mode=st->getGroupViewMode();
+    QString mode = st->getGroupViewMode();
     if (mode==ICON_VIEW_MODE) switchToIconView();
-    if (mode==LIST_VIEW_MODE) switchToListView();
+    else switchToListView();
 }
 
 GroupObjectDialog::~GroupObjectDialog()
@@ -213,7 +228,7 @@ void GroupObjectDialog::iconViewCurrentChanged(QListWidgetItem *itm)
         return;
     }
     int obj_id = itm->data(Qt::UserRole).toInt();
-    FWObject *o = mw->db()->findInIndex(obj_id);
+    FWObject *o = m_project->db()->findInIndex(obj_id);
     selectedObject = o;
 }
 
@@ -226,7 +241,7 @@ void GroupObjectDialog::listViewCurrentChanged(QTreeWidgetItem *itm)
         return;
     }
     int obj_id = itm->data(0, Qt::UserRole).toInt();
-    FWObject *o = mw->db()->findInIndex(obj_id);
+    FWObject *o = m_project->db()->findInIndex(obj_id);
     selectedObject = o;
 }
 
@@ -255,7 +270,7 @@ void GroupObjectDialog::insertObject(FWObject *o)
 	if(obj_id==cp_id) return;
     }
 
-    addIcon(o, ! m_project->isSystem(obj) );
+    addIcon(o, ! FWBTree().isSystem(obj) );
 
     changed();
 }
@@ -311,33 +326,26 @@ void GroupObjectDialog::getHelpName(QString *str)
 
 void GroupObjectDialog::loadFWObject(FWObject *o)
 {
-    obj=o;
+    obj = o;
     Group *g = Group::cast(obj);
     assert(g!=NULL);
-    if (ServiceGroup::cast(obj)!=NULL)
-    {
-        m_dialog->icon->setPixmap(QIcon(":/Icons/ServiceGroup/icon").pixmap(
-                                      25,25));
-    }
-    else
-    {
-        m_dialog->icon->setPixmap(QIcon(":/Icons/ObjectGroup/icon").pixmap(
-                                      25,25));
-    }
+
     init=true;
 
     m_dialog->obj_name->setText( QString::fromUtf8(g->getName().c_str()) );
     m_dialog->comment->setText( QString::fromUtf8(g->getComment().c_str()) );
 
-    m_dialog->obj_name->setEnabled( !m_project->isSystem(obj) );
-    m_dialog->comment->setEnabled(  !m_project->isSystem(obj) );
+    m_dialog->obj_name->setEnabled( !FWBTree().isSystem(obj) );
+    m_dialog->comment->setEnabled(  !FWBTree().isSystem(obj) );
 
     listView->clear();
-
     iconView->clear();
 
+    listView->setDB(o->getRoot());
+    iconView->setDB(o->getRoot());
+
     iconView->setResizeMode( QListWidget::Adjust );
-    iconView->setGridSize ( QSize(50, 40) );
+    iconView->setGridSize( QSize(50, 40) );
 
     switch (vt)
     {
@@ -355,13 +363,14 @@ void GroupObjectDialog::loadFWObject(FWObject *o)
     for (FWObject::iterator i=g->begin(); i!=g->end(); i++)
         addIcon( *i );
 
+    listView->header()->resizeSections(QHeaderView::ResizeToContents);
 
     //apply->setEnabled( false );
 
-    m_dialog->obj_name->setEnabled(!o->isReadOnly() && !m_project->isSystem(o));
+    m_dialog->obj_name->setEnabled(!o->isReadOnly() && !FWBTree().isSystem(o));
     setDisabledPalette(m_dialog->obj_name);
 
-    m_dialog->comment->setEnabled(!o->isReadOnly() && !m_project->isSystem(o));
+    m_dialog->comment->setEnabled(!o->isReadOnly() && !FWBTree().isSystem(o));
     setDisabledPalette(m_dialog->comment);
 
 //    listView->setEnabled(!o->isReadOnly());
@@ -370,21 +379,45 @@ void GroupObjectDialog::loadFWObject(FWObject *o)
 //    iconView->setEnabled(!o->isReadOnly());
     setDisabledPalette(iconView);
 
-
     init=false;
-}
 
-void GroupObjectDialog::changed()
-{
-    //if (!init) apply->setEnabled( true );
+    if (FWBTree().isSystem(g))
+    {
+        m_dialog->newButton->hide();
+        return;
+    }
 
-    emit changed_sign();
+    if (new_object_menu)
+    {
+        new_object_menu->clear();
+        m_dialog->newButton->setMenu(NULL);
+        delete new_object_menu;
+    }
+
+    new_object_menu = new QMenu(this);
+
+    int add_to_group_id = g->getId();
+
+    list<string> types_list;
+    g->getAllowedTypesOfChildren(types_list);
+    foreach(string tn, types_list)
+    {
+        if (tn == FWObjectReference::TYPENAME ||
+            tn == FWServiceReference::TYPENAME ||
+            tn == FWIntervalReference::TYPENAME) continue;
+        if (fwbdebug)
+            qDebug() << "Adding type" << tn.c_str() << "to the new object menu";
+        m_project->m_panel->om->addNewObjectMenuItem(
+            new_object_menu, tn.c_str(), "", add_to_group_id);
+    }
+
+    m_dialog->newButton->setMenu( new_object_menu );
+    m_dialog->newButton->show();
 }
 
 void GroupObjectDialog::validate(bool *res)
 {
     *res=true;
-    if (!isTreeReadWrite(this,obj)) { *res=false; return; }
     if (!validateName(this, obj, m_dialog->obj_name->text()))
     {
         *res=false;
@@ -392,26 +425,21 @@ void GroupObjectDialog::validate(bool *res)
     }
 }
 
-void GroupObjectDialog::isChanged(bool*)
-{
-    //*res=(!init && apply->isEnabled());
-}
 
-void GroupObjectDialog::libChanged()
-{
-    changed();
-}
 
 void GroupObjectDialog::applyChanges()
 {
     if (fwbdebug) qDebug("GroupObjectDialog::applyChanges");
 
-    string oldname=obj->getName();
-    obj->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
-    obj->setComment(
+    std::auto_ptr<FWCmdChange> cmd( new FWCmdChange(m_project, obj));
+    FWObject* new_state = cmd->getNewState();
+
+    string oldname = obj->getName();
+    new_state->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
+    new_state->setComment(
         string(m_dialog->comment->toPlainText().toUtf8().constData()) );
 
-    init=true;
+    //init=true;
 
     set<int> oldobj;
     set<int> newobj;
@@ -440,11 +468,16 @@ void GroupObjectDialog::applyChanges()
 
     for (set<int>::iterator k=diff.begin(); k!=diff.end(); ++k)
     {
-        FWObject *o = mw->db()->findInIndex(*k);
-        if (m_project->isSystem(obj))
-            mw->delObj(o, false);
-        else
-            obj->removeRef(o);
+        FWObject *o = m_project->db()->findInIndex(*k);
+        // Note: FWBTree::isSystem() would not work for new_state because
+        // it is not part of the tree and isSystem() relies on the tree path
+        if (FWBTree().isSystem(obj))
+        {
+            m_project->m_panel->om->deleteObject(o);
+        } else
+        {
+            new_state->removeRef(o);
+        }
     }
 
     diff.clear();
@@ -457,29 +490,28 @@ void GroupObjectDialog::applyChanges()
 
     for (set<int>::iterator k1=diff.begin(); k1!=diff.end(); ++k1)
     {
-        FWObject *o = mw->db()->findInIndex(*k1);
-        if (m_project->isSystem(o))
-            mw->pasteTo(obj, o);
-        else
-            obj->addRef(o);
+        FWObject *o = m_project->db()->findInIndex(*k1);
+        if (FWBTree().isSystem(obj))
+        {
+            m_project->pasteTo(new_state, o);
+        } else
+        {
+            new_state->addRef(o);
+        }
     }
 
-    mw->updateObjName(obj, QString::fromUtf8(oldname.c_str()));
+    saveColumnWidths();
 
-    //apply->setEnabled( false );
-    mw->updateLastModifiedTimestampForAllFirewalls(obj);
-
-    if (fwbdebug) qDebug("GroupObjectDialog::applyChanges done");
-}
-
-void GroupObjectDialog::discardChanges()
-{
-    loadFWObject(obj);
+    if (!cmd->getOldState()->cmp(new_state, true))
+    {
+        if (obj->isReadOnly()) return;
+        m_project->undoStack->push(cmd.release());
+    }
 }
 
 void GroupObjectDialog::switchToIconView()
 {
-    if (vt == Icon) return;
+    //if (vt == Icon) return;
     vt = Icon;
 
     if ( ! m_dialog->iconViewBtn->isChecked() ) m_dialog->iconViewBtn->toggle();
@@ -491,7 +523,7 @@ void GroupObjectDialog::switchToIconView()
 
 void GroupObjectDialog::switchToListView()
 {
-    if (vt == List) return;
+    //if (vt == List) return;
     vt = List;
 
     if ( ! m_dialog->listViewBtn->isChecked() ) m_dialog->listViewBtn->toggle();
@@ -501,34 +533,17 @@ void GroupObjectDialog::switchToListView()
     st->setGroupViewMode(LIST_VIEW_MODE);
 }
 
+// This method is attached to the context menu item "Edit"
 void GroupObjectDialog::openObject()
 {
     if (selectedObject!=NULL)
     {
-        mw->openObject( selectedObject );
-        mw->editObject( selectedObject );
-    }
-}
-
-void GroupObjectDialog::openObject(QTreeWidgetItem *itm)
-{
-    int obj_id = itm->data(0, Qt::UserRole).toInt();
-    FWObject *o = mw->db()->findInIndex(obj_id);
-    if (o!=NULL)
-    {
-        mw->openObject( o );
-        mw->editObject( o );
-    }
-}
-
-void GroupObjectDialog::openObject(QListWidgetItem *itm)
-{
-    int obj_id = itm->data(Qt::UserRole).toInt();
-    FWObject *o = mw->db()->findInIndex(obj_id);
-    if (o!=NULL)
-    {
-        mw->openObject( o );
-        mw->editObject( o );
+        QCoreApplication::postEvent(
+            m_project, new showObjectInTreeEvent(selectedObject->getRoot()->getFileName().c_str(),
+                                                 selectedObject->getId()));
+        QCoreApplication::postEvent(
+            mw, new openObjectInEditorEvent(selectedObject->getRoot()->getFileName().c_str(),
+                                            selectedObject->getId()));
     }
 }
 
@@ -547,7 +562,7 @@ void GroupObjectDialog::dropped(QDropEvent *ev)
 
         // see comment in ObjectTreeView.cpp explaining the purpose of
         // flag process_mouse_release_event
-        ObjectTreeView *otv = mw->getCurrentObjectTree();
+        ObjectTreeView *otv = m_project->getCurrentObjectTree();
         otv->ignoreNextMouseReleaseEvent();
 
     }
@@ -561,7 +576,7 @@ void GroupObjectDialog::iconContextMenu(const QPoint & pos)
     if (itm)
     {
         int obj_id = itm->data(Qt::UserRole).toInt();
-        o = mw->db()->findInIndex(obj_id);
+        o = m_project->db()->findInIndex(obj_id);
         selectedObject = o;
     }
     setupPopupMenu(iconView->mapToGlobal(pos));
@@ -575,7 +590,7 @@ void GroupObjectDialog::listContextMenu(const QPoint & pos)
     if (itm)
     {
         int obj_id = itm->data(0, Qt::UserRole).toInt();
-        o = mw->db()->findInIndex(obj_id);
+        o = m_project->db()->findInIndex(obj_id);
         selectedObject = o;
     }
     setupPopupMenu(listView->viewport()->mapToGlobal(pos));
@@ -599,14 +614,14 @@ void GroupObjectDialog::setupPopupMenu(const QPoint &pos)
     QAction *delID =popup->addAction(tr("Delete"),this, SLOT(deleteObj()));
 
     copyID->setEnabled(selectedObject!=NULL &&
-                          ! m_project->isSystem(selectedObject) );
+                          ! FWBTree().isSystem(selectedObject) );
     cutID->setEnabled(selectedObject!=NULL &&
-                          ! m_project->isSystem(obj) &&
+                          ! FWBTree().isSystem(obj) &&
                           ! obj->isReadOnly() );
-    pasteID->setEnabled(! m_project->isSystem(obj) &&
+    pasteID->setEnabled(! FWBTree().isSystem(obj) &&
                           ! obj->isReadOnly() );
     delID->setEnabled(selectedObject!=NULL &&
-                          ! m_project->isSystem(obj) &&
+                          ! FWBTree().isSystem(obj) &&
                           ! obj->isReadOnly() );
 
     popup->exec( pos );
@@ -618,9 +633,9 @@ void GroupObjectDialog::copyObj()
     for(vector<int>::iterator it=selectedObjects.begin(); 
         it!=selectedObjects.end(); ++it)
     {
-        FWObject* selectedObject = mw->db()->findInIndex(*it);
+        FWObject* selectedObject = m_project->db()->findInIndex(*it);
 
-        if (selectedObject!=NULL && ! m_project->isSystem(selectedObject) )
+        if (selectedObject!=NULL && ! FWBTree().isSystem(selectedObject) )
         {
             FWObjectClipboard::obj_clipboard->add(selectedObject,
                                                   this->m_project );
@@ -642,7 +657,7 @@ void GroupObjectDialog::pasteObj()
     for (i= FWObjectClipboard::obj_clipboard->begin();
          i!=FWObjectClipboard::obj_clipboard->end(); ++i)
     {
-        insertObject( mw->db()->findInIndex(i->first) );
+        insertObject( m_project->db()->findInIndex(i->first) );
     }
 }
 
@@ -658,7 +673,7 @@ void GroupObjectDialog::deleteObj()
         if (fwbdebug)
             qDebug("GroupObjectDialog::deleteObj()  (*it)=%d", (*it));
 
-        FWObject* selectedObject = mw->db()->findInIndex(*it);
+        FWObject* selectedObject = m_project->db()->findInIndex(*it);
         int o_id = selectedObject->getId();
 
         for (int it=0; it<listView->topLevelItemCount(); ++it)
@@ -696,21 +711,6 @@ void GroupObjectDialog::saveColumnWidths()
     st->setGroupViewColumns(s);
 }
 
-/* ObjectEditor class connects its slot to this signal and does all
- * the verification for us, then accepts (or not) the event. So we do
- * nothing here and defer all the processing to ObjectEditor
- */
-void GroupObjectDialog::closeEvent(QCloseEvent *e)
-{
-    saveColumnWidths();
-    emit close_sign(e);
-}
-
-void GroupObjectDialog::hideEvent(QHideEvent *)
-{
-    saveColumnWidths();
-}
-
 void GroupObjectDialog::selectObject(FWObject *o)
 {
     int o_id = o->getId();
@@ -735,3 +735,18 @@ void GroupObjectDialog::selectObject(FWObject *o)
         }
     }
 }
+
+void GroupObjectDialog::newObject()
+{
+    m_dialog->newButton->showMenu();
+}
+
+void GroupObjectDialog::itemDoubleClicked(QListWidgetItem*)
+{
+    openObject();
+}
+
+void GroupObjectDialog::itemDoubleClicked(QTreeWidgetItem*, int)
+{
+    openObject();
+}
