@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland <vadim@fwbuilder.org>
 
-  $Id: Help.h 935 2009-05-16 00:27:55Z vadim $
+  $Id: Help.h 2543 2010-02-16 02:06:37Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,27 +28,40 @@
 #define __HELP_H_
 
 #include "../../config.h"
-#include "SimpleTextView.h"
+#include "ui_helpview_q.h"
 
-#include <QFile>
+#include <QMainWindow>
+#include <QStringList>
+#include <QUrl>
+#include <QByteArray>
 
 
-class Help : public SimpleTextView
+class Help : public QMainWindow
 {
-    
+    Q_OBJECT;
+
+    QStringList paths;
+    QByteArray window_geometry;
+    Qt::WindowFlags flags;
+    bool load_links_in_browser;
+
 public:
+    Ui::HelpView_q *m_dialog;
     
-    Help(QWidget *parent, const QString &_file, const QString &title);
+    Help(QWidget *parent, const QString &title, bool load_links_in_browser=true);
+    virtual ~Help();
+
+    void setSource(const QUrl &url) { m_dialog->textview->setSource(url); }
+    QString findHelpFile(const QString &file_base_name);
+
+    void setName(const QString &name);
     
-    virtual ~Help() {};
-
-    void scrollToAnchor(const QString &anchor);
-
-    static bool getHelpFileContents(const QString &help_file,
-                                    QString &contents);
-    static bool getFile(const QString &help_file, const QString &locale,
-                        QFile &file);
+    virtual void closeEvent(QCloseEvent *event);
+    virtual void hideEvent(QHideEvent *event);
+    virtual void showEvent(QShowEvent *event);
 
+    static Help* getHelpWindow(QWidget *parent);
+    static Help* help_window;
 };
 
 #endif 
