@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: HostDialog.h 952 2009-05-20 21:54:45Z vadim $
+  $Id: HostDialog.h 1891 2009-11-24 03:40:50Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -29,45 +29,29 @@
 
 #include "../../config.h"
 #include <ui_hostdialog_q.h>
+#include "BaseObjectDialog.h"
 #include <QWidget>
 
 #include "fwbuilder/FWObject.h"
 
 class ProjectPanel;
 
-class HostDialog : public QWidget
+class HostDialog : public BaseObjectDialog
 {
-    Q_OBJECT
+    Q_OBJECT;
 
-    libfwbuilder::FWObject *obj;
-    bool init;
-    bool modified;
     Ui::HostDialog_q *m_dialog;
-    ProjectPanel *m_project;
 
  public:
-    HostDialog(ProjectPanel *project, QWidget *parent);
+    HostDialog(QWidget *parent);
     ~HostDialog();
     
 public slots:
-    virtual void changed();
-    virtual void libChanged();
     virtual void applyChanges();
-    virtual void discardChanges();
     virtual void loadFWObject(libfwbuilder::FWObject *obj);
     virtual void validate(bool*);
-    virtual void isChanged(bool*);
-    virtual void closeEvent(QCloseEvent *e);
     virtual void getHelpName(QString*);
 
- signals:
-/**
- * This signal is emitted from closeEvent, ObjectEditor connects
- * to this signal to make checks before the object editor can be closed
- * and to store its position on the screen
- */
-    void close_sign(QCloseEvent *e);
-    void changed_sign(); 
 };
 
 #endif // HOSTDIALOG_H
