@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ICMPServiceDialog.cpp 952 2009-05-20 21:54:45Z vadim $
+  $Id: ICMPServiceDialog.cpp 1932 2009-11-26 03:26:24Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -31,18 +31,21 @@
 #include "ProjectPanel.h"
 #include "FWBTree.h"
 #include "ICMPServiceDialog.h"
+#include "FWCmdChange.h"
 
 #include "fwbuilder/Library.h"
 #include "fwbuilder/ICMPService.h"
 #include "fwbuilder/ICMP6Service.h"
 
+#include <memory>
+
 #include <qlineedit.h>
 #include <qspinbox.h>
 #include <qcheckbox.h>
 #include <qtextedit.h>
 #include <qcombobox.h>
 #include <qpushbutton.h>
-#include "FWBSettings.h"
+#include <QUndoStack>
 
 #include <iostream>
 
@@ -50,8 +53,8 @@
 using namespace std;
 using namespace libfwbuilder;
 
-ICMPServiceDialog::ICMPServiceDialog(ProjectPanel *project, QWidget *parent) :
-        QWidget(parent), m_project(project)
+ICMPServiceDialog::ICMPServiceDialog(QWidget *parent) :
+        BaseObjectDialog(parent)
 {
     m_dialog = new Ui::ICMPServiceDialog_q;
     m_dialog->setupUi(this);
@@ -74,17 +77,17 @@ void ICMPServiceDialog::loadFWObject(FWObject *o)
     ICMPService *s = dynamic_cast<ICMPService*>(obj);
     assert(s!=NULL);
 
-    if (ICMP6Service::isA(o))
-    {
-        m_dialog->editorTitle->setText("ICMP6 Service");
-        m_dialog->editorLabel->setPixmap(
-            QPixmap(QString::fromUtf8(":/Icons/ICMP6Service/icon")));
-    } else
-    {
-        m_dialog->editorTitle->setText("ICMP Service");
-        m_dialog->editorLabel->setPixmap(
-            QPixmap(QString::fromUtf8(":/Icons/ICMPService/icon")));
-    }
+    // if (ICMP6Service::isA(o))
+    // {
+    //     m_dialog->editorTitle->setText("ICMP6 Service");
+    //     m_dialog->editorLabel->setPixmap(
+    //         QPixmap(QString::fromUtf8(":/Icons/ICMP6Service/icon")));
+    // } else
+    // {
+    //     m_dialog->editorTitle->setText("ICMP Service");
+    //     m_dialog->editorLabel->setPixmap(
+    //         QPixmap(QString::fromUtf8(":/Icons/ICMPService/icon")));
+    // }
 
     init=true;
 
@@ -110,58 +113,30 @@ void ICMPServiceDialog::loadFWObject(FWObject *o)
     init=false;
 }
 
-void ICMPServiceDialog::changed()
-{
-    //apply->setEnabled( true );
-    emit changed_sign();
-}
-
 void ICMPServiceDialog::validate(bool *res)
 {
     *res=true;
-    if (!isTreeReadWrite(this,obj)) { *res=false; return; }
     if (!validateName(this,obj,m_dialog->obj_name->text())) { *res=false; return; }
 }
 
-void ICMPServiceDialog::isChanged(bool*)
-{
-    //*res=(!init && apply->isEnabled());
-}
 
-void ICMPServiceDialog::libChanged()
-{
-    changed();
-}
 
 void ICMPServiceDialog::applyChanges()
 {
-    string oldname=obj->getName();
-    obj->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
-    obj->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
+    std::auto_ptr<FWCmdChange> cmd( new FWCmdChange(m_project, obj));
+    FWObject* new_state = cmd->getNewState();
 
-    obj->setInt("type", m_dialog->icmpType->value() );
-    obj->setInt("code", m_dialog->icmpCode->value() );
+    string oldname = obj->getName();
+    new_state->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
+    new_state->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
 
-    mw->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
+    new_state->setInt("type", m_dialog->icmpType->value() );
+    new_state->setInt("code", m_dialog->icmpCode->value() );
 
-    //apply->setEnabled( false );
-    mw->updateLastModifiedTimestampForAllFirewalls(obj);
+    if (!cmd->getOldState()->cmp(new_state, true))
+    {
+        if (obj->isReadOnly()) return;
+        m_project->undoStack->push(cmd.release());
+    }
 }
 
-void ICMPServiceDialog::discardChanges()
-{
-    loadFWObject(obj);
-}
-
-
-/* ObjectEditor class connects its slot to this signal and does all
- * the verification for us, then accepts (or not) the event. So we do
- * nothing here and defer all the processing to ObjectEditor
- */
-void ICMPServiceDialog::closeEvent(QCloseEvent *e)
-{
-    emit close_sign(e);
-
-}
-
-
