@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IOSImporter.cpp 332 2008-07-06 20:11:03Z vadim $
+  $Id: IOSImporter.cpp 2650 2010-03-04 03:56:46Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -46,6 +46,9 @@
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/RuleElement.h"
 
+#include <QtDebug>
+
+using namespace std;
 using namespace libfwbuilder;
 
 IOSImporter::IOSImporter(FWObject *lib,
@@ -278,8 +281,8 @@ std::pair<int,int> IOSImporter::convertPortSpec(const std::string &port_op,
     std::string portspec = strip(port_spec);
 
     if (fwbdebug)
-        qDebug(QString("Convert TCP/UDP port spec: port_op=%1 port_spec=%2").
-               arg(port_op.c_str()).arg(port_spec.c_str()).toAscii().constData());
+        qDebug() << QString("Convert TCP/UDP port spec: port_op=%1 port_spec=%2").
+            arg(port_op.c_str()).arg(port_spec.c_str());
 
     if (portop=="" && portspec=="") return std::pair<int,int>(0, 0);
 
@@ -360,7 +363,11 @@ FWObject* IOSImporter::createUDPService()
 void IOSImporter::merge_rule::operator()(FWObject* r)
 {
     PolicyRule *rule = PolicyRule::cast(r);
-    assert(rule!=NULL);
+    // Note that Policy object can have children that are objects of
+    // classes PolicyRule and RuleSetOptions. If r does not cast to
+    // PolicyRule, then it must be RuleSetOptions and we should just
+    // skip it.
+    if (rule==NULL) return;
 
     target_ruleset->add(rule);
 //    target_ruleset->renumberRules();
@@ -385,8 +392,8 @@ Firewall* IOSImporter::finalize()
 
     if (haveFirewallObject())
     {
-        FWObject *f = getFirewallObject();
-        f->setStr("host_OS", "ios");
+        FWObject *fw = getFirewallObject();
+        fw->setStr("host_OS", "ios");
 
         FWObject *policy =
             getFirewallObject()->getFirstByType(Policy::TYPENAME);
@@ -396,9 +403,8 @@ Firewall* IOSImporter::finalize()
         {
             if (fwbdebug)
             {
-                qDebug("Setting interface and direction for all rules");
-                qDebug(QString("all_rulesets.size()=%1").
-                       arg(all_rulesets.size()).toAscii().constData());
+                qDebug() << "Setting interface and direction for all rules";
+                qDebug() << "all_rulesets.size()=" << all_rulesets.size();
             }
 
             std::map<const std::string,UnidirectionalRuleSet*>::iterator i;
@@ -408,12 +414,9 @@ Firewall* IOSImporter::finalize()
 
                 if (fwbdebug)
                 {
-                    qDebug(QString("  irs->name=%1").
-                           arg(irs->name.c_str()).toAscii().constData());
-                    qDebug(QString("  irs->intf_dir.size()=%1").
-                           arg(irs->intf_dir.size()).toAscii().constData());
-                    qDebug(QString("  irs->ruleset->size()=%1").
-                           arg(irs->ruleset->size()).toAscii().constData());
+                    qDebug() << "  irs->name=" << irs->name.c_str();
+                    qDebug() << "  irs->intf_dir.size()=" << irs->intf_dir.size();
+                    qDebug() << "  irs->ruleset->size()=" << irs->ruleset->size();
                 }
 
                 // optimization: If we have several interfaces for
@@ -497,14 +500,16 @@ Firewall* IOSImporter::finalize()
                         if (intf!=NULL)
                         {
                             if (fwbdebug)
-                                qDebug(QString("    interface=%1").
-                                       arg(intf->getName().c_str()).toAscii().constData());
-                            std::for_each(irs->ruleset->begin(),
-                                          irs->ruleset->end(),
-                                          merge_rule(irs->name,
-                                                     intf,
-                                                     direction,
-                                                     policy)
+                                qDebug() << "    interface=" 
+                                         << intf->getName().c_str();
+
+                            std::for_each(
+                                irs->ruleset->begin(),
+                                irs->ruleset->end(),
+                                merge_rule(irs->name,
+                                           intf,
+                                           direction,
+                                           policy)
                             );
                         }
                     }
@@ -519,6 +524,13 @@ Firewall* IOSImporter::finalize()
             }
         }
 
+        list<FWObject*> l2 = fw->getByType(Policy::TYPENAME);
+        for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i)
+        {
+            RuleSet *rs = RuleSet::cast(*i);
+            rs->renumberRules();
+        }
+
         return getFirewallObject();
     }
     else
