@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IPTImporter.cpp 1036 2009-06-04 20:35:53Z vadim $
+  $Id: IPTImporter.cpp 2652 2010-03-04 05:06:35Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -59,6 +59,9 @@
 #include "fwbuilder/FWServiceReference.h"
 #include "fwbuilder/CustomService.h"
 
+#include <QtDebug>
+
+using namespace std;
 using namespace libfwbuilder;
 
 IPTImporter::IPTImporter(FWObject *lib,
@@ -234,16 +237,18 @@ FWObject* IPTImporter::createIPService()
 std::pair<int,int> IPTImporter::convertPortRange(str_tuple &range,
                                                  const char *proto)
 {
-    return std::pair<int,int>(convertPort(range.first,proto),
-                              convertPort(range.second,proto));
+    return std::pair<int,int>(convertPort(range.first, proto, 0),
+                              convertPort(range.second, proto, 65535));
 }
 
 int IPTImporter::convertPort(const std::string &port_spec,
-                             const char *proto)
+                             const char *proto,
+                             int default_port)
 {
     int port = 0;
     std::string ps = strip(port_spec);
-    if (ps=="") return 0;
+    if (ps == "") return 0;
+    if (ps == ":") return default_port;
 
     struct servent *se = getservbyname(ps.c_str(), proto);
     if (se!=NULL)
@@ -272,13 +277,15 @@ FWObject* IPTImporter::createTCPUDPService(str_tuple &src_range,
 {
     if (fwbdebug)
     {
-        qDebug(QString("Creating %1 service").arg(proto.c_str()).toAscii().constData());
-        qDebug(QString("src range: %1 - %2")
-               .arg(src_range.first.c_str())
-               .arg(src_range.second.c_str()).toAscii().constData());
-        qDebug(QString("dst range: %1 - %2")
-               .arg(dst_range.first.c_str())
-               .arg(dst_range.second.c_str()).toAscii().constData());
+        qDebug() << "Creating service " << proto.c_str();
+        qDebug() << "src range: "
+                 << src_range.first.c_str()
+                 << " - "
+                 << src_range.second.c_str();
+        qDebug() << "dst range: "
+                 << dst_range.first.c_str()
+                 << " - "
+                 << dst_range.second.c_str();
     }
 
 //     std::string name = proto + " "
@@ -351,7 +358,9 @@ FWObject* IPTImporter::createTCPUDPService(const std::string &proto)
         std::string name = proto + " group " + s.str();
 
         if (fwbdebug)
-            qDebug(QString("Group of %1 services with name '%2', sig '%3'").arg(proto.c_str()).arg(name.c_str()).arg(sig.c_str()).toAscii().constData());
+            qDebug() <<
+                QString("Group of %1 services with name '%2', sig '%3'")
+                .arg(proto.c_str()).arg(name.c_str()).arg(sig.c_str());
 
         ServiceGroup *sg = ServiceGroup::cast(createObject(ServiceGroup::TYPENAME, name));
         for (FWObject::iterator j=olist.begin(); j!=olist.end(); ++j)
@@ -865,14 +874,28 @@ Firewall* IPTImporter::finalize()
 
     if (haveFirewallObject())
     {
-        FWObject *f = getFirewallObject();
-        f->setStr("host_OS", "linux24");
+        FWObject *fw = getFirewallObject();
+        fw->setStr("host_OS", "linux24");
 
-        FWOptions  *fwopt = Firewall::cast(f)->getOptionsObject();
+        FWOptions  *fwopt = Firewall::cast(fw)->getOptionsObject();
         assert(fwopt!=NULL);
 
         fwopt->setBool("firewall_is_part_of_any_and_networks", false);
 
+        list<FWObject*> l2 = fw->getByType(Policy::TYPENAME);
+        for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i)
+        {
+            RuleSet *rs = RuleSet::cast(*i);
+            rs->renumberRules();
+        }
+
+        l2 = fw->getByType(NAT::TYPENAME);
+        for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i)
+        {
+            RuleSet *rs = RuleSet::cast(*i);
+            rs->renumberRules();
+        }
+
         return getFirewallObject();
     }
     else
