@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IPTImporter.h 647 2008-11-14 07:03:21Z vadim $
+  $Id: IPTImporter.h 2652 2010-03-04 05:06:35Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -62,7 +62,7 @@ class IPTImporter : public Importer {
     virtual libfwbuilder::FWObject* createUDPService();
 
     std::pair<int,int> convertPortRange(str_tuple &range, const char *proto);
-    int convertPort(const std::string &port_spec, const char *proto);
+    int convertPort(const std::string &port_spec, const char *proto, int default_port);
 
     virtual void addSrv();
 
