@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Importer.cpp 1036 2009-06-04 20:35:53Z vadim $
+  $Id: Importer.cpp 2654 2010-03-04 05:32:41Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -31,16 +31,16 @@
 
 #include "Importer.h"
 
+#include <string>
 #include <ios>
 #include <iostream>
 #include <algorithm>
 
-#include <qstring.h>
-
 #include "fwbuilder/FWObject.h"
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/Address.h"
+#include "fwbuilder/DNSName.h"
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/IPService.h"
 #include "fwbuilder/ICMPService.h"
@@ -57,7 +57,31 @@
 #include "ProjectPanel.h"
 #include "FWBTree.h"
 
+#include <QString>
+#include <QtDebug>
+
+
 using namespace libfwbuilder;
+using namespace std;
+
+// a functor to join list<string> into a string with separator sep
+class join : public std::unary_function<std::string, void>
+{
+    std::string *result;
+    std::string  separator;
+public:
+    join(std::string *res, const std::string &s)
+    { result = res; separator = s; }
+    void operator()(std::string &s);
+};
+
+void join::operator()(string &s)
+{
+    if (!result->empty()) *result += separator;
+    *result += s;
+}
+
+
 
 FWObject* Importer::createObject(const std::string &objType,
                                  const std::string &objName)
@@ -840,7 +864,8 @@ FWObject* Importer::createGroupOfInterfaces(
     std::for_each(interfaces.begin(), interfaces.end(), join(&sig, "_"));
 
     if (fwbdebug)
-        qDebug(QString("Interface group with name '%1', sig '%2'").arg(name.c_str()).arg(sig.c_str()).toAscii().constData());
+        qDebug() << QString("Interface group with name '%1', sig '%2'")
+            .arg(name.c_str()).arg(sig.c_str());
 
     if (all_objects.count(sig)!=0) return all_objects[sig];
 
@@ -857,22 +882,42 @@ FWObject* Importer::createGroupOfInterfaces(
 FWObject* Importer::createAddress(const std::string &addr,
                                   const std::string &netmask)
 {
-    std::string comment = "Imported from " + getFirewallObject()->getName() +
-        " " + addr + "/" + netmask;
     std::string sig = std::string("addr-") + addr + "/" + netmask;
     if (all_objects.count(sig)!=0) return all_objects[sig];
 
     if ( netmask == InetAddr::getAllOnes().toString() )
     {
-        Address *a;
-        std::string name = std::string("h-") + addr;
-        a = Address::cast(createObject(IPv4::TYPENAME, name));
-        a->setAddress(InetAddr(addr));
-        a->setNetmask(InetAddr(InetAddr::getAllOnes()));
-        a->setComment(comment);
-        all_objects[sig] = a;
-        *logger << "Address object: " << name << "\n";
-        return a;
+        string name;
+        try
+        {
+            InetAddr obj_addr(addr); // testing if string converts to an address
+            name = std::string("h-") + addr;
+            Address *a = Address::cast(createObject(IPv4::TYPENAME, name));
+            a->setAddress(obj_addr);
+            a->setNetmask(InetAddr(InetAddr::getAllOnes()));
+            string comment = "Imported from " + getFirewallObject()->getName() +
+                " " + addr + "/" + netmask;
+            a->setComment(comment);
+            all_objects[sig] = a;
+            *logger << "Address object: " << name << "\n";
+            return a;
+        } catch(FWException &ex)
+        {
+            // address text line can not be converted to ipv4 address.
+            // Since parsers do not understand ipv6 yet, assume this
+            // is a host address and create DNSName object
+            name = addr;
+            DNSName *da = DNSName::cast(createObject(DNSName::TYPENAME, name));
+            da->setSourceName(addr);
+            da->setRunTime(true);
+            string comment = "Imported from " + getFirewallObject()->getName() +
+                " " + addr;
+            da->setComment(comment);
+            all_objects[sig] = da;
+            *logger << "DNSName object: " << name << "\n";
+            return da;
+        }
+
     } else
     {
         Network *net;
@@ -918,6 +963,9 @@ FWObject* Importer::createAddress(const std::string &addr,
             }
         }
 
+        string comment = "Imported from " + getFirewallObject()->getName() +
+            " " + addr + "/" + netmask;
+
         net->setComment(comment);
         all_objects[sig] = net;
         *logger << "Network object: " << name << "\n";
