@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: NetworkDialog.cpp 952 2009-05-20 21:54:45Z vadim $
+  $Id: NetworkDialog.cpp 2512 2010-02-10 23:46:54Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -31,6 +31,7 @@
 #include "NetworkDialog.h"
 #include "ProjectPanel.h"
 #include "FWBSettings.h"
+#include "FWCmdChange.h"
 
 #include "fwbuilder/Library.h"
 #include "fwbuilder/Network.h"
@@ -38,6 +39,8 @@
 #include "fwbuilder/FWException.h"
 #include "fwbuilder/Inet6AddrMask.h"
 
+#include <memory>
+
 #include <qlineedit.h>
 #include <qspinbox.h>
 #include <qcheckbox.h>
@@ -45,12 +48,15 @@
 #include <qcombobox.h>
 #include <qmessagebox.h>
 #include <qpushbutton.h>
+#include <QUndoStack>
+#include <QtDebug>
+#include <QApplication>
+
 
-#include "FWWindow.h"
 using namespace std;
 using namespace libfwbuilder;
 
-NetworkDialog::NetworkDialog(ProjectPanel *project, QWidget *parent) : QWidget(parent), m_project(project)
+NetworkDialog::NetworkDialog(QWidget *parent) : BaseObjectDialog(parent)
 {
     m_dialog = new Ui::NetworkDialog_q;
     m_dialog->setupUi(this);
@@ -77,7 +83,16 @@ void NetworkDialog::loadFWObject(FWObject *o)
     m_dialog->netmask->setText( s->getNetmaskPtr()->toString().c_str() );
     m_dialog->comment->setText( QString::fromUtf8(s->getComment().c_str()) );
 
-    //apply->setEnabled( false );
+    // See #893 No need to show address and mask 0.0.0.0 to the user
+    // if the object is "Any", especially because the same object is
+    // used as "any" for both ipv4 and ipv6 rules. It can be confusing
+    // if they see address "0.0.0.0" while they want to find object
+    // "any" for ipv6.
+    if (obj->getId() == FWObjectDatabase::ANY_ADDRESS_ID)
+        m_dialog->object_attributes->hide();
+    else
+        m_dialog->object_attributes->show();
+
 
     m_dialog->obj_name->setEnabled(!o->isReadOnly());
     setDisabledPalette(m_dialog->obj_name);
@@ -95,20 +110,15 @@ void NetworkDialog::loadFWObject(FWObject *o)
     init=false;
 }
 
-void NetworkDialog::changed()
-{
-    //apply->setEnabled( true );
-    emit changed_sign();
-}
-
 void NetworkDialog::validate(bool *result)
 {
-    *result=true;
+    if (fwbdebug) qDebug() << "NetworkDialog::validate";
+
+    *result = true;
 
-    if (!isTreeReadWrite(this,obj)) { *result=false; return; }
     if (!validateName(this,obj,m_dialog->obj_name->text()))
     {
-        *result=false;
+        *result = false;
         return;
     }
 
@@ -116,74 +126,106 @@ void NetworkDialog::validate(bool *result)
     assert(s!=NULL);
     try
     {
-        InetAddr( m_dialog->address->text().toLatin1().constData() );
+        InetAddr( m_dialog->address->text().toStdString() );
     } catch (FWException &ex)
     {
-        *result=false;
-        QMessageBox::critical(
-            this, "Firewall Builder",
-            tr("Illegal IP address '%1'").arg(m_dialog->address->text()),
-            tr("&Continue"), 0, 0,
-            0 );
+        *result = false;
+        if (QApplication::focusWidget() != NULL)
+        {
+            blockSignals(true);
+            QMessageBox::critical(
+                this, "Firewall Builder",
+                tr("Illegal IP address '%1'").arg(m_dialog->address->text()),
+                tr("&Continue"), 0, 0,
+                0 );
+            blockSignals(false);
+        }
+        return;
     }
+
+    InetAddr addr(m_dialog->address->text().toStdString());
+
     try
     {
         QString len = m_dialog->netmask->text() ;
         bool ok = false ;
-        int ilen = len.toInt (&ok);
+        int ilen = len.toInt(&ok);
         if (ok)
         {
+            // permit netmask 0.0.0.0 if the address is also 0.0.0.0
+            if (addr.isAny() && ilen == 0) return;
+
             if (ilen>0 && ilen < 32)
             {
                 return ;
             }
             else
             {
-                *result=false;
+                *result = false;
                 QMessageBox::critical(
                     this, "Firewall Builder",
                     tr("Illegal netmask '%1'").arg( m_dialog->netmask->text() ),
                     tr("&Continue"), 0, 0,
                     0 );
-                
+                return;
             }
         }
-        InetAddr( m_dialog->netmask->text().toLatin1().constData() );
+
+        InetAddr nm( m_dialog->netmask->text().toStdString() );
+        if (nm.isAny())
+        {
+            // permit netmask 0.0.0.0 if the address is also 0.0.0.0
+            if (addr.isAny()) return;
+            {
+                if (QApplication::focusWidget() != NULL)
+                {
+                    blockSignals(true);
+                    // Do not allow netmask of 0 bits See #251
+                    QMessageBox::critical(
+                        this, "Firewall Builder",
+                        tr("Network object should not have netmask '0.0.0.0'"),
+                        tr("&Continue"), 0, 0,
+                        0 );
+                    blockSignals(false);
+                }
+                return;
+            }
+        }
+
     } catch (FWException &ex)
     {
 
-        *result=false;
-        QMessageBox::critical(
-            this, "Firewall Builder",
-            tr("Illegal netmask '%1'").arg( m_dialog->netmask->text() ),
-            tr("&Continue"), 0, 0,
-            0 );
+        *result = false;
+        if (QApplication::focusWidget() != NULL)
+        {
+            blockSignals(true);
+            QMessageBox::critical(
+                this, "Firewall Builder",
+                tr("Illegal netmask '%1'").arg( m_dialog->netmask->text() ),
+                tr("&Continue"), 0, 0,
+                0 );
+            blockSignals(false);
+        }
     }
 }
 
-void NetworkDialog::isChanged(bool*)
-{
-    //*result=(!init && apply->isEnabled());
-}
 
-void NetworkDialog::libChanged()
-{
-    changed();
-}
 
 void NetworkDialog::applyChanges()
 {
-    Network *s = dynamic_cast<Network*>(obj);
+    std::auto_ptr<FWCmdChange> cmd( new FWCmdChange(m_project, obj));
+    FWObject* new_state = cmd->getNewState();
+
+    Network *s = dynamic_cast<Network*>(new_state);
     assert(s!=NULL);
 
     string oldname = obj->getName();
-    obj->setName(string(m_dialog->obj_name->text().toUtf8().constData()));
-    obj->setComment(string(
-                        m_dialog->comment->toPlainText().toUtf8().constData()));
+    new_state->setName(string(m_dialog->obj_name->text().toUtf8().constData()));
+    new_state->setComment(string(
+                              m_dialog->comment->toPlainText().toUtf8().constData()));
     try
     {
-        s->setAddress(
-            InetAddr(m_dialog->address->text().toLatin1().constData()) );
+        s->setAddress(InetAddr(m_dialog->address->text().toStdString()));
     } catch (FWException &ex)
     {
 // exception thrown if user types illegal m_dialog->address or
@@ -201,8 +243,7 @@ void NetworkDialog::applyChanges()
         }
         else
         {
-            s->setNetmask(
-                InetAddr(m_dialog->netmask->text().toLatin1().constData()) );
+            s->setNetmask(InetAddr(m_dialog->netmask->text().toStdString()));
         }
     } catch (FWException &ex)
     {
@@ -211,26 +252,12 @@ void NetworkDialog::applyChanges()
 //        bool ok = false ;
     }
 
-    mw->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
-
-    //apply->setEnabled( false );
-    mw->updateLastModifiedTimestampForAllFirewalls(obj);
-}
-
-void NetworkDialog::discardChanges()
-{
-    loadFWObject(obj);
-}
-
-
-/* ObjectEditor class connects its slot to this signal and does all
- * the verification for us, then accepts (or not) the event. So we do
- * nothing here and defer all the processing to ObjectEditor
- */
-void NetworkDialog::closeEvent(QCloseEvent *e)
-{
-    emit close_sign(e);
-
+    if (!cmd->getOldState()->cmp(new_state, true))
+    {
+        if (obj->isReadOnly()) return;
+        m_project->undoStack->push(cmd.release());
+    }
+    
 }
 
 void NetworkDialog::addressEntered()
@@ -238,7 +265,7 @@ void NetworkDialog::addressEntered()
     try
     {
         QString addr = m_dialog->address->text();
-        InetAddrMask address_and_mask(string(addr.toLatin1().constData()));
+        InetAddrMask address_and_mask(string(addr.toStdString()));
         if (addr.contains('/'))
         {
             m_dialog->address->setText(
