@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectManipulator.h 973 2009-05-29 16:15:31Z vadim $
+  $Id: ObjectManipulator.h 2566 2010-02-19 03:06:25Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,7 +17,7 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
@@ -30,6 +30,7 @@
 
 #include "../../config.h"
 #include "global.h"
+#include "utils.h"
 
 #include <qdialog.h>
 #include <qtreewidget.h>
@@ -42,21 +43,28 @@
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/ObjectGroup.h"
 
+#include "UsageResolver.h"
+
 #include <stack>
 #include <set>
 
 class ObjectTreeView;
 class ObjectTreeViewItem;
 class QComboBox;
-class QMenu;
+class QPixmap;
+class QUndoCommand;
+
 class ProjectPanel;
 
-namespace libfwbuilder 
+
+namespace libfwbuilder
 {
     class Firewall;
+    class Cluster;
     class Library;
 }
 
+
 class HistoryItem {
     ObjectTreeViewItem *itm;
     int                 objId;
@@ -68,90 +76,104 @@ class HistoryItem {
     int  id()   { return objId; }
 };
 
-/*class ObjToolTip : public QToolTip {
+class ObjectManipulator : public QWidget
+{
+    Q_OBJECT;
 
-    ObjectTreeView *otv;
- public:
-    ObjToolTip(ObjectTreeView *widget);
-    virtual ~ObjToolTip() {}
-    virtual void maybeTip(const QPoint &p);
-};*/
-
-class ObjectManipulator : public QWidget/*ObjectManipulator_q*/ {
-
-    Q_OBJECT
-
-    std::vector<libfwbuilder::FWObject*>          idxToLibs;
-    std::vector<QTreeWidget*>                     idxToTrees;
-    QSet<int> ids ;     
-    std::stack<HistoryItem>                       history;
+    std::vector<libfwbuilder::FWObject*> idxToLibs;
+    std::vector<QTreeWidget*> idxToTrees;
+    int previous_lib_index;
+    QSet<int> ids ;
+    std::stack<HistoryItem> history;
     int cacheHits;
 
-    libfwbuilder::FWObject                        *currentObj;
-    ObjectTreeView                                *current_tree_view;
-    
-    int                                            treeWidth;
-    int                                            treeHeight;
+    //libfwbuilder::FWObject *currentObj;
+    ObjectTreeView *current_tree_view;
+
+    int treeWidth;
+    int treeHeight;
+
+    bool active;
+
+    QMenu *popup_menu;
 
-    bool                                           active;
-    
 /* this is a reverse idex of all objects in all trees. We use it to
  * quickly locate given object in the tree and open it
  */
     std::map<libfwbuilder::FWObject*, ObjectTreeViewItem*> allItems;
-    
+
     ProjectPanel *m_project;
 
     int dedup_marker_global_counter;
-    
+
+    void buildNewObjectMenu();
+
     ObjectTreeViewItem* insertObject(ObjectTreeViewItem *itm,
                                      libfwbuilder::FWObject *obj);
     void insertSubtree( ObjectTreeViewItem *itm,libfwbuilder::FWObject *obj );
 
-    void removeObjectFromTreeView(libfwbuilder::FWObject *obj );
-
     QString getTreeLabel( libfwbuilder::FWObject *obj );
 
-    void addTreePage(libfwbuilder::FWObject *lib);
     void showObjectInTree(ObjectTreeViewItem *otvi);
 
+    void setObjectIcon(libfwbuilder::FWObject *obj, QPixmap *pm);
+
     int  getIdxForLib(libfwbuilder::FWObject*);
-    void addLib( libfwbuilder::FWObject *lib, QTreeWidget* otv=NULL);
-    void removeLib(libfwbuilder::FWObject*);
     void removeLib(int idx);
     void updateCreateObjectMenu(libfwbuilder::FWObject* lib);
+
     void makeNameUnique(libfwbuilder::FWObject* p,libfwbuilder::FWObject* obj);
 
+    /* find the name of the interface that was created last */
+    QString findNewestInterfaceName(libfwbuilder::FWObject *parent);
+
     libfwbuilder::FWObject* actuallyCreateObject(
         libfwbuilder::FWObject *parent,
         const QString &objType,
         const QString &objName,
-        libfwbuilder::FWObject *copyFrom=NULL);
+        libfwbuilder::FWObject *copyFrom=NULL,
+        QUndoCommand* macro = 0);
 
-    void autorename(libfwbuilder::FWObject *obj,bool ask=true);
+    void autorename(libfwbuilder::FWObject *obj);
     void extractFirewallsFromGroup(libfwbuilder::ObjectGroup *gr,
                                    std::set<libfwbuilder::Firewall*> &fo);
-    
+
     libfwbuilder::FWObject* actuallyPasteTo(libfwbuilder::FWObject *target,
                                             libfwbuilder::FWObject *obj,
                                             std::map<int,int> &map_ids);
 
-    bool validateForPaste(libfwbuilder::FWObject *target,
-                          libfwbuilder::FWObject *obj);
-    
+    void actuallyDeleteObject(libfwbuilder::FWObject *obj, QUndoCommand* macro = 0);
+
     void findWhereUsedRecursively(libfwbuilder::FWObject *obj,
                                   libfwbuilder::FWObject *top,
                                   std::set<libfwbuilder::FWObject*> &resset);
-    
+
+    void refreshSubtree(QTreeWidgetItem *parent, QTreeWidgetItem *itm);
+
+    libfwbuilder::FWObject* newLibrary(QUndoCommand* macro);
+    libfwbuilder::FWObject* newHost(QUndoCommand* macro);
+    libfwbuilder::FWObject* newInterface(QUndoCommand* macro);
+    libfwbuilder::FWObject* newFirewall(QUndoCommand* macro);
+    libfwbuilder::FWObject* newCluster(QUndoCommand* macro, bool fromSelected = false);
+    libfwbuilder::FWObject* newClusterIface(QUndoCommand* macro);
+    libfwbuilder::FWObject* newStateSyncClusterGroup(QUndoCommand* macro);
+    libfwbuilder::FWObject* newFailoverClusterGroup(QUndoCommand* macro);
+    libfwbuilder::FWObject* newInterfaceAddress(QUndoCommand* macro);
+    libfwbuilder::FWObject* newInterfaceAddressIPv6(QUndoCommand* macro);
+    libfwbuilder::FWObject* newPhysicalAddress(QUndoCommand* macro);
+    libfwbuilder::FWObject* newPolicyRuleSet (QUndoCommand* macro);
+    libfwbuilder::FWObject* newNATRuleSet (QUndoCommand* macro);
+
+   
 public slots:
      virtual void libChanged(int l);
      virtual void switchingTrees(QWidget* w);
      virtual void currentTreePageChanged(int i);
 
+     void newClusterFromSelected();
+     
      void selectionChanged(QTreeWidgetItem *cur);
 
-     void info();
-
      /**
       * open object obj in the editor. Does not open editor panel
       * if it is closed. Asks FWWindow permission to own editor.
@@ -171,79 +193,69 @@ public slots:
      void closeObject();
 
      void editSelectedObject();
-     
+     void openSelectedRuleSet();
+
      void contextMenuRequested(const QPoint &pos);
 
-     
+     /*
+      * Internal: this method is used in actuallyPasteTo(). This
+      * method checks if the target object is appropriate and replaces
+      * it with parent if needed. Also does validation and shows error
+      * dialogs if validation fails. Returns new parent or NULL if
+      * validation fails.
+      */
+     libfwbuilder::FWObject* prepareForInsertion(libfwbuilder::FWObject *target,
+                                                 libfwbuilder::FWObject *obj);
+
      libfwbuilder::FWObject* createObject(const QString &objType,
                                           const QString &objName,
-                                          libfwbuilder::FWObject *copyFrom=NULL);
-     
+                                          libfwbuilder::FWObject *copyFrom=NULL,
+                                          QUndoCommand* macro = 0);
+
      libfwbuilder::FWObject* createObject(libfwbuilder::FWObject *parent,
                                           const QString &objType,
                                           const QString &objName,
-                                          libfwbuilder::FWObject *copyFrom=NULL);
-     
+                                          libfwbuilder::FWObject *copyFrom=NULL,
+                                          QUndoCommand* macro = 0);
 
-     void newLibrary();
      void newObject();
-     void newFirewall();
-     void newHost();
-     void newInterface();
-     void newNetwork();
-     void newNetworkIPv6(); 
-     void newAddress();
-     void newAddressIPv6();
-     void newInterfaceAddress();
-     void newInterfaceAddressIPv6();
-     void newPhysicalAddress();
-     void newAddressRange();
-     void newObjectGroup();
-     void newDNSName();
-     void newAddressTable();
-     
-     void newCustom();
-     void newIP();
-     void newICMP();
-     void newICMP6();
-     void newTCP();
-     void newUDP();
-     void newTagService();
-     void newUserService();
-     void newServiceGroup();
-     
-     void newInterval();
-     void newPolicyRuleSet ();
-     void newNATRuleSet ();
+
+     void createNewObject();
+
      void copyObj();
      void cutObj();
      void pasteObj();
-     void deleteObj();
+     void delObj(QUndoCommand* macro = 0);
      void dumpObj();
      void compile();
      void install();
+     void transferfw();
 
      void duplicateObj(QAction*);
      void moveObj(QAction*);
-     
+
      void groupObjects();
-     
-     void openObject(QTreeWidgetItem *otvi);
-     void openObject(libfwbuilder::FWObject *obj);
-     void restoreSelection(bool same_widget);
+
+     void openObjectInTree(QTreeWidgetItem *otvi);
+     void openObjectInTree(libfwbuilder::FWObject *obj);
 
      void find();
      void findObject();
-     
+
      virtual void back();
      virtual void lockObject();
      virtual void unlockObject();
      virtual void simulateInstall();
      virtual void findWhereUsedSlot();
 
-     
+
 public:
 
+     ObjectManipulator(QWidget *parent);
+     ~ObjectManipulator();
+
+     void setupProject(ProjectPanel *project);
+
      void libChangedById(int id);
      void changeFirstNotSystemLib();
      std::vector<QTreeWidget*> getTreeWidgets() { return idxToTrees;};
@@ -251,53 +263,61 @@ public:
      void filterFirewallsFromSelection(
          std::vector<libfwbuilder::FWObject*> &so,
          std::set<libfwbuilder::Firewall*> &fo);
-     void autorename(libfwbuilder::FWObject *obj,
+
+     QString getStandardName(libfwbuilder::FWObject *parent,
+                             const std::string &objtype,
+                             const std::string &namesuffix);
+
+     QString makeNameUnique(libfwbuilder::FWObject* parent,
+                            const QString &obj_name, const QString &obj_type);
+
+     void autorename(std::list<libfwbuilder::FWObject*> &obj_list,
                      const std::string &objtype,
                      const std::string &namesuffix);
-    
-     ObjectManipulator( QWidget *parent);
-     ~ObjectManipulator();
+     void autorenameVlans(std::list<libfwbuilder::FWObject*> &obj_list);
+
+     void reload();
 
      void loadObjects();
      void clearObjects();
-    
-     bool validateDialog();
-     void invalidateDialog();
 
      void reopenCurrentItemParent();
 
-     void insertObjectInTree(libfwbuilder::FWObject *parent,
-                             libfwbuilder::FWObject *obj);
+     void insertSubtree(libfwbuilder::FWObject *parent,
+                        libfwbuilder::FWObject *obj);
+     void removeObjectFromTreeView(libfwbuilder::FWObject *obj );
 
-     void openObject(libfwbuilder::FWObject *obj, bool register_in_history);
-     void openObject(ObjectTreeViewItem *otvi,    bool register_in_history);
+     void addLib(libfwbuilder::FWObject *lib);
+     void removeLib(libfwbuilder::FWObject *lib);
+     void openLibForObject(libfwbuilder::FWObject *obj);
+
+     void openObjectInTree(libfwbuilder::FWObject *obj, bool register_in_history);
+     void openObjectInTree(ObjectTreeViewItem *otvi,    bool register_in_history);
 
      libfwbuilder::FWObject* duplicateObject(libfwbuilder::FWObject *target,
-                                             libfwbuilder::FWObject *obj,
-                                             const QString &name = QString::null,
-                                             bool  askForAutorename=true);
+                                             libfwbuilder::FWObject *obj);
 
      void moveObject(libfwbuilder::FWObject *target,
                      libfwbuilder::FWObject *obj);
 
      void moveObject(const QString &targetLibName, libfwbuilder::FWObject *obj);
 
+     // This method is called toprocess event objectMovedEvent. this event is
+     // posted by FWCmdMoveObject after the object is moved from one
+     // part of the tree to another, including when an object moves to the
+     // Deleted Objects library
+     void objectMoved(libfwbuilder::FWObject *obj);
+
      libfwbuilder::FWObject* pasteTo(libfwbuilder::FWObject *target,
                                      libfwbuilder::FWObject *obj);
-     
-     libfwbuilder::FWObject* getOpened() { return currentObj; }
 
      void updateLibColor(libfwbuilder::FWObject *lib);
      void updateLibName(libfwbuilder::FWObject *lib);
 
-     void updateObjName(libfwbuilder::FWObject *obj,
-                        const QString &oldName,
-                        bool  askForAutorename=true);
-     void updateObjName(libfwbuilder::FWObject *obj,
-                        const QString &oldName,
-                        const QString &oldLabel,
-                        bool  askForAutorename=true);
-    
+     void autoRenameChildren(libfwbuilder::FWObject *obj,
+                             const QString &oldName);
+    void updateObjectInTree(libfwbuilder::FWObject *obj, bool subtree=false);
+
      ObjectTreeView* getCurrentObjectTree();
      libfwbuilder::FWObject* getSelectedObject();
 
@@ -318,7 +338,7 @@ public:
       */
      void closeSystemLib();
 
-     void delObj(libfwbuilder::FWObject *obj,bool openobj=true);
+     void deleteObject(libfwbuilder::FWObject *obj, QUndoCommand* macro = 0);
 
      /**
       * select whatever object is current in the tree (used to restore
@@ -331,21 +351,25 @@ public:
       */
      void unselect();
 
-     /**
-      * returns true if anything is selected in the tree
-      */
-     bool isSelected();
-
      /**
       * controls whether "Deleted Objects" library is shown
       */
      void showDeletedObjects(bool f);
 
 
+     /**
+      * checks if currently selected object can be locked. Note that
+      * if its parent or any object all the way up to the library is
+      * read-only, then the state of the current object can not be
+      * changed which makes it not lockable.
+      */
+     bool isCurrentObjectLockable();
+     bool isCurrentObjectUnlockable();
+     
      /**
       * get boolean flags that describe state of the menu items.
       * Can be used for both pop-up context menu and the main menu.
-      */ 
+      */
      void getMenuState(bool haveMoveTargets,
                        bool &dupMenuItem,
                        bool &moveMenuItem,
@@ -354,45 +378,33 @@ public:
                        bool &delMenuItem,
                        bool &newMenuItem,
                        bool &inDeletedObjects);
-    
-     void updateLastModifiedTimestampForOneFirewall(libfwbuilder::FWObject *o);
-     void updateLastModifiedTimestampForAllFirewalls(libfwbuilder::FWObject *o);
+
+     bool getDeleteMenuState(libfwbuilder::FWObject *obj);
+
      void updateLastInstalledTimestamp(libfwbuilder::FWObject *o);
      void updateLastCompiledTimestamp(libfwbuilder::FWObject *o);
-    
+
      std::list<libfwbuilder::Firewall*> findFirewallsForObject(
          libfwbuilder::FWObject *o);
-     void findAllFirewalls (std::list<libfwbuilder::Firewall *> &fws);
+
+     void findAllFirewalls(std::list<libfwbuilder::Firewall*> &fws);
+
+     std::list<libfwbuilder::Cluster*> findClustersUsingFirewall(libfwbuilder::FWObject *fw);
+     void findAllClusters(std::list<libfwbuilder::Cluster*> &fws);
 
      void loadExpandedTreeItems();
      void saveExpandedTreeItems();
 
-signals:
-/**
- * the dialog class should have a slot that can load object's data
- * into dialog elements when ObjectManipulator emits this signal
- */
-     void loadObject_sign(libfwbuilder::FWObject *);
+     void loadSectionSizes();
+     void saveSectionSizes();
 
-/**
- * the dialog class should have a slot that can verify data entered by
- * user in the dialog elements when ObjectManipulator emits this
- * signal. The validation result is returned in variable "bool *res"
- */
-     void validate_sign(bool *res);
-
-/**
- * the dialog class should have a slot that can verify if the data in
- * dialog has been edited.
- */
-     void isChanged_sign(bool *res);
-
-/**
- * the dialog class should have a slot that applies changes made by
- * the user and saves data in the object.
- */
-     void applyChanges_sign();
+     void setAttributesColumnEnabled(bool f);
 
+     QAction* addNewObjectMenuItem(QMenu *menu,
+                                   const char* type_name,
+                                   const QString &text=QString(),
+                                   int add_to_group_id=-1);
+                                  
 };
 
 #endif
