@@ -2,11 +2,11 @@
 
                           Firewall Builder
 
-                 Copyright (C) 2003 NetCitadel, LLC
+                 Copyright (C) 2009 NetCitadel, LLC
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: PrefsDialog.cpp 1474 2009-09-18 17:22:54Z vadim $
+  $Id: PrefsDialog.cpp 2466 2010-01-30 23:55:51Z yalovoy $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,20 +28,22 @@
 #include "global.h"
 #include "check_update_url.h"
 #include "../../VERSION.h"
-#include "../../build_num"
 
 #include "utils.h"
+#include "platforms.h"
 
 #include "PrefsDialog.h"
 #include "FWBSettings.h"
-#include "listOfLibraries.h"
 #include "FWWindow.h"
+#include "ProjectPanel.h"
 #include "HttpGet.h"
+#include "RuleSetView.h"
+
+#include "fwbuilder/Resources.h"
 
 #include <qlineedit.h>
 #include <qfiledialog.h>
 #include <qcombobox.h>
-#include <qcheckbox.h>
 #include <qlistwidget.h>
 #include <qdir.h>
 #include <qlabel.h>
@@ -58,6 +60,10 @@
 
 #include <QTreeWidgetItem>
 #include <QUrl>
+#include <QCheckBox>
+#include <QFrame>
+#include <QTableWidget>
+#include <QHeaderView>
 
 #include <sstream>
 
@@ -111,13 +117,25 @@ PrefsDialog::PrefsDialog(QWidget *parent) : QDialog(parent)
     m_dialog->tooltipDelay->setValue( st->getTooltipDelay() );
 
     m_dialog->deletedObj->setChecked( st->getBool("UI/ShowDeletedObjects") );
+    m_dialog->attributesInTree->setChecked( st->getBool("UI/ShowObjectsAttributesInTree") );
+
+    m_dialog->new_dns_name_compile_tm->setChecked(
+        st->getBool("Objects/DNSName/useCompileTimeForNewObjects"));
+    m_dialog->new_dns_name_run_tm->setChecked(
+        ! st->getBool("Objects/DNSName/useCompileTimeForNewObjects"));
+    m_dialog->use_name_for_dns_record->setChecked(
+        st->getBool("Objects/DNSName/useNameForDNSRecord"));
+
+    m_dialog->new_addr_tbl_compile_tm->setChecked(
+        st->getBool("Objects/AddressTable/useCompileTimeForNewObjects"));
+    m_dialog->new_addr_tbl_run_tm->setChecked(
+        ! st->getBool("Objects/AddressTable/useCompileTimeForNewObjects"));
+
 
     m_dialog->emptyRCSLog->setChecked( st->getRCSLogState() );
-    m_dialog->autosave->setChecked( st->getAutoSave() );
 
     m_dialog->autosaveFile->setChecked(st->getBool("Environment/autoSaveFile"));
-    m_dialog->autosaveInterval->setValue(
-        st->getInt("Environment/autoSaveFilePeriod"));
+    m_dialog->autosaveInterval->setValue(st->getInt("Environment/autoSaveFilePeriod"));
 
     m_dialog->dataFileCompression->setChecked( st->getCompression() );
 
@@ -125,7 +143,9 @@ PrefsDialog::PrefsDialog(QWidget *parent) : QDialog(parent)
 
     m_dialog->sshPath->setText( st->getSSHPath() );
     m_dialog->scpPath->setText( st->getSCPPath() );
-   
+    m_dialog->rememberSshPass->setChecked( st->getBool("Environment/RememberSshPassEnabled") );
+
+    m_dialog->showTips->setChecked( st->getBool("UI/NoStartTip") );
 
 // set label icons colors and text strings using user's settings
 
@@ -183,9 +203,11 @@ PrefsDialog::PrefsDialog(QWidget *parent) : QDialog(parent)
     rulesFont = st->getRulesFont();
     treeFont = st->getTreeFont();
     uiFont = st->getUiFont();
+    compilerOutputFont = st->getCompilerOutputFont();
 
     m_dialog->rulesFontDescr->setText(getFontDescription(rulesFont));
     m_dialog->treeFontDescr->setText(getFontDescription(treeFont));
+    m_dialog->compilerOutputFontDescr->setText(getFontDescription(compilerOutputFont));
 
     m_dialog->chClipComment->setChecked(st->getClipComment() );
 
@@ -196,6 +218,56 @@ PrefsDialog::PrefsDialog(QWidget *parent) : QDialog(parent)
 #if !defined(Q_OS_WIN32)
     m_dialog->plink_hint->hide();
 #endif
+
+    // Fill lists of platforms and host OS
+
+    QMap<QString,QString> platforms = getAllPlatforms(false);
+    QMap<QString,QString> os = getAllOS(false);
+
+    m_dialog->enabled_platforms->setRowCount(platforms.size());
+    m_dialog->enabled_platforms->setColumnCount(1);
+    int row = 0;
+    QMap<QString,QString>::iterator it;
+    for (it=platforms.begin(); it!=platforms.end(); ++it)
+    {
+        QString name = it.key();
+        QString readable_name = it.value();
+        QTableWidgetItem *cb = new QTableWidgetItem(readable_name);
+        m_dialog->enabled_platforms->setItem(row, 0, cb);
+
+        QString res_status = Resources::platform_res[name.toStdString()]->getResourceStr(
+            "/FWBuilderResources/Target/status/").c_str();
+        QString prefs_status = st->getTargetStatus(name, res_status);
+        cb->setCheckState((prefs_status=="disabled") ? Qt::Unchecked : Qt::Checked);
+
+        cb->setData(Qt::UserRole, name);
+        row++;
+    }
+    m_dialog->enabled_platforms->horizontalHeader()->setStretchLastSection(true);
+    m_dialog->enabled_platforms->sortItems(0);
+    m_dialog->enabled_platforms->update();
+
+    m_dialog->enabled_os->setRowCount(os.size());
+    m_dialog->enabled_os->setColumnCount(1);
+    row = 0;
+    for (it=os.begin(); it!=os.end(); ++it)
+    {
+        QString name = it.key();
+        QString readable_name = it.value();
+        QTableWidgetItem *cb = new QTableWidgetItem(readable_name);
+        m_dialog->enabled_os->setItem(row, 0, cb);
+
+        QString res_status = Resources::os_res[name.toStdString()]->getResourceStr(
+            "/FWBuilderResources/Target/status/").c_str();
+        QString prefs_status = st->getTargetStatus(name, res_status);
+        cb->setCheckState((prefs_status=="disabled") ? Qt::Unchecked : Qt::Checked);
+
+        cb->setData(Qt::UserRole, name);
+        row++;
+    }
+    m_dialog->enabled_os->horizontalHeader()->setStretchLastSection(true);
+    m_dialog->enabled_os->sortItems(0);
+    m_dialog->enabled_os->update();
 }
 
 QString PrefsDialog::getFontDescription(const QFont &font)
@@ -281,12 +353,20 @@ void PrefsDialog::changeTreeFont()
     m_dialog->treeFontDescr->setText(getFontDescription(treeFont));
 }
 
+void PrefsDialog::changeCompilerOutputFont()
+{
+    changeFont(compilerOutputFont);
+    m_dialog->compilerOutputFontDescr->setText(getFontDescription(compilerOutputFont));
+}
+
 void PrefsDialog::changeFont(QFont &font)
 {
     bool ok;
     QFont f = QFontDialog::getFont(&ok, font, this);
     if (ok)
+    {
         font = f;
+    }
 }
 
 void PrefsDialog::findWDir()
@@ -327,12 +407,29 @@ void PrefsDialog::accept()
     st->setObjTooltips( m_dialog->objTooltips->isChecked() );
     st->setTooltipDelay( m_dialog->tooltipDelay->value() );
 
-    st->setBool("UI/ShowDeletedObjects", m_dialog->deletedObj->isChecked() );
+    st->setBool("UI/ShowDeletedObjects", m_dialog->deletedObj->isChecked());
+    st->setBool("UI/ShowObjectsAttributesInTree", m_dialog->attributesInTree->isChecked());
+
+    ProjectPanel *pp = mw->activeProject();
+    if (pp)
+    {
+        pp->m_panel->om->setAttributesColumnEnabled(
+            m_dialog->attributesInTree->isChecked());
+        if (m_dialog->attributesInTree->isChecked())
+            pp->m_panel->om->loadSectionSizes();
+    }
+
+    st->setBool("Objects/DNSName/useCompileTimeForNewObjects",
+                m_dialog->new_dns_name_compile_tm->isChecked());
+    st->setBool("Objects/DNSName/useNameForDNSRecord",
+                m_dialog->use_name_for_dns_record->isChecked());
+
+    st->setBool("Objects/AddressTable/useCompileTimeForNewObjects",
+                m_dialog->new_addr_tbl_compile_tm->isChecked());
 
 //  QToolTip::setWakeUpDelay( st->getTooltipDelay()*1000 );
 
     st->setRCSLogState( m_dialog->emptyRCSLog->isChecked() );
-    st->setAutoSave( m_dialog->autosave->isChecked() );
 
     st->setBool("Environment/autoSaveFile", m_dialog->autosaveFile->isChecked());
     st->setInt("Environment/autoSaveFilePeriod", m_dialog->autosaveInterval->value() );
@@ -366,6 +463,7 @@ void PrefsDialog::accept()
     st->setRulesFont(rulesFont);
     st->setTreeFont(treeFont);
     st->setUiFont(uiFont);
+    st->setCompilerOutputFont(compilerOutputFont);
 
     st->setClipComment(m_dialog->chClipComment->isChecked());
     st->setCheckUpdates(m_dialog->checkUpdates->isChecked());
@@ -373,6 +471,24 @@ void PrefsDialog::accept()
 
     st->setSSHPath( m_dialog->sshPath->text() );
     st->setSCPPath( m_dialog->scpPath->text() );
+    st->setBool("Environment/RememberSshPassEnabled", m_dialog->rememberSshPass->isChecked());
+    
+    for (int row=0; row < m_dialog->enabled_platforms->rowCount(); ++row)
+    {
+        QTableWidgetItem *itm = m_dialog->enabled_platforms->item(row, 0);
+        st->setTargetStatus(itm->data(Qt::UserRole).toString(),
+                            (itm && itm->checkState() == Qt::Unchecked) ?
+                            "disabled" : "active");
+    }
+
+    QStringList disabled_os;
+    for (int row=0; row < m_dialog->enabled_os->rowCount(); ++row)
+    {
+        QTableWidgetItem *itm = m_dialog->enabled_os->item(row, 0);
+        st->setTargetStatus(itm->data(Qt::UserRole).toString(),
+                            (itm && itm->checkState() == Qt::Unchecked) ?
+                            "disabled" : "active");
+    }
 
     if (!wd.isEmpty())
     {
@@ -380,10 +496,18 @@ void PrefsDialog::accept()
         d.mkdir( wd );
     }
 
+    if (pp)
+    {
+        RuleSetView* rsv = pp->getCurrentRuleSetView();
+        if (rsv)
+            rsv->updateAll();
+    }
+
     mw->setupAutoSave();
     mw->showDeletedObjects(st->getBool("UI/ShowDeletedObjects"));
     mw->updateTreeFont();
 //    app->setFont(st->getTreeFont());
+    st->setBool("UI/NoStartTip", m_dialog->showTips->isChecked());
     QDialog::accept();
 }
 
