@@ -6,7 +6,7 @@
 
   Author:  alek@codeminders.com
 
-  $Id: ProjectPanel.h 973 2009-05-29 16:15:31Z vadim $
+  $Id: ProjectPanel.h 2588 2010-02-19 19:29:04Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -26,9 +26,13 @@
 #ifndef PROJECTPANEL_H
 #define PROJECTPANEL_H
 
-#include "ObjectEditor.h"
 #include "ui_projectpanel_q.h"
 
+#include <set>
+#include <map>
+
+
+
 namespace libfwbuilder {
     class FWObjectDatabase;
     class Firewall;
@@ -43,6 +47,8 @@ namespace libfwbuilder {
 class QWidget;
 class QMdiSubWindow;
 class QTextEdit;
+class QUndoStack;
+
 class ObjectTreeView;
 class ObjectManipulator;
 class findDialog;
@@ -50,9 +56,6 @@ class FWWindow;
 class RuleSetView;
 class RCS;
 
-class FindObjectWidget;
-class FindWhereUsedWidget;
-class listOfLibraries;
 class FWBTree;
 
 #define DEFAULT_H_SPLITTER_POSITION 250
@@ -60,72 +63,71 @@ class FWBTree;
 
 
 class ProjectPanel: public QWidget {
-    Q_OBJECT
+    Q_OBJECT;
 
-    FWWindow                               *mainW;
-    RCS                                    *rcs;
-    listOfLibraries                        *addOnLibs;
-    FWBTree                                *objectTreeFormat;
-    bool                                    systemFile;
-    bool                                    safeMode;
-    bool                                    editingStandardLib;
-    bool                                    editingTemplateLib;
-    bool                                    ruleSetRedrawPending;
-    bool                                    closing ;
-    bool                                    ready;
-    QString                                 startupFileName;
+    FWWindow *mainW;
+    RCS *rcs;
+    bool systemFile;
+    bool safeMode;
+    bool editingStandardLib;
+    bool editingTemplateLib;
+    bool ruleSetRedrawPending;
+    bool ready;
     
-    libfwbuilder::FWObjectDatabase         *objdb;
-    QWidget                                *editorOwner;
+    libfwbuilder::FWObjectDatabase *objdb;
     
-    //QTextEdit                              *oi;
-    //ObjectManipulator                      *om; 
-    ObjectEditor                           *oe;
-    findDialog                             *fd;
-    libfwbuilder::FWObject                 *shownInInfo;
+    findDialog *fd;
         
-    QTimer                                 *autosaveTimer;
+    QTimer *autosaveTimer;
     std::map<libfwbuilder::FWObject*, RuleSetView*> ruleSetViews;
-    int                                     ruleSetTabIndex;
+    int ruleSetTabIndex;
     
-    libfwbuilder::FWObject                 *visibleFirewall;
-    libfwbuilder::RuleSet                  *visibleRuleSet ;
-    std::vector<libfwbuilder::FWObject*>    firewalls;
+    libfwbuilder::FWObject *visibleFirewall;
+    libfwbuilder::RuleSet *visibleRuleSet ;
+    std::vector<libfwbuilder::FWObject*> firewalls;
+
+    int lastFirewallIdx;
+    bool changingTabs;
+    QString noFirewalls;
+    bool loading_state;
+
+    // set of object IDs for objects that have been modified recently.
+    // Method updateLastModifiedTimestampForAllFirewalls() uses this
+    // to update lastModified timestamp of all firewalls using these objects.
+    std::set<int> lastModifiedTimestampChangePool;
+
+    // IDs of objects that need to be updated in the tree. Method
+    // updateObjectInTree() processes them. The key in the map is object ID,
+    // value is a boolean flag, true means need to update the object and 
+    // subtree under it.
+    std::map<int, bool> updateObjectsInTreePool;
+
+    bool treeReloadPending;
 
-    int                                     lastFirewallIdx;
-    bool                                    changingTabs;
-    QString                                 noFirewalls;
-    bool                                    enableAvtoSaveState;
-    
 public:
 
     QMdiSubWindow *mdiWindow;
     Ui::ProjectPanel_q *m_panel;
-    FindObjectWidget *findObjectWidget;
-    FindWhereUsedWidget *findWhereUsedWidget;
     QSet<QString> copySet;
 
+    QUndoStack *undoStack;
     
     void readyStatus(bool f) { ready=f; }
 
     libfwbuilder::RuleSet* getCurrentRuleSet () {return visibleRuleSet;};
     RuleSetView* getCurrentRuleSetView() ;
 
-    ProjectPanel* clone(ProjectPanel * cln);
-
-    void openRuleSet(libfwbuilder::FWObject *obj);
+    void openRuleSet(libfwbuilder::FWObject *obj, bool immediately=false);
     void closeRuleSet(libfwbuilder::FWObject *obj)
     { if ((libfwbuilder::FWObject*)(visibleRuleSet)==obj) visibleRuleSet=NULL;};
     
-    FWBTree * getFWTree () {return objectTreeFormat;};
-    bool isClosing (){return closing;};
-
-
     ProjectPanel(QWidget *parent);
     ~ProjectPanel();
 
 
     void initMain(FWWindow *main);
+    void reset();
+    
     void loadObjects();
     void loadObjects(libfwbuilder::FWObjectDatabase *db);
     void clearObjects();
@@ -136,13 +138,9 @@ public:
     libfwbuilder::RuleElement* getRE(libfwbuilder::Rule* r, int col );
 
     //wrapers for some ObjectManipulator functions
-    libfwbuilder::FWObject* getOpened();
     
     libfwbuilder::FWObject*  getCurrentLib();
 
-    void insertObjectInTree(libfwbuilder::FWObject *parent,
-                            libfwbuilder::FWObject *obj);
-    
     libfwbuilder::FWObject* createObject(const QString &objType,
                                          const QString &objName,
                                          libfwbuilder::FWObject *copyFrom=NULL);
@@ -152,6 +150,7 @@ public:
                                          const QString &objName,
                                          libfwbuilder::FWObject *copyFrom=NULL);
 
+    void updateObjectInTree(libfwbuilder::FWObject *obj, bool subtree=false);
 
     FWWindow* getWindow (){ return mainW;}     
     void moveObject(libfwbuilder::FWObject *target,
@@ -160,41 +159,18 @@ public:
     void moveObject(const QString &targetLibName,
                     libfwbuilder::FWObject *obj);
 
-    void autorename(libfwbuilder::FWObject *obj,
-                    const std::string &objtype,
-                    const std::string &namesuffix);
+    void registerModifiedObject(libfwbuilder::FWObject *o);
+    void registerObjectToUpdateInTree(libfwbuilder::FWObject *o, bool update_subtree);
+    void registerTreeReloadRequest();
+    void registerRuleSetRedrawRequest();
 
-    void updateLibColor(libfwbuilder::FWObject *lib);
-    void updateLibName(libfwbuilder::FWObject *lib);
-
-    void updateObjName(libfwbuilder::FWObject *obj,
-                       const QString &oldName,
-                       bool  askForAutorename=true);
-    void updateObjName(libfwbuilder::FWObject *obj,
-                       const QString &oldName,
-                       const QString &oldLabel,
-                       bool  askForAutorename=true);
-
-    void updateLastModifiedTimestampForOneFirewall(libfwbuilder::FWObject *o);
-    void updateLastModifiedTimestampForAllFirewalls(libfwbuilder::FWObject *o);   
-    void updateLastInstalledTimestamp(libfwbuilder::FWObject *o);
-    void updateLastCompiledTimestamp(libfwbuilder::FWObject *o);
-       
-    void loadDataFromFw(libfwbuilder::Firewall *fw);
-    
     libfwbuilder::FWObject* pasteTo(libfwbuilder::FWObject *target,
                                     libfwbuilder::FWObject *obj);
-    void delObj(libfwbuilder::FWObject *obj,bool openobj=true);
+
     ObjectTreeView* getCurrentObjectTree();
-    void openObject(QTreeWidgetItem *otvi);
-    void openObject(libfwbuilder::FWObject *obj);
-    bool editObject(libfwbuilder::FWObject *obj);
+
     void findAllFirewalls (std::list<libfwbuilder::Firewall *> &fws);
 
-    libfwbuilder::FWObject* duplicateObject(libfwbuilder::FWObject *target,
-                                            libfwbuilder::FWObject *obj,
-                                            const QString &name = QString::null,
-                                            bool  askForAutorename=true);
     void showDeletedObjects(bool f);
     void select();
     void unselect();
@@ -211,46 +187,21 @@ public:
     void setManipulatorFocus();
     void clearManipulatorFocus();
     
-    //wrapers for some Object Editor functions
-    bool isEditorVisible();
-    bool isEditorModified();
-    
-    void showEditor();
-    void hideEditor();
-    void closeEditor();
-    
-    void openEditor(libfwbuilder::FWObject *o);
-    void openOptEditor(libfwbuilder::FWObject *, ObjectEditor::OptType t);
-    void blankEditor();
-    
-    libfwbuilder::FWObject* getOpenedEditor();
-    ObjectEditor::OptType getOpenedOptEditor();
-    
-    void selectObjectInEditor(libfwbuilder::FWObject *o);
-
-    void actionChangedEditor(libfwbuilder::FWObject *o);
-    bool validateAndSaveEditor();
     //find dialog functions wrapers
     void setFDObject(libfwbuilder::FWObject *o);
     
-    void info(libfwbuilder::FWObject *o, bool forced = false);    
     void resetFD();
     
     void clearFirewallTabs();
+    void closeRuleSetPanel();
     void ensureObjectVisibleInRules(libfwbuilder::FWReference *obj);
-    void ensureRuleIsVisible(libfwbuilder::Rule *rule, int col=0);
     
     libfwbuilder::FWObject* getVisibleFirewall() { return visibleFirewall; }
     RuleSetView* getRuleSetViews(libfwbuilder::FWObject *o) 
     {return ruleSetViews[o];};
 
-    void updateRuleSetViewSelection();
-    void updateRuleSetView();
-    void updateRuleOptions();
-    void updateTreeViewItemOrder();
     int  findFirewallInList(libfwbuilder::FWObject *f);
     void updateFirewallName();
-    void scheduleRuleSetRedraw();
     void selectRules();
     void unselectRules();
     void editCopy();
@@ -259,7 +210,18 @@ public:
     void editPaste();
     
     bool saveIfModified();
-    QString getDestDir(const QString &fname);
+
+    // semi-intelligent way to guess most appropriate
+    // destination directory for various file save or file open
+    // operations. If working directory is configured in
+    // preferences, then getDestDir returns that. If it is not
+    // configured and file name is given on the command line,
+    // directory where that file is located is returned. If
+    // parameter filename is empty, then current directory
+    // is returned (however on windows and mac userDataDir is returned)
+
+    QString getDestDir(const QString &filename);
+    
     QString chooseNewFileName(const QString &fname, const QString &title);
     void setFileName(const QString &fname);
     void restoreDepends(libfwbuilder::FWObject *obj_old,
@@ -267,11 +229,15 @@ public:
                         const std::map<int,
                         libfwbuilder::FWObject *> &objByIds);
 
-    QString getPageTitle();
+    QString getPageTitle(bool file_path=false);
+
+    void setActive();
     
  public slots:
     void newObject();
-    void info();
+
+    void topLevelChangedForTreePanel(bool topLevel);
+    void visibilityChangedForTreePanel(bool topLevel);
     
     virtual void back();
     virtual void lockObject();
@@ -291,54 +257,50 @@ public:
 
     virtual void reopenFirewall();
     virtual void redrawRuleSets();
-    virtual void changeInfoStyle();
     virtual void restoreRuleSetTab();
 
     virtual void fileProp();
     virtual bool fileNew();
-    virtual bool fileOpen();
 
     virtual void fileClose();
     virtual void fileSave();
     virtual void fileSaveAs();
-    virtual void fileExit();
     virtual void fileCommit();
     virtual void fileDiscard();
     virtual void fileAddToRCS();
     
-    virtual void startupLoad();
-    
     virtual void fileImport();
     virtual void fileCompare();
     virtual void fileExport();
     
-    virtual void closeAuxiliaryPanel();
-    virtual void closeEditorPanel();
-    virtual void openEditorPanel();
-    
-    virtual void search();
-
-    virtual void compile(std::set<libfwbuilder::Firewall * > vf);
+    virtual void compile(std::set<libfwbuilder::Firewall*> vf);
     virtual void compile();
-    virtual void install(std::set<libfwbuilder::Firewall * > vf);
+    virtual void install(std::set<libfwbuilder::Firewall*> vf);
     virtual void install();
-     
-    virtual void rollBackSelectionSameWidget();
-    virtual void rollBackSelectionDifferentWidget();
+    virtual void transferfw(std::set<libfwbuilder::Firewall*> vf);
+    virtual void transferfw();
+
     void splitterMoved ( int pos, int index );
-    void stateChanged(Qt::WindowStates oldState, Qt::WindowStates newState);
 
     virtual void autoSave();
-    
- signals:
-     void restoreSelection_sign(bool same_widget);
+    virtual void compileThis();
+    virtual void installThis();
 
+    void updateLastModifiedTimestampForAllFirewalls();
+    void updateObjectInTree();
+    void reloadTree();
+
+#if QT_VERSION < 0x040500
+    void setTreeDockPosition();
+#endif
+
+    void splitterPositionChanged(int, int);
 
 public:
     QString getFileName();
     bool editingLibrary();
     void createRCS( const QString &filename);
-    void loadFromRCS(RCS *rcs);
+    bool loadFromRCS(RCS *rcs);
     void loadStandardObjects();
     bool loadFile(const QString &fileName, bool load_rcs_head);
 
@@ -347,6 +309,9 @@ public:
     void saveState();
     void loadState(bool open_objects=true);
 
+    void saveMainSplitter();
+    void loadMainSplitter();
+    
     void loadOpenedRuleSet();
     void saveOpenedRuleSet();
 
@@ -354,7 +319,7 @@ public:
     void loadLastOpenedLib();
     
     bool checkin(bool unlock);
-    void loadLibrary(const std::string &libfpath);
+    libfwbuilder::FWObject* loadLibrary(const std::string &libfpath);
     
     bool exportLibraryTest(std::list<libfwbuilder::FWObject*> &selectedLibs);
     void exportLibraryTo(QString fname,
@@ -366,39 +331,23 @@ public:
                        std::list<libfwbuilder::FWReference*> &extRefs);
     
     void setSafeMode(bool f) { safeMode=f; }
-    void setStartupFileName(const QString &fn);
     void setupAutoSave();
-    QString getCurrentFileName();
     RCS * getRCS();
-    void findObject(libfwbuilder::FWObject *);
-    void findWhereUsed(libfwbuilder::FWObject *);
     QString printHeader();
-    bool requestEditorOwnership(QWidget *w,
-                                libfwbuilder::FWObject *o,
-                                ObjectEditor::OptType   otype,
-                                bool validate = true);
-    void releaseEditor();
-    void connectEditor(QWidget *w);
-
-    bool isSystem(libfwbuilder::FWObject *obj);
-    bool isStandardId(libfwbuilder::FWObject *obj);    
-    bool validateForInsertion(libfwbuilder::FWObject *target,libfwbuilder::FWObject *obj);
-    bool getCopyMenuState(const QString &objPath);
-    bool getCutMenuState(const QString &objPath);
-    bool getPasteMenuState(const QString &objPath);
-    bool getDeleteMenuState(libfwbuilder::FWObject *obj);
-    libfwbuilder::FWObject* createNewLibrary(libfwbuilder::FWObjectDatabase *db);
 
+    void toggleViewTree(bool f);
 
 protected:
     int oldState ;
-    virtual void showEvent( QShowEvent *ev);
-    virtual void hideEvent( QHideEvent *ev);
-    virtual void closeEvent( QCloseEvent * );
-    virtual void resizeEvent ( QResizeEvent * event );
+    virtual void showEvent(QShowEvent *ev);
+    virtual void hideEvent(QHideEvent *ev);
+    virtual void closeEvent(QCloseEvent *ev);
+    virtual void resizeEvent(QResizeEvent *ev);
+    virtual bool event(QEvent *ev);
 
     void setMainSplitterPosition(int w1, int w2);
-    void setObjInfoSplitterPosition(int w1, int w2);
+    void collapseTree();
+    void collapseRules();
 };
 
 #endif
