@@ -7,7 +7,7 @@
   Author:  alek@codeminders.com
            refactoring and bugfixes: vadim@fwbuilder.org
 
-  $Id: ProjectPanel_file_ops.cpp 958 2009-05-21 17:55:48Z vadim $
+  $Id: ProjectPanel_file_ops.cpp 2644 2010-03-03 01:04:24Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,6 +28,7 @@
 #include "global.h"
 #include "utils.h"
 #include "utils_no_qt.h"
+#include "events.h"
 
 #include <fwbuilder/RuleSet.h>
 #include <fwbuilder/Policy.h>
@@ -52,15 +53,17 @@
 #include "ObjConflictResolutionDialog.h"
 #include "LibExportDialog.h"
 #include "longTextDialog.h"
+#include "FWBTree.h"
 
-//#include "listOfLibraries.h"
-
+#include <QtDebug>
 #include <QMdiSubWindow>
 #include <QMdiArea>
 #include <QStatusBar>
 #include <QTextStream>
 #include <QFile>
+#include <QFileInfo>
 #include <QTimer>
+#include <QUndoStack>
 
 #include <list>
 #include <algorithm>
@@ -69,6 +72,9 @@
 
 #include <libxml/tree.h>
 
+#include "memcheck.h"
+
+
 #define LONG_ERROR_CUTOFF 1024
 
 using namespace Ui;
@@ -77,7 +83,10 @@ using namespace std;
 
 bool ProjectPanel::saveIfModified()
 {
-    if (db() && db()->isDirty() && rcs && !rcs->getFileName().isEmpty())
+    if (fwbdebug) qDebug() << "ProjectPanel::saveIfModified()";
+
+    if (db() && db()->isDirty())
+//    if (db() && db()->isDirty() && rcs && !rcs->getFileName().isEmpty())
     {
         QString message = "Some objects have been modified but not saved.\n";
         message += "Do you want to save ";
@@ -108,11 +117,39 @@ QString ProjectPanel::chooseNewFileName(const QString &fname,
 {
     QString destdir = getDestDir(fname);
 
-    // Note that QFileDialog::getSaveFileName asks for confirmation
-    // if the file already exists.
-    QString fn = QFileDialog::getSaveFileName( this, title, destdir,
-       tr( "FWB Files (*.fwb);;All Files (*)" ) );
+    // when file open dialog is created using static function
+    // QFileDialog::getSaveFileName, its behavior is different on
+    // Linux and Mac (did not check on windows)
 
+    // Native dialog usd by Qt on Mac automatically adds .fwb suffix
+    // to the file name user enters if it does not have any suffix. It
+    // checks for the conflicts with exitsing files _after_ the suffix
+    // has been added. On Linux dialog created by the static function
+    // QFileDialog::getSaveFileName does not add suffix and checks for
+    // conflicts using the name without one. Since I used to add
+    // suffix here but did not check for the conflict again after
+    // that, it was possible for the user to enter name with no suffix
+    // and that way overwrite old file without warning. Will avoid
+    // static fucntion and instead build dialog manually and use
+    // setDefaultSuffix() to enforce suffix.
+
+    QFileDialog fd(this);
+    fd.setFileMode(QFileDialog::AnyFile);
+    fd.setDefaultSuffix("fwb");
+    fd.setFilter(tr( "FWB Files (*.fwb);;All Files (*)" ) );
+    fd.setWindowTitle(title);
+    fd.setDirectory(destdir);
+    fd.setAcceptMode(QFileDialog::AcceptSave);
+
+    QString fn;
+    if (fd.exec())
+    {
+        QStringList fileNames = fd.selectedFiles();
+        fn = fileNames.front();
+    }
+
+    QFileInfo finfo(fn);
+    if (finfo.suffix().isEmpty()) fn += ".fwb";
     return fn;
 }
 
@@ -146,79 +183,27 @@ bool ProjectPanel::fileNew()
         setFileName(nfn);
         save();
         setupAutoSave();
+
+        QCoreApplication::postEvent(mw, new updateSubWindowTitlesEvent());
     }
 
     if (fwbdebug)
         qDebug("ProjectPanel::fileNew()  rcs=%p  rcs->getFileName()='%s'",
                rcs, rcs->getFileName().toAscii().constData());
 
+    // reset actions, including Save() which should now
+    // be inactive
+    mw->prepareFileMenu();
+
     return (rcs!=NULL);
 }
 
-bool ProjectPanel::fileOpen()
-{
-/*
- * there appears to be a bug on Mac OS X that causes main window to
- * switch to a different MDI subwindow if user opens "File Open"
- * dialog and cancels operation by clicking "Cancel". This does not
- * happen on other platforms so this must be Mac only thing. To work
- * around the issue we save pointer to the MDI subwindow that was
- * active before activation of the "OpenFile" dialog and restore it if
- * user cancelled operation.
- *
- * This only happens in fileOpen(). fileImport() uses the same
- * QFileDialog dialog but does not trigger this effect. Perhaps this
- * happens because we call fileOpen from newly created but still
- * hidden ProjectPanel object. This is the only difference I can think
- * of.
- */
-    QMdiSubWindow *current_active_window = mainW->getMdiArea()->activeSubWindow();
-
-    QString dir;
-
-/*
- * Pick default directory where to look for the file to open.
- * 1) if "work directory" is configured in preferences, always use it
- * 2) if it is blank, use the same directory where currently opened file is
- * 3) if this is the first file to be opened, get directory where the user opened
- *    during last session from settings using st->getOpenFileDir
- */
-
-    dir = st->getWDir();
-    if (fwbdebug) qDebug("Choosing directory for file open 1: %s",
-                         dir.toStdString().c_str());
-
-    if (dir.isEmpty() && !mw->getCurrentFileName().isEmpty())
-        dir = getFileDir(mw->getCurrentFileName());
-    if (fwbdebug) qDebug("Choosing directory for file open 2: %s",
-                         dir.toStdString().c_str());
-
-    if (dir.isEmpty()) dir = st->getOpenFileDir();
-    if (fwbdebug) qDebug("Choosing directory for file open 3: %s",
-                         dir.toStdString().c_str());
-
-    QString fileName = QFileDialog::getOpenFileName(
-        mainW,
-        tr("Open File"),
-        dir,
-        "FWB files (*.fwb *.fwl *.xml);;All Files (*)");
-    
-    if (fileName.isEmpty())
-    {
-        mainW->getMdiArea()->setActiveSubWindow(current_active_window);
-        return false;
-    }
-
-    bool res = loadFile(fileName, false);
-    if (res) mw->updateOpenRecentMenu(fileName);
-
-    return res;
-}
-
 bool ProjectPanel::loadFile(const QString &fileName, bool load_rcs_head)
 {
-    if (fwbdebug) qDebug("ProjectPanel::loadFile  fileName=%s load_rcs_head=%d",
-                         fileName.toLocal8Bit().constData(), load_rcs_head);
+    if (fwbdebug)
+        qDebug() << "ProjectPanel::loadFile  fileName="
+                 << fileName.toLocal8Bit()
+                 << "load_rcs_head=" << load_rcs_head;
 
     RCSFilePreview  fp(this);
     bool hasRCS = fp.showFileRLog(fileName);
@@ -237,7 +222,12 @@ bool ProjectPanel::loadFile(const QString &fileName, bool load_rcs_head)
     if (dlg_res!=QDialog::Accepted) return false;
 
     if (!saveIfModified() || !checkin(true)) return false;
-    if (!systemFile && rcs!=NULL) fileClose();
+
+    if (!systemFile && rcs!=NULL)
+    {
+        if (mw->isEditorVisible()) mw->hideEditor();
+        reset();
+    }
 
     //try to get simple rcs instance from RCS preview
     RCS *new_rcs = fp.getSelectedRev();
@@ -250,47 +240,33 @@ bool ProjectPanel::loadFile(const QString &fileName, bool load_rcs_head)
     try
     {
         new_rcs->co();
+        if (loadFromRCS(new_rcs)) 
+        {
+            if (new_rcs->isTemp())
+                unlink(new_rcs->getFileName().toLocal8Bit().constData());
+
+            st->setOpenFileDir(getFileDir(fileName));
+            return true;
+        }
+
     } catch (FWException &ex)
     {
         return false;
     }
 
-    loadFromRCS(new_rcs);
-
-    if (new_rcs->isTemp())
-        unlink(new_rcs->getFileName().toLocal8Bit().constData());
-
-    st->setOpenFileDir(getFileDir(fileName));
-
-    return true;
+    return false;
 }
 
 void ProjectPanel::fileClose()
 {
     if (fwbdebug) qDebug("ProjectPanel::fileClose(): start");
-
-    closing = true ;
-
-    findObjectWidget->init();
-    if (isEditorVisible()) hideEditor();
-
+    if (mw->isEditorVisible()) mw->hideEditor();
     if (!saveIfModified() || !checkin(true))  return;
-
-    if (rcs) delete rcs;
-    rcs=NULL;
-
-    if (fwbdebug) qDebug("ProjectPanel::fileClose(): clearing widgets");
-
-    FWObjectClipboard::obj_clipboard->clear();
-
+    reset();
     mdiWindow->close();
-
-    firewalls.clear();
-    visibleFirewall = NULL;
-    visibleRuleSet = NULL;
-    clearFirewallTabs();
-    clearObjects();
-
+    // reset actions, including Save() which should now
+    // be inactive
+    mw->prepareFileMenu();
     if (fwbdebug) qDebug("ProjectPanel::fileClose(): done");
 }
 
@@ -307,17 +283,12 @@ void ProjectPanel::autoSave()
 
 void ProjectPanel::fileSave()
 {
-    QStatusBar *sb = mainW->statusBar();
-    sb->showMessage( tr("Saving data to file...") );
-    QApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
     save();
-    sb->clearMessage();
-    QApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
 }
 
 void ProjectPanel::fileSaveAs()
 {
-    if (isEditorVisible()) hideEditor();
+    if (mw->isEditorVisible()) mw->hideEditor();
 
 /* we need to save data into the current file before we save it into a
  * new file, provided we do have current file
@@ -349,21 +320,15 @@ void ProjectPanel::fileSaveAs()
     }
 }
 
-void ProjectPanel::fileExit()
-{
-    if (saveIfModified() && checkin(true))
-    {
-        saveState();
-        if (rcs) delete rcs;
-        qApp->quit();
-    }
-}
-
 void ProjectPanel::fileCommit()
 {
     save();
     if (!checkin(true))  return;
     rcs->co();
+
+    // reset actions, including Save() which should now
+    // be inactive
+    mw->prepareFileMenu();
 }
 
 /*
@@ -373,11 +338,11 @@ void ProjectPanel::fileCommit()
 void ProjectPanel::fileDiscard()
 {
     if (QMessageBox::warning(this, "Firewall Builder",
-      tr("This operation discards all changes that have been saved\n"
-         "into the file so far, closes it and replaces it with a clean\n"
-         "copy of its head revision from RCS.\n"
+      tr("This operation discards all changes that have been saved "
+         "into the file so far, closes it and replaces it with a clean "
+         "copy of its head revision from RCS."
          "\n"
-         "All changes will be lost if you do this.\n\n"),
+         "All changes will be lost if you do this.\n"),
       tr("&Discard changes"),
       tr("&Cancel"), QString::null,
       1 )==0 )
@@ -394,8 +359,7 @@ void ProjectPanel::fileDiscard()
         /* do everything fileClose() does except do not close mdiWindow
          * because we'll need it again to reopen the file into
          */
-        findObjectWidget->init();
-        if (isEditorVisible()) hideEditor();
+        if (mw->isEditorVisible()) mw->hideEditor();
 
         if (rcs) delete rcs;
         rcs=NULL;
@@ -412,6 +376,10 @@ void ProjectPanel::fileDiscard()
          * open, which it isn't because we reset rcs to NULL
          */
         loadFile(fname, false);
+
+        // reset actions, including Save() which should now
+        // be inactive
+        mw->prepareFileMenu();
     }
 }
 
@@ -442,7 +410,13 @@ void ProjectPanel::fileAddToRCS()
             0, 1 );
     }
 
-    setWindowTitle(getPageTitle());
+    // reset actions, including Save() which should now
+    // be inactive
+    mw->prepareFileMenu();
+
+    //setWindowTitle(getPageTitle());
+    QCoreApplication::postEvent(mw, new updateSubWindowTitlesEvent());
+
 }
 
 void ProjectPanel::fileImport()
@@ -457,9 +431,18 @@ void ProjectPanel::fileImport()
 
     if (fname.isEmpty()) return;   // Cancel  - keep working with old file
 
-    loadLibrary( fname.toLocal8Bit().constData() );
+    FWObject *new_lib = loadLibrary( fname.toLocal8Bit().constData() );
 
     loadObjects();
+
+    // reset actions, including Save() which should now
+    // be inactive
+    mw->prepareFileMenu();
+
+    //QCoreApplication::postEvent(this, new openLibraryForObjectEvent(
+    //                                getFileName(), new_lib->getId()));
+
+    m_panel->om->openLib(new_lib);
 }
 
 
@@ -567,7 +550,7 @@ void ProjectPanel::fileCompare()
                            tr( "TXT Files (*.txt);;All Files (*)" ));
 
             if (fwbdebug)
-                qDebug( QString("Saving report to %1").arg(fn).toAscii().constData() );
+                qDebug() << QString("Saving report to %1").arg(fn);
 
             if (fn.isEmpty() ) return ;   // Cancel
 
@@ -778,7 +761,7 @@ bool ProjectPanel::exportLibraryTest(list<FWObject*> &selectedLibs)
             }
             s = s + "\n";
 
-            if (fwbdebug) qDebug(s.toAscii().constData());
+            if (fwbdebug) qDebug() << s;
 
             objlist = objlist + s;
         }
@@ -871,9 +854,14 @@ void ProjectPanel::findExternalRefs(FWObject *lib,
     }
 }
 
-void ProjectPanel::loadLibrary(const string &libfpath)
+/**
+ * Load library or several libraries from an external file. Return
+ * pointer to the last new imported library.
+ */
+FWObject* ProjectPanel::loadLibrary(const string &libfpath)
 {
     MessageBoxUpgradePredicate upgrade_predicate(mainW);
+    FWObject *last_new_lib = NULL;
 
     try
     {
@@ -883,9 +871,44 @@ void ProjectPanel::loadLibrary(const string &libfpath)
         FWObject *dobj = ndb->findInIndex(FWObjectDatabase::DELETED_OBJECTS_ID);
         if (dobj) ndb->remove(dobj, false);
 
+        set<int> duplicate_ids;
+        db()->findDuplicateIds(ndb, duplicate_ids);
+
+        map<int, int> id_mapping;
+        for (set<int>::iterator it=duplicate_ids.begin(); it!=duplicate_ids.end();
+             ++it)
+        {
+            FWObject *obj = ndb->findInIndex(*it);
+            assert(obj!=NULL);
+            int new_id = FWObjectDatabase::generateUniqueId();
+            obj->setId(new_id);
+            id_mapping[*it] = new_id;
+
+            // cerr << "Duplicate ID: " << *it 
+            //      << " " << FWObjectDatabase::getStringId(*it)
+            //      << obj->getPath()
+            //      << endl;
+        }
+        ndb->fixReferences(ndb, id_mapping);
+
+        int new_lib_id = -1;
+
+        // check for duplicate library names
+        FWObjectTypedChildIterator it2 = ndb->findByType(Library::TYPENAME);
+        for (; it2!=it2.end(); ++it2)
+        {
+            QString new_name = m_panel->om->makeNameUnique(
+                db(), QString::fromUtf8((*it2)->getName().c_str()), Library::TYPENAME);
+            (*it2)->setName(string(new_name.toUtf8()));
+            if ((*it2)->getId() != FWObjectDatabase::STANDARD_LIB_ID)
+                new_lib_id = (*it2)->getId();
+        }
+
         MergeConflictRes mcr(this);
         db()->merge(ndb, &mcr);
         delete ndb;
+
+        last_new_lib = db()->findInIndex(new_lib_id);
     } catch(FWException &ex)
     {
         QString error_txt = ex.toString().c_str();
@@ -902,6 +925,7 @@ void ProjectPanel::loadLibrary(const string &libfpath)
             tr("&Continue"), QString::null,QString::null,
             0, 1 );
     }
+    return last_new_lib;
 }
 
 /*
@@ -911,8 +935,6 @@ void ProjectPanel::loadStandardObjects()
 {
     if (fwbdebug) qDebug("ProjectPanel::load(): start");
 
-    QStatusBar *sb = mainW->statusBar();
-
     editingStandardLib = false;
     editingTemplateLib = false;
 
@@ -924,11 +946,16 @@ void ProjectPanel::loadStandardObjects()
     {
 // need to drop read-only flag on the database before I load new objects
 
+        if (objdb)
+        {
+            objdb->destroyChildren();
+            delete objdb;
+        }
+
         objdb = new FWObjectDatabase();
         objdb->setReadOnly( false );
 
-        sb->showMessage( tr("Loading system objects...") );
-        QApplication::processEvents(QEventLoop::ExcludeUserInputEvents,100);
+        mw->showStatusBarMessage(tr("Loading system objects..."));
 
 // always load system objects
         if (fwbdebug)
@@ -939,7 +966,8 @@ void ProjectPanel::loadStandardObjects()
 
         if (fwbdebug) qDebug("ProjectPanel::load(): create User library");
 
-        FWObject *userLib = createNewLibrary( objdb );
+        FWObject *userLib = FWBTree().createNewLibrary(objdb);
+
         userLib->setName("User");
         userLib->setStr("color","#d2ffd0");
 
@@ -948,7 +976,8 @@ void ProjectPanel::loadStandardObjects()
 
         createRCS("");
 
-        setWindowTitle(getPageTitle());
+        //setWindowTitle(getPageTitle());
+        QCoreApplication::postEvent(mw, new updateSubWindowTitlesEvent());
 
         loadObjects();
         setupAutoSave();
@@ -965,14 +994,10 @@ void ProjectPanel::loadStandardObjects()
             tr("&Continue"), QString::null,QString::null,
             0, 1 );
     }
-
-    sb->clearMessage();
 }
 
-void ProjectPanel::loadFromRCS(RCS *_rcs)
+bool ProjectPanel::loadFromRCS(RCS *_rcs)
 {
-    QStatusBar *sb = mainW->statusBar();
-
     resetFD();
 
     editingStandardLib = false;
@@ -990,33 +1015,36 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
         /* load the data file */
         systemFile = false;
 
+        clearObjects();
+
+        if (objdb)
+        {
+            objdb->destroyChildren();
+            delete objdb;
+        }
+
         objdb = new FWObjectDatabase();
 
 // need to drop read-only flag on the database before I load new objects
         objdb->setReadOnly( false );
 
 // always loading system objects
-        sb->showMessage( tr("Loading system objects...") );
-        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+        mw->showStatusBarMessage(tr("Loading system objects...") );
 
         objdb->load( sysfname, &upgrade_predicate, librespath);
         objdb->setFileName("");
 
 // objects from a data file are in database ndb
 
-        sb->showMessage( tr("Reading and parsing data file...") );
-        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+        mw->showStatusBarMessage(tr("Reading and parsing data file..."));
 
         FWObjectDatabase *ndb = new FWObjectDatabase();
         ndb->load(rcs->getFileName().toLocal8Bit().constData(),
                   &upgrade_predicate,librespath);
         time_t   oldtimestamp = ndb->getTimeLastModified();
 
-        sb->clearMessage();
-        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
-
 /* loadingLib is true if user wants to open a library or master library file */
-        bool loadingLib         = editingLibrary();
+        bool loadingLib = editingLibrary();
 
         if (fwbdebug)
         {
@@ -1043,22 +1071,18 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
             }
         }
 
-        sb->showMessage( tr("Merging with system objects...") );
-        QCoreApplication::processEvents(
-            QEventLoop::ExcludeUserInputEvents, 100);
-
+        mw->showStatusBarMessage(tr("Merging with system objects...") );
+        
         MergeConflictRes mcr(mainW);
         objdb->merge(ndb, &mcr);
 
+        ndb->destroyChildren();
         delete ndb;
 
         objdb->setFileName(rcs->getFileName().toLocal8Bit().constData());
         objdb->resetTimeLastModified(oldtimestamp);
         objdb->setDirty(false);
 
-        sb->clearMessage();
-        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents,100);
-
         if (fwbdebug)
         {
             qDebug("* Merge is done");
@@ -1110,7 +1134,7 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
             }
             QString newFileName = ofinfo.dir().absolutePath()
                 + "/" + ofinfo.completeBaseName() + ".fwb";
-
+            
             bool needToRename = true;
 
             /* need these dances with symlinks to fix bug #1008956:
@@ -1147,7 +1171,7 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
                      */
                     QMessageBox::warning(
                         this,"Firewall Builder",
-                        tr("Firewall Builder 2 uses file extension '.fwb' and\n"
+                        tr("Firewall Builder uses file extension '.fwb' and\n"
                            "needs to rename old data file '%1' to '%2',\n"
                            "but file '%3' already exists.\n"
                            "Choose a different name for the new file.")
@@ -1172,7 +1196,7 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
                             0, 1 );
 
                         loadStandardObjects();
-                        return;
+                        return false;
                     }
                     nfinfo.setFile(newFileName);
                 }
@@ -1182,7 +1206,7 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
 
                 QMessageBox::warning(
                 this,"Firewall Builder",
-                tr("Firewall Builder 2 uses file extension '.fwb'. Your data"
+                tr("Firewall Builder uses file extension '.fwb'. Your data"
                    "file '%1' \nhas been renamed '%2'")
                 .arg(fn).arg(newFileName),
                 tr("&Continue"), QString::null,QString::null,
@@ -1195,7 +1219,8 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
         rcs->setFileName(fn);
         db()->setFileName(fn.toLocal8Bit().constData());
 
-        setWindowTitle(getPageTitle());
+        //setWindowTitle(getPageTitle());
+        //QCoreApplication::postEvent(mw, new updateSubWindowTitlesEvent());
 
         mainW->disableActions(m_panel->ruleSets->count()!=0);
 
@@ -1204,7 +1229,7 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
             qDebug("ProjectPanel::load(): load complete dirty=%d "
                    "last_modified=%s",
                    db()->isDirty(), ctime(&last_modified));
-
+        
     } catch(FWException &ex)
     {
         QString trans = ex.getProperties()["failed_transformation"].c_str();
@@ -1231,7 +1256,7 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
                 0, 1 );
         } else
         {
-            QString error_txt = ex.toString().c_str();
+            QString error_txt = QString::fromUtf8(ex.toString().c_str());
             if (error_txt.length() > LONG_ERROR_CUTOFF) 
             {
                 error_txt.truncate(LONG_ERROR_CUTOFF);
@@ -1248,7 +1273,7 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
         }
         // load standard objects so the window does not remain empty
         loadStandardObjects();
-        return;
+        return false;
     }
 
     db()->setReadOnly( rcs->isRO() || rcs->isTemp() );
@@ -1256,19 +1281,15 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
 // clear dirty flag for all objects, recursively
     if (!forceSave)  db()->setDirty(false);
 
-    sb->showMessage( tr("Building object tree...") );
+    mw->showStatusBarMessage(tr("Building object tree..."));
     QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
 
     loadObjects();
     QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
 
-    sb->showMessage( tr("Indexing...") );
-    QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
+    mw->showStatusBarMessage(tr("Indexing...") );
     db()->reIndex();
 
-    sb->clearMessage();
-    QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
-
     setupAutoSave();
 
     time_t last_modified = db()->getTimeLastModified();
@@ -1276,6 +1297,7 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
         qDebug("ProjectPanel::load(): all done: "
                "dirty=%d last_modified=%s",
                db()->isDirty(), ctime(&last_modified));
+    return true;
 }
 
 bool ProjectPanel::checkin(bool unlock)
@@ -1299,8 +1321,12 @@ bool ProjectPanel::checkin(bool unlock)
                 tr("Checking file %1 in RCS").arg(rcs->getFileName()) +
                 QString("</b>")
             );
+
             if ( fsd_dialog->exec()== QDialog::Rejected )
             {
+                if (fwbdebug)
+                    qDebug() << "RCSFileSaveDialog_q  user hit Cancel";
+
                 delete fsd_dialog;
                 return false;
             }
@@ -1348,6 +1374,8 @@ void ProjectPanel::save()
                rcs->isTemp(),
                rcs->getFileName().toLocal8Bit().constData());
 
+    //undoStack->clear();
+
     if (!rcs->isRO() && !rcs->isTemp())
     {
         try
@@ -1357,9 +1385,23 @@ void ProjectPanel::save()
             else
             {
 /* editingLibfile is true if user edits a library or master library file */
+
+                mw->showStatusBarMessage(
+                    tr("Saving data to file %1").arg(rcs->getFileName()));
+
                 bool editingLibfile = editingLibrary();
 
-                if (st->getDontSaveStdLib())  // this is now default
+/* ****************************************************************
+ *
+ *      REMOVE THIS
+ *
+ * or may be not. The savings of not storing standard objects in each file
+ * are minimal but this code seems to be leaking too
+ *
+ ******************************************************************
+ */
+                //if (st->getDontSaveStdLib())  // this is now default
+                if (false)
                 {
                     list<FWObject*> userLibs;
                     list<FWObject*> ll = db()->getByType(Library::TYPENAME);
@@ -1398,13 +1440,19 @@ void ProjectPanel::save()
 
                     ndb->resetTimeLastModified( db()->getTimeLastModified() );
                     xmlSetCompressMode(st->getCompression() ? 9 : 0);
+
                     ndb->saveFile(
                         rcs->getFileName().toLocal8Bit().constData());
 
                     delete ndb;
 
                     QApplication::restoreOverrideCursor();
-
+                    // reset "dirty" flag only after we actually save the data
+                    // fixes #389
+                    db()->setDirty(false);
+                    // and reset actions, including Save() which should now
+                    // be inactive
+                    mw->prepareFileMenu();
                 } else
                 {
                     QApplication::setOverrideCursor(QCursor( Qt::WaitCursor));
@@ -1414,7 +1462,6 @@ void ProjectPanel::save()
                     QApplication::restoreOverrideCursor();
                 }
             }
-            db()->setDirty(false);
         }
         catch (FWException &ex)
         {
