@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland vadim@fwbuilder.org
 
-  $Id: RCS.cpp 686 2008-12-08 17:13:50Z svn $
+  $Id: RCS.cpp 2644 2010-03-03 01:04:24Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -42,6 +42,7 @@
 #include <qapplication.h>
 #include <qeventloop.h>
 #include <qtextcodec.h>
+#include <QtDebug>
 
 #include <stdlib.h>
 
@@ -64,16 +65,17 @@
 
 #include <iostream>
 
+
 using namespace std;
 using namespace libfwbuilder;
 
-QString     RCS::rcs_file_name     = "";
-QString     RCS::rlog_file_name    = "";
+QString     RCS::rcs_file_name = "";
+QString     RCS::rlog_file_name = "";
 QString     RCS::rcsdiff_file_name = "";
-QString     RCS::ci_file_name      = "";
-QString     RCS::co_file_name      = "";
-
-RCSEnvFix*  RCS::rcsenvfix         = NULL;
+QString     RCS::ci_file_name = "";
+QString     RCS::co_file_name = "";
+RCSEnvFix*  RCS::rcsenvfix = NULL;
+bool        RCS::rcs_available = false;
 
 /***********************************************************************
  *
@@ -192,7 +194,7 @@ RCSEnvFix::RCSEnvFix()
     TZOffset = tzsign + TZOffset;
 
     if (fwbdebug)
-	qDebug("tzoffset: %d TZOffset: '%s'",tzoffset,TZOffset.toAscii().constData());
+    qDebug("tzoffset: %d TZOffset: '%s'",tzoffset,TZOffset.toAscii().constData());
 
 #ifdef _WIN32
 /* need this crap because Windows does not set environment variable TZ
@@ -217,12 +219,14 @@ RCSEnvFix::RCSEnvFix()
 
 /* also need to set env variable USER for rcs tools, but if the user name
  * contains spaces, replace them with underscores (like "John Smith")
+ *
+ * global variable QString user_name is set in common/init.cpp
  */
-    QString uname=getUserName();
+    QString uname = user_name;
 
-    env.push_back( QString("USER=")+uname);
-    env.push_back( QString("LOGNAME=")+uname);
-    if (getenv("TMP")!=NULL)
+    env.push_back( QString("USER=") + uname);
+    env.push_back( QString("LOGNAME=") + uname);
+    if (getenv("TMP") != NULL)
         env.push_back( QString("TMP=") + getenv("TMP"));
     if (getenv("TEMP")!=NULL)
         env.push_back( QString("TEMP=") + getenv("TEMP"));
@@ -239,10 +243,9 @@ QStringList* RCSEnvFix::getEnv()
  * class RCS
  *
  ***********************************************************************/
-RCS::RCS(const QString &file)
-{
-    if (rcsenvfix==NULL) rcsenvfix = new RCSEnvFix();
 
+void RCS::init()
+{
     if (rcs_file_name=="")
     {
 #ifdef _WIN32
@@ -270,7 +273,37 @@ RCS::RCS(const QString &file)
 #endif
     }
 
-    filename      = file;
+    // now check if rcs tools are available. To test, try to run rlog
+    // with no arguments
+
+    QStringList arglist;
+
+    QProcess rcs_proc;
+    rcs_proc.start( rlog_file_name, arglist );
+    rcs_proc.waitForStarted();
+
+    if (rcs_proc.state() != QProcess::Running)
+    {
+        rcs_proc.close();
+        // rlog (and probably other RCS tools) are unavailable
+        if (fwbdebug) qDebug() << "RCS tools unavailable";
+        rcs_available = false;
+        return;
+    }
+
+    rcs_proc.waitForFinished();
+    rcs_proc.close();
+
+    rcs_available = true;
+}
+
+RCS::RCS(const QString &file)
+{
+    if (rcsenvfix==NULL) rcsenvfix = new RCSEnvFix();
+
+    QFileInfo fi(file);
+    if (fi.exists()) filename = fi.canonicalFilePath();
+    else filename = file;
     checked_out   = false;
     locked        = false;
     inrcs         = false;
@@ -286,6 +319,13 @@ RCS::RCS(const QString &file)
     connect(proc, SIGNAL(readyReadStandardError()),
             this,  SLOT(readFromStderr() ) );
 
+    if (!fi.exists())
+    {
+        inrcs = false;
+        tracking_file = true;
+        return;
+    }
+
     try
     {
         QString  rcspath = filename.left( filename.lastIndexOf("/") );
@@ -362,7 +402,7 @@ RCS::RCS(const QString &file)
                 r.log += log_rx.cap(1);
 
             r.log.replace('\r',"");
-            
+
 
             if (r.rev != "")
             {
@@ -390,6 +430,7 @@ RCS::RCS(const QString &file)
 
 RCS::~RCS()
 {
+    delete ciproc;
     delete proc;
 }
 
@@ -422,8 +463,11 @@ void RCS::readFromStderr()
 
 void RCS::setFileName(const QString &fn)
 {
-    filename=fn;
-    if (fwbdebug)  qDebug("RCS::setFileName fn = %s",fn.toAscii().constData());
+    QFileInfo fi(fn);
+    if (fi.exists()) filename = fi.canonicalFilePath();
+    else filename = fn;
+    if (fwbdebug)
+        qDebug() << "RCS::setFileName fn =" << fn << "filename =" << filename;
 }
 
 /*********************************************************************
@@ -432,7 +476,7 @@ void RCS::setFileName(const QString &fn)
 
 void RCS::abandon()
 {
-    if (!isInRCS()) return;
+    if (!isInRCS() || !rcs_available) return;
 
 /* check out head revision and unlock it */
     QStringList arglist;
@@ -483,9 +527,19 @@ void RCS::abandon()
  */
 void RCS::add() throw(libfwbuilder::FWException)
 {
-    int      i=filename.lastIndexOf("/");
-    QString  rcspath=filename.left(i);
-    QDir     rcsdir;
+    int i = filename.lastIndexOf("/");
+    QString rcspath = filename.left(i);
+    QDir rcsdir;
+
+    if (!rcs_available)
+    {
+        QString err = QObject::tr("RCS tools are unavailable");
+        if (fwbdebug) qDebug() << err;
+        throw(FWException(err.toStdString()));
+    }
+
+    if (fwbdebug) qDebug() << "RCS::add()  will run " << rcs_file_name;
+
     rcsdir.cd(rcspath);
 
     if (!rcsdir.exists("RCS")) rcsdir.mkdir("RCS");
@@ -531,13 +585,15 @@ void RCS::add() throw(libfwbuilder::FWException)
         }
     }
     QByteArray outp = proc->readAllStandardOutput();
-    QString msg=QObject::tr("Fatal error during initial RCS checkin of file %1 :\n %2\nExit status %3")
-	.arg(filename).arg(outp.data()).arg(proc->exitCode());
+    QString msg = QObject::tr("Fatal error during initial RCS checkin of file %1 :\n %2\nExit status %3")
+        .arg(filename).arg(outp.data()).arg(proc->exitCode());
     throw(FWException( msg.toLatin1().constData()  ));
 }
 
 bool RCS::isInRCS()
 {
+    if (!rcs_available) return false;
+
     if (tracking_file) return inrcs;
 
     QStringList arglist;
@@ -560,11 +616,11 @@ bool RCS::isInRCS()
     {
 /* exist status '1' means the file is not in RCS */
         inrcs=false;
-	if (fwbdebug)
-	{
+    if (fwbdebug)
+    {
           QByteArray outp = proc->readAllStandardOutput();
           qDebug("Error running rlog: %s",outp.data());
-	}
+    }
         return false;
     }
     inrcs=true;
@@ -608,7 +664,7 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
 {
 /* first check if filename is already in RCS */
 
-    if (!isInRCS()) return false;
+    if (!rcs_available || !isInRCS()) return false;
 
     if (ro)
     {
@@ -660,20 +716,20 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
                 strcat(tname,"_temp_XXXXXX");
                 int fd = mkstemp(tname);
 #endif
-		if (fd<0)
-		{
+        if (fd<0)
+        {
                     QString err = tr("Error creating temporary file ")+tname+QString(" :\n")+strerror(errno);
                     QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
                     throw(FWException(err.toLatin1().constData()));
-		}
+        }
 #ifdef _WIN32
                 if (_write(fd,stdoutBuffer.toLatin1().constData(),stdoutBuffer.length() )<0)
-		{
-		    _close(fd);
+        {
+            _close(fd);
 #else
                 if ( write(fd,stdoutBuffer.toLatin1().constData(),stdoutBuffer.length() )<0)
-		{
-		    close(fd);
+        {
+            close(fd);
 #endif
                     QString err = tr("Error writing to temporary file ")+tname+QString(" :\n")+strerror(errno);
                     QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
@@ -698,7 +754,8 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
 
     } else
     {
-        QString me=getUserName();
+        /* global variable QString user_name is set in common/init.cpp */
+        QString me = user_name;
         if (locked)
         {
 /* the file is already locked, can not just check it out like that */
@@ -796,7 +853,7 @@ bool RCS::ci( const QString &_lm,
               bool unlock) throw(libfwbuilder::FWException)
 {
 /* first check if filename is already in RCS */
-    if (!isInRCS()) return false;
+    if (!rcs_available || !isInRCS()) return false;
 
     QString logmsg = _lm;
 
@@ -896,6 +953,9 @@ bool RCS::ci( const QString &_lm,
  */
 QString RCS::rlog() throw(libfwbuilder::FWException)
 {
+    if (!rcs_available)
+        throw(FWException(QObject::tr("RCS tools are unavailable").toStdString()));
+
     QStringList arglist;
 
     arglist << QString("-z") + rcsenvfix->getTZOffset() << filename;
@@ -914,7 +974,10 @@ QString RCS::rlog() throw(libfwbuilder::FWException)
     proc->waitForStarted();
 
     if (proc->state() != QProcess::Running)
+    {
+        proc->close();
         throw(FWException("Fatal error running rlog "));
+    }
 
     if (fwbdebug) qDebug("Running rlog");
 
@@ -939,8 +1002,11 @@ QStringList RCS::rcsdiff(const QString&) throw(libfwbuilder::FWException)
     return temp.split("\n");
 }
 
-bool        RCS::isDiff(const QString &rev) throw(libfwbuilder::FWException)
+bool RCS::isDiff(const QString &rev) throw(libfwbuilder::FWException)
 {
+    if (!rcs_available)
+        throw(FWException(QObject::tr("RCS tools are unavailable").toStdString()));
+
     QStringList arglist;
 
     arglist << "-q";
@@ -989,4 +1055,3 @@ QString RCS::getSelectedRev()
     if (isInRCS()) return selectedRev;
     return "";
 }
-
