@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHIOS.h 304 2008-07-03 02:19:16Z vadim $
+  $Id: SSHIOS.h 2251 2009-12-23 01:44:15Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -30,11 +30,13 @@
 #include "../../config.h"
 #include "global.h"
 
-#include "SSHPIX.h"
+#include "SSHCisco.h"
 
-class SSHIOS : public SSHPIX {
+#include <QString>
 
-    Q_OBJECT
+class SSHIOS : public SSHCisco {
+
+    Q_OBJECT;
 
 public:
     
