@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHSession.cpp 769 2009-02-21 22:42:38Z vadim $
+  $Id: SSHSession.cpp 2261 2009-12-23 22:56:52Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -42,6 +42,7 @@
 #include <qtextcodec.h>
 #include <qapplication.h>
 #include <qdatetime.h>
+#include <QtDebug>
 
 #include <iostream>
 
@@ -50,6 +51,8 @@
 #endif
 
 
+//#define STATE_MACHINE_DEBUG 1
+
 using namespace std;
 
 const char  *SSHSession::newKeyOpenSSH  ="Are you sure you want to continue connecting (yes/no)?";
@@ -100,7 +103,7 @@ SSHSession::SSHSession(QWidget *_par,
     testRun=false;
     stripComments=false;
     wdir="";
-    conffile="";
+    script="";
     backupFile="";
     save_diff="";
     diff_pgm="";
@@ -232,13 +235,13 @@ void SSHSession::setOptions(instConf *cnf)
     setVerbose(cnf->verbose);
     setBackup(cnf->backup);
     setBackupFile(cnf->backup_file);
-    setIncr(cnf->incremental);
+    //setIncr(cnf->incremental);
     setDryRun(cnf->dry_run);
     setSaveStandby(cnf->saveStandby);
     setTestRun(cnf->testRun);
     setStripComments(cnf->stripComments);
     setWDir(cnf->wdir);
-    setConfFile(cnf->conffile);
+    setScript(cnf->script);
     setSaveDiff(cnf->save_diff);
     setDiffPgm(cnf->diff_pgm);
     setDiffFile(cnf->diff_file);
@@ -247,43 +250,98 @@ void SSHSession::setOptions(instConf *cnf)
 void SSHSession::terminate()
 {
     if (fwbdebug)
-        qDebug("SSHSession::terminate     this=%p  proc=%p   heartBeatTimer=%p",
-               this,proc,heartBeatTimer);
+        qDebug() << "SSHSession::terminate     this=" << this
+                 << "proc=" << proc
+                 << "heartBeatTimer=" << heartBeatTimer;
 
     stopHeartBeat();
 
     if (proc!=NULL)
     {
-//        disconnect(proc,SIGNAL(readyReadStdout()),
-//                   this,SLOT(readFromStdout() ) );
-//        disconnect(proc,SIGNAL(readyReadStderr()),
-//                   this,SLOT(readFromStderr() ) );
-
         disconnect(proc,SIGNAL(finished(int, QProcess::ExitStatus)),
                    this,SLOT(finished(int) ) );
 
         if (fwbdebug)
-            qDebug("SSHSession::terminate   terminating child process");
+            qDebug() << "SSHSession::terminate   terminating child process";
+
 #ifdef _WIN32
         if (proc->pid() != NULL)
 #else
-            if (proc->pid() != -1)
+        if (proc->pid() != -1)
 #endif
         {
-            // process is stll alive, killing
-            QString s=QString(proc->readAllStandardOutput());
-            if (!quiet)
+            if (proc->state() == QProcess::Running)
             {
-                s.replace('\r',"");
-                emit printStdout_sign(s);
+                Q_PID pid = proc->pid();
+                if (fwbdebug) qDebug() << "Process is running pid=" << pid;
+                emit printStdout_sign(QString("Stopping background process"));
+
+                /*
+                 * on windows proc->terminate() posts a WM_CLOSE
+                 * message to all toplevel windows of the child
+                 * process. However, since our child process is a
+                 * console app (ssh client), this does nothing. Need
+                 * to use proc->kill() on windows right away to avoid
+                 * timeout.
+                 */
+#ifdef _WIN32
+                proc->kill();
+#else
+                proc->terminate();
+#endif
+
+                if (fwbdebug)
+                    qDebug() << "Terminate signal sent, waiting for it to finish";
+
+                int time_to_wait = 20;
+                for (int timeout = 0; timeout < time_to_wait; timeout++)
+                {
+                    if (proc->state() != QProcess::Running) break;
+                    // print countdown only if we've been waiting more than 3 sec
+                    if (timeout > 3)
+                        emit printStdout_sign(
+                            QString(
+                                "Background process is still running. "
+                                "Will wait %1 sec").arg(time_to_wait - timeout));
+
+                    QString s = QString(proc->readAllStandardOutput());
+                    if (!quiet)
+                    {
+                        s.replace('\r',"");
+                        emit printStdout_sign(s);
+                    }
+                    QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+#ifdef _WIN32
+                    Sleep(1000);
+#else
+                    sleep(1);
+#endif
+                }
+
+                if (fwbdebug) qDebug() << "Reading last output buffers";
+
+                QString s = QString(proc->readAllStandardOutput());
+                if (!quiet)
+                {
+                    s.replace('\r',"");
+                    emit printStdout_sign(s);
+                }
+
+                if (fwbdebug) qDebug() << "done reading I/O buffers";
+
+                if (proc->state() == QProcess::Running)
+                {
+                    if (fwbdebug) qDebug() << "Still running, killing";
+                    proc->kill();
+                }
             }
-            proc->kill();
+
             delete proc;
-            proc=NULL;
-            retcode=-1;
-//            processExited();
+            proc = NULL;
+            retcode = -1;
         }
     }
+
     if (fwbdebug) qDebug("SSHSession::terminate   done");
 }
 
@@ -317,7 +375,7 @@ void SSHSession::sendLine()
     {
         string s = input.front();
         s = s + "\n";
-#if 0
+#if STATE_MACHINE_DEBUG
         if (fwbdebug)
             qDebug("SSHSession::sendLine : %d lines to go -- %s",
                    int(input.size()), s.c_str());
@@ -411,23 +469,15 @@ void SSHSession::readFromStdout()
 
         stdoutBuffer.append(buf);
 
+        if (fwbdebug)
+            qDebug("SSHSession::readFromStdout: '%s'", buf.toAscii().constData());
+
         bool endsWithLF = buf.endsWith("\n");
         QString lastLine = "";
 
         // split on LF
         QStringList bufLines = buf.split("\n", QString::KeepEmptyParts);
 
-#if 0
-        if (fwbdebug)
-        {
-            qDebug("- - - - - - - - - - - - - - - - - - - - - - - - - -");
-            qDebug("basize='%d'",basize);
-            qDebug("buffer='%s'",buf.toAscii().constData());
-            qDebug("endsWithLF=%d",endsWithLF);
-            qDebug("bufLines.size()=%d",bufLines.size());
-        }
-#endif
-
         // if buf ends with a LF character, the last element in the list is
         // an empty string
         if (endsWithLF && bufLines.last().isEmpty())  bufLines.pop_back();
@@ -440,16 +490,27 @@ void SSHSession::readFromStdout()
             bufLines.pop_back();
         }
 
-        // elements that are left in the list  are all complete lines of text
-        for (QStringList::Iterator i=bufLines.begin(); i!=bufLines.end(); ++i)
+        if (bufLines.size() > 0)
         {
-            QString s = pendingLogLine + *i + "\n";
+            /*
+             * elements that are left in the list are all complete
+             * lines of text.
+             *
+             * IMPORTANT: QT processes events when we emit signal
+             * here. This means SSHSession::readFromStdout() (this
+             * method) gets called recursively. If we print log lines
+             * one by one, more log lines will be printed after the
+             * first and they end up appearing in a strange order.
+             *
+             * See bug #465
+             */
+            QString s = pendingLogLine + bufLines.join("\n");
+            pendingLogLine = "";
             if (!quiet)
             {
-                s.replace('\r',"");
+                s.replace('\r', "");
                 emit printStdout_sign(s);
             }
-            pendingLogLine = "";
         }
 
         pendingLogLine += lastLine;
@@ -514,9 +575,11 @@ void SSHSession::finished(int retcode)
 
 bool SSHSession::cmpPrompt(const QString &str, const QString &prompt)
 {
+#if STATE_MACHINE_DEBUG
     if (fwbdebug)
         qDebug("SSHSession::cmpPrompt: str='%s' prompt='%s'",
                str.toAscii().constData(),prompt.toAscii().constData());
+#endif
 
     bool res = false;
     if (!str.isEmpty())
@@ -528,25 +591,41 @@ bool SSHSession::cmpPrompt(const QString &str, const QString &prompt)
             res = (s.lastIndexOf(prompt,-1) != -1);
         }
     }
+#if STATE_MACHINE_DEBUG
     if (fwbdebug) qDebug("SSHSession::cmpPrompt: res=%d",res);
+#endif
     return res;
 }
 
 bool SSHSession::cmpPrompt(const QString &str,const QRegExp &prompt)
 {
+#if STATE_MACHINE_DEBUG
     if (fwbdebug)
         qDebug("SSHSession::cmpPrompt: str='%s' prompt='%s' (regexp)",
                str.toAscii().constData(),prompt.pattern().toAscii().constData());
-
+#endif
     if (str.isEmpty()) return false;
 
     bool res=(str.lastIndexOf(prompt,-1)!=-1);
-
+#if STATE_MACHINE_DEBUG
     if (fwbdebug)
         qDebug("SSHSession::cmpPrompt: res=%d",res);
-
+#endif
     return res;
 }
 
 
+void SSHSession::sendCommand(const QString &cmd)
+{
+    stdoutBuffer = "";
+    if (!dry_run)
+    {
+        proc->write((cmd + "\n").toAscii());
+    } else
+    {
+        emit printStdout_sign(QString("[DRY RUN] %1\n").arg(cmd));
+        proc->write("\n");
+    }
+}
+
 
