@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHUnx.cpp 769 2009-02-21 22:42:38Z vadim $
+  $Id: SSHUnx.cpp 2639 2010-03-01 04:39:27Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -37,6 +37,7 @@
 #include <qmessagebox.h>
 #include <qapplication.h>
 #include <qeventloop.h>
+#include <QtDebug>
 
 #include <iostream>
 
@@ -51,7 +52,8 @@ SSHUnx::SSHUnx(QWidget *_par,
 {
     normal_prompt="> ";
     enable_prompt="# ";
-    pwd_prompt="'s password: ";
+    pwd_prompt_1="'s password: ";
+    pwd_prompt_2="'s password: ";
     thinkfinger_pwd_prompt="Password or swipe finger:";
     epwd_prompt="Password: ";
     ssh_pwd_prompt="'s password: ";
@@ -82,10 +84,12 @@ SSHUnx::~SSHUnx()
 
 bool SSHUnx::checkForErrors(QStringList *errptr)
 {
+#if STATE_MACHINE_DEBUG
     if (fwbdebug)
         qDebug(
             QString("SSHUnx::stateMachine:  Checking for errors. Buffer='%1'").
             arg(stdoutBuffer).toAscii().constData());
+#endif
 
     for (QStringList::const_iterator i=errptr->begin(); i!=errptr->end(); ++i)
     {
@@ -130,9 +134,10 @@ bool SSHUnx::checkForErrors()
 void SSHUnx::stateMachine()
 {
     if (checkForErrors()) return;
-
+#if STATE_MACHINE_DEBUG
     if (fwbdebug)
         qDebug("SSHUnx::stateMachine: state=%d",state);
+#endif
 
     //entry:
     switch (state)
@@ -171,7 +176,7 @@ void SSHUnx::stateMachine()
             goto push_files;
         }
 
-        if (fwbdebug) qDebug(stdoutBuffer.toAscii().constData());
+        if (fwbdebug) qDebug() << stdoutBuffer;
 
         QString fingerprint;
         //int n1,n2;
