@@ -6,7 +6,7 @@
 
   Author:  alek@codeminders.com
 
-  $Id: UserDialog.cpp 952 2009-05-20 21:54:45Z vadim $
+  $Id: UserDialog.cpp 1932 2009-11-26 03:26:24Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -29,6 +29,7 @@
 
 #include "FWBTree.h"
 #include "UserDialog.h"
+#include "FWCmdChange.h"
 
 #include "ProjectPanel.h"
 #include "fwbuilder/Library.h"
@@ -36,6 +37,8 @@
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/FWException.h"
 
+#include <memory>
+
 #include <qlineedit.h>
 #include <qspinbox.h>
 #include <qcheckbox.h>
@@ -43,13 +46,14 @@
 #include <qcombobox.h>
 #include <qmessagebox.h>
 #include <qpushbutton.h>
-#include "FWBSettings.h"
+#include <QUndoStack>
+
 
-#include "FWWindow.h"
 using namespace std;
 using namespace libfwbuilder;
 
-UserDialog::UserDialog(ProjectPanel *project, QWidget *parent) : QWidget(parent), m_project(project)
+
+UserDialog::UserDialog(QWidget *parent) : BaseObjectDialog(parent)
 {
     m_dialog = new Ui::UserDialog_q;
     m_dialog->setupUi(this);
@@ -90,62 +94,35 @@ void UserDialog::loadFWObject(FWObject *o)
     init=false;
 }
 
-void UserDialog::changed()
-{
-    //apply->setEnabled( true );
-    emit changed_sign();
-}
-
 void UserDialog::validate(bool *res)
 {
     *res=true;
 
-    if (!isTreeReadWrite(this,obj)) { *res=false; return; }
     if (!validateName(this,obj,m_dialog->obj_name->text())) { *res=false; return; }
 
     UserService *s = dynamic_cast<UserService*>(obj);
     assert(s!=NULL);
 }
 
-void UserDialog::isChanged(bool*)
-{
-    //*res=(!init && apply->isEnabled());
-}
 
-void UserDialog::libChanged()
-{
-    changed();
-}
 
 void UserDialog::applyChanges()
 {
-    UserService *s = dynamic_cast<UserService*>(obj);
+    std::auto_ptr<FWCmdChange> cmd( new FWCmdChange(m_project, obj));
+    FWObject* new_state = cmd->getNewState();
+
+    UserService *s = dynamic_cast<UserService*>(new_state);
     assert(s!=NULL);
 
-    string oldname=obj->getName();
+    string oldname = obj->getName();
     s->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
     s->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
     s->setUserId( string(m_dialog->userid->text().toUtf8().constData()) );
 
-    mw->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
-
-    //apply->setEnabled( false );
-    mw->updateLastModifiedTimestampForAllFirewalls(obj);
-}
-
-void UserDialog::discardChanges()
-{
-    loadFWObject(obj);
-}
-
-
-/* ObjectEditor class connects its slot to this signal and does all
- * the verification for us, then accepts (or not) the event. So we do
- * nothing here and defer all the processing to ObjectEditor
- */
-void UserDialog::closeEvent(QCloseEvent *e)
-{
-    emit close_sign(e);
-
+    if (!cmd->getOldState()->cmp(new_state, true))
+    {
+        if (obj->isReadOnly()) return;
+        m_project->undoStack->push(cmd.release());
+    }
 }
 
