@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: global.h 777 2009-03-07 03:47:30Z vadim $
+  $Id: global.h 2383 2010-01-18 21:33:26Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -29,14 +29,16 @@
 #include "../../VERSION.h"
 #include <string>
 
-class  QApplication;
-class  FWWindow;
-class  FWBSettings;
-class  listOfLibraries;
+#include <QEvent>
 
-extern QApplication      *app;
-extern FWWindow          *mw;
-extern FWBSettings       *st;
+class QString;
+class QApplication;
+class FWWindow;
+class FWBSettings;
+
+extern QApplication *app;
+extern FWWindow *mw;
+extern FWBSettings *st;
 
 extern std::string     appRootDir;
 extern std::string     userDataDir;
@@ -48,10 +50,9 @@ extern std::string     tempfname;
 extern std::string     argv0;
 extern std::string     ee;
 extern int             fwbdebug;
-extern int             registered;
-extern bool            gui_experiment1;
+extern QString         build_num;
+extern QString         user_name;
 
-#define SETTINGS_PATH_PREFIX  "/3.0"
 
 #ifdef NDEBUG
 #  undef NDEBUG
