@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: instDialog.h 1373 2009-08-31 18:58:37Z vadim $
+  $Id: instDialog.h 2377 2010-01-17 23:09:51Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -36,17 +36,23 @@
 
 #include "instOptionsDialog.h"
 #include "FirewallInstaller.h"
+#include "ProjectPanel.h"
+#include "Configlet.h"
 
 #include <qstring.h>
 #include <qstringlist.h>
 #include <qprocess.h>
+#include <QRegExp>
 
 #include <fstream>
 #include <set>
 #include <map>
 #include <list>
 
+#include "fwbuilder/FWObjectDatabase.h"
+
 class FirewallInstaller;
+class instBatchOptionsDialog;
 
 class QEventLoop;
 class QTextEdit;
@@ -55,41 +61,56 @@ class QCheckListItem;
 class QPushButton;
 class QProgressBar;
 class QStringList;
+class QTreeWidgetItem;
 class QTextCharFormat;
 //class QCheckTableItem;
 
 namespace libfwbuilder
 {
     class Firewall;
+    class Cluster;
 }
 
 enum BatchOperation {BATCH_INSTALL, BATCH_COMPILE} ;
 enum Page1Operation {INST_DLG_COMPILE, INST_DLG_INSTALL};
+enum InstDialogState {NONE, COMPILE_DONE, INSTALL_DONE };
 
-typedef std::map<libfwbuilder::Firewall *,QTableWidgetItem *> t_tableMap;
-typedef std::list<libfwbuilder::Firewall *> t_fwList;
 typedef std::pair<QString,QString> t_procMess; // first - compilation result, second - installation result;
-typedef std::set<libfwbuilder::Firewall*> t_fwSet;
+
+#define FIREWALL_NAME_COLUMN 0
+#define COMPILE_CHECKBOX_COLUMN 1
+#define INSTALL_CHECKBOX_COLUMN 2
+#define LAST_MODIFIED_COLUMN 3
+#define LAST_COMPILED_COLUMN 4
+#define LAST_INSTALLED_COLUMN 5
+
 
 class instDialog : public QDialog, public FakeWizard
 {
 
-    Q_OBJECT
+    Q_OBJECT;
+
+    void init(QWidget*);
 
     Ui::instDialog_q *m_dialog;
     instConf cnf;
     Page1Operation page_1_op;
+    InstDialogState state;
     FirewallInstaller *installer;
-        
+    ProjectPanel *project;
+    instBatchOptionsDialog *batch_inst_opt_dlg;
+    bool compile_only;
+
     // proc is used to launch external oprocess, such as compiler or
     // user-defined installer script
     QProcess       proc;
 
     QString fwb_prompt;
     
-    t_fwSet reqFirewalls;
+    std::set<libfwbuilder::Firewall*> reqFirewalls;
 
-    t_fwList firewalls;
+    std::list<libfwbuilder::Firewall*> firewalls;
+    std::list<libfwbuilder::Cluster*> clusters;
     
     std::list<libfwbuilder::Firewall*> compile_fw_list;
     std::list<libfwbuilder::Firewall*>::size_type compile_list_initial_size;
@@ -97,10 +118,9 @@ class instDialog : public QDialog, public FakeWizard
     std::list<libfwbuilder::Firewall*>::size_type install_list_initial_size;
     
     std::map<int,QTreeWidgetItem*> opListMapping;
-    
-    t_tableMap compileMapping;
-    t_tableMap installMapping;
-    
+    std::list<QRegExp> error_re;
+    std::list<QRegExp> warning_re;
+
     QString path; //path of the program to execute
 //    QStringList args; //arguments for that program
         
@@ -115,10 +135,13 @@ class instDialog : public QDialog, public FakeWizard
     bool compileFlag;
     bool customScriptFlag;
     bool showSelectedFlag;
+    bool finished;
+    bool onlySelected;
 
-    QTextEdit *currentLog;
+    QTextBrowser *currentLog;
     QTextCharFormat normal_format;
     QTextCharFormat error_format;
+    QTextCharFormat warning_format;
     QTextCharFormat highlight_format;
     
     QPushButton *currentSaveButton;
@@ -131,8 +154,7 @@ class instDialog : public QDialog, public FakeWizard
 
     
     void fillCompileSelectList();
-    void selectAll(t_tableMap &mapping);
-    void deselectAll(t_tableMap &mappin);
+    void setSelectStateAll(int column, Qt::CheckState);
 
     void fillCompileOpList();
     void fillCompileUIList();
@@ -141,18 +163,24 @@ class instDialog : public QDialog, public FakeWizard
 
     bool checkSSHPathConfiguration(libfwbuilder::Firewall*);
     
-    //void analyseInstallQueue(bool &fPix, bool &fCustInst);
-    libfwbuilder::Firewall *findFirewallbyListItem(QTreeWidgetItem* item);
-    libfwbuilder::Firewall *findFirewallbyTableItem(QTableWidgetItem *item);
+    //libfwbuilder::Firewall *findFirewallbyTableItem(QTableWidgetItem *item);
     
     void setSuccessState(QTreeWidgetItem *item);
     void setFailureState(QTreeWidgetItem *item);
     void setErrorState(QTreeWidgetItem *item);
     void setInProcessState(QTreeWidgetItem *item);
 
+    bool checkIfNeedToCompile(libfwbuilder::Firewall *fw);
+    bool checkIfNeedToInstall(libfwbuilder::Firewall *fw);
+    QTreeWidgetItem *createTreeItem(QTreeWidgetItem* parent,
+                                    libfwbuilder::Firewall *fw);
+    void setFlags(QTreeWidgetItem* item);
+
+    virtual void closeEvent(QCloseEvent *event);
+ 
  public:
    
-    instDialog(QWidget* p, BatchOperation op, t_fwSet reqFirewalls_);
+    instDialog(QWidget *p, bool install, bool onlySelected, std::set<libfwbuilder::Firewall*> fws);
     virtual ~instDialog();
     
     void summary();
@@ -167,10 +195,10 @@ class instDialog : public QDialog, public FakeWizard
     bool runInstaller(libfwbuilder::Firewall *fw);
 
     QStringList prepareArgForCompiler(libfwbuilder::Firewall *fw);
-    bool tableHasChecked();
+    bool tableHasCheckedItems();
     void clearReqFirewalls();
     void addReqFirewall(libfwbuilder::Firewall *f);
-    QString replaceMacrosInCommand(const QString &cmd);
+    void replaceMacrosInCommand(Configlet *configlet);
     QString getActivationCmd();
     
     void enableStopButton();
@@ -179,6 +207,8 @@ class instDialog : public QDialog, public FakeWizard
     void setUpProcessToCompile();
     void setUpProcessToInstall();
     bool executeCommand(const QString &path, QStringList &args);
+
+    bool isFinished() { return finished; }
     
 protected:
     
@@ -186,7 +216,7 @@ protected:
     virtual void hideEvent( QHideEvent *ev);
 
     bool getInstOptions(libfwbuilder::Firewall *fw);
-    bool getBatchInstOptions();
+    bool getBatchInstOptions(libfwbuilder::Firewall *first_fw);
     
     void prepareInstConf(libfwbuilder::Firewall *fw);
 
@@ -196,7 +226,7 @@ protected:
     void readInstallerOptionsFromFirewallObject(libfwbuilder::Firewall *fw);
     void readInstallerOptionsFromDialog(libfwbuilder::Firewall *fw,
                                                 instOptionsDialog *dlg);
-    void completeInstallerOptions();
+    bool verifyManagementAddress();
 
     void storeInstallerOptions();
     void findFirewalls();
@@ -223,10 +253,8 @@ public slots:
     void updateProgressBar(int n,bool setsize);
 
     void saveLog();
-    void togleDetailMC();
 
     void readFromStdout();
-    //virtual void readFromStderr();
     void selectAllFirewalls();
     void deselectAllFirewalls();
     
@@ -238,10 +266,14 @@ public slots:
     
     void stopCompile();
     void stopInstall();
-    void findFirewallInCompileLog(QTreeWidgetItem*);
-    void showSelected();
-    void tableValueChanged(int row, int col);
-    
+
+    void tableItemChanged(QTreeWidgetItem * item, int column);
+    void findFirewallInCompileLog(QTreeWidgetItem* item);
+
+    void logItemClicked(QUrl);
+
+signals:
+    void activateRule(ProjectPanel*, QString, QString, int);
 };
 
 
