@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: instOptionsDialog.cpp 643 2008-11-11 06:11:11Z vadim $
+  $Id: instOptionsDialog.cpp 2639 2010-03-01 04:39:27Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -42,9 +42,13 @@
 #include <qlayout.h>
 #include <qspinbox.h>
 #include <qframe.h>
+#include <QtDebug>
 
 #include <stdlib.h>
 
+#include "FWBSettings.h"
+#include "FWWindow.h"
+
 using namespace std;
 using namespace libfwbuilder;
 
@@ -55,11 +59,33 @@ instOptionsDialog::instOptionsDialog(QWidget *parent, instConf *_cnf) :
     m_dialog->setupUi(this);
     cnf = _cnf;
 
-    m_dialog->pwd->setEchoMode( QLineEdit::Password );
-    m_dialog->epwd->setEchoMode( QLineEdit::Password );
+    int fw_id = -1;
+    if (cnf->fwobj)
+    {
+        // Note cnf->fwobj == NULL during batch install
+        fw_id = cnf->fwobj->getId();
+    }
+
+    QString username = cnf->user;
+    bool savePassEnabled = st->getBool("Environment/RememberSshPassEnabled");
+    if (savePassEnabled && !username.isEmpty())
+    {
+        m_dialog->rememberPass->setEnabled(true);
+        m_dialog->rememberPass->setChecked(true);
+        QPair<QString, QString> passwds = mw->passwords[qMakePair(fw_id, username)];
+        m_dialog->pwd->setText(passwds.first);
+        m_dialog->epwd->setText(passwds.second);
+    }
+    else
+    {
+        m_dialog->rememberPass->setEnabled(false);
+        m_dialog->rememberPass->setChecked(false);
+    }
+
+    m_dialog->pwd->setEchoMode(QLineEdit::Password);
+    m_dialog->epwd->setEchoMode(QLineEdit::Password);
 
     m_dialog->uname->setText( cnf->user );
-    m_dialog->incr->setChecked( cnf->incremental );
     m_dialog->test->setChecked( cnf->dry_run );
     m_dialog->backupConfigFile->setText( cnf->backup_file );
     m_dialog->saveDiff->setChecked( cnf->save_diff );
@@ -101,29 +127,35 @@ instOptionsDialog::instOptionsDialog(QWidget *parent, instConf *_cnf) :
         );
 
         QString platform = cnf->fwobj->getStr("platform").c_str();
+        string version = cnf->fwobj->getStr("version");
 
         if (platform=="pix" || platform=="fwsm" || platform=="iosacl")
         {
             m_dialog->copyFWB->hide();
-            // Hide elements of installOptions dialog for which we do not
-            // have commands
-            QString cmd = cnf->getCmdFromResource("schedule_rollback");
-            // option "schedule_rollback" is currently used to control rollback
-            // behavior only for pix, fwsm and ios
-            if (cmd.isEmpty())
+            if (platform == "iosacl")
             {
-                m_dialog->rollback->hide();
-                m_dialog->rollbackTime->hide();
-                m_dialog->rollbackTimeUnit->hide();
-                m_dialog->cancelRollbackIfSuccess->hide();
+                if (XMLTools::version_compare(version, "12.4") >= 0)
+                {
+                    m_dialog->rollback->setText("Schedule roll back using EEM in ");
+                    m_dialog->cancelRollbackIfSuccess->setText(
+                        "Cancel rollback if policy activation was successfull");
+                } else
+                {
+                    m_dialog->rollback->setText("Schedule reboot in ");
+                    m_dialog->cancelRollbackIfSuccess->setText(
+                        "Cancel reboot if policy activation was successfull");
+                }
             }
-
             if (platform=="iosacl") m_dialog->PIXgroupBox->hide();
-
         } else
         {
+            m_dialog->rollback->setText("Schedule reboot in ");
+            m_dialog->cancelRollbackIfSuccess->setText(
+                "Cancel reboot if policy activation was successfull");
             m_dialog->epwd->hide();
+            m_dialog->epwdLbl->hide();
             m_dialog->PIXgroupBox->hide();
+            m_dialog->test->hide();  // dry run option
             // cancelling rollback at the end of activation is currently
             // only supported on pix,fwsm and ios
             m_dialog->cancelRollbackIfSuccess->hide();
@@ -147,19 +179,20 @@ instOptionsDialog::instOptionsDialog(QWidget *parent, instConf *_cnf) :
     if (fwbdebug)
     {
         QSize sz = sizeHint();
-        qDebug(QString("instOptionsDialog:  sizeHint: %1x%2").arg(sz.width()).arg(sz.height()).toAscii().constData());
+        qDebug() << QString("instOptionsDialog:  sizeHint: %1x%2")
+            .arg(sz.width()).arg(sz.height());
         sz = minimumSizeHint();
-        qDebug(QString("instOptionsDialog:  minimumSizeHint: %1x%2").
-                arg(sz.width()).arg(sz.height()).toAscii().constData());
+        qDebug() << QString("instOptionsDialog:  minimumSizeHint: %1x%2")
+            .arg(sz.width()).arg(sz.height());
 
         QRect bfr;
 
         bfr = m_dialog->titleFrame->geometry();
-        qDebug(QString("instOptionsDialog:  titleFrame: top=%1 bottom=%2").
-                arg(bfr.top()).arg(bfr.bottom()).toAscii().constData());
+        qDebug() << QString("instOptionsDialog:  titleFrame: top=%1 bottom=%2")
+            .arg(bfr.top()).arg(bfr.bottom());
         bfr = m_dialog->buttonsFrame->geometry();
-        qDebug(QString("instOptionsDialog:  buttonsFrame: top=%1 bottom=%2").
-                arg(bfr.top()).arg(bfr.bottom()).toAscii().constData());
+        qDebug() << QString("instOptionsDialog:  buttonsFrame: top=%1 bottom=%2")
+            .arg(bfr.top()).arg(bfr.bottom());
     }
 
     //resize( minimumSizeHint() );
@@ -169,6 +202,24 @@ instOptionsDialog::instOptionsDialog(QWidget *parent, instConf *_cnf) :
     //dlg->setFixedHeight( dlg->minimumSizeHint().height() );
 }
 
+void instOptionsDialog::savePassword()
+{
+    int fw_id = -1;
+    if (cnf->fwobj)
+    {
+        // Note cnf->fwobj == NULL during batch install
+        fw_id = cnf->fwobj->getId();
+    }
+
+    if ( m_dialog->rememberPass->isChecked() )
+        mw->passwords[qMakePair(fw_id, m_dialog->uname->text())] =
+                 qMakePair(m_dialog->pwd->text(), m_dialog->epwd->text());
+    else
+        mw->passwords.remove(qMakePair(fw_id, m_dialog->uname->text()));
+
+    st->setBool("Environment/RememberSshPass", m_dialog->rememberPass->isChecked());
+}
+
 instOptionsDialog::~instOptionsDialog()
 {
     delete m_dialog;
