@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: instOptionsDialog.h 353 2008-07-13 01:26:39Z vadim $
+  $Id: instOptionsDialog.h 1667 2009-10-30 21:39:54Z a2k $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -43,22 +43,15 @@ class instOptionsDialog : public QDialog
     instOptionsDialog(QWidget *parent, instConf *_cnf);
     ~instOptionsDialog();
 
+    void savePassword();
+
     QString getUName();
     QString getPWD();
     QString getEPWD();
     
     Ui::instOptionsDialog_q *m_dialog;
 
- public slots:
-    
- signals:
-/**
- * This signal is emitted from closeEvent, ObjectEditor connects
- * to this signal to make checks before the object editor can be closed
- * and to store its position on the screen
- */
-   // void close_sign(QCloseEvent *e);
-    
+   
 };
 
 #endif 
