@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: linux24AdvancedDialog.cpp 454 2008-08-06 15:50:49Z vadim $
+  $Id: linux24AdvancedDialog.cpp 2202 2009-12-19 04:51:57Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,9 +28,13 @@
 #include "platforms.h"
 
 #include "linux24AdvancedDialog.h"
+#include "FWCmdChange.h"
 
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Management.h"
+#include "fwbuilder/Resources.h"
+
+#include <memory>
 
 #include <qcheckbox.h>
 #include <qspinbox.h>
@@ -39,6 +43,7 @@
 #include <qlineedit.h>
 #include <qstackedwidget.h>
 #include <qregexp.h>
+#include <QUndoStack>
 
 #include "FWWindow.h"
 #include "Help.h"
@@ -56,8 +61,14 @@ linux24AdvancedDialog::linux24AdvancedDialog(QWidget *parent,FWObject *o)
 {
     m_dialog = new Ui::linux24AdvancedDialog_q;
     m_dialog->setupUi(this);
+    setWindowModality(Qt::WindowModal);
     obj=o;
 
+    string host_os = obj->getStr("host_OS");
+    string description = Resources::os_res[host_os]->
+        getResourceStr("/FWBuilderResources/Target/description");
+    setWindowTitle(QObject::tr("%1 advanced settings").arg(description.c_str()));
+
     FWOptions *fwopt=(Firewall::cast(obj))->getOptionsObject();
     assert(fwopt!=NULL);
 
@@ -146,6 +157,15 @@ linux24AdvancedDialog::linux24AdvancedDialog(QWidget *parent,FWObject *o)
     data.registerOption( m_dialog->linux24_path_logger,
                          fwopt,
                          "linux24_path_logger");
+    data.registerOption( m_dialog->linux24_path_vconfig,
+                         fwopt,
+                         "linux24_path_vconfig");
+    data.registerOption( m_dialog->linux24_path_brctl,
+                         fwopt,
+                         "linux24_path_brctl");
+    data.registerOption( m_dialog->linux24_path_ifenslave,
+                         fwopt,
+                         "linux24_path_ifenslave");
     data.registerOption( m_dialog->linux24_path_modprobe,
                          fwopt,
                          "linux24_path_modprobe");
@@ -156,8 +176,19 @@ linux24AdvancedDialog::linux24AdvancedDialog(QWidget *parent,FWObject *o)
                          fwopt,
                          "linux24_path_ip6tables_restore");
 
+    data.registerOption(m_dialog->conntrack_max,
+                        fwopt,
+                        "linux24_conntrack_max");
+    data.registerOption(m_dialog->conntrack_hashsize,
+                        fwopt,
+                        "linux24_conntrack_hashsize");
+    data.registerOption(m_dialog->conntrack_tcp_be_liberal,
+                        fwopt,
+                        "linux24_conntrack_tcp_be_liberal", threeStateMapping);
 
     data.loadAll();
+
+    m_dialog->tabWidget->setCurrentIndex(0);
 }
 
 /*
@@ -165,15 +196,22 @@ linux24AdvancedDialog::linux24AdvancedDialog(QWidget *parent,FWObject *o)
  */
 void linux24AdvancedDialog::accept()
 {
-    FWOptions *fwopt=(Firewall::cast(obj))->getOptionsObject();
-    assert(fwopt!=NULL);
+    ProjectPanel *project = mw->activeProject();
+    std::auto_ptr<FWCmdChange> cmd( new FWCmdChange(project, obj));
 
-    Management *mgmt=(Firewall::cast(obj))->getManagementObject();
+    // new_state  is a copy of the fw object
+    FWObject* new_state = cmd->getNewState();
+    FWOptions* fwoptions = Firewall::cast(new_state)->getOptionsObject();
+    assert(fwoptions!=NULL);
+
+    Management *mgmt = (Firewall::cast(new_state))->getManagementObject();
     assert(mgmt!=NULL);
 
-    data.saveAll();
+    data.saveAll(fwoptions);
 
-    mw->updateLastModifiedTimestampForAllFirewalls(obj);
+    if (!cmd->getOldState()->cmp(new_state, true))
+        project->undoStack->push(cmd.release());
+    
     QDialog::accept();
 }
 
@@ -184,11 +222,14 @@ void linux24AdvancedDialog::reject()
 
 void linux24AdvancedDialog::help()
 {
-    Help *h = new Help(this, "linux24AdvancedDialog", "Host type Linux 2.4/2.6");
-    h->show();
     QString tab_title = m_dialog->tabWidget->tabText(
         m_dialog->tabWidget->currentIndex());
-    h->scrollToAnchor(tab_title.replace('/', '-').replace(' ', '-').toLower());
+    QString anchor = tab_title.replace('/', '-').replace(' ', '-').toLower();
+    Help *h = Help::getHelpWindow(this);
+    h->setName("Host type Linux 2.4/2.6");
+    h->setSource(QUrl("linux24AdvancedDialog.html#" + anchor));
+    h->raise();
+    h->show();
 }
 
 
