@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: newFirewallDialog.cpp 1358 2009-08-27 16:59:30Z vadim $
+  $Id: newFirewallDialog.cpp 2481 2010-02-01 20:30:05Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -30,9 +30,11 @@
 #include "platforms.h"
 
 #include "newFirewallDialog.h"
-#include "FWWindow.h"
 #include "ObjConflictResolutionDialog.h"
 #include "upgradePredicate.h"
+#include "FWBSettings.h"
+#include "FWBTree.h"
+#include "events.h"
 
 #include "fwbuilder/Library.h"
 #include "fwbuilder/Firewall.h"
@@ -46,6 +48,7 @@
 #include <qtextedit.h>
 #include <qcombobox.h>
 #include <qpushbutton.h>
+#include <qtoolbutton.h>
 #include <qradiobutton.h>
 #include <qcheckbox.h>
 #include <qtreewidget.h>
@@ -58,7 +61,9 @@
 #include <qpixmapcache.h>
 #include <qfiledialog.h>
 #include <qdir.h>
+#include <QtDebug>
 
+#include <algorithm>
 #include <iostream>
 
 // must be the last for win
@@ -69,25 +74,33 @@ using namespace std;
 
 
 
-newFirewallDialog::newFirewallDialog() : QDialog()
+newFirewallDialog::newFirewallDialog(FWObject *_p) : QDialog()
 {
+    parent = _p;
+    db = parent->getRoot();
+
     m_dialog = new Ui::newFirewallDialog_q;
     m_dialog->setupUi(this);
 
+    possible_inside_interface_labels.push_back("inside");
+    possible_inside_interface_labels.push_back("GREEN");
+    possible_inside_interface_labels.push_back("green");
+
+    possible_outside_interface_labels.push_back("outside");
+    possible_outside_interface_labels.push_back("RED");
+    possible_outside_interface_labels.push_back("red");
+
+    possible_dmz_interface_labels.push_back("dmz");
+    possible_dmz_interface_labels.push_back("ORANGE");
+    possible_dmz_interface_labels.push_back("orange");
+
+
     setControlWidgets(this, m_dialog->stackedWidget,
                       m_dialog->nextButton,
                       m_dialog->finishButton,
                       m_dialog->backButton,
                       m_dialog->cancelButton,
                       m_dialog->titleLabel);
-    /*connect( m_dialog->nextButton, SIGNAL( clicked() ),
-             this, SLOT( nextClicked() ));
-    connect( m_dialog->backButton, SIGNAL( clicked() ),
-             this, SLOT( backClicked() ));
-    connect( m_dialog->finishButton, SIGNAL( clicked() ),
-             this, SLOT( finishClicked() ));
-    connect( m_dialog->cancelButton, SIGNAL( clicked() ),
-             this, SLOT( cancelClicked() ));*/
 
     nfw = NULL;
     tmpldb = NULL;
@@ -104,31 +117,50 @@ newFirewallDialog::newFirewallDialog() : QDialog()
     m_dialog->templaterFilePath->setText(tempfname.c_str());
     m_dialog->templaterFrame->setVisible(false);
 
-/* fill in platform */
-    setPlatform(m_dialog->platform, "" );
+    /* fill in platform. Since iptables is the most popular, start with
+     * it.
+     */
+    QString new_fw_platform = st->getNewFirewallPlatform();
 
-/* fill in host OS  */
-    setHostOS(m_dialog->hostOS, "" );
+    /* if new_fw_platform is empty, the drop-down list will have empty
+     * item which will be current. This is so only on the first run of
+     * the program because it remembers chosen platform and uses it on
+     * subsequent runs.
+     */
+    setPlatform(m_dialog->platform, new_fw_platform);
+
+    /* fill in host OS  */
+    setHostOS(m_dialog->hostOS, readPlatform(m_dialog->platform), "");
 
     setNextEnabled( 0, false );
+
     /*for (int i=0; i<pageCount(); ++i)
         setHelpEnabled( i, false );*/
-
+/*
     //m_dialog->iface_list->setItemMargin( 1 );
     m_dialog->iface_list->setAllColumnsShowFocus( true );
 
     //m_dialog->iface_sl_list->setItemMargin( 1 );
-    m_dialog->iface_sl_list->setAllColumnsShowFocus( true );
     m_dialog->iface_dyn->setToolTip(wordWrap(tr("Check option 'dynamic address' for the interface that gets its IP address dynamically via DHCP or PPP protocol.") ,80 ));
     m_dialog->iface_unnum->setToolTip(wordWrap(tr("Check option 'Unnumbered interface' for the interface that does not have an IP address. Examples of interfaces of this kind are those used to terminate PPPoE or VPN tunnels.") ,80 ));
+*/
+    m_dialog->iface_sl_list->setAllColumnsShowFocus( true );
     m_dialog->obj_name->setFocus();
 
+
+    currentTemplate = NULL;
+    this->m_dialog->interfaceEditor1->clear();
+    this->m_dialog->interfaceEditor2->clear();
+    this->m_dialog->interfaceEditor1->closeTab();
+    this->m_dialog->interfaceEditor2->closeTab();//->removeTab(0);
+
     showPage(0);
+
 }
 
 void newFirewallDialog::browseTemplate()
 {
-    QString fileName = QFileDialog::getOpenFileName(this, tr("FWBuilder template files"), "", tr("FWBuilder template files (*.xml *.fwb)"));
+    QString fileName = QFileDialog::getOpenFileName(this, tr("FWBuilder template files"), "", tr("FWBuilder template files (*.xml *.fwb *.fwl)"));
     if (fileName=="")
         return ;
     QDir dir (fileName);
@@ -169,8 +201,11 @@ void newFirewallDialog::changed()
     int p = currentPage();
     if (p==0)
     {
-        setNextEnabled( p, !m_dialog->obj_name->text().isEmpty() );
-        
+        setNextEnabled(p,
+                       !m_dialog->obj_name->text().isEmpty() &&
+                       !readPlatform(m_dialog->platform).isEmpty()
+        );
+        setHostOS(m_dialog->hostOS, readPlatform(m_dialog->platform), "");
     }
 
     if (p==1)
@@ -193,23 +228,6 @@ void newFirewallDialog::changed()
         f = m_dialog->use_manual->isChecked() || snmpPollCompleted;
         setNextEnabled( 1, f );
     }
-
-    if (p==2)
-    {
-        if (m_dialog->iface_dyn->isChecked() ||
-            m_dialog->iface_unnum->isChecked() ||
-            m_dialog->iface_bridgeport->isChecked())
-        {
-            m_dialog->iface_addr->clear();
-            m_dialog->iface_addr->setEnabled(false);
-            m_dialog->iface_netmask->clear();
-            m_dialog->iface_netmask->setEnabled(false);
-        } else
-        {
-            m_dialog->iface_addr->setEnabled(true);
-            m_dialog->iface_netmask->setEnabled(true);
-        }
-    }
 }
 
 void  newFirewallDialog::monitor()
@@ -232,6 +250,8 @@ void  newFirewallDialog::monitor()
 
     map<int, InterfaceData>* intf = q->getInterfaces();
     map<int, InterfaceData>::iterator i;
+    this->m_dialog->interfaceEditor1->clear();
+    this->m_dialog->interfaceEditor1->removeTab(0);
     for (i=intf->begin(); i!=intf->end(); ++i)
     {
         InterfaceData* idata = &(i->second);
@@ -239,27 +259,11 @@ void  newFirewallDialog::monitor()
         if ( idata->ostatus )
         {
             idata->guessLabel(readPlatform(m_dialog->platform).toStdString());
-
-            QString dn;
-            if (idata->isDyn)        dn+="dyn";
-            if (idata->isUnnumbered) dn+="unn";
-            if (idata->isBridgePort) dn+="bridge";
-
-            QStringList qsl;
-            qsl << idata->name.c_str() << idata->label.c_str();
-            // TODO: redesign the way we pass information from the
-            // snmp crawler down the line so that multiple ip
-            // addresses per interface can be processed.
-            if (idata->addr_mask.size())
-                qsl << idata->addr_mask.front()->getAddressPtr()->toString().c_str()
-                    << idata->addr_mask.front()->getNetmaskPtr()->toString().c_str();
-            else
-                qsl << "" << "";
-
-            qsl << dn << idata->mac_addr.c_str();
-            new QTreeWidgetItem(m_dialog->iface_list, qsl);
+            this->m_dialog->interfaceEditor1->addInterfaceFromData(idata);
         }
     }
+    if ( this->m_dialog->interfaceEditor1->count() == 0 )
+        this->m_dialog->interfaceEditor1->addNewInterface();
 
     delete q;
     q=NULL;
@@ -278,7 +282,7 @@ void newFirewallDialog::getInterfacesViaSNMP()
     if (q!=NULL || getInterfacesBusy) return;
 
     snmpPollCompleted=false;
-    m_dialog->iface_list->clear();
+    m_dialog->interfaceEditor1->clear();
 
     string rcomm=m_dialog->snmp_community->text().toLatin1().constData();
 
@@ -336,6 +340,11 @@ void newFirewallDialog::getInterfacesViaSNMP()
 #endif
 }
 
+bool interfaceCompare(libfwbuilder::Interface *first, libfwbuilder::Interface *second)
+{
+    return first->getName() < second->getName();
+}
+
 bool newFirewallDialog::appropriate(const int page) const
 {
     int p  = page;
@@ -356,6 +365,22 @@ bool newFirewallDialog::appropriate(const int page) const
 
 void newFirewallDialog::nextClicked()
 {
+    if ( currentPage() == 4 )
+    {
+        if (m_dialog->templateList->currentItem() == NULL)
+        {
+            QMessageBox::warning(
+                    this,"Firewall Builder",
+                    tr("Please select template"),
+                    tr("&Continue"), QString::null,QString::null,
+                    0, 1 );
+            showPage(4);
+            return;
+        }
+    }
+    if ( currentPage() == 2 )
+        if ( !this->m_dialog->interfaceEditor1->isValid() )
+            return;
     if (nextRelevant( currentPage() ) > -1)
         showPage(nextRelevant( currentPage() ));
 }
@@ -372,9 +397,23 @@ void newFirewallDialog::showPage(const int page)
 
     int p = page;
 
+    if (fwbdebug)
+        qDebug("newFirewallDialog::showPage  page=%d", page);
+
 // p is a page number _after_ it changed
     switch (p)
     {
+    case 0:
+        // we get here if user hits "Back" on page 4 (where they
+        // choose template object)
+        if (tmpldb!=NULL)
+        {
+            m_dialog->templateList->clear();
+            delete tmpldb;
+            tmpldb = NULL;
+        }
+        break;
+
     case 1:
     {
         changed();  // to properly enable/disable widgets
@@ -383,12 +422,20 @@ void newFirewallDialog::showPage(const int page)
 
     case 2:
     {
-        m_dialog->iface_name->setFocus();
-
         if (!Resources::getTargetCapabilityBool(
                 readPlatform(m_dialog->platform).toLatin1().constData(),
                 "security_levels") )
         {
+
+        this->m_dialog->interfaceEditor1->setExplanation(
+            tr("Dynamic interface gets its IP address by means "
+               "of DHCP or PPP protocol and does not require an "
+               "address here. Regular interface has statically "
+               "configured IP address which should be entered on "
+               "this page. Interface can have several IPv4 and "
+               "IPv6 addresses.")
+        );
+
 /* if chosen fw platform does not support security levels,
  * this is the last page
  */
@@ -406,6 +453,7 @@ void newFirewallDialog::showPage(const int page)
             return;
         }
 
+        // Edit security levels
         fillInterfaceSLList();
 
         setNextEnabled( 3, false );
@@ -415,17 +463,29 @@ void newFirewallDialog::showPage(const int page)
 
     case 4:
     {
-        setFinishEnabled( 4, true );
+        // Show firewall templates
+        setFinishEnabled( 4, false );
+        setNextEnabled( 4, true );
 /* load templates if not loaded */
         if (tmpldb==NULL)
         {
 
             MessageBoxUpgradePredicate upgrade_predicate(this);
-
             tmpldb = new FWObjectDatabase();
             tmpldb->setReadOnly( false );
-            tmpldb->load( m_dialog->templaterFilePath->text().toAscii().data(),
-                          &upgrade_predicate, librespath);
+            try
+            {
+                tmpldb->load( m_dialog->templaterFilePath->text().toAscii().data(),
+                              &upgrade_predicate, librespath);
+            }
+            catch (FWException &ex)
+            {
+                QMessageBox::critical(
+                    this,"Firewall Builder",
+                    tr("Error loading template library:\n%1").arg(ex.toString().c_str()),
+                    tr("&Continue"), QString::null,QString::null,
+                    0, 1 );
+            }
         }
 
         list<FWObject*> fl;
@@ -434,9 +494,7 @@ void newFirewallDialog::showPage(const int page)
         for ( ; libiter!=libiter.end(); ++libiter)
             findFirewalls(*libiter, fl, false);
 
-
-        QString icn = QString(
-            Resources::global_res->getObjResourceStr(fl.front(), "icon-tree").c_str() );
+        QString icn = ":/Icons/Firewall/icon-tree";
 
         m_dialog->templateList->clear();
 
@@ -459,63 +517,98 @@ void newFirewallDialog::showPage(const int page)
             m_dialog->templateList->addItem( twi );
             templates[ m_dialog->templateList->item( m_dialog->templateList->count()-1 ) ]=o;
         }
-        m_dialog->templateList->setCurrentItem(0);
         m_dialog->templateList->setFocus();
+        m_dialog->templateList->setCurrentItem(0);
         break;
     }
+
+    case 5:
+    {
+        // Edit interfaces of the template object
+        createFirewallFromTemplate();
+        setFinishEnabled( 5, true );
+
+        this->m_dialog->interfaceEditor2->setExplanation(
+            tr("Here you can change IP address of the template interface "
+               "to match addresses used on your network. "
+               "Interface can have several IPv4 and "
+               "IPv6 addresses.")
+        );
+
+        this->m_dialog->interfaceEditor2->clear();
+        this->m_dialog->interfaceEditor2->closeTab();//->removeTab(0);
+        this->m_dialog->interfaceEditor2->setCornerWidgetsVisible(false);
+        QList<Interface*> interfaces;
+        FWObjectTypedChildIterator intiter = nfw->findByType(Interface::TYPENAME);
+        for ( ; intiter != intiter.end(); ++intiter )
+            interfaces.append(Interface::cast(*intiter));
+        sort(interfaces.begin(), interfaces.end(), interfaceCompare);
+        foreach(Interface* intr, interfaces)
+            m_dialog->interfaceEditor2->addInterface(intr);
+    }
     }
 }
 
 void newFirewallDialog::fillInterfaceSLList()
 {
-
-    QTreeWidgetItem *itm = m_dialog->iface_list->topLevelItem(0);// firstChild();
-
-    int itm_index = 0;
-
     m_dialog->iface_sl_list->clear();
-
-    while (itm!=NULL)
+    foreach(EditedInterfaceData iface,
+            this->m_dialog->interfaceEditor1->getData().values() +
+            this->m_dialog->interfaceEditor1->getNewData())
     {
         InterfaceData idata;
 
-        idata.name         =  itm->text(0).toLatin1().constData();
-        idata.label        =  itm->text(1).toLatin1().constData();
-        idata.isDyn        =  itm->text(4).indexOf("Dyn")!=-1;
-        idata.isUnnumbered =  itm->text(4).indexOf("Unn")!=-1;
-        idata.isBridgePort =  itm->text(4).indexOf("Bridge")!=-1;
-
-        InetAddrMask *iam = new InetAddrMask();
-        if (!idata.isDyn && !idata.isUnnumbered && !idata.isBridgePort)
+        idata.name  = iface.name.toStdString();
+        idata.label = iface.label.toStdString();
+        AddressInfo address;
+        bool gotIPv4 = false;
+        foreach(AddressInfo addr, iface.addresses.values())
         {
-            iam->setAddress(itm->text(2).toLatin1().constData());
+            if (addr.ipv4)
+            {
+                address = addr;
+                gotIPv4 = true;
+                break;
+            }
         }
-        idata.addr_mask.push_back(iam);
 
-        try
+        InetAddrMask *iam;// = new InetAddrMask();
+        if (iface.type == 0 && iface.addresses.size() != 0)
         {
-            idata.guessSecurityLevel(
-                readPlatform(m_dialog->platform).toStdString() );
+            if (!gotIPv4) address = iface.addresses.values().first();
+            if ( address.ipv4 )
+                iam = new InetAddrMask(InetAddr(address.address.toStdString()), InetAddr(address.netmask.toStdString()));
+            else
+            {
+                iam = new InetAddrMask(InetAddr(AF_INET6, address.address.toStdString()), InetAddr(AF_INET6, address.netmask.toStdString()));
+            }
+            idata.addr_mask.push_back(iam);
         }
-        catch (FWException &ex)
-        {
-            QMessageBox::warning(
-                this,"Firewall Builder", ex.toString().c_str(),
-                "&Continue", QString::null, QString::null, 0, 1 );
 
-            showPage( 2 );
-            return;
+        if (gotIPv4)
+        {
+            try
+            {
+                idata.guessSecurityLevel( readPlatform(m_dialog->platform).toStdString() );
+            }
+            catch (FWException &ex)
+            {
+                QMessageBox::warning( this,"Firewall Builder", ex.toString().c_str(),
+                                      "&Continue", QString::null, QString::null, 0, 1 );
+                showPage( 2 );
+                return;
+            }
         }
+        else idata.securityLevel = 0;
+
 
         QStringList qsl;
-        qsl << idata.name.c_str()
-            << idata.label.c_str()
-            << idata.addr_mask.front()->getAddressPtr()->toString().c_str()
+        qsl << iface.name//idata.name.c_str()
+            << iface.label//idata.label.c_str()
+            << address.address
             << QString::number(idata.securityLevel);
-        new QTreeWidgetItem(m_dialog->iface_sl_list, qsl);
 
-        itm_index++;
-        itm=m_dialog->iface_list->topLevelItem(itm_index);
+        new QTreeWidgetItem(m_dialog->iface_sl_list, qsl);
     }
 }
 
@@ -528,11 +621,13 @@ void newFirewallDialog::templateSelected(QListWidgetItem *itm)
     if (templates.size()==0) return;
     FWObject *o = templates[itm];
     if (o==NULL) return;
+    this->m_dialog->interfaceEditor2->setTemplate(o);
+    currentTemplate = o;
 
     Firewall *fw = Firewall::cast(o);
 
     m_dialog->templateComment->clear();
-    QString s=QString("<a name=\"top\">\n") + fw->getComment().c_str();
+    QString s = QString("<a name=\"top\">\n") + fw->getComment().c_str();
     m_dialog->templateComment->append( s );
     m_dialog->templateComment->scrollToAnchor("top");
 
@@ -543,21 +638,29 @@ void newFirewallDialog::templateSelected(QListWidgetItem *itm)
     for (FWObject::iterator i=ll.begin(); i!=ll.end(); i++)
     {
         Interface *intf = Interface::cast( *i );
-        if (intf->getLabel()=="outside")
+        if (std::find(possible_outside_interface_labels.begin(),
+                      possible_outside_interface_labels.end(),
+                      intf->getLabel()) != possible_outside_interface_labels.end())
         {
             haveOutside=true;
             m_dialog->intfOutsideLine->show();
             m_dialog->intfOutsideText->show();
             fillInterfaceData(intf,m_dialog->intfOutsideText);
         }
-        if (intf->getLabel()=="inside")
+
+        if (std::find(possible_inside_interface_labels.begin(),
+                      possible_inside_interface_labels.end(),
+                      intf->getLabel()) != possible_inside_interface_labels.end())
         {
             haveInside=true;
             m_dialog->intfInsideLine->show();
             m_dialog->intfInsideText->show();
             fillInterfaceData(intf,m_dialog->intfInsideText);
         }
-        if (intf->getLabel()=="dmz")
+
+        if (std::find(possible_dmz_interface_labels.begin(),
+                      possible_dmz_interface_labels.end(),
+                      intf->getLabel()) != possible_dmz_interface_labels.end())
         {
             haveDMZ=true;
             m_dialog->intfDMZLine->show();
@@ -592,15 +695,13 @@ void newFirewallDialog::fillInterfaceData(Interface *intf, QTextBrowser *qte)
     else
         if (intf->isUnnumbered()) s +=  tr("Unnumbered interface");
         else
-            if (intf->isBridgePort()) s +=  tr("Bridge port");
-            else
-            {
-                const InetAddr *addr = intf->getAddressPtr();
-                QString addr_str = (addr) ? addr->toString().c_str() : "";
-                const InetAddr *netm = intf->getNetmaskPtr();
-                QString netm_str = (netm) ? netm->toString().c_str() : "";
-                s += QString("%1/%2").arg(addr_str).arg(netm_str);
-            }
+        {
+            const InetAddr *addr = intf->getAddressPtr();
+            QString addr_str = (addr) ? addr->toString().c_str() : "";
+            const InetAddr *netm = intf->getNetmaskPtr();
+            QString netm_str = (netm) ? netm->toString().c_str() : "";
+            s += QString("%1/%2").arg(addr_str).arg(netm_str);
+        }
     s += "</td>";
     s += "</tr>";
     s += "</table>";
@@ -618,7 +719,7 @@ bool newFirewallDialog::validateAddressAndMask(const QString &addr,
     {
         QMessageBox::warning(
             this,"Firewall Builder",
-            tr("Illegal address '%1/%2'").arg(addr).arg(netm),
+            tr("Invalid address '%1/%2'").arg(addr).arg(netm),
             "&Continue", QString::null, QString::null, 0, 1 );
         return false;
     }
@@ -632,10 +733,10 @@ bool newFirewallDialog::validateAddressAndMask(const QString &addr,
             {
                 QMessageBox::warning(
                     this,"Firewall Builder",
-                    tr("Illegal address '%1/%2'").arg(addr).arg(netm),
+                    tr("Invalid address '%1/%2'").arg(addr).arg(netm),
                     "&Continue", QString::null, QString::null, 0, 1 );
                 return false;
-            }          
+            }
         }
         else
         {
@@ -647,93 +748,13 @@ bool newFirewallDialog::validateAddressAndMask(const QString &addr,
     {
         QMessageBox::warning(
             this,"Firewall Builder",
-            tr("Illegal address '%1/%2'").arg(addr).arg(netm),
+            tr("Invalid address '%1/%2'").arg(addr).arg(netm),
             "&Continue", QString::null, QString::null, 0, 1 );
         return false;
     }
     return true;
 }
 
-
-void newFirewallDialog::addInterface()
-{
-    QString dn = "";
-    if (m_dialog->iface_dyn->isChecked())        dn+="Dyn";
-    if (m_dialog->iface_unnum->isChecked())      dn+="Unn";
-    if (m_dialog->iface_bridgeport->isChecked()) dn+="Bridge";
-
-    QString addr;
-    QString netm;
-
-    if (!m_dialog->iface_dyn->isChecked() &&
-        !m_dialog->iface_unnum->isChecked() &&
-        !m_dialog->iface_bridgeport->isChecked())
-    {
-        addr = m_dialog->iface_addr->text();
-        netm = m_dialog->iface_netmask->text();
-
-        if (addr.isEmpty())
-            addr = QString(InetAddr::getAny().toString().c_str());
-        if (netm.isEmpty())
-            netm = QString(InetAddr::getAny().toString().c_str());
-        if (!validateAddressAndMask(addr, netm)) return;
-
-    }
-    QStringList qsl;
-    qsl << m_dialog->iface_name->text()
-        << m_dialog->iface_label->text()
-        << addr << netm << dn
-        << m_dialog->iface_physaddr->text();
-
-    new QTreeWidgetItem(m_dialog->iface_list, qsl);
-}
-
-void newFirewallDialog::selectedInterface(QTreeWidgetItem*cur,QTreeWidgetItem*)
-{
-    QTreeWidgetItem *itm = cur; //current item
-    if (itm)
-    {
-        m_dialog->iface_name->setText( itm->text(0) );
-        m_dialog->iface_label->setText( itm->text(1) );
-        m_dialog->iface_addr->setText( itm->text(2) );
-        m_dialog->iface_netmask->setText( itm->text(3) );
-        m_dialog->iface_reg->setChecked( itm->text(4).isEmpty() );
-        m_dialog->iface_dyn->setChecked( itm->text(4).indexOf("Dyn")!=-1 );
-        m_dialog->iface_unnum->setChecked( itm->text(4).indexOf("Unn")!=-1 );
-        m_dialog->iface_bridgeport->setChecked( itm->text(4).indexOf("Bridge")!=-1 );
-        m_dialog->iface_physaddr->setText( itm->text(5) );
-    }
-}
-
-void newFirewallDialog::updateInterface()
-{
-    QString dn = "";
-    if (m_dialog->iface_dyn->isChecked())   dn+="Dyn";
-    if (m_dialog->iface_unnum->isChecked()) dn+="Unn";
-    if (m_dialog->iface_bridgeport->isChecked()) dn+="Bridge";
-
-    QTreeWidgetItem *itm = m_dialog->iface_list->currentItem();
-    if (itm==NULL) return;
-    QString addr = m_dialog->iface_addr->text();
-    QString netm = m_dialog->iface_netmask->text();
-    if (!validateAddressAndMask(addr, netm)) return;
-
-    itm->setText( 0 , m_dialog->iface_name->text() );
-    itm->setText( 1 , m_dialog->iface_label->text() );
-    itm->setText( 2 , m_dialog->iface_addr->text() );
-    itm->setText( 3 , m_dialog->iface_netmask->text() );
-    itm->setText( 4 , dn );
-    itm->setText( 5 , m_dialog->iface_physaddr->text() );
-}
-
-void newFirewallDialog::deleteInterface()
-{
-    QTreeWidgetItem *itm = m_dialog->iface_list->currentItem();
-    if (itm==NULL) return;
-    m_dialog->iface_list->takeTopLevelItem(
-            m_dialog->iface_list->indexOfTopLevelItem(itm) );
-}
-
 void newFirewallDialog::adjustSL(QTreeWidgetItem *itm1)
 {
 // interface 1 is above 2. Adjust their security levels accordingly
@@ -787,6 +808,19 @@ void newFirewallDialog::downInterface()
 
 void newFirewallDialog::cancelClicked()
 {
+    if (nfw)
+    {
+        parent->remove(nfw, false);
+        delete nfw;
+        nfw = NULL;
+    }
+
+    if (tmpldb)
+    {
+        delete tmpldb;
+        tmpldb = NULL;
+    }
+
     reject();
 }
 
@@ -794,46 +828,36 @@ void newFirewallDialog::finishClicked()
 {
     int p = currentPage();
 
+    if (fwbdebug)
+        qDebug() << "newFirewallDialog::finishClicked()"
+                 << "p=" << p;
+
+    if ( p == 5 )
+        if ( !this->m_dialog->interfaceEditor2->isValid() )
+            return;
+
+    string platform = readPlatform(m_dialog->platform).toAscii().constData();
+    string host_os = readHostOS(m_dialog->hostOS).toAscii().constData();
+
+    st->setNewFirewallPlatform(platform.c_str());
+
     if (p==2)  fillInterfaceSLList();
 
-    if (p==4)
+    if (p==4 || p==5)
     {
         // Creating from a template
-        QListWidgetItem *itm = m_dialog->templateList->currentItem();
-        FWObject *template_fw=templates[itm];
-        assert (template_fw!=NULL);
+        if (nfw==NULL) createFirewallFromTemplate();
+        else
+            changedAddressesInNewFirewall();
 
-        FWObject *no = mw->duplicateObject(mw->getCurrentLib(),
-                                           template_fw,
-                                           m_dialog->obj_name->text(),
-                                           false );  // do not ask to autorename
-        if (no==NULL)
-        {
-            QDialog::accept();
-            return;
-        }
-
-        map<string,string> platforms = Resources::getPlatforms();
-        map<string,string>::iterator i;
-        for (i=platforms.begin(); i!=platforms.end(); i++)
-            Resources::setDefaultTargetOptions(i->first, Firewall::cast(no) );
-
-        map<string,string> OSs = Resources::getOS();
-        for (i=OSs.begin(); i!=OSs.end(); i++)
-            Resources::setDefaultTargetOptions(i->first, Firewall::cast(no) );
-
-        no->setStr("platform",
-                   readPlatform(m_dialog->platform).toLatin1().constData() );
-        no->setStr("host_OS",
-                   readHostOS(m_dialog->hostOS).toLatin1().constData() );
-
-        nfw=Firewall::cast(no);
     } else
     {
         // Create from interface list (obtained either manually or via snmp)
+        if ( !this->m_dialog->interfaceEditor1->isValid() )
+            return;
 
         FWObject *o;
-        o = mw->createObject(Firewall::TYPENAME, m_dialog->obj_name->text() );
+        o = db->create(Firewall::TYPENAME);
 
         if (o==NULL)
         {
@@ -841,127 +865,104 @@ void newFirewallDialog::finishClicked()
             return;
         }
 
-        map<string,string> platforms = Resources::getPlatforms();
-        map<string,string>::iterator i;
-        for (i=platforms.begin(); i!=platforms.end(); i++)
-            Resources::setDefaultTargetOptions( i->first , Firewall::cast(o) );
-
-        map<string,string> OSs = Resources::getOS();
-        for (i=OSs.begin(); i!=OSs.end(); i++)
-            Resources::setDefaultTargetOptions( i->first  , Firewall::cast(o) );
-
-        o->setStr("platform",
-                  readPlatform(m_dialog->platform).toLatin1().constData() );
-        o->setStr("host_OS",
-                  readHostOS(m_dialog->hostOS).toLatin1().constData() );
+        o->setName( string( m_dialog->obj_name->text().toUtf8().constData() ) );//.toStdString());
+        parent->add(o);
 
         nfw = Firewall::cast(o);
 
+        o->setStr("platform", platform);
+        Resources::setDefaultTargetOptions(platform , nfw);
+
+        o->setStr("host_OS", host_os);
+        Resources::setDefaultTargetOptions(host_os , nfw);
+
 /* create interfaces */
 
-        int itm_index = 0;
-        QTreeWidgetItem *itm = m_dialog->iface_list->topLevelItem(itm_index);
-
-        while (itm!=NULL)
+        foreach(EditedInterfaceData iface, this->m_dialog->interfaceEditor1->getNewData())
         {
-            QString name    =  itm->text(0);
-            QString label   =  itm->text(1);
-            QString addr    =  itm->text(2);
-            QString netmask =  itm->text(3);
-            bool    dyn     =  itm->text(4).indexOf("Dyn")!=-1;
-            bool    unnum   =  itm->text(4).indexOf("Unn")!=-1;
-            bool    bridgeport =  itm->text(4).indexOf("Bridge")!=-1;
-            QString physaddr=  itm->text(5);
+            QString name     =  iface.name;
+            QString label    =  iface.label;
+            bool    dyn      =  iface.type == 1;
+            bool    unnum    =  iface.type == 2;
+            QString physaddr =  iface.mac;
 
-            QList<QTreeWidgetItem*> ltwi = m_dialog->iface_sl_list->findItems(
-                name , Qt::MatchExactly );
+            QList<QTreeWidgetItem*> ltwi = m_dialog->iface_sl_list->findItems( name , Qt::MatchExactly );
             assert(!ltwi.empty());
             QTreeWidgetItem *itm2 = ltwi[0];
             assert(itm2!=NULL);
 
             int sl = itm2->text(3).toInt();
 
-            Interface *oi = Interface::cast(mw->db()->create(Interface::TYPENAME));
+            Interface *oi = Interface::cast(db->create(Interface::TYPENAME));
             assert(oi!=NULL);
 
             nfw->add(oi);
 
-            oi->setName(name.toStdString());
-            oi->setLabel(label.toStdString());
+            oi->setName( string(name.toUtf8().constData()) );
+            oi->setLabel( string(label.toUtf8().constData()) );
+            oi->setComment( string(iface.comment.toUtf8().constData()) );
 
             oi->setDyn(dyn);
             oi->setUnnumbered(unnum);
-            oi->setBridgePort(bridgeport);
             oi->setSecurityLevel(sl);
 
-            mw->insertObjectInTree(nfw, oi);
-
+            //mw->insertObjectInTree(nfw, oi);
             if (fwbdebug)
                 qDebug("Adding interface %s: security_level=%d",
                        oi->getName().c_str(), sl);
 
-            if (!dyn && !unnum && !bridgeport &&
-                !addr.isEmpty() && addr!="0.0.0.0")
+            if (iface.type == 0)
             {
-                if (addr.indexOf(':')!=-1)
+                foreach(AddressInfo address, iface.addresses)
                 {
-                    // ipv6 address
-                    QString addrname = QString("%1:%2:ip6")
-                        .arg(m_dialog->obj_name->text()).arg(name);
-                    IPv6 *oa = IPv6::cast(mw->db()->create(IPv6::TYPENAME));
-                    oi->add(oa);
-                    oa->setName(addrname.toStdString());
-                    oa->setAddress(
-                        InetAddr(AF_INET6, addr.toLatin1().constData()) );
-                    bool ok = false ;
-                    int inetmask = netmask.toInt(&ok);
-                    if (ok)
+                    if (address.address == "0.0.0.0") continue;
+                    if (address.ipv4)
                     {
-                        oa->setNetmask( InetAddr(AF_INET6, inetmask) );
+                        string addrname = string( QString("%1:%2:ip").arg(QString(m_dialog->obj_name->text())).arg(name).toUtf8().constData() );
+                        IPv4 *oa = IPv4::cast(db->create(IPv4::TYPENAME));
+                        oi->add(oa);
+                        oa->setName(addrname);
+                        oa->setAddress( InetAddr(address.address.toLatin1().constData()) );
+                        bool ok = false ;
+                        int inetmask = address.netmask.toInt(&ok);
+                        if (ok)
+                        {
+                            oa->setNetmask( InetAddr(inetmask) );
+                        }
+                        else
+                        {
+                            oa->setNetmask( InetAddr(address.netmask.toLatin1().constData()) );
+                        }
                     }
                     else
                     {
-                        oa->setNetmask(
-                            InetAddr(AF_INET6, netmask.toLatin1().constData()));
+                        string addrname = string ( QString("%1:%2:ip6").arg(QString(m_dialog->obj_name->text())).arg(name).toUtf8().constData() );
+                        IPv6 *oa = IPv6::cast(db->create(IPv6::TYPENAME));
+                        oi->add(oa);
+                        oa->setName(addrname);
+                        oa->setAddress(InetAddr(AF_INET6, address.address.toLatin1().constData()) );
+                        bool ok = false ;
+                        int inetmask = address.netmask.toInt(&ok);
+                        if (ok)
+                        {
+                            oa->setNetmask( InetAddr(AF_INET6, inetmask) );
+                        }
+                        else
+                        {
+                            oa->setNetmask(InetAddr(AF_INET6, address.netmask.toLatin1().constData()));
+                        }
                     }
-
-                    mw->insertObjectInTree(oi, oa);
-
-                } else
-                {
-                    QString addrname = QString("%1:%2:ip").arg(
-                        m_dialog->obj_name->text()).arg(name);
-                    IPv4 *oa = IPv4::cast(mw->db()->create(IPv4::TYPENAME));
-                    oi->add(oa);
-                    oa->setName(addrname.toStdString());
-                    oa->setAddress( InetAddr(addr.toLatin1().constData()) );
-                    bool ok = false ;
-                    int inetmask = netmask.toInt(&ok);
-                    if (ok)
-                    {
-                        oa->setNetmask( InetAddr(inetmask) );
-                    }
-                    else
-                    {
-                        oa->setNetmask(
-                            InetAddr(netmask.toLatin1().constData()) );
-                    }
-
-                    mw->insertObjectInTree(oi, oa);
                 }
             }
-
-            itm_index++;
-            itm = m_dialog->iface_list->topLevelItem(itm_index);
         }
-        mw->reloadAllWindowsWithFile(mw->activeProject());
     }
-    if (unloadTemplatesLib)
+
+    if (tmpldb!=NULL)
     {
         delete tmpldb;
         tmpldb = NULL;
-        unloadTemplatesLib=false;
     }
+
     QDialog::accept();
 }
 
