@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: newGroupDialog.cpp 332 2008-07-06 20:11:03Z vadim $
+  $Id: newGroupDialog.cpp 1487 2009-09-23 17:00:48Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -29,9 +29,6 @@
 #include "utils.h"
 
 #include "newGroupDialog.h"
-#include "FWWindow.h"
-
-#include "fwbuilder/FWObjectDatabase.h"
 
 #include <qlineedit.h>
 #include <qapplication.h>
@@ -46,12 +43,14 @@ newGroupDialog::~newGroupDialog()
     delete m_dialog;
 }
 
-newGroupDialog::newGroupDialog(QWidget *parent) : QDialog(parent)
+newGroupDialog::newGroupDialog(QWidget *parent, FWObjectDatabase *_db) : QDialog(parent)
 {
+    db = _db;
+
     m_dialog = new Ui::newGroupDialog_q;
     m_dialog->setupUi(this);
 
-    fillLibraries(m_dialog->libs,mw->db(),true);   // only read-write libs
+    fillLibraries(m_dialog->libs, db, true);   // only read-write libs
     m_dialog->obj_name->setFocus();
 }
 
