@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: newHostDialog.h 304 2008-07-03 02:19:16Z vadim $
+  $Id: newHostDialog.h 1833 2009-11-20 15:07:12Z a2k $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -58,7 +58,9 @@ class newHostDialog : public QDialog, public FakeWizard
     libfwbuilder::Logger               *logger;
     libfwbuilder::SNMP_interface_query *q;
     QTimer                             *timer;
+    libfwbuilder::FWObjectDatabase     *db;
     libfwbuilder::FWObjectDatabase     *tmpldb;
+    libfwbuilder::FWObject             *parent;
     std::map<QListWidgetItem*, libfwbuilder::FWObject*> templates;
     bool                                unloadTemplatesLib;
     bool                                getInterfacesBusy;
@@ -69,7 +71,7 @@ class newHostDialog : public QDialog, public FakeWizard
     bool validateAddressAndMask(const QString &addr, const QString &netm);
 
  public:
-    newHostDialog();
+    newHostDialog(libfwbuilder::FWObject *parent);
     virtual ~newHostDialog();
 
     libfwbuilder::Host* getNewHost() { return nhst; };
@@ -78,16 +80,12 @@ class newHostDialog : public QDialog, public FakeWizard
     void showPage(const int page);
 
 public slots:
-    virtual void addInterface();
-    virtual void updateInterface();
-    virtual void deleteInterface();
     virtual void changed();
 //    virtual void selectedInterface(QTreeWidgetItem *cur);
     virtual void getInterfacesViaSNMP();    
     virtual void monitor();
 //    virtual void templateSelected(QListWidgetItem *cur);
 
-    virtual void selectedInterface(QTreeWidgetItem*,QTreeWidgetItem*);
     virtual void templateSelected(QListWidgetItem *itm);
     void browseTemplate();
     void useStandartTemplate();
