@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: pixAdvancedDialog.cpp 585 2008-10-09 05:08:41Z vadim $
+  $Id: pixAdvancedDialog.cpp 2259 2009-12-23 21:53:29Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -34,13 +34,19 @@
 #include "SimpleTextEditor.h"
 #include "FWWindow.h"
 #include "FWBSettings.h"
+#include "FWCmdChange.h"
 
+#include "CompilerDriver_pix.h"
+
+#include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Management.h"
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/XMLTools.h"
 
+#include <memory>
+
 #include <qcheckbox.h>
 #include <qspinbox.h>
 #include <qcombobox.h>
@@ -52,16 +58,14 @@
 #include <qtabwidget.h>
 #include <qlistwidget.h>
 #include <qlabel.h>
-#include <qprocess.h>
-#include <qfile.h>
 
 #include <iostream>
 #include <sstream>
 
-#include <libxml/xmlmemory.h>
 
 using namespace std;
 using namespace libfwbuilder;
+using namespace fwcompiler;
 
 
 pixAdvancedDialog::pixAdvancedDialog(QWidget*, FWObject *o)//(parent)
@@ -70,48 +74,8 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget*, FWObject *o)//(parent)
     m_dialog->setupUi(static_cast<QDialog*>(this));
     obj=o;
 
-    Firewall  *fw=Firewall::cast(obj);
-    FWOptions *fwopt=fw->getOptionsObject();
-    string compiler=fwopt->getStr("compiler");
-    if (compiler=="")
-    {
-        compiler=Resources::platform_res[fw->getStr("platform")]->getCompiler();
-    }
-/*
- * On Unix compilers are installed in the standard place and are
- * accessible via PATH. On Windows and Mac they get installed in
- * unpredictable directories and need to be found
- *
- * first, check if user specified an absolute path for the compiler,
- * then check  if compiler is registsred in preferences, and if not,
- * look for it in appRootDir and if it is not there, rely on PATH
- */
-#if defined(Q_OS_WIN32) ||  defined(Q_OS_MACX)
-
-    if ( ! QFile::exists( compiler.c_str() ) )
-    {
-        string ts = string("Compilers/")+compiler;
-        QString cmppath = st->getStr( ts.c_str() );
-        if (!cmppath.isEmpty()) compiler=cmppath.toLatin1().constData();
-        else
-        {
-            /* try to find compiler in appRootDir. */
-            string ts =  getPathToBinary(compiler);
-            if ( QFile::exists( ts.c_str() ) )
-                compiler = ts;
-        }
-    }
-#endif
-
-    fwb_pix_proc = new QProcess();
-
-    connect(fwb_pix_proc, SIGNAL(readyReadStandardOutput()), this,  SLOT(readFromStdout() ) );
-    connect(fwb_pix_proc, SIGNAL(readyReadStandardError()), this,  SLOT(readFromStderr() ) );
-    connect(fwb_pix_proc, SIGNAL(stateChanged( QProcess::ProcessState )),   this,  SLOT(fwb_pix_Finished( QProcess::ProcessState ) ) );
-    connect(fwb_pix_proc, SIGNAL(bytesWritten(qint64)),   this,  SLOT(allXMLSent() ) );
-
-    compilerPath = compiler.c_str();
-    argumentList << "-f" << "-" << "-I" << fw->getName().c_str();
+    //Firewall  *fw=Firewall::cast(obj);
+    //FWOptions *fwopt=fw->getOptionsObject();
 
     string vers="version_"+obj->getStr("version");
     string platform = obj->getStr("platform");   // could be 'pix' or 'fwsm'
@@ -206,11 +170,11 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget*, FWObject *o)//(parent)
     syslogFacilityMapping.push_back("LOCAL7");
     syslogFacilityMapping.push_back("23");
 
-
-
-    FWOptions *fwoptions=(Firewall::cast(obj))->getOptionsObject();
+    FWOptions *fwoptions = (Firewall::cast(obj))->getOptionsObject();
     assert(fwoptions!=NULL);
 
+/* Page Script */
+
     bool f1=fwoptions->getBool("pix_acl_basic");
     bool f2=fwoptions->getBool("pix_acl_no_clear");
     bool f3=fwoptions->getBool("pix_acl_substitution");
@@ -232,10 +196,13 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget*, FWObject *o)//(parent)
     Management *mgmt=(Firewall::cast(obj))->getManagementObject();
     assert(mgmt!=NULL);
 
+    data.registerOption( m_dialog->short_script, fwoptions, "short_script");
+
 /* Page "Compiler Options" */
 
     bool outboundACLSupported= (Resources::platform_res[platform]->getResourceBool(
-                  "/FWBuilderResources/Target/options/"+vers+"/pix_outbound_acl_supported") );
+                  "/FWBuilderResources/Target/options/" +
+                  vers + "/pix_outbound_acl_supported"));
 
     if (outboundACLSupported)
         m_dialog->pix_emulate_out_acl->hide();
@@ -243,8 +210,9 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget*, FWObject *o)//(parent)
         m_dialog->pix_generate_out_acl->hide();
 
     m_dialog->tabWidget->setTabEnabled(8,false); //Disable tab
-    data.registerOption(m_dialog->ipv4before_2,    fwoptions, "ipv4_6_order", QStringList() <<  "IPv4 before IPv6" <<"ipv4_first" << "IPv6 before IPv4" << "ipv6_first");
-
+    data.registerOption(m_dialog->ipv4before_2, fwoptions,
+                        "ipv4_6_order",
+                        QStringList() <<  "IPv4 before IPv6" <<"ipv4_first" << "IPv6 before IPv4" << "ipv6_first");
 
     data.registerOption( m_dialog->outputFileName, fwoptions,
                          "output_file");
@@ -330,6 +298,10 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget*, FWObject *o)//(parent)
     data.registerOption( m_dialog->sshArgs, fwoptions, "sshArgs");
     data.registerOption( m_dialog->scpArgs, fwoptions, "scpArgs");
 
+    data.registerOption( m_dialog->use_scp, fwoptions, "use_scp");
+    data.registerOption( m_dialog->filesystem, fwoptions, "filesystem");
+    data.registerOption( m_dialog->filesystem, fwoptions, "firewall_dir");
+
     PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();
 
     m_dialog->installScript->setText(     pis->getCommand().c_str());
@@ -658,6 +630,8 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget*, FWObject *o)//(parent)
     loadFixups();
     updateFixupCommandsDisplay();
     scriptACLModeChanged();
+
+    m_dialog->tabWidget->setCurrentIndex(0);
 }
 
 pixAdvancedDialog::~pixAdvancedDialog()
@@ -785,60 +759,15 @@ void pixAdvancedDialog::displayCommands()
  */
     saveFixups();
 
-    xmlChar  *buffer;
-    int       bufsize;
-    mw->db()->saveToBuffer(&buffer,&bufsize);
-    proc_buffer = (char*)buffer;
-    FREEXMLBUFF(buffer);
-
-    fwb_pix_proc->start(compilerPath, argumentList);
-    if ( !fwb_pix_proc->waitForStarted() )
-    {
-        m_dialog->pix_generated_fixup->append(
-            tr("Error: Policy compiler for PIX is not installed") );
-    }
-
-    fwb_pix_proc->write(proc_buffer.toAscii());
+    CompilerDriver_pix driver(obj->getRoot());
+    driver.setTargetId(FWObjectDatabase::getStringId(obj->getId()));
+    string inspectors = driver.protocolInspectorCommands();
+    m_dialog->pix_generated_fixup->setText(inspectors.c_str());
 }
 
-void pixAdvancedDialog::allXMLSent()
-{
-    fwb_pix_proc->closeWriteChannel();
-}
-
-void pixAdvancedDialog::readFromStdout()
-{
-    m_dialog->pix_generated_fixup->append( QString( fwb_pix_proc->readAllStandardOutput() ) );
-}
-
-void pixAdvancedDialog::readFromStderr()
-{
-    m_dialog->pix_generated_fixup->append( QString( fwb_pix_proc->readAllStandardError() ) );
-}
-
-void pixAdvancedDialog::fwb_pix_Finished( QProcess::ProcessState newState )
-{
-    if (newState != QProcess::NotRunning) return;
-
-    if (fwb_pix_proc->exitStatus() != QProcess::NormalExit)
-        m_dialog->pix_generated_fixup->append( tr("Compiler error") );
-}
-
-
 void pixAdvancedDialog::updateFixupCommandsDisplay()
 {
-
-    m_dialog->pix_generated_fixup->setText("");
-    return;
-
-    QString stdoutBuffer;
-
-    fwb_pix_proc->start(compilerPath, argumentList);
-    if ( !fwb_pix_proc->waitForStarted() )
-    {
-        m_dialog->pix_generated_fixup->append(  tr("Error: Policy compiler for PIX is not installed") );
-    }
-    return;
+    displayCommands();
 }
 
 void pixAdvancedDialog::fixupCmdChanged()
@@ -851,13 +780,19 @@ void pixAdvancedDialog::fixupCmdChanged()
  */
 void pixAdvancedDialog::accept()
 {
-    FWOptions *options=(Firewall::cast(obj))->getOptionsObject();
-    assert(options!=NULL);
+    ProjectPanel *project = mw->activeProject();
+    std::auto_ptr<FWCmdChange> cmd( new FWCmdChange(project, obj));
 
-    Management *mgmt=(Firewall::cast(obj))->getManagementObject();
+    // new_state  is a copy of the fw object
+    FWObject* new_state = cmd->getNewState();
+    FWOptions* fwoptions = Firewall::cast(new_state)->getOptionsObject();
+    assert(fwoptions!=NULL);
+
+    Management *mgmt = (Firewall::cast(new_state))->getManagementObject();
     assert(mgmt!=NULL);
 
-    data.saveAll();
+    data.saveAll(fwoptions);
+
     saveFixups();
 
 //    PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();
@@ -865,7 +800,7 @@ void pixAdvancedDialog::accept()
 //    pis->setArguments( installScriptArgs->text() );
 
     // find first interface marked as "management"
-    const InetAddr *mgmt_addr = Firewall::cast(obj)->getManagementAddress();
+    const InetAddr *mgmt_addr = Firewall::cast(new_state)->getManagementAddress();
     if (mgmt_addr)
         mgmt->setAddress(*mgmt_addr);
 
@@ -886,15 +821,17 @@ void pixAdvancedDialog::accept()
             v=m_dialog->syslog_device_id_string_val->text();
         }
 
-        options->setStr("pix_syslog_device_id_opt",s.toLatin1().constData());
-        options->setStr("pix_syslog_device_id_val",v.toLatin1().constData());
+        fwoptions->setStr("pix_syslog_device_id_opt",s.toLatin1().constData());
+        fwoptions->setStr("pix_syslog_device_id_val",v.toLatin1().constData());
     }
 
     PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();
     pis->setCommand( m_dialog->installScript->text().toLatin1().constData() );
     pis->setArguments( m_dialog->installScriptArgs->text().toLatin1().constData() );
 
-    mw->updateLastModifiedTimestampForAllFirewalls(obj);
+    if (!cmd->getOldState()->cmp(new_state, true))
+        project->undoStack->push(cmd.release());
+    
     QDialog::accept();
 }
 
