@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: platforms.cpp 1154 2009-07-14 21:26:34Z vadim $
+  $Id: platforms.cpp 2647 2010-03-04 02:06:35Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,11 +27,18 @@
 #include "global.h"
 
 #include "platforms.h"
+#include "FWBSettings.h"
+#include "interfaceProperties.h"
+#include "interfacePropertiesObjectFactory.h"
 
-#include <qobject.h>
-#include <qstringlist.h>
+#include <QObject>
+#include <QStringList>
+#include <QComboBox>
+#include <QtDebug>
 
+#include "fwbuilder/Cluster.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/Interface.h"
 #include "fwbuilder/FWOptions.h"
 #include "fwbuilder/Management.h"
 #include "fwbuilder/Resources.h"
@@ -39,12 +46,15 @@
 #include "fwbuilder/Policy.h"
 
 #include <algorithm>
+#include <iostream>
+#include <memory>
+
 
 using namespace std;
 using namespace libfwbuilder;
 
-QStringList emptyList;
 
+QStringList emptyList;
 QStringList logLevels;
 QStringList logFacilities;
 QStringList actionsOnReject;
@@ -200,8 +210,10 @@ bool isDefaultPolicyRuleOptions(FWOptions *opt)
 	    res= ( opt->getStr("log_prefix").empty() &&
                    opt->getStr("log_level").empty()      &&
 		   opt->getInt("limit_value")<=0         &&
+		   ! opt->getBool("limit_value_not")     &&
 		   opt->getInt("limit_burst")<=0         &&
 		   opt->getInt("connlimit_value")<=0     &&
+		   ! opt->getBool("connlimit_above_not") &&
 		   opt->getInt("connlimit_masklen")<=0   &&
 
 		   opt->getStr("hashlimit_name").empty() &&
@@ -214,7 +226,7 @@ bool isDefaultPolicyRuleOptions(FWOptions *opt)
 
                    opt->getInt("ulog_nlgroup")<=1        &&
                    opt->getStr("limit_suffix").empty()   &&
-                   ! opt->getBool("firewall_is_part_of_any_and_networks"));
+                   opt->getStr("firewall_is_part_of_any_and_networks") == "");
 	}
 
 	if (platform=="pix" || platform=="fwsm")
@@ -235,27 +247,37 @@ bool isDefaultPolicyRuleOptions(FWOptions *opt)
 
 	if (platform=="pf")
         {
-            string vers=p->getStr("version");
-            if (vers=="4.x")
+            string version = p->getStr("version");
+            bool ge_4_0 = XMLTools::version_compare(version, "4.0")>=0;
+            bool ge_4_5 = XMLTools::version_compare(version, "4.5")>=0;
+            if (ge_4_5)
             {
-                res= ( opt->getStr("log_prefix").empty()       &&
-                       opt->getInt("pf_rule_max_state")<=0     &&
-                       ! opt->getBool("pf_source_tracking")    &&
-                       opt->getInt("pf_max_src_conn")<=0       &&
-                       opt->getInt("pf_max_src_conn_rate_num")<=0       &&
-                       opt->getInt("pf_max_src_conn_rate_seconds")<=0 &&
-                       ! opt->getBool("pf_keep_state") &&
-                       ! opt->getBool("pf_sloppy_tracker")
+                res = (!opt->getBool("pf_no_sync") && opt->getBool("pf_pflow"));
+            }
+
+            if (ge_4_0)
+            {
+                res = res &&
+                    ( opt->getStr("log_prefix").empty()       &&
+                      opt->getInt("pf_rule_max_state")<=0     &&
+                      ! opt->getBool("pf_source_tracking")    &&
+                      opt->getInt("pf_max_src_conn")<=0       &&
+                      opt->getInt("pf_max_src_conn_rate_num")<=0       &&
+                      opt->getInt("pf_max_src_conn_rate_seconds")<=0 &&
+                      ! opt->getBool("pf_keep_state") &&
+                      ! opt->getBool("pf_sloppy_tracker")
                 );
             }else
             {
-                res= ( opt->getStr("log_prefix").empty()       &&
-                       opt->getInt("pf_rule_max_state")<=0     &&
-                       ! opt->getBool("pf_source_tracking")    &&
-                       opt->getInt("pf_max_src_conn")<=0       &&
-                       opt->getInt("pf_max_src_conn_rate_num")<=0       &&
-                       opt->getInt("pf_max_src_conn_rate_seconds")<=0 &&
-                       ! opt->getBool("pf_sloppy_tracker")
+                res = res &&
+                    ( opt->getStr("log_prefix").empty()       &&
+                      opt->getInt("pf_rule_max_state")<=0     &&
+                      ! opt->getBool("pf_source_tracking")    &&
+                      opt->getInt("pf_max_src_conn")<=0       &&
+                      opt->getInt("pf_max_src_conn_rate_num")<=0       &&
+                        opt->getInt("pf_max_src_conn_rate_seconds")<=0 &&
+                      ! opt->getBool("pf_keep_state") &&
+                      ! opt->getBool("pf_sloppy_tracker")
                 );
             }
 	}
@@ -291,9 +313,8 @@ bool isDefaultPolicyRuleOptions(FWOptions *opt)
         // all rules are stateless for IOS ACL
 	if (platform=="iosacl")
         {
-	    res = true; // ignore "stateless" option
-	}
-
+            res = !opt->getBool("iosacl_add_mirror_rule");
+        }
     }
     return res;
 }
@@ -311,11 +332,15 @@ bool isDefaultNATRuleOptions(FWOptions *opt)
 
     QString platform = p->getStr("platform").c_str();
 
-//    if (fwbdebug)
-//        qDebug(QString("Options object type: %1   platform: %2   pf_pool_type_none: '%3'").arg(opt->getTypeName()).arg(platform).arg(opt->getStr("pf_pool_type_none").c_str()));
-
     if (NATRuleOptions::isA(opt))
     {
+	if (platform=="iptables")
+        {
+            res = !opt->getBool("ipt_use_snat_instead_of_masq") &&
+                !opt->getBool("ipt_nat_random") &&
+                !opt->getBool("ipt_nat_persistent");
+        }
+
 	if (platform=="pf")
         {
             // if "pf_pool_type_none" is undefined, then all others
@@ -350,17 +375,16 @@ bool isDefaultRoutingRuleOptions(FWOptions *opt)
 
 QString getVersionString(const QString &platform, const QString &version)
 {
-    list<QStringPair> vl = getVersionsForPlatform(platform);
+    list<QStringPair> vl;
+    getVersionsForPlatform(platform, vl);
     list<QStringPair>::iterator li =
         std::find_if(vl.begin(),vl.end(),findFirstInQStringPair(version));
     QString readableVersion = (li!=vl.end())?li->second:"";
     return readableVersion;
 }
 
-list<QStringPair> getVersionsForPlatform(const QString &platform)
+void getVersionsForPlatform(const QString &platform, std::list<QStringPair> &res)
 {
-    list<QStringPair> res;
-
 /* versions are defined here instead of the resource files so that
  * strings could be localized. We use strings that can be localized
  * only for iptables but define versions for all platforms here for
@@ -395,7 +419,10 @@ list<QStringPair> getVersionsForPlatform(const QString &platform)
                 res.push_back(QStringPair("","- any -"));
                 res.push_back(QStringPair("3.x", QObject::tr("3.x")));
 		res.push_back(QStringPair("ge_3.7", QObject::tr("3.7 to 3.9")));
-                res.push_back(QStringPair("4.x", QObject::tr("4.x")));
+                res.push_back(QStringPair("4.0", QObject::tr("4.0 to 4.2")));
+                res.push_back(QStringPair("4.3", QObject::tr("4.3")));
+                res.push_back(QStringPair("4.5", QObject::tr("4.5")));
+                res.push_back(QStringPair("4.6", QObject::tr("4.6 and later")));
 /* add pf versions here */
             } else
             {
@@ -415,8 +442,160 @@ list<QStringPair> getVersionsForPlatform(const QString &platform)
             }
         }
     }
+}
 
-    return res;
+/*
+ * ticket #58: move state sync types and failover types to resource files.
+ *
+ * Note: this function fills in list of QString pairs, each pair is 
+ * <protocol_name>,<user_readable_protocol_name>
+ * The second element in the pair is for QComboBox and is visible to the user.
+ */
+void getStateSyncTypesForOS(const QString &host_os, std::list<QStringPair> &res)
+{
+    Resources* os_res = Resources::os_res[host_os.toStdString()];
+    if (os_res==NULL) return;
+    list<string> protocols;
+    os_res->getResourceStrList("/FWBuilderResources/Target/protocols/state_sync",
+                               protocols);
+    _repackStringList(protocols, res);
+}
+
+void getFailoverTypesForOS(const QString &host_os, std::list<QStringPair> &res)
+{
+    Resources* os_res = Resources::os_res[host_os.toStdString()];
+    if (os_res==NULL) return;
+    list<string> protocols;
+    os_res->getResourceStrList("/FWBuilderResources/Target/protocols/failover",
+                               protocols);
+    _repackStringList(protocols, res);
+}
+
+void getInterfaceTypes(Interface *iface, list<QStringPair> &res)
+{
+    FWObject *fw = iface->getParent();
+    string host_os = fw->getStr("host_OS");
+    Resources* os_res = Resources::os_res[host_os];
+    if (os_res==NULL) return;
+    list<string> interface_types;
+
+    if (Cluster::isA(fw))
+    {
+        os_res->getResourceStrList("/FWBuilderResources/Target/interfaces/cluster",
+                                   interface_types);
+    } else
+    {
+        os_res->getResourceStrList("/FWBuilderResources/Target/interfaces/firewall",
+                                   interface_types);
+    }
+    _repackStringList(interface_types, res);
+}
+
+/*
+ * Return list of types of subinterfaces that given interface can have
+ *
+ * @iface an Interface object. This is not a subinterface, this is a
+ * regular interface. This function returns list of subinterface types
+ * this interface can have.
+ *
+ * @res a list of pairs of QString, each pair is <type>,<description>
+ */
+void getSubInterfaceTypes(Interface *iface, list<QStringPair> &res)
+{
+    FWObject *p = iface->getParentHost();
+    assert(p!=NULL);
+
+    QString host_os = p->getStr("host_OS").c_str();
+    Resources* os_res = Resources::os_res[host_os.toStdString()];
+    if (os_res==NULL) return;
+
+    FWOptions *ifopt;
+    ifopt = Interface::cast(iface)->getOptionsObject();
+    string parent_type = ifopt->getStr("type");
+
+    // empty parent type is equivalent to "ethernet" for backwards
+    // compatibility
+    if (parent_type.empty()) parent_type = "ethernet";
+
+    QString obj_name = iface->getName().c_str();
+
+    list<string> interface_types;
+    os_res->getResourceStrList(
+        "/FWBuilderResources/Target/subinterfaces/" + parent_type,
+        interface_types);
+    _repackStringList(interface_types, res);
+}
+ 
+void setInterfaceTypes(QComboBox *iface_type,
+                       Interface *iface,
+                       const QString &current_type)
+{
+    bool this_is_subinterface = Interface::isA(iface->getParent());
+    list<QStringPair> mapping;
+    if (this_is_subinterface)
+        getSubInterfaceTypes(Interface::cast(iface->getParent()), mapping);
+    else getInterfaceTypes(iface, mapping);
+
+    // #335 : if interface name matches naming convention for vlan
+    // interfaces and vlan type is in the list that came from the
+    // resource file, then leave only vlan in the list we return.
+    // Note that if resource file says this subint can not be vlan, we
+    // dan't return vlan type on the list even if its name looks like
+    // it could be one.
+    FWObject *p = iface->getParentHost();
+    assert(p!=NULL);
+    QString host_os = p->getStr("host_OS").c_str();
+    QString obj_name = iface->getName().c_str();
+
+    Resources* os_res = Resources::os_res[p->getStr("host_OS")];
+    string os_family = p->getStr("host_OS");
+    if (os_res!=NULL)
+        os_family = os_res->getResourceStr("/FWBuilderResources/Target/family");
+
+    std::auto_ptr<interfaceProperties> int_prop(
+        interfacePropertiesObjectFactory::getInterfacePropertiesObject(
+            os_family));
+    if (int_prop->looksLikeVlanInterface(obj_name))
+    {
+        QString parent_name = iface->getParent()->getName().c_str();
+        QString err;
+        if (int_prop->isValidVlanInterfaceName(obj_name, parent_name, err))
+        {
+            // iface can be valid vlan interface. Leave only vlan type
+            // in the list if it was there to begin with. 
+            for (list<QStringPair>::iterator it=mapping.begin();
+                 it!=mapping.end(); ++it)
+            {
+                QString itype = it->first;
+                QString rtype = it->second;
+                if (itype == "8021q")
+                {
+                    mapping.clear();
+                    mapping.push_back(QStringPair(itype, rtype));
+                    mapping.push_back(QStringPair("unknown", "Unknown"));
+                    break;
+                }
+            }
+        }
+    }
+
+    list<QStringPair>::iterator it;
+    int idx = 0;
+    int unknown_idx = 0;
+    int current_idx = -1;
+    for (it = mapping.begin(); it != mapping.end(); it++)
+    {
+        if (it->first == "unknown") unknown_idx = idx;
+        iface_type->addItem(it->second);
+        iface_type->setItemData(idx,  QVariant(it->first));
+        if (current_type == it->first)
+            current_idx = idx;
+        idx++;
+    }
+    if (current_idx >= 0)
+        iface_type->setCurrentIndex(current_idx);
+    else
+        iface_type->setCurrentIndex(unknown_idx);
 }
 
 /* currently we return the same list for all platforms */
@@ -493,6 +672,16 @@ QString getScreenName(QString s, const QStringList &sl)
     return res;
 }
 
+QString getRuleAction(Rule *rule)
+{
+    PolicyRule *policy_rule = PolicyRule::cast(rule);
+    NATRule *nat_rule = NATRule::cast(rule);
+    string act;
+    if (policy_rule) act = policy_rule->getActionAsString();
+    if (nat_rule) act = nat_rule->getActionAsString();
+    return act.c_str();
+}
+
 /*
  * will remap names of some actions to make it clear what commands or
  * configuration language keywords they will be translated to for the
@@ -503,59 +692,28 @@ QString getScreenName(QString s, const QStringList &sl)
  * files and are not pulled using Rule::getActionAsString.
  */
 
-QString getActionNameForPlatform(PolicyRule::Action action,
-                                 const QString &platform)
+QString getActionNameForPlatform(Firewall *fw, Rule *rule)
 {
-    QString action_name = "";
-    switch (action)
+    if (fw==NULL) return "";
+    PolicyRule *policy_rule = PolicyRule::cast(rule);
+    NATRule *nat_rule = NATRule::cast(rule);
+    string act;
+    if (policy_rule) act = policy_rule->getActionAsString();
+    if (nat_rule) act = nat_rule->getActionAsString();
+    return getActionNameForPlatform(fw, act);
+}
+
+QString getActionNameForPlatform(Firewall *fw, const std::string &action)
+{
+    if (fw==NULL) return "";
+    string platform = fw->getStr("platform");
+    string name;
+    try  
     {
-    case PolicyRule::Accept:     action_name = QObject::tr("Accept");   break;
-
-    case PolicyRule::Deny:       action_name = QObject::tr("Deny");     break;
-
-    case PolicyRule::Reject:     action_name = QObject::tr("Reject");   break;
-
-    case PolicyRule::Scrub:      action_name = QObject::tr("Scrub");    break;
-
-    case PolicyRule::Return:     action_name = QObject::tr("Return");   break;
-
-    case PolicyRule::Skip:       action_name = QObject::tr("Skip");     break;
-
-    case PolicyRule::Continue:   action_name = QObject::tr("Continue"); break;
-
-    case PolicyRule::Modify:     action_name = QObject::tr("Modify");   break;
-
-    case PolicyRule::Classify:   action_name = QObject::tr("Classify"); break;
-
-    case PolicyRule::Custom:     action_name = QObject::tr("Custom");   break;
-
-    case PolicyRule::Branch:
-        action_name = QObject::tr("Branch");
-        if (platform=="iptables") action_name = QObject::tr("Chain");
-        if (platform=="pf")       action_name = QObject::tr("Anchor");
-        break;
-
-    case PolicyRule::Accounting:
-        action_name = QObject::tr("Accounting");
-        if (platform=="ipf" || platform=="ipfw") action_name = QObject::tr("Count");
-        break;
-
-    case PolicyRule::Tag:
-        action_name = QObject::tr("Tag");
-        if (platform=="iptables") action_name = QObject::tr("Mark");
-        break;
-
-    case PolicyRule::Pipe:
-        action_name = QObject::tr("Pipe");
-        if (platform=="iptables") action_name = QObject::tr("Queue");
-        break;
-    case PolicyRule::Route:
-        action_name = QObject::tr("Routing");
-        break;
-    default:
-        ;
-    }
-    return action_name;
+        name = Resources::getTargetCapabilityStr(
+            platform, "actions/" + action + "/description");
+    } catch (FWException &ex) { }
+    return name.c_str();
 }
 
 /*
@@ -614,3 +772,213 @@ QString getReadableRuleElementName(const string &rule_element_type_name)
 
     return QString();
 }
+
+QMap<QString,QString> getAllPlatforms(bool filter)
+{
+    QMap<QString,QString> res;
+    map<string,string> platforms = Resources::getPlatforms();
+    map<string,string>::iterator i;
+    for (i=platforms.begin(); i!=platforms.end(); i++)
+    {
+        QString name = i->first.c_str();
+        QString res_status = Resources::platform_res[name.toStdString()]->getResourceStr(
+            "/FWBuilderResources/Target/status/").c_str();
+        QString status = st->getTargetStatus(name, res_status);
+        if (filter && status == "disabled") continue;
+        res[name] = i->second.c_str();
+    }
+    return res;
+}
+
+QMap<QString,QString> getAllOS(bool filter)
+{
+    QMap<QString,QString> res;
+    map<string,string> OSs = Resources::getOS();
+    map<string,string>::iterator i;
+    for (i=OSs.begin(); i!=OSs.end(); i++)
+    {
+        QString name = i->first.c_str();
+        QString res_status = Resources::os_res[name.toStdString()]->getResourceStr(
+            "/FWBuilderResources/Target/status/").c_str();
+        QString status = st->getTargetStatus(name, res_status);
+        if (filter && status == "disabled") continue;
+        res[name] = i->second.c_str();
+    }
+    return res;
+}
+
+QString readPlatform(QComboBox *platform)
+{
+    return platform->itemData(platform->currentIndex()).toString();
+}
+
+QString readHostOS(QComboBox *hostOS)
+{
+    return hostOS->itemData(hostOS->currentIndex()).toString();
+}
+
+/*
+ * Fill combobox widget <platform> with items that exist in resources.
+ * If second argument is not an empty string, make corresponding item current.
+ * If it is an empty string, add an empty item on top to the combo box and make
+ * it current.
+ */
+void setPlatform(QComboBox *platform, const QString &pl)
+{
+    platform->clear();
+    // platforms maps platform name (pix) to readable name (Cisco PIX)
+    QMap<QString,QString> platforms = getAllPlatforms();
+
+    QMap<QString,QString>::iterator i;
+
+    // platform_mapping maps key (<group>.<platform name>) to pair
+    // <group>, <platform name>
+    QMap<QString, QPair<QString, QString> > platform_mapping;
+    QStringList platform_keys;
+
+    for (i=platforms.begin(); i!=platforms.end(); i++)
+    {
+        QString group =
+            Resources::platform_res[i.key().toLatin1().constData()]->
+                getResourceStr("/FWBuilderResources/Target/group").c_str();
+        QString key = group + "." + i.key();
+        platform_mapping[key] = QPair<QString,QString>(group, i.key());
+        platform_keys.push_back(key);
+    }
+
+    qSort(platform_keys);
+
+    QStringList::iterator iter;
+    int ind = 0;
+    int cp = 0;
+
+    if (pl.isEmpty())
+    {
+        platform->addItem("", "");
+        cp++;
+    }
+
+    QString current_group = "";
+    for (iter=platform_keys.begin(); iter!=platform_keys.end(); iter++)
+    {
+        if (fwbdebug) qDebug() << *iter;
+
+        QString group = platform_mapping[*iter].first;
+        QString platform_name = platform_mapping[*iter].second;
+
+        if (platforms.count(platform_name) == 0) continue;
+
+        if (group != current_group)
+        {
+            current_group = group;
+#if (QT_VERSION > 0x040500)
+            platform->insertSeparator(cp);  // QT before 4.4.? does not support separator in QComboBox
+#else
+            platform->addItem("");
+#endif
+            cp++;
+        }
+
+        platform->addItem(platforms[platform_name], platform_name);
+        // note that if pl is "", then no real platform name will
+        // match it and ind will remain 0, which makes the top item in
+        // the combobox current.
+        if ( pl == platform_name ) ind = cp;
+        cp++;
+    }
+    platform->setCurrentIndex( ind );
+}
+
+/*
+ * Fill in "host os" combo box with list of os supported for the given
+ * platform and make current host os item current.
+ *
+ * If platform == "", then use all known host OS but also add
+ *  empty item on top of the combobox and make that item current.
+ * If os == "", make the first OS in the list current.
+ */
+void setHostOS(QComboBox *hostOS, const QString &platform, const QString &os)
+{
+    hostOS->clear();
+
+    QStringList supported_os_list;
+
+    if (!platform.isEmpty())
+    {
+        Resources *platform_res = Resources::platform_res[platform.toLatin1().constData()];
+        if (!platform_res)
+            platform_res = Resources::platform_res["unknown"];
+
+        QString supported_os = platform_res->
+            getResourceStr("/FWBuilderResources/Target/supported_os").c_str();
+
+        if (fwbdebug)
+            qDebug("supported_os %s", supported_os.toLatin1().constData());
+
+        if (supported_os.isEmpty())
+        {
+            // something is broken, we have no supported host OS for
+            // this platform.  Just add os to the combo box and return
+            if (fwbdebug)
+                qDebug("No supported host OS for platform %s",
+                       platform.toLatin1().constData());
+
+            hostOS->addItem(os, os);
+            hostOS->setCurrentIndex(0);
+            return;
+        }
+        supported_os_list = supported_os.split(",");
+
+        int cp = 0;
+        int ind = 0;
+        QMap<QString,QString> OSs = getAllOS();
+        QStringList::iterator os_iter;
+        for (os_iter=supported_os_list.begin();
+             os_iter!=supported_os_list.end(); ++os_iter)
+        {
+            QString os_code = *os_iter;
+            if (OSs.count(os_code) > 0)
+            {
+                hostOS->addItem( OSs[os_code], os_code);
+                if ( os == os_code ) ind = cp;
+                cp++;
+            }
+        }
+        hostOS->setCurrentIndex( ind );
+        return;
+    }
+
+    // platform is empty
+
+    int cp = 0;
+    int ind = 0;
+
+    hostOS->addItem("", "");
+    cp++;
+
+    QMap<QString,QString> OSs = getAllOS();
+    QMap<QString,QString>::iterator i;
+    for (i=OSs.begin(); i!=OSs.end(); i++)
+    {
+        hostOS->addItem( i.value(), i.key() );
+        if ( os == i.key() ) ind = cp;
+        cp++;
+    }
+
+    hostOS->setCurrentIndex( ind );
+}
+
+void _repackStringList(list<string> &list1, list<QStringPair> &list2)
+{
+    list2.clear();
+    foreach(string p, list1)
+    {
+        QString str = QString(p.c_str());
+        QStringList pl = str.split(",");
+        if (pl.size() == 1)
+            list2.push_back(QStringPair(str, str));
+        else
+            list2.push_back(QStringPair(pl[0], pl[1]));
+    }
+}
+
