@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: utils.cpp 748 2009-01-24 07:17:05Z vadim $
+  $Id: utils.cpp 2639 2010-03-01 04:39:27Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -37,11 +37,14 @@
 #include <qnamespace.h>
 #include <QHostInfo>
 #include <qhostaddress.h>
-#include <qapplication.h>
 #include <qpixmapcache.h>
-//Added by qt3to4:
+
 #include <QList>
 #include <QPixmap>
+#include <QApplication>
+#include <QtDebug>
+
+#include "FWBSettings.h"
 
 #include "fwbuilder/dns.h"
 #include "fwbuilder/FWObject.h"
@@ -69,19 +72,12 @@
 #include <algorithm>
 #include <cctype>
 
+#include "FWBTree.h"
+
 using namespace std;
 using namespace libfwbuilder;
 
 
-class mapValEQPredicate {
-    string descr;
-    public:
-    mapValEQPredicate(const string &d) { descr=d; }
-    bool operator()(pair<string,string> _d) { return (descr == _d.second); }
-};
-
-
-
 QAction* addPopupMenuItem(QObject *res,
                      QMenu* menu,
                      const QString &resourceIconPath,
@@ -209,10 +205,13 @@ bool isTreeReadWrite(QWidget *parent, FWObject *obj)
 {
     if (obj->isReadOnly())
     {
-        QMessageBox::warning(parent, "Firewall Builder",
- QObject::tr("Impossible to apply changes because object is located in read-only\npart of the tee or data file was opened read-only"),
- QObject::tr("&Continue"), 0, 0,
- 0, 2 );
+        QMessageBox::warning(
+            parent, "Firewall Builder",
+            QObject::tr("Impossible to apply changes because object is "
+                        "located in read-only\npart of the tree or data "
+                        "file was opened read-only"),
+            QObject::tr("&Continue"), 0, 0,
+            0, 2 );
 
         return false;
     }
@@ -222,19 +221,47 @@ bool isTreeReadWrite(QWidget *parent, FWObject *obj)
 /*
  * compare names as QString objects to catch non-ascii names
  */
-bool validateName(QWidget *parent,
-                  libfwbuilder::FWObject *obj,const QString &newname)
+bool validateName(QWidget *parent, FWObject *obj, const QString &newname)
 {
     FWObject *p = obj->getParent();
     for (FWObject::iterator i=p->begin(); i!=p->end(); ++i)
     {
         FWObject *o1= *i;
-        if (QString(o1->getName().c_str())==newname && o1!=obj)
+        // Another hack: we need to be able to create policy and nat
+        // ruleset objects with name "ftp-proxy/*" for PF. Allow
+        // objects of different type to have the same name.
+
+        if (o1 == obj || o1->getTypeName() != obj->getTypeName()) continue;
+
+        if (QString(o1->getName().c_str()) == newname)
         {
-            QMessageBox::warning(parent, "Firewall Builder",
- QObject::tr("Object with name '%1' already exists, please choose different name.").
-                                 arg(o1->getName().c_str()),
-                                 QObject::tr("&Continue editing"), NULL, NULL, 0, 2 );
+            /*
+             * when we open this warning dialog, the dialog class
+             * loses focus and obj_name lineEdit widget sends signal
+             * "editingfinished" again.  To the user this looks like the
+             * warning dialog popped up twice (in fact two copies of the
+             * same warning dialog appear at the same time, one exactly on
+             * top of another). To avoid this, block signals for the
+             * duration while we show the dialog. Note that documentation
+             * does not mention that QObject::blockSignals() affects not
+             * only the widget but all its children, but it seems to work
+             * that way. Tested with Qt 4.6.1. See #1171
+             */
+
+            // show warning dialog only if app has focus
+            if (QApplication::focusWidget() != NULL)
+            {
+                parent->blockSignals(true);
+            
+                QMessageBox::warning(
+                    parent, "Firewall Builder",
+                    QObject::tr("Object with name '%1' already exists, "
+                                "please choose different name.").
+                    arg(o1->getName().c_str()),
+                    QObject::tr("&Continue"), NULL, NULL, 0, 2 );
+                
+                parent->blockSignals(false);
+            }
             return false;
         }
     }
@@ -252,45 +279,6 @@ QString quoteString(const QString &str)
     return res;
 }
 
-QString getUserName()
-{
-    QString uname;
-
-#ifdef _WIN32
-
-#define INFO_BUFFER_SIZE 32767
-
-TCHAR  infoBuf[INFO_BUFFER_SIZE];
-DWORD  bufCharCount = INFO_BUFFER_SIZE;
-
-  bufCharCount = INFO_BUFFER_SIZE;
-  if( GetUserName( infoBuf, &bufCharCount ) )
-  {
-#ifdef UNICODE
-      uname = QString::fromUtf16((ushort*)infoBuf);
-#else
-      uname = QString::fromLocal8Bit(infoBuf);
-#endif
-  }
-
-  uname = uname.replace(' ','_');
-
-#else
-
-    char *lname = getenv("LOGNAME");
-    if (lname!=NULL)
-        uname = QString(lname);
-    else
-    {
-        struct passwd *pwd = getpwuid(getuid());
-        assert(pwd);
-        uname = QString(pwd->pw_name);
-    }
-#endif
-    return uname;
-}
-
-
 QString getFileDir(const QString &file)
 {
     int sn1  = file.lastIndexOf("/",-1);
@@ -310,77 +298,6 @@ QString getFileDir(const QString &file)
     return dir;
 }
 
-QMap<QString,QString> getAllPlatforms()
-{
-    QMap<QString,QString> res;
-
-    map<string,string> platforms = Resources::getPlatforms();
-    map<string,string>::iterator i;
-    for (i=platforms.begin(); i!=platforms.end(); i++)
-        res[ i->first.c_str() ] = i->second.c_str();
-
-    return res;
-}
-
-QMap<QString,QString> getAllOS()
-{
-    QMap<QString,QString> res;
-
-    map<string,string> OSs = Resources::getOS();
-    map<string,string>::iterator i;
-    for (i=OSs.begin(); i!=OSs.end(); i++)
-        res[ i->first.c_str() ] = i->second.c_str();
-
-    return res;
-}
-
-QString readPlatform(QComboBox *platform)
-{
-    map<string,string> platforms = Resources::getPlatforms();
-    map<string,string>::iterator i1 = std::find_if( platforms.begin(), platforms.end(),
-                         mapValEQPredicate(static_cast<const char*>(platform->currentText().toLatin1())));
-    return (*i1).first.c_str();
-}
-
-void setPlatform(QComboBox *platform,const QString &pl)
-{
-    platform->clear();
-    int cp=0;
-    QMap<QString,QString> platforms = getAllPlatforms();
-    QMap<QString,QString>::iterator i;
-    int ind=0;
-    for (i=platforms.begin(); i!=platforms.end(); i++,cp++)
-    {
-        platform->addItem( i.value() );
-        if ( pl == i.key() ) ind = cp;
-    }
-    platform->setCurrentIndex( ind );
-}
-
-QString readHostOS(QComboBox *hostOS)
-{
-    map<string,string> OSs       = Resources::getOS();
-    map<string,string>::iterator i2 = std::find_if( OSs.begin(), OSs.end(),
-                           mapValEQPredicate(static_cast<const char*>(hostOS->currentText().toLatin1())));
-    return (*i2).first.c_str();
-}
-
-void setHostOS(QComboBox *hostOS,const QString &os)
-{
-    hostOS->clear();
-
-    int cp=0;
-    QMap<QString,QString> OSs = getAllOS();
-    QMap<QString,QString>::iterator i;
-    int ind=0;
-    for (i=OSs.begin(); i!=OSs.end(); i++,cp++)
-    {
-        hostOS->addItem( i.value() );
-        if ( os == i.key() ) ind = cp;
-    }
-    hostOS->setCurrentIndex( ind );
-}
-
 void setDisabledPalette(QWidget *w)
 {
     QPalette    pal=w->palette();
@@ -416,8 +333,8 @@ QString getAddrByName(const QString &name, int af_type)
             return QString(results.front().toString().c_str());
     } catch (FWException &e)
     {
-        qDebug("utils::getAddrByName: Can not convert address to string");
-        qDebug(e.toString().c_str());
+        qDebug() << "utils::getAddrByName: Can not convert address to string";
+        qDebug() << e.toString().c_str();
     }
     return "";
 
@@ -527,3 +444,64 @@ void loadIcon(QPixmap &pm, libfwbuilder::FWObject *obj)
     }
 }
 
+void LoadPixmap(const QString path, QPixmap &where)
+{
+    if ( ! QPixmapCache::find( path, where ) )
+    {
+        where.load( path );
+        if (where.width() == 0)
+            qDebug("pixmap load failed: %s", path.toAscii().constData());
+        QPixmapCache::insert( path, where );
+    }
+}
+
+QPixmap LoadPixmap(const QString path)
+{
+
+    QPixmap p;
+    LoadPixmap(path, p);
+    return p;
+}
+
+QString calculateIconName(const QString &_icn, bool negation)
+{
+    QString icn = ":/Icons/" + _icn + "/icon";
+
+    if (negation)
+    {
+        icn = icn + "-neg";
+    }
+
+    if (FWBSettings::SIZE16X16 == st->getIconsInRulesSize())
+    {
+        return icn+"-tree";
+    }
+
+    return icn;
+}
+
+void doSetObjectIcon(libfwbuilder::FWObject *obj, QPixmap *pm, int icon_size)
+{
+    QString icn_alias;
+    QString icn_sfx;
+
+    switch (icon_size)
+    {
+    case 0: icn_sfx = "icon-tree"; break;
+    case 2: icn_sfx = "icon-big"; break;
+    default: icn_sfx = "icon"; break;
+    }
+
+    if (obj->getRO())
+        icn_alias = ":/Icons/lock";
+    else
+    {
+        if (FWBTree().isStandardFolder(obj))
+            icn_alias = ":/Icons/SystemGroup/" + icn_sfx;
+        else
+            icn_alias = QString(":/Icons/") + obj->getTypeName().c_str() + "/" + icn_sfx;
+    }
+
+    LoadPixmap(icn_alias, *pm);
+}
+
