@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: utils_no_qt.h 408 2008-07-25 03:30:28Z vadim $
+  $Id: utils_no_qt.h 2174 2009-12-17 01:12:32Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -61,6 +61,10 @@ extern void findFirewalls(libfwbuilder::FWObject *o,
                           std::list<libfwbuilder::FWObject*> &fwlist,
                           bool skip_system_libs=true);
 
+extern void findClusters(libfwbuilder::FWObject *o,
+                         std::list<libfwbuilder::FWObject*> &fwlist,
+                         bool skip_system_libs=true);
+
 extern void findHosts(libfwbuilder::FWObject *o,
                       std::list<libfwbuilder::FWObject*> &fwlist,
                       bool skip_system_libs=true);
@@ -78,16 +82,4 @@ extern std::string strip(const std::string &s);
 
 extern std::string getPathToBinary(const std::string &pgm_name);
 
-// a functor to join list<string> into a string with separator sep
-class join : public std::unary_function<std::string, void>
-{
-    std::string *result;
-    std::string  separator;
-public:
-    join(std::string *res, const std::string &s)
-    { result = res; separator = s; }
-    void operator()(std::string &s);
-};
-
-
 #endif
