@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_pf.cpp 1048 2009-06-09 22:36:41Z vadim $
+  $Id: NATCompiler_pf.cpp 2554 2010-02-17 02:43:23Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -42,6 +42,8 @@
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/AddressTable.h"
+#include "fwbuilder/FailoverClusterGroup.h"
+#include "fwbuilder/Cluster.h"
 
 #include <iostream>
 #include <iomanip>
@@ -61,14 +63,12 @@ int NATCompiler_pf::prolog()
 
     if ( n>0 ) 
     {
-	bool found_ext=false;
 	list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
 	for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) 
         {
 	    Interface *iface=dynamic_cast<Interface*>(*i);
 	    assert(iface);
 
-	    if ( iface->isExt() ) found_ext=true;
             if ( iface->isDyn())
             {
                 iface->setBool("use_var_address",true);
@@ -82,11 +82,6 @@ int NATCompiler_pf::prolog()
                     iface->remove(*j);
             }
 	}
-
-	if (!found_ext)
-	    abort(
-                "At least one interface should be marked as external, "
-                "can not configure NAT");
     }
 
 /* pseudo-host with ip address 127.0.0.1  We'll use it for redirection
@@ -100,7 +95,6 @@ int NATCompiler_pf::prolog()
     IPv4::cast(loopback_address)->setAddress(InetAddr::getLoopbackAddr());
 
     dbcopy->add(loopback_address,false);
-    cacheObj(loopback_address);
 
     if (tables)
     {
@@ -115,14 +109,13 @@ int NATCompiler_pf::prolog()
 string NATCompiler_pf::debugPrintRule(libfwbuilder::Rule *r)
 {
     NATRule   *rule=NATRule::cast(r);
-    Interface *rule_iface = getCachedFwInterface(rule->getInterfaceId());
+    FWObject *rule_iface = dbcopy->findInIndex(rule->getInterfaceId());
 
     return NATCompiler::debugPrintRule(rule)+
         " "+string( (rule_iface!=NULL)?rule_iface->getName():"") +
         " (type="+rule->getRuleTypeAsString()+")";
 }
 
-
 bool NATCompiler_pf::NATRuleType::processNext()
 {
     NATRule *rule=getNext(); if (rule==NULL) return false;
@@ -130,13 +123,39 @@ bool NATCompiler_pf::NATRuleType::processNext()
 
     if (rule->getRuleType()!=NATRule::Unknown) return true;
 
+    RuleElementTSrc *tsrcre = rule->getTSrc();
     RuleElementTDst *tdstre = rule->getTDst();
+    RuleElementTSrv *tsrvre = rule->getTSrv();
 
     Service  *osrv=compiler->getFirstOSrv(rule);
                                              
     Address *tsrc = compiler->getFirstTSrc(rule);
     Address *tdst = compiler->getFirstTDst(rule);
-    Service  *tsrv=compiler->getFirstTSrv(rule);
+    Service *tsrv=compiler->getFirstTSrv(rule);
+
+    if (rule->getAction() == NATRule::Branch)
+    {
+	rule->setRuleType(NATRule::NATBranch);
+        if (!tsrcre->isAny() || !tdstre->isAny() || !tsrvre->isAny())
+        {
+            tsrcre->clearChildren();
+            tsrcre->setAnyElement();
+
+            tdstre->clearChildren();
+            tdstre->setAnyElement();
+
+            tsrvre->clearChildren();
+            tsrvre->setAnyElement();
+
+            compiler->warning(
+                    rule,
+                    "Translated Src, Dst and Srv are ignored in the NAT "
+                    "rule with action 'Branch'");
+        }
+        return true;
+    }
+
+
 
     if (tsrc->isAny() && tdst->isAny() && tsrv->isAny())
     {
@@ -208,7 +227,7 @@ bool NATCompiler_pf::NATRuleType::processNext()
 	return true;
     }
 
-    compiler->abort(_("Unsupported translation. Rule: ")+rule->getLabel());
+    compiler->abort(rule, "Unsupported translation.");
    
     return false;
 }
@@ -297,7 +316,6 @@ bool NATCompiler_pf::splitSDNATRule::processNext()
                         compiler->dbcopy->create(tsrv->getTypeName()));
                     match_service->setName(tsrv->getName() + "_dport");
                     compiler->dbcopy->add(match_service);
-                    compiler->cacheObj(match_service); // to keep cache consistent
                     match_service->setDstRangeStart(tu_tsrv->getDstRangeStart());
                     match_service->setDstRangeEnd(tu_tsrv->getDstRangeEnd());
                 }
@@ -323,7 +341,6 @@ bool NATCompiler_pf::splitSDNATRule::processNext()
 bool NATCompiler_pf::VerifyRules::processNext()
 {
     NATRule *rule=getNext(); if (rule==NULL) return false;
-    tmp_queue.push_back(rule);
 
     RuleElementOSrc  *osrc=rule->getOSrc();  assert(osrc);
     RuleElementODst  *odst=rule->getODst();  assert(odst);
@@ -333,72 +350,138 @@ bool NATCompiler_pf::VerifyRules::processNext()
     RuleElementTDst  *tdst=rule->getTDst();  assert(tdst);
     RuleElementTSrv  *tsrv=rule->getTSrv();  assert(tsrv);
 
-//    if (rule->getRuleType()==NATRule::LB)
-//        compiler->abort(_("Load balancing rules are not supported. Rule ")+rule->getLabel());
-
     if (rule->getRuleType()==NATRule::DNAT && odst->size()!=1)
-	compiler->abort(_("There should be no more than one object in original destination in the rule ")+rule->getLabel());
-
-//    if (rule->getRuleType()==NATRule::SNAT && tsrc->size()!=1)
-//	compiler->abort(_("There should be no more than one object in translated source in the rule ")+rule->getLabel());
+    {
+	compiler->abort(
+            rule, 
+            "There should be no more than one object in original destination");
+        return true;
+    }
 
     if (osrv->getNeg())
-        compiler->abort(_("Negation in original service is not supported. Rule ")+rule->getLabel());
+    {
+        compiler->abort(
+            rule, 
+            "Negation in original service is not supported.");
+        return true;
+    }
 
     /* bug #1276083: "Destination NAT rules". this restriction is not
      * true at least as of OpenBSD 3.5
      *
     if (rule->getRuleType()==NATRule::DNAT && osrv->isAny()) 
-	compiler->abort(_("Service must be specified for destination translation rule. Rule ")+rule->getLabel());
+	compiler->abort("Service must be specified for destination translation rule. Rule "+rule->getLabel());
     */
 
     if (rule->getRuleType()==NATRule::DNAT && osrv->isAny() && !tsrv->isAny())
-	compiler->abort(_("Can not translate 'any' into a specific service. Rule ")+rule->getLabel());
+    {
+	compiler->abort(
+            rule, 
+            "Can not translate 'any' into a specific service.");
+        return true;
+    }
 
     if (tsrc->getNeg())
-        compiler->abort(_("Can not use negation in translated source. Rule ")+rule->getLabel());
+    {
+        compiler->abort(
+            rule, 
+            "Can not use negation in translated source.");
+        return true;
+    }
 
     if (tdst->getNeg())
-        compiler->abort(_("Can not use negation in translated destination. Rule ")+rule->getLabel());
+    {
+        compiler->abort(
+            rule, 
+            "Can not use negation in translated destination.");
+        return true;
+    }
 
     if (tsrv->getNeg())
-        compiler->abort(_("Can not use negation in translated service. Rule ")+rule->getLabel());
+    {
+        compiler->abort(
+            rule, 
+            "Can not use negation in translated service.");
+        return true;
+    }
 
     if (tsrv->size()!=1) 
-	compiler->abort(_("Translated service should be 'Original' or should contain single object. Rule: ")+rule->getLabel());
+    {
+	compiler->abort(
+            rule, 
+            "Translated service should be 'Original' or should contain single object.");
+        return true;
+    }
 
     FWObject *o=tsrv->front();
     if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     if ( Group::cast(o)!=NULL)
-	compiler->abort(_("Can not use group in translated service. Rule ")+rule->getLabel());
+    {
+	compiler->abort(
+            rule, 
+            "Can not use group in translated service.");
+        return true;
+    }
 
 #if 0
     if (rule->getRuleType()==NATRule::SNAT ) 
     {
         Address* o1=compiler->getFirstTSrc(rule);
         if ( Network::cast(o1)!=NULL || AddressRange::cast(o1)!=NULL )
-            compiler->abort(_("Can not use network or address range object in translated source. Rule ")+rule->getLabel());
+            compiler->abort("Can not use network or address range object in translated source. Rule "+rule->getLabel());
     }
 #endif
 
     if (rule->getRuleType()==NATRule::SNAT ) 
     {
         if (tsrc->isAny())
-            compiler->abort("Source translation rule needs an address in Translated Source. Rule " + rule->getLabel());
+        {
+            compiler->abort(rule, 
+                            "Source translation rule needs an address in "
+                            "Translated Source.");
+            return true;
+        }
+
+        FWObject *o = FWReference::getObject(tsrc->front());
+        if (Interface::isA(o) && Interface::cast(o)->isUnnumbered())
+        {
+            compiler->abort(rule,
+                            "Can not use unnumbered interface in "
+                            "Translated Source of a Source translation rule.");
+            return true;
+        }
+
     }
 
     if (rule->getRuleType()==NATRule::DNAT || rule->getRuleType()==NATRule::Redirect ) 
     {
         if (tdst->isAny())
-            compiler->abort("Destination translation rule needs an address in Translated Destination. Rule " + rule->getLabel());
+        {
+            compiler->abort(
+                rule, 
+                "Destination translation rule needs an address in "
+                "Translated Destination.");
+            return true;
+        }
 
         if ( tdst->size()!=1)
-            compiler->abort(_("There should be no more than one object in translated destination in the rule ")+rule->getLabel());
+        {
+            compiler->abort(
+                rule, 
+                "There should be no more than one object in translated destination");
+            return true;
+        }
 
         Address* o1=compiler->getFirstTDst(rule);
         if ( Network::cast(o1)!=NULL || AddressRange::cast(o1)!=NULL )
-            compiler->abort(_("Can not use network or address range object in translated destination. Rule ")+rule->getLabel());
+        {
+            compiler->abort(
+                rule, 
+                "Can not use network or address range object in translated destination.");
+            return true;
+        }
+
     }
 
 
@@ -408,7 +491,13 @@ bool NATCompiler_pf::VerifyRules::processNext()
         Network *a2=Network::cast(compiler->getFirstTSrc(rule));
         if ( a1==NULL || a2==NULL ||
              a1->getNetmaskPtr()->getLength()!=a2->getNetmaskPtr()->getLength() )
-            compiler->abort(_("Original and translated source should both be networks of the same size . Rule ")+rule->getLabel());
+        {
+            compiler->abort(
+                rule, 
+                "Original and translated source should both be networks of the same size.");
+            return true;
+        }
+
     }
 
     if (rule->getRuleType()==NATRule::DNetnat && !tsrc->isAny() ) 
@@ -417,9 +506,39 @@ bool NATCompiler_pf::VerifyRules::processNext()
         Network *a2=Network::cast(compiler->getFirstTDst(rule));
         if ( a1==NULL || a2==NULL ||
              a1->getNetmaskPtr()->getLength()!=a2->getNetmaskPtr()->getLength() )
-            compiler->abort(_("Original and translated destination should both be networks of the same size . Rule ")+rule->getLabel());
+        {
+            compiler->abort(
+                rule, 
+                "Original and translated destination should both be networks of the same size.");
+            return true;
+        }
+
     }
 
+    if (rule->getRuleType()==NATRule::NATBranch ) 
+    {
+        RuleSet *branch = rule->getBranch();
+        if (branch == NULL)
+        {
+            compiler->abort(
+                rule, 
+                "Action 'Branch' needs NAT rule set to point to");
+            return true;
+        } else
+        {
+            if (!NAT::isA(branch))
+            {
+                compiler->abort(
+                    rule, 
+                    "Action 'Branch' must point to a NAT rule set "
+                    "(points to " + branch->getTypeName() + ")");
+                return true;
+            }
+
+        }
+    }
+
+    tmp_queue.push_back(rule);
     return true;
 }
 
@@ -428,9 +547,10 @@ bool NATCompiler_pf::splitOnOSrv::processNext()
     NATRule *rule=getNext(); if (rule==NULL) return false;
 
     RuleElementOSrv  *osrv=rule->getOSrv();  assert(osrv);
-    if (osrv->size()!=1) {
-
-	for(list<FWObject*>::iterator i=osrv->begin(); i!=osrv->end(); ++i) {
+    if (osrv->size()!=1)
+    {
+	for(list<FWObject*>::iterator i=osrv->begin(); i!=osrv->end(); ++i)
+        {
 	    FWObject *o= *i;
 //	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
@@ -544,49 +664,43 @@ bool NATCompiler_pf::splitForTSrc::processNext()
 }
 
 
+bool NATCompiler_pf::assignInterfaceToNATRule(Rule *rule, Address *addr)
+{
+    if (Interface::isA(addr) || IPv4::isA(addr))
+    {
+        FWObject *p = addr;
+        while ( p && ! Interface::isA(p) ) p = p->getParent();
+        Interface *intf = Interface::cast(p);
+
+        if (intf && intf->isFailoverInterface())
+        {
+            FailoverClusterGroup *fg = FailoverClusterGroup::cast(
+                intf->getFirstByType(FailoverClusterGroup::TYPENAME));
+            if (fg)
+                intf = fg->getInterfaceForMemberFirewall(fw);
+        }
+
+        if (intf && intf->isChildOf(fw))
+        {
+            rule->setInterfaceId(intf->getId());
+            rule->setInterfaceStr(intf->getName());
+            return true;
+        }
+    }
+    return false;
+}
+
 bool NATCompiler_pf::AssignInterface::processNext()
 {
+    NATCompiler_pf *pf_comp=dynamic_cast<NATCompiler_pf*>(compiler);
     NATRule *rule=getNext(); if (rule==NULL) return false;
 
-    if (regular_interfaces.empty())
-    {
-        int n=0;
-        list<FWObject*> l2=compiler->fw->getByType(Interface::TYPENAME);
-        for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) 
-        {
-            Interface *iface=Interface::cast(*i);
-            assert(iface);
-
-            if (iface->isLoopback() ||
-                iface->isUnnumbered() ||
-                iface->isBridgePort()) continue;
-
-            if (n) regular_interfaces+=",";
-            regular_interfaces+= iface->getName();
-            n++;
-        }
-        if (n>1) regular_interfaces="{ "+regular_interfaces+" }";
-    }
-
     switch ( rule->getRuleType() )
     {
     case NATRule::SNAT:
     {
-	Address *a=compiler->getFirstTSrc(rule);
-#if 0
-        Interface *iface = compiler->findInterfaceFor(a,compiler->fw);
-        if (iface!=NULL)
+        if (pf_comp->assignInterfaceToNATRule(rule, compiler->getFirstTSrc(rule)))
         {
-            rule->setInterfaceId( iface->getId() );
-            tmp_queue.push_back(rule);
-            return true;
-        }
-#endif
-        if ( (Interface::isA(a) || IPv4::isA(a)) && a->isChildOf(compiler->fw))
-        {
-            FWObject *p=a;
-            while ( ! Interface::isA(p) ) p=p->getParent();
-            rule->setInterfaceId( p->getId() );
             tmp_queue.push_back(rule);
             return true;
         }
@@ -600,13 +714,8 @@ bool NATCompiler_pf::AssignInterface::processNext()
 
     case NATRule::DNAT:
     {
-	Address *a=compiler->getFirstODst(rule);
-
-        if ( (Interface::isA(a) || IPv4::isA(a)) && a->isChildOf(compiler->fw))
+        if (pf_comp->assignInterfaceToNATRule(rule, compiler->getFirstODst(rule)))
         {
-            FWObject *p=a;
-            while ( ! Interface::isA(p) ) p=p->getParent();
-            rule->setInterfaceId( p->getId() );
             tmp_queue.push_back(rule);
             return true;
         }
@@ -658,7 +767,6 @@ bool NATCompiler_pf::ReplaceFirewallObjectsODst::processNext()
  */
             if (! interface_->isLoopback() ) cl.push_back(interface_);
 
-//	    if (interface_->isExt()) cl.push_back(interface_);
 	}
 	if ( ! cl.empty() ) {
 	    rel->clearChildren();
@@ -723,9 +831,9 @@ bool NATCompiler_pf::ReplaceFirewallObjectsTSrc::processNext()
                 {
                     char errmsg[1024];
                     sprintf(errmsg,
-_("Could not find suitable interface for the NAT rule %s. Perhaps all interfaces are unnumbered?"), 
+"Could not find suitable interface for the NAT rule %s. Perhaps all interfaces are unnumbered?", 
                             rule->getLabel().c_str() );
-                            compiler->abort(errmsg);
+                    compiler->abort(rule, errmsg);
                 }
             }
 	}
@@ -773,9 +881,9 @@ bool NATCompiler_pf::ReplaceObjectsTDst::processNext()
             if (loopback_address==NULL)
             {
                 char errstr[1024];
-                sprintf(errstr, _("Can not configure redirection NAT rule %s because loopback interface is missing.") ,
+                sprintf(errstr, "Can not configure redirection NAT rule %s because loopback interface is missing." ,
                         rule->getLabel().c_str() );
-                compiler->abort(errstr);
+                compiler->abort(rule, errstr);
             }
 
             rel->clearChildren();
@@ -888,9 +996,10 @@ bool NATCompiler_pf::processMultiAddressObjectsInRE::processNext()
             {
                 if (re->size()>1 && neg)
                 {
-                    string err = "AddressTable object can not be used with negation in combination with other objects in the same rule element. Rule ";
-                    err += rule->getLabel();
-                    compiler->abort(err);
+                    string err = "AddressTable object can not be used with "
+                        "negation in combination with other objects "
+                        "in the same rule element.";
+                    compiler->abort(rule, err);
                 }
                 o->setBool("pf_table",true);
                 string tblname = o->getName();
@@ -903,8 +1012,8 @@ bool NATCompiler_pf::processMultiAddressObjectsInRE::processNext()
     {
         string err;
         err = "Can not process MultiAddress object in rule " +
-            rule->getLabel() + " : " + ex.toString();
-        compiler->abort( err );
+            rule->getLabel() + ". Error: " + ex.toString();
+        compiler->abort(rule,  err);
     }
 
     if (!cl.empty())
@@ -944,20 +1053,42 @@ void NATCompiler_pf::checkForDynamicInterfacesOfOtherObjects::findDynamicInterfa
     list<FWObject*> cl;
     for (list<FWObject*>::iterator i1=re->begin(); i1!=re->end(); ++i1) 
     {
-        FWObject *o   = *i1;
+        FWObject *o = *i1;
         FWObject *obj = o;
         if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
-        Interface  *ifs   =Interface::cast( obj );
+        Interface  *ifs = Interface::cast(obj);
 
-        if (ifs!=NULL && ifs->isDyn() && ! ifs->isChildOf(compiler->fw))        
+        if (ifs && Cluster::isA(ifs->getParent()))
+        {
+            FailoverClusterGroup *failover_group =
+                FailoverClusterGroup::cast(
+                    ifs->getFirstByType(FailoverClusterGroup::TYPENAME));
+            if (failover_group)
+            {
+                for (FWObjectTypedChildIterator it =
+                         failover_group->findByType(FWObjectReference::TYPENAME);
+                     it != it.end(); ++it)
+                {
+                    Interface *member_iface = Interface::cast(FWObjectReference::getObject(*it));
+                    assert(member_iface);
+                    if (member_iface->isChildOf(compiler->fw))
+                    {
+                        ifs = member_iface;
+                        break;
+                    }
+                }
+            }
+        }
+
+        if (ifs && ifs->isDyn() && ! ifs->isChildOf(compiler->fw))        
         {
             char errstr[2048];
-            sprintf(errstr,_("Can not build rule using dynamic interface '%s' of the object '%s' because its address in unknown. Rule %s"),
+            sprintf(errstr,
+                    "Can not build rule using dynamic interface '%s' of the object '%s' because its address is unknown.",
                     ifs->getName().c_str(), 
-                    ifs->getParent()->getName().c_str(),
-                    rule->getLabel().c_str() );
+                    ifs->getParent()->getName().c_str());
 
-            compiler->abort(errstr);
+            compiler->abort(rule, errstr);
         }
     }
 }
@@ -1004,10 +1135,10 @@ void NATCompiler_pf::compile()
 {
     bool manage_virtual_addr=fwopt->getBool("manage_virtual_addr");
 
-    cout << " Compiling " << fw->getName();
-    if (!getRuleSetName().empty())  cout << " ruleset " << getRuleSetName();
-    if (ipv6) cout << ", IPv6";
-    cout <<  endl << flush;
+    string banner = " Compiling NAT rules for " + fw->getName();
+    if (!getRuleSetName().empty())  banner += " ruleset " + getRuleSetName();
+    if (ipv6) banner += ", IPv6";
+    info(banner);
 
     try {
 
@@ -1016,6 +1147,8 @@ void NATCompiler_pf::compile()
         add( new Begin());
         add( new printTotalNumberOfRules() );
 
+        add( new singleRuleFilter());
+    
         add( new recursiveGroupsInOSrc("check for recursive groups in OSRC") );
         add( new recursiveGroupsInODst("check for recursive groups in ODST") );
         add( new recursiveGroupsInOSrv("check for recursive groups in OSRV") );
@@ -1077,6 +1210,13 @@ void NATCompiler_pf::compile()
 
         add( new ExpandMultipleAddresses( "expand multiple addresses" ) );
 
+        // we might get empty RE after expanding multiple addresses,
+        // for example when unnumbered interface is used in TSRC. Note
+        // that VerifyRules should not allow this, but we may still
+        // get here in the test mode. Calling dropRuleWithEmptyRE works
+        // as a fail-safe and prevents crash.
+        add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
+
 	if ( manage_virtual_addr )
             add( new addVirtualAddress("add virtual addresses for NAT rules"));
 
@@ -1090,6 +1230,9 @@ void NATCompiler_pf::compile()
         add( new AssignInterface( "assign rules to interfaces" ) );
         add( new convertInterfaceIdToStr("prepare interface assignments") );
 
+        add( new checkForObjectsWithErrors(
+                 "check if we have objects with errors in rule elements"));
+
         add( new createTables("create tables"));
 //        add( new PrintTables(       "print tables"     ) );
 
@@ -1099,7 +1242,8 @@ void NATCompiler_pf::compile()
         runRuleProcessors();
 
 
-    } catch (FWException &ex) {
+    } catch (FWException &ex)
+    {
 	error(ex.toString());
 	exit(1);
     }
