@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipfw.cpp 709 2008-12-25 20:38:53Z vadim $
+  $Id: PolicyCompiler_ipfw.cpp 2353 2010-01-16 00:26:01Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -55,17 +55,14 @@ int PolicyCompiler_ipfw::prolog()
     anytcp=dbcopy->createTCPService();
     anytcp->setId(FWObjectDatabase::generateUniqueId()); // ANY_TCP_OBJ_ID);
     dbcopy->add(anytcp,false);
-    cacheObj(anytcp); // to keep cache consistent
 
     anyudp=dbcopy->createUDPService();
     anyudp->setId(FWObjectDatabase::generateUniqueId()); //ANY_UDP_OBJ_ID);
     dbcopy->add(anyudp,false);
-    cacheObj(anyudp); // to keep cache consistent
 
     anyicmp=dbcopy->createICMPService();
     anyicmp->setId(FWObjectDatabase::generateUniqueId()); //ANY_ICMP_OBJ_ID);
     dbcopy->add(anyicmp,false);
-    cacheObj(anyicmp); // to keep cache consistent
 
 
     return n;
@@ -237,8 +234,9 @@ bool PolicyCompiler_ipfw::doSrvNegation::processNext()
 
     RuleElementSrv *srv=rule->getSrv();
 
-    if (srv->getNeg()) {
-	throw FWException(_("Negation in Srv is not implemented. Rule: ")+rule->getLabel());
+    if (srv->getNeg())
+    {
+	compiler->abort(rule, "Negation in Srv is not implemented");
 	return false;
     }
     tmp_queue.push_back(rule);
@@ -466,16 +464,26 @@ bool PolicyCompiler_ipfw::eliminateDuplicateRules::processNext()
     PolicyCompiler *pcomp=dynamic_cast<PolicyCompiler*>(compiler);
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
+    // Note that if rule has "any" in Interface column, it is
+    // implemented as reference to the AnyNetwork object. In this case
+    // Compiler::getFirstItf() returns NULL.
+    Interface *intf_rule = compiler->getFirstItf(rule);
+    int intf_id_rule = (intf_rule) ? intf_rule->getId() : -1;
+
     if ( ! rule->getBool("skip_check_for_duplicates"))
     {
         for (deque<PolicyRule*>::iterator i=rules_seen_so_far.begin(); i!=rules_seen_so_far.end(); ++i)
         {
             PolicyRule *r=(*i);
             if ( r->getBool("skip_check_for_duplicates") ) continue;
-            if (r->getInterfaceId()==rule->getInterfaceId() &&
+
+            Interface *intf_r = compiler->getFirstItf(r);
+            int intf_id_r = (intf_r) ? intf_r->getId() : -1;
+
+            if (intf_id_r==intf_id_rule &&
                 r->getAction()==rule->getAction()   &&
                 r->getLogging()==rule->getLogging() &&
-                pcomp->cmpRules(*r,*rule) ) 
+                pcomp->cmpRules(*r,*rule) )
             {
 //                cout << "---------------------------------------" << endl;
 //                cout << pcomp->debugPrintRule(r) << endl;
@@ -504,7 +512,10 @@ bool PolicyCompiler_ipfw::processMultiAddressObjectsInRE::processNext()
         if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
         if (atrt!=NULL && atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
-            compiler->abort("Run-time AddressTable objects are not supported. Rule " + rule->getLabel());
+            compiler->abort(
+                
+                    rule, 
+                    "Run-time AddressTable objects are not supported.");
     }
 
     tmp_queue.push_back(rule);
@@ -514,10 +525,10 @@ bool PolicyCompiler_ipfw::processMultiAddressObjectsInRE::processNext()
 
 void PolicyCompiler_ipfw::compile()
 {
-    cout << " Compiling " << fw->getName();
-    if (!getRuleSetName().empty())  cout << " ruleset " << getRuleSetName();
-    if (ipv6) cout << ", IPv6";
-    cout <<  endl << flush;
+    string banner = " Compiling " + fw->getName();
+    if (!getRuleSetName().empty())  banner += " ruleset " + getRuleSetName();
+    if (ipv6) banner += ", IPv6";
+    info(banner);
 
     try {
 
@@ -526,7 +537,8 @@ void PolicyCompiler_ipfw::compile()
 	addDefaultPolicyRule();
         bool check_for_recursive_groups=true;
 
-        if ( fw->getOptionsObject()->getBool("check_shading")) 
+        if ( fw->getOptionsObject()->getBool("check_shading") &&
+             ! inSingleRuleCompileMode())
         {
             add( new Begin("Detecting rule shadowing"));
             add( new printTotalNumberOfRules());
@@ -553,12 +565,16 @@ void PolicyCompiler_ipfw::compile()
             add( new swapMultiAddressObjectsInDst(
                      " swap MultiAddress -> MultiAddressRunTime in Dst") );
 
-            add( new ExpandMultipleAddressesInSRC(
+            add( new ExpandMultipleAddressesInSrc(
                      "expand objects with multiple addresses in SRC"));
-            add( new ExpandMultipleAddressesInDST(
+            add( new ExpandMultipleAddressesInDst(
                      "expand objects with multiple addresses in DST"));
             add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
             add( new ConvertToAtomic("convert to atomic rules"));
+
+            add( new checkForObjectsWithErrors(
+                     "check if we have objects with errors in rule elements"));
+
             add( new DetectShadowing("Detect shadowing"));
             add( new simplePrintProgress());
 
@@ -570,6 +586,8 @@ void PolicyCompiler_ipfw::compile()
         add( new Begin());
         add( new printTotalNumberOfRules());
 
+        add( new singleRuleFilter());
+
         if (check_for_recursive_groups)
         {
             add( new recursiveGroupsInSrc("check for recursive grps in SRC"));
@@ -618,6 +636,7 @@ void PolicyCompiler_ipfw::compile()
                  "check for a special cases with dynamic interface"));
         add( new addressRanges("expand address range objects"));
         add( new splitServices("split rules with different protocols"));
+        add( new splitIpOptions("split rules with multiple IPService objects with options"));
         add( new separateTCPWithFlags("separate TCP services with flags"));
         add( new separateSrcPort("split on TCP and UDP with source ports"));
         add( new separatePortRanges("split services with port ranges"));
@@ -632,13 +651,18 @@ void PolicyCompiler_ipfw::compile()
 
 	add( new calculateNum("calculate rule numbers "));
         add( new convertInterfaceIdToStr("prepare interface assignments"));
+
+        add( new checkForObjectsWithErrors(
+                 "check if we have objects with errors in rule elements"));
+
         add( new PrintRule("generate ipf code"));
         add( new simplePrintProgress());
 
         runRuleProcessors();
 
 
-    } catch (FWException &ex) {
+    } catch (FWException &ex)
+    {
 	error(ex.toString());
 	exit(1);
     }
@@ -647,16 +671,38 @@ void PolicyCompiler_ipfw::compile()
 string PolicyCompiler_ipfw::debugPrintRule(Rule *r)
 {
     PolicyRule *rule=PolicyRule::cast(r);
-    string s= PolicyCompiler::debugPrintRule(rule);
+    ostringstream s;
 
-    int  iface = rule->getInterfaceId();
-    if (iface > -1)
+    s << PolicyCompiler::debugPrintRule(rule);
+
+    RuleElementItf *intf_re = rule->getItf();
+    string rule_interfaces;
+    int intf_count = 0;
+    for (FWObject::iterator it=intf_re->begin(); it!=intf_re->end(); ++it)
     {
-	Interface *rule_iface = getCachedFwInterface( iface );
-        s += " intf: "+rule_iface->getName();
+        FWObject *o   = *it;
+        if (FWReference::cast(o)!=NULL) o = FWReference::cast(o)->getPointer();
+        rule_interfaces += " " + o->getName();
+        intf_count++;
     }
+    if (intf_count > 0)
+    {
+        s << " intf: ";
+        if (intf_count > 1) s << "{ ";
+        s << rule_interfaces;
+        if (intf_count > 1) s << " }";
+    }
+    else
+        s << " intf: ?";
 
-    return s;
+    // int  iface = rule->getInterfaceId();
+    // if (iface > -1)
+    // {
+    //     FWObject *rule_iface = dbcopy->findInIndex( iface );
+    //     s += " intf: "+rule_iface->getName();
+    // }
+
+    return s.str();
 }
 
 
