@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf.cpp 1359 2009-08-27 17:10:20Z vadim $
+  $Id: PolicyCompiler_pf.cpp 2438 2010-01-26 03:45:14Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -40,6 +40,8 @@
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/AddressTable.h"
+#include "fwbuilder/FailoverClusterGroup.h"
+#include "fwbuilder/StateSyncClusterGroup.h"
 
 #include <algorithm>
 #include <functional>
@@ -57,7 +59,7 @@ string PolicyCompiler_pf::myPlatformName() { return "pf"; }
 int PolicyCompiler_pf::prolog()
 {
     if (fw->getStr("platform")!=myPlatformName() ) 
-	abort(_("Unsupported platform ") + fw->getStr("platform") );
+	abort("Unsupported platform " + fw->getStr("platform") );
 
     list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
     for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) 
@@ -72,9 +74,9 @@ int PolicyCompiler_pf::prolog()
             {
                 char errstr[256];
                 sprintf(errstr,
-                        _("Dynamic interface %s should not have an IP address object attached to it. This IP address object will be ignored."),
+                        "Dynamic interface %s should not have an IP address object attached to it. This IP address object will be ignored.",
                         iface->getName().c_str() );
-                warning( errstr );
+                warning(errstr );
                 for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) 
                     iface->remove(*j);
             }
@@ -191,9 +193,10 @@ bool PolicyCompiler_pf::processMultiAddressObjectsInRE::processNext()
             {
                 if (re->size()>1 && neg)
                 {
-                    string err = "AddressTable object can not be used with negation in combination with other objects in the same rule element. Rule ";
-                    err += rule->getLabel();
-                    compiler->abort(err);
+                    compiler->abort(rule,
+                                    "AddressTable object can not be used "
+                                    "with negation in combination with "
+                                    "other objects in the same rule element.");
                 }
                 string tblname = o->getName();
                 string tblID = tblname + "_addressTableObject";
@@ -205,9 +208,8 @@ bool PolicyCompiler_pf::processMultiAddressObjectsInRE::processNext()
     } catch(FWException &ex)  // TableFactory::registerTable throws exception
     {
         string err;
-        err = "Can not process MultiAddress object in rule " +
-            rule->getLabel() + " : " + ex.toString();
-        compiler->abort( err );
+        err = "Can not process MultiAddress object: " + ex.toString();
+        compiler->abort(rule, err);
     }
 
     if (!maddr_runtime.empty())
@@ -259,7 +261,8 @@ bool PolicyCompiler_pf::splitIfFirewallInSrc::processNext()
 	FWObject *obj = NULL;
 //	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
 	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
-	if (obj==NULL)  throw FWException(_("Broken Src object in rule: ")+rule->getLabel());
+	if (obj==NULL)
+            compiler->abort(rule, "Broken Src object");
 
 	if (obj->getId()==compiler->getFwId()) {
 	    fw_in_src=o;   // can not remove right now because remove invalidates iterator
@@ -304,7 +307,8 @@ bool PolicyCompiler_pf::splitIfFirewallInDst::processNext()
 	FWObject *obj = NULL;
 //	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
 	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
-	if (obj==NULL) throw FWException(_("Broken Dst in rule: ")+rule->getLabel());
+	if (obj==NULL)
+            compiler->abort(rule, "Broken Dst");
 
 	if (obj->getId()==compiler->getFwId()) {
 	    fw_in_dst=o;   // can not remove right now because remove invalidates iterator
@@ -347,7 +351,7 @@ bool PolicyCompiler_pf::fillDirection::processNext()
  * as long as it misses interface - we need to determine direction
  * again anyway.
  */
-    if (rule->getDirection() == PolicyRule::Undefined)   // || rule->getInterfaceId()==-1 )
+    if (rule->getDirection() == PolicyRule::Undefined)
         rule->setDirection( PolicyRule::Both );
 
 /*
@@ -369,7 +373,7 @@ bool PolicyCompiler_pf::fillDirection::processNext()
         //int fwid = compiler->getFwId();
 
         if (src==NULL || dst==NULL)
-            compiler->abort("Broken src or dst in rule "+rule->getLabel());
+            compiler->abort(rule, "Broken src or dst");
 
         if (!src->isAny() && !dst->isAny() &&
             compiler->complexMatch(compiler->fw, src) &&
@@ -378,12 +382,14 @@ bool PolicyCompiler_pf::fillDirection::processNext()
         if (!src->isAny() && compiler->complexMatch(compiler->fw, src))
         {
             rule->setDirection( PolicyRule::Outbound );
-            compiler->warning("Changing rule direction due to self reference in rule "+rule->getLabel());
+            compiler->warning(
+                                  rule, "Changing rule direction due to self reference");
         }
         if (!dst->isAny() && compiler->complexMatch(compiler->fw, dst))
         {
             rule->setDirection( PolicyRule::Inbound );
-            compiler->warning("Changing rule direction due to self reference in rule "+rule->getLabel());
+            compiler->warning(
+                                  rule, "Changing rule direction due to self reference");
         }
     }
     return true;
@@ -395,10 +401,10 @@ bool PolicyCompiler_pf::fillDirection::processNext()
  */
 void PolicyCompiler_pf::addDefaultPolicyRule()
 {
-    if (getSourceRuleSet()->isTop())
+    if (getSourceRuleSet()->isTop() && !inSingleRuleCompileMode())
     {
-        if ( getCachedFwOpt()->getBool("mgmt_ssh") &&
-             !getCachedFwOpt()->getStr("mgmt_addr").empty() )
+        if (getCachedFwOpt()->getBool("mgmt_ssh") &&
+             !getCachedFwOpt()->getStr("mgmt_addr").empty())
         {
             PolicyRule *r;
             TCPService *ssh = dbcopy->createTCPService();
@@ -407,7 +413,6 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
 
             ssh->setName("mgmt_ssh");
             dbcopy->add(ssh,false);
-            cacheObj(ssh); // to keep cache consistent
 
             string mgmt_addr = getCachedFwOpt()->getStr("mgmt_addr");
             InetAddr  addr;
@@ -435,19 +440,18 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
             {
                 char errstr[256];
                 sprintf(errstr,
-                        _("Invalid address for the backup ssh access: '%s'"),
-                        mgmt_addr.c_str() );
-                abort( errstr );
+                        "Invalid address for the backup ssh access: '%s'",
+                        mgmt_addr.c_str());
+                abort(errstr);
             }
 
             Network *mgmt_workstation = dbcopy->createNetwork();
             mgmt_workstation->setName("mgmt_addr");
-            mgmt_workstation->setAddress( addr );
-            mgmt_workstation->setNetmask( netmask );
+            mgmt_workstation->setAddress(addr);
+            mgmt_workstation->setNetmask(netmask);
 //        IPv4 *mgmt_workstation = IPv4::cast(dbcopy->create(IPv4::TYPENAME));
-//        mgmt_workstation->setAddress( getCachedFwOpt()->getStr("mgmt_addr") );
+//        mgmt_workstation->setAddress(getCachedFwOpt()->getStr("mgmt_addr"));
             dbcopy->add(mgmt_workstation,false);
-            cacheObj(mgmt_workstation); // to keep cache consistent
 
 
             r= dbcopy->createPolicyRule();
@@ -477,11 +481,14 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
             combined_ruleset->push_front(r);
         }
 
+        insertCarpRule();
+        insertPfsyncRule();
+
         PolicyRule *r = dbcopy->createPolicyRule();
         FWOptions *ruleopt;
         temp_ruleset->add(r);
         r->setAction(PolicyRule::Deny);
-        r->setLogging( getCachedFwOpt()->getBool("fallback_log") );
+        r->setLogging(getCachedFwOpt()->getBool("fallback_log"));
         r->setDirection(PolicyRule::Both);
         r->setPosition(10000);
         r->setComment("   fallback rule ");
@@ -501,17 +508,16 @@ bool PolicyCompiler_pf::SpecialServices::processNext()
 
     RuleElementSrv *srv=rule->getSrv();
 
-    for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++) {
+    for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++)
+    {
 	FWObject *o= *i;
 	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	Service *s=Service::cast( o );
 	assert(s);
 
-	if (IPService::cast(s)!=NULL  && rule->getAction()==PolicyRule::Accept) {
-	    if (s->getBool("rr")        ||
-		s->getBool("ssrr")      ||
-		s->getBool("ts") )
-		rule->setBool("allow_opts",true);
+	if (IPService::cast(s)!=NULL  && rule->getAction()==PolicyRule::Accept)
+        {
+            rule->setBool("allow_opts", IPService::constcast(s)->hasIpOptions());
 	}
     }
     return true;
@@ -555,7 +561,9 @@ bool PolicyCompiler_pf::ProcessScrubOption::processNext()
 	    ruleopt->setBool("scrub",false);
 	    tmp_queue.push_back(rule);
 
-	    throw FWException(_("Rule option 'scrub' is supported only for rules with action 'Accept'. Rule: ")+rule->getLabel());
+	    compiler->abort(rule,
+                            "Rule option 'scrub' is supported only for rules "
+                            "with action 'Accept'");
 
 	    return true;
 	}
@@ -693,8 +701,10 @@ bool PolicyCompiler_pf::doSrvNegation::processNext()
 
     RuleElementSrv *srv=rule->getSrv();
 
-    if (srv->getNeg()) {
-	throw FWException(_("Negation in Srv is not implemented. Rule: ")+rule->getLabel());
+    if (srv->getNeg())
+    {
+	compiler->abort(rule,
+                        "Negation in Srv is not implemented");
 	return true;
     }
     tmp_queue.push_back(rule);
@@ -713,6 +723,7 @@ bool PolicyCompiler_pf::addLoopbackForRedirect::processNext()
 
     if (pf_comp->redirect_rules_info==NULL)
         compiler->abort(
+            rule, 
             "addLoopbackForRedirect needs a valid pointer to "
             "the list<NATCompiler_pf::redirectRuleInfo> object");
 
@@ -781,12 +792,12 @@ void PolicyCompiler_pf::checkForDynamicInterfacesOfOtherObjects::findDynamicInte
             ! ifs->getParent()->getBool("pf_table") )
         {
             char errstr[2048];
-            sprintf(errstr,_("Can not build rule using dynamic interface '%s' of the object '%s' because its address in unknown. Rule %s"),
+            sprintf(errstr,"Can not build rule using dynamic interface '%s' of the object '%s' because its address in unknown. Rule %s",
                     ifs->getName().c_str(), 
                     ifs->getParent()->getName().c_str(),
                     rule->getLabel().c_str() );
 
-            throw FWException(errstr);
+            compiler->abort(rule, errstr);
         }
     }
 }
@@ -944,7 +955,7 @@ bool PolicyCompiler_pf::separateServiceObject::processNext()
 
 bool PolicyCompiler_pf::separateSrcPort::condition(const Service *srv)
 {
-    if ( TCPService::isA(srv) || UDPService::isA(srv) )
+    if ( TCPService::isA(srv) || UDPService::isA(srv))
     {
         int srs = TCPUDPService::constcast(srv)->getSrcRangeStart();
         int sre = TCPUDPService::constcast(srv)->getSrcRangeEnd();
@@ -969,10 +980,10 @@ bool PolicyCompiler_pf::separateTOS::condition(const Service *srv)
 
 void PolicyCompiler_pf::compile()
 {
-    cout << " Compiling " << fw->getName();
-    if (!getRuleSetName().empty())  cout << " ruleset " << getRuleSetName();
-    if (ipv6) cout << ", IPv6";
-    cout <<  endl << flush;
+    string banner = " Compiling " + fw->getName();
+    if (!getRuleSetName().empty())  banner += " ruleset " + getRuleSetName();
+    if (ipv6) banner += ", IPv6";
+    info(banner);
 
     try 
     {
@@ -981,147 +992,159 @@ void PolicyCompiler_pf::compile()
 	addDefaultPolicyRule();
         bool check_for_recursive_groups=true;
 
-        if ( fw->getOptionsObject()->getBool ("check_shading") ) 
+        if (fw->getOptionsObject()->getBool("check_shading") &&
+            ! inSingleRuleCompileMode())
         {
-            add( new Begin                       ("Detecting rule shadowing" ));
-            add( new printTotalNumberOfRules     () );
+            add(new Begin("Detecting rule shadowing"));
+            add(new printTotalNumberOfRules());
 
-            add( new ItfNegation(            "process negation in Itf"  ) );
-            add( new InterfacePolicyRules(
+            add(new ItfNegation("process negation in Itf"));
+            add(new InterfacePolicyRules(
                      "process interface policy rules and store interface ids"));
 
-            add( new recursiveGroupsInSrc("check for recursive groups in SRC"));
-            add( new recursiveGroupsInDst("check for recursive groups in DST"));
-            add( new recursiveGroupsInSrv("check for recursive groups in SRV"));
+            add(new recursiveGroupsInSrc("check for recursive groups in SRC"));
+            add(new recursiveGroupsInDst("check for recursive groups in DST"));
+            add(new recursiveGroupsInSrv("check for recursive groups in SRV"));
             check_for_recursive_groups=false;
 
-            add( new ExpandGroups("expand groups"          ) );
-            add( new dropRuleWithEmptyRE(
+            add(new ExpandGroups("expand groups"));
+            add(new dropRuleWithEmptyRE("drop rules with empty rule elements"));
+            add(new eliminateDuplicatesInSRC("eliminate duplicates in SRC"));
+            add(new eliminateDuplicatesInDST("eliminate duplicates in DST"));
+            add(new eliminateDuplicatesInSRV("eliminate duplicates in SRV"));
+
+            add(new swapAddressTableObjectsInSrc(
+                     "AddressTable -> MultiAddressRunTime in Src"));
+            add(new swapAddressTableObjectsInDst(
+                     "AddressTable -> MultiAddressRunTime in Dst"));
+
+            add(new swapMultiAddressObjectsInSrc(
+                     "MultiAddress -> MultiAddressRunTime in Src"));
+            add(new swapMultiAddressObjectsInDst(
+                     "MultiAddress -> MultiAddressRunTime in Dst"));
+
+            add(new ExpandMultipleAddressesInSrc(
+                     "expand objects with multiple addresses in SRC"));
+            add(new ExpandMultipleAddressesInDst(
+                     "expand objects with multiple addresses in DST"));
+            add(new dropRuleWithEmptyRE(
                      "drop rules with empty rule elements"));
-            add( new eliminateDuplicatesInSRC("eliminate duplicates in SRC") );
-            add( new eliminateDuplicatesInDST("eliminate duplicates in DST") );
-            add( new eliminateDuplicatesInSRV("eliminate duplicates in SRV") );
+            add(new ConvertToAtomic("convert to atomic rules"));
 
-            add( new swapAddressTableObjectsInSrc(
-                     "AddressTable -> MultiAddressRunTime in Src") );
-            add( new swapAddressTableObjectsInDst(
-                     "AddressTable -> MultiAddressRunTime in Dst") );
+            add( new checkForObjectsWithErrors(
+                     "check if we have objects with errors in rule elements"));
 
-            add( new swapMultiAddressObjectsInSrc(
-                     "MultiAddress -> MultiAddressRunTime in Src") );
-            add( new swapMultiAddressObjectsInDst(
-                     "MultiAddress -> MultiAddressRunTime in Dst") );
-
-            add( new ExpandMultipleAddressesInSRC(
-                     "expand objects with multiple addresses in SRC" ) );
-            add( new ExpandMultipleAddressesInDST(
-                     "expand objects with multiple addresses in DST" ) );
-            add( new dropRuleWithEmptyRE(
-                     "drop rules with empty rule elements"));
-            add( new ConvertToAtomic             ("convert to atomic rules") );
-            add( new DetectShadowing             ("Detect shadowing"       ) );
-            add( new simplePrintProgress         (                         ) );
+            add(new DetectShadowing("Detect shadowing"));
+            add(new simplePrintProgress());
 
             runRuleProcessors();
             deleteRuleProcessors();
         }
 
-        add( new Begin() );
-        add( new printTotalNumberOfRules() );
+        add(new Begin());
+        add(new printTotalNumberOfRules());
 
-//        add( new printScrubRule            (" Defragmentation"             ));
+        add( new singleRuleFilter());
+
+//        add(new printScrubRule(" Defragmentation"));
         if (check_for_recursive_groups)
         {
-            add( new recursiveGroupsInSrc("check for recursive groups in SRC"));
-            add( new recursiveGroupsInDst("check for recursive groups in DST"));
-            add( new recursiveGroupsInSrv("check for recursive groups in SRV"));
+            add(new recursiveGroupsInSrc("check for recursive groups in SRC"));
+            add(new recursiveGroupsInDst("check for recursive groups in DST"));
+            add(new recursiveGroupsInSrv("check for recursive groups in SRV"));
         }
 
-        add( new emptyGroupsInSrc(           "check for empty groups in SRC" ));
-        add( new emptyGroupsInDst(           "check for empty groups in DST" ));
-        add( new emptyGroupsInSrv(           "check for empty groups in SRV" ));
+        add(new emptyGroupsInSrc("check for empty groups in SRC"));
+        add(new emptyGroupsInDst("check for empty groups in DST"));
+        add(new emptyGroupsInSrv("check for empty groups in SRV"));
 
-//        add( new doSrcNegation(         "process negation in Src"          ));
-//        add( new doDstNegation(         "process negation in Dst"          ));
-	add( new doSrvNegation(         "process negation in Srv"            ));
+//        add(new doSrcNegation("process negation in Src"));
+//        add(new doDstNegation("process negation in Dst"));
+	add(new doSrvNegation("process negation in Srv"));
 
 // ExpandGroups opens groups, as well as groups in groups etc.
-        add( new ExpandGroups(          "expand groups"                      ));
-        add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
+        add(new ExpandGroups("expand groups"));
+        add(new dropRuleWithEmptyRE("drop rules with empty rule elements"));
 
-        add( new CheckForTCPEstablished(
-                 "check for TCPService objects with flag \"established\"") );
+        add(new CheckForTCPEstablished(
+                 "check for TCPService objects with flag \"established\""));
             
-        add( new eliminateDuplicatesInSRC("eliminate duplicates in SRC"      ));
-        add( new eliminateDuplicatesInDST("eliminate duplicates in DST"      ));
-        add( new eliminateDuplicatesInSRV("eliminate duplicates in SRV"      ));
+        add(new eliminateDuplicatesInSRC("eliminate duplicates in SRC"));
+        add(new eliminateDuplicatesInDST("eliminate duplicates in DST"));
+        add(new eliminateDuplicatesInSRV("eliminate duplicates in SRV"));
 
-        add( new swapAddressTableObjectsInSrc(
-                 "AddressTable -> MultiAddressRunTime in Src") );
-        add( new swapAddressTableObjectsInDst(
-                 "AddressTable -> MultiAddressRunTime in Dst") );
+        add(new swapAddressTableObjectsInSrc(
+                 "AddressTable -> MultiAddressRunTime in Src"));
+        add(new swapAddressTableObjectsInDst(
+                 "AddressTable -> MultiAddressRunTime in Dst"));
 
-        add( new swapMultiAddressObjectsInSrc(
-                 "MultiAddress -> MultiAddressRunTime in Src") );
-        add( new swapMultiAddressObjectsInDst(
-                 "MultiAddress -> MultiAddressRunTime in Dst") );
+        add(new swapMultiAddressObjectsInSrc(
+                 "MultiAddress -> MultiAddressRunTime in Src"));
+        add(new swapMultiAddressObjectsInDst(
+                 "MultiAddress -> MultiAddressRunTime in Dst"));
 
-        add( new processMultiAddressObjectsInSrc(
-                 "process MultiAddress objects in Src") );
-        add( new processMultiAddressObjectsInDst(
-                 "process MultiAddress objects in Dst") );
+        add(new processMultiAddressObjectsInSrc(
+                 "process MultiAddress objects in Src"));
+        add(new processMultiAddressObjectsInDst(
+                 "process MultiAddress objects in Dst"));
 
-        add( new ItfNegation(         "process negation in Itf"  ) );
-        add( new InterfacePolicyRules(
-                 "process interface policy rules and store interface ids") );
+        add(new replaceFailoverInterfaceInItf("replace carp interfaces"));
 
-	add( new splitIfFirewallInSrc("split rule if firewall is in Src"   ));
-	add( new splitIfFirewallInDst("split rule if firewall is in Dst"   ));
-	add( new fillDirection("determine directions"               ));
+        add(new ItfNegation("process negation in Itf"));
+
+        //add(new InterfacePolicyRules(
+        //    "process interface policy rules and store interface ids"));
+
+	add(new splitIfFirewallInSrc("split rule if firewall is in Src"));
+	add(new splitIfFirewallInDst("split rule if firewall is in Dst"));
+	add(new fillDirection("determine directions"));
 
 // commented out for bug #2828602
 // ... and put back per #2844561
 // both bug reports/patches are by Tom Judge (tomjudge on sourceforge)
 	add( new SplitDirection("split rules with direction 'both'"  ));
 
-        add( new addLoopbackForRedirect(
-                 "add loopback to rules that permit redirected services" ) );
-	add( new ExpandMultipleAddresses(
-                 "expand objects with multiple addresses" ) );
-        add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
-        add( new checkForDynamicInterfacesOfOtherObjects(
-                 "check for dynamic interfaces of other hosts and firewalls" ));
-        add( new MACFiltering("verify for MAC address filtering"   ));
-        add( new checkForUnnumbered("check for unnumbered interfaces"    ));
-	add( new addressRanges("expand address range objects"       ));
-	add( new splitServices("split rules with different protocols"));
-	add( new separateTCPWithFlags("separate TCP services with flags"    ));
-        add( new separateSrcPort("split on TCP and UDP with source ports"));
-        add( new separateTagged("split on TagService"));
-        add( new separateTOS("split on IPService with TOS"));
+        add(new addLoopbackForRedirect(
+                 "add loopback to rules that permit redirected services"));
+	add(new ExpandMultipleAddresses(
+                 "expand objects with multiple addresses"));
+        add(new dropRuleWithEmptyRE("drop rules with empty rule elements"));
+        add(new checkForDynamicInterfacesOfOtherObjects(
+                 "check for dynamic interfaces of other hosts and firewalls"));
+        add(new MACFiltering("verify for MAC address filtering"));
+        add(new checkForUnnumbered("check for unnumbered interfaces"));
+	add(new addressRanges("expand address range objects"));
+	add(new splitServices("split rules with different protocols"));
+	add(new separateTCPWithFlags("separate TCP services with flags"));
+        add(new separateSrcPort("split on TCP and UDP with source ports"));
+        add(new separateTagged("split on TagService"));
+        add(new separateTOS("split on IPService with TOS"));
 
         if (ipv6)
             add( new DropIPv4Rules("drop ipv4 rules"));
         else
             add( new DropIPv6Rules("drop ipv6 rules"));
 
-	add( new verifyCustomServices(
-                 "verify custom services for this platform"));
-//	add( new ProcessScrubOption(    "process 'scrub' option"         ));
-	add( new SpecialServices(       "check for special services"     ));
-	add( new setQuickFlag(          "set 'quick' flag"               ));
-        add( new checkForZeroAddr(      "check for zero addresses"       ));
-        add( new convertInterfaceIdToStr("prepare interface assignments" ));
+	add(new verifyCustomServices("verify custom services for this platform"));
+//	add(new ProcessScrubOption("process 'scrub' option"));
+	add(new SpecialServices("check for special services"));
+	add(new setQuickFlag("set 'quick' flag"));
+        add(new checkForZeroAddr("check for zero addresses"));
+        add(new convertInterfaceIdToStr("prepare interface assignments"));
 
-        add( new createTables(      "create tables"    ));
-//        add( new PrintTables(       "print tables"     ));
+        add( new checkForObjectsWithErrors(
+                 "check if we have objects with errors in rule elements"));
 
-        add( new PrintRule(             "generate pf code" ));
-        add( new simplePrintProgress() );
+        add(new createTables("create tables"));
+//        add(new PrintTables("print tables"));
+
+        add(new PrintRule("generate pf code"));
+        add(new simplePrintProgress());
 
         runRuleProcessors();
 
-    } catch (FWException &ex) {
+    } catch (FWException &ex)
+    {
 	error(ex.toString());
 	exit(1);
     }
@@ -1131,3 +1154,81 @@ void PolicyCompiler_pf::compile()
 void PolicyCompiler_pf::epilog()
 {
 }
+
+void PolicyCompiler_pf::insertCarpRule()
+{
+    /* Add CARP-Service to database */
+    IPService* carp_service = IPService::cast(dbcopy->create(IPService::TYPENAME));
+    carp_service->setComment("CARP service");
+    carp_service->setProtocolNumber(112);
+    dbcopy->add(carp_service);
+
+    FWObjectTypedChildIterator interfaces = fw->findByType(Interface::TYPENAME);
+    for (; interfaces != interfaces.end(); ++interfaces) 
+    {
+        Interface *iface = Interface::cast(*interfaces);
+
+        if (iface->isFailoverInterface())
+        {
+            FWObject *failover_group =
+                iface->getFirstByType(FailoverClusterGroup::TYPENAME);
+            if (failover_group->getStr("type") == "carp")
+            {
+                /* Add automatic rules for CARP 
+                 * Rule should be associated with physical interface
+                 */
+                
+                string phys_iface_name = iface->getOptionsObject()->getStr("base_device");
+                Interface *phys_iface = Interface::cast(
+                    fw->findObjectByName(Interface::TYPENAME, phys_iface_name));
+                if (phys_iface)
+                {
+                    PolicyRule *rule = 
+                        addMgmtRule(NULL, NULL, carp_service, phys_iface,
+                                    PolicyRule::Both, PolicyRule::Accept, "CARP");
+                    FWOptions *ruleopt = rule->getOptionsObject();
+                    assert(ruleopt!=NULL);
+                    ruleopt->setBool("firewall_is_part_of_any_and_networks", false);
+                } else
+                {
+                    warning(
+                        "Can not find interface " + phys_iface_name + 
+                        " for the CARP interface " + iface->getName() +
+                        " of the cluster");
+                }
+            }
+        }
+    }
+}
+
+void PolicyCompiler_pf::insertPfsyncRule()
+{
+    /* Add pfsync service to database */
+    IPService* pfsync_service = IPService::cast(dbcopy->create(IPService::TYPENAME));
+    pfsync_service->setComment("pfsync service");
+    pfsync_service->setProtocolNumber(240);
+    dbcopy->add(pfsync_service);
+
+    FWObjectTypedChildIterator interfaces = fw->findByType(Interface::TYPENAME);
+    for (; interfaces != interfaces.end(); ++interfaces) 
+    {
+        Interface *iface = Interface::cast(*interfaces);
+        if (iface->getOptionsObject()->getBool("state_sync_group_member"))
+        {
+            FWObject *state_sync_group = dbcopy->findInIndex(
+                dbcopy->getIntId(
+                    iface->getOptionsObject()->getStr("state_sync_group_id")));
+            assert(state_sync_group!=NULL);
+            if (state_sync_group && state_sync_group->getStr("type") == "pfsync")
+            {
+                PolicyRule *rule = addMgmtRule(NULL, NULL, pfsync_service, iface,
+                                               PolicyRule::Both,
+                                               PolicyRule::Accept, "pfsync");
+                FWOptions *ruleopt = rule->getOptionsObject();
+                assert(ruleopt!=NULL);
+                ruleopt->setBool("firewall_is_part_of_any_and_networks", false);
+            }
+        }
+    }
+}
+
