@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf_writers.cpp 1302 2009-08-10 22:33:16Z vadim $
+  $Id: PolicyCompiler_pf_writers.cpp 2504 2010-02-07 05:24:21Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -44,21 +44,18 @@
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/DNSName.h"
 #include "fwbuilder/AddressTable.h"
+#include "fwbuilder/XMLTools.h"
 
 #include <iostream>
-#if __GNUC__ > 3 || \
-    (__GNUC__ == 3 && (__GNUC_MINOR__ > 2 || (__GNUC_MINOR__ == 2 ) ) ) || \
-    _MSC_VER
-#  include <streambuf>
-#else
-#  include <streambuf.h>
-#endif
 #include <iomanip>
 #include <fstream>
 #include <sstream>
 
 #include <assert.h>
 
+#include <QStringList>
+
+
 using namespace libfwbuilder;
 using namespace fwcompiler;
 using namespace std;
@@ -76,7 +73,8 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
     FWOptions *ruleopt =rule->getOptionsObject();
     Service *srv=compiler->getFirstSrv(rule);    assert(srv);
 
-    switch (rule->getAction()) {
+    switch (rule->getAction())
+    {
     case PolicyRule::Accept:  
     case PolicyRule::Tag:
     case PolicyRule::Classify:
@@ -91,18 +89,22 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
 
     case PolicyRule::Reject: 
 	if (TCPService::isA(srv)) compiler->output << "block return-rst ";
-	else {
+	else
+        {
 	    string aor=ruleopt->getStr("action_on_reject");
 	    string code;
-	    if ( aor.find("ICMP")!=string::npos ) {
+	    if ( aor.find("ICMP")!=string::npos )
+            {
 		code="return-icmp ";
-		if (aor.find("unreachable")!=string::npos ) {
+		if (aor.find("unreachable")!=string::npos )
+                {
  		    if (aor.find("net")!=string::npos)      code=code+"( 0 ) ";
 		    if (aor.find("host")!=string::npos)     code=code+"( 1 ) ";
 		    if (aor.find("protocol")!=string::npos) code=code+"( 2 ) ";
 		    if (aor.find("port")!=string::npos)     code=code+"( 3 ) ";
 		}
-		if (aor.find("prohibited")!=string::npos ) {
+		if (aor.find("prohibited")!=string::npos )
+                {
  		    if (aor.find("net")!=string::npos)      code=code+"( 9 ) ";
 		    if (aor.find("host")!=string::npos)     code=code+"( 10 ) ";
 		}
@@ -122,21 +124,23 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
     {
         RuleSet *ruleset = rule->getBranch();
         if (ruleset==NULL)
-            compiler->abort(string("Branching rule ") + rule->getLabel() +
-                            " refers ruleset that does not exist");
-        string ruleset_name = ruleset->getName();
-        if (ruleset_name.find("/*")!=string::npos)
+        {
+            compiler->abort(
+                    rule, 
+                    "Branching rule refers ruleset that does not exist");
+            // if we are in test mode or single-rule compile mode
+            compiler->output << "anchor \"UNDEFINED\" ";
+        }else
+        {
+            string ruleset_name = ruleset->getName();
             compiler->output << "anchor \"" << ruleset_name << "\" ";
-        else
-            compiler->output << "anchor " << ruleset_name << " ";
+        }
         break;
     }
     default:
         compiler->abort(
-            string("Unknown action '") + rule->getActionAsString()
-            + "' in rule " + rule->getLabel()
-        );
-//        compiler->output << rule->getActionAsString() << " ";
+                rule, 
+                string("Unknown action ") + rule->getActionAsString());
     }
 }
 
@@ -152,9 +156,12 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
 	string ro = ruleopt->getStr(prefix+"_route_option");
         if (ruleopt->getBool("pf_fastroute") && ro != "none")
 	{
-            compiler->abort("Cannot use fastroute and route method in "
-                            "the same rule because they are mutually "
-                            "exclusive. Rule " + rule->getLabel());
+            compiler->abort(
+                
+                    rule, 
+                    "Cannot use fastroute and route methods in "
+                    "the same rule because they are mutually "
+                    "exclusive.");
 	} else if (ruleopt->getBool("pf_fastroute") && ro == "none")
         {
             compiler->output << "fastroute ";
@@ -166,8 +173,11 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
             if (!ro.empty())
             {
                 if (roif.empty())
-                    compiler->abort("Interface specification is required "
-                                    "for action Route. Rule " + rule->getLabel());
+                    compiler->abort(
+                        
+                            rule, 
+                            "Interface specification is required "
+                            "for action Route.");
 
                 if (ro == "route_through")
                     compiler->output << "route-to ";
@@ -176,8 +186,11 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
                 else if (ro == "route_copy_through")
                     compiler->output << "dup-to ";
                 else
-                    compiler->abort("Unknown option for rule action Route: '" + 
-                                    ro + "' in rule "+rule->getLabel());
+                    compiler->abort(
+                        
+                            rule, 
+                            "Unknown option for rule action Route: '" + 
+                            ro + "'");
             		
                 compiler->output << "{ ";
 
@@ -208,7 +221,9 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
                             } catch (FWException &ex)
                             {
                                 compiler->abort(
-        "Illegal IP address for next hop in rule "+rule->getLabel());
+                                    
+                                        rule, 
+                                        "Illegal IP address for next hop");
                             }
                             try
                             {
@@ -234,7 +249,9 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
                             } catch (FWException &ex)
                             {
                                 compiler->abort(
-        "Illegal netmask for next hop in rule "+rule->getLabel());
+                                    
+                                        rule, 
+                                        "Illegal netmask for next hop");
                             }
                         } else
                         {
@@ -245,7 +262,9 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
                             } catch (FWException &ex)
                             {
                                 compiler->abort(
-        "Illegal IP address for next hop in rule "+rule->getLabel());
+                                    
+                                        rule, 
+                                        "Illegal IP address for next hop");
                             }
                             route_member++;
                         }
@@ -253,18 +272,27 @@ void PolicyCompiler_pf::PrintRule::_printRouteOptions(PolicyRule *rule)
                 }
                 if (route_member < 1)
                 {
-                    compiler->abort("No router specified rule action Route: '"+ 
-                                    ro + "' in rule "+rule->getLabel());
+                    compiler->abort(
+                        
+                            rule, 
+                            "No router specified rule action Route: '"+ 
+                            ro + "'");
                 }
                 if (route_member >= 2 && (roload.empty() || roload == "none"))
                 {
-                    compiler->abort("More than one router specified without load balancing for rule action Route: '" + 
-                                    ro + "' in rule "+rule->getLabel());
+                    compiler->abort(
+                        
+                            rule, 
+                            "More than one router specified without load balancing for rule action Route: '" + 
+                            ro + "'");
                 }
                 if (route_member == 1 && ((!roload.empty()) && roload != "none"))
                 {
-                    compiler->abort("Only one router specified with load balancing for rule action Route: '" + 
-                                    ro + "' in rule "+rule->getLabel());
+                    compiler->abort(
+                        
+                            rule, 
+                            "Only one router specified with load balancing for rule action Route: '" + 
+                            ro + "'");
                 }
 
                 compiler->output << "} ";
@@ -370,14 +398,17 @@ string PolicyCompiler_pf::PrintRule::_printLogPrefix(PolicyRule *rule,
  * %C - chain name
  */
     string::size_type n;
-    if (rule && (n=s.find("%N"))!=string::npos ) {
+    if (rule && (n=s.find("%N"))!=string::npos )
+    {
         std::ostringstream s1;
         s1 << rule->getPosition();
         s.replace(n,2,s1.str());
     }
-    if (rule && (n=s.find("%A"))!=string::npos ) {
+    if (rule && (n=s.find("%A"))!=string::npos )
+    {
         std::ostringstream s1;
-        switch (rule->getAction()) {
+        switch (rule->getAction())
+        {
         case PolicyRule::Accept:  s1 << "ACCEPT"; break;
         case PolicyRule::Deny:    s1 << "DROP";   break;
         case PolicyRule::Reject:  s1 << "REJECT"; break;
@@ -386,29 +417,67 @@ string PolicyCompiler_pf::PrintRule::_printLogPrefix(PolicyRule *rule,
         }
         s.replace(n,2,s1.str());
     }
-    if (rule && (n=s.find("%I"))!=string::npos ) {
+    if (rule && (n=s.find("%I"))!=string::npos )
+    {
         std::ostringstream s1;
-        string rule_iface =  rule->getInterfaceStr();
-        if (rule_iface!="") 
+
+        RuleElementItf *intf_re = rule->getItf();
+        string rule_interfaces;
+        if (!intf_re->isAny())
         {
-            s1 << rule_iface;
-            s.replace(n,2,s1.str());
-        } else
-            s.replace(n,2,"global");
+            for (FWObject::iterator it=intf_re->begin(); it!=intf_re->end(); ++it)
+            {
+                FWObject *o   = *it;
+                if (FWReference::cast(o)!=NULL) o = FWReference::cast(o)->getPointer();
+                rule_interfaces += " " + o->getName();
+            }
+        }
+        if (!rule_interfaces.empty()) 
+            s.replace(n, 2, rule_interfaces);
+        else
+            s.replace(n, 2, "global");
+
+        // string rule_iface =  rule->getInterfaceStr();
+        // if (rule_iface!="") 
+        // {
+        //     s1 << rule_iface;
+        //     s.replace(n,2,s1.str());
+        // } else
+        //     s.replace(n,2,"global");
     }
-    if (rule && (n=s.find("%C"))!=string::npos ) {
+    if (rule && (n=s.find("%C"))!=string::npos )
+    {
         s.replace(n,2,"");  // there is no chain in PF and friends
     }
 
     return "\"" + s + "\" ";
 }
 
-
 void PolicyCompiler_pf::PrintRule::_printInterface(PolicyRule *rule)
 {
-    string       iface_name = rule->getInterfaceStr();
-    if (iface_name!="") 
-	compiler->output << "on " << iface_name << " ";
+    RuleElementItf *intf_re = rule->getItf();
+    string rule_interfaces;
+    int intf_count = 0;
+
+    if (!intf_re->isAny())
+    {
+        for (FWObject::iterator it=intf_re->begin(); it!=intf_re->end(); ++it)
+        {
+            FWObject *o   = *it;
+            if (FWReference::cast(o)!=NULL) o = FWReference::cast(o)->getPointer();
+            rule_interfaces += " " + o->getName();
+            intf_count++;
+        }
+        compiler->output << "on";
+        if (intf_count > 1) compiler->output << " {";
+        compiler->output << rule_interfaces;
+        if (intf_count > 1) compiler->output << " }";
+        compiler->output << " ";
+    }
+
+    // string iface_name = rule->getInterfaceStr();
+    // if (iface_name!="") 
+    //     compiler->output << "on " << iface_name << " ";
 }
 
 // print address family
@@ -454,8 +523,8 @@ string PolicyCompiler_pf::PrintRule::_printPort(int rs,int re,bool neg)
     if (rs<0) rs=0;
     if (re<0) re=0;
 
-    if (!neg) {
-
+    if (!neg)
+    {
 	if (rs>0 || re>0)
         {
             if (rs>re && re==0) re=rs;
@@ -465,7 +534,8 @@ string PolicyCompiler_pf::PrintRule::_printPort(int rs,int re,bool neg)
 		if (rs==0 && re!=0)      str << "<= " << re;
 		else
 		    if (rs!=0 && re==65535)  str << ">= " << rs;
-		    else {
+		    else
+                    {
 /* 
  * port range. Operator '><' defines range in a such way that boundaries
  * are not included. Since we assume it is inclusive, let's move boundaries 
@@ -475,15 +545,17 @@ string PolicyCompiler_pf::PrintRule::_printPort(int rs,int re,bool neg)
 			str << rs << " >< " << re;
 		    }
 	}
-    } else {
-
-	if (rs>0 || re>0) {
+    } else
+    {
+	if (rs>0 || re>0)
+        {
 	    if (rs==re)  str << "!= " << rs;
 	    else
 		if (rs==0 && re!=0)      str << "> " << re;
 		else
 		    if (rs!=0 && re==65535)  str << "< " << rs;
-		    else {
+		    else
+                    {
 			str << rs << " <> " << re;
 		    }
 	}
@@ -587,7 +659,9 @@ void PolicyCompiler_pf::PrintRule::_printDstService(RuleElementSrv  *rel)
             string dscp = ip->getDSCPCode();
             if (!tos.empty()) compiler->output << " tos " << tos << " ";
             if (!dscp.empty())
-                compiler->abort("PF does not support DSCP matching");
+                compiler->abort(
+                                    rel->getParent(),
+                                    "PF does not support DSCP matching");
         }
     } else 
     {
@@ -766,7 +840,7 @@ void PolicyCompiler_pf::PrintRule::_printSrcAddr(RuleElementSrc  *rel)
                << rule->getLabel()
                << "'  rel->front(): "
                << oref->getPointerId();
-        compiler->abort(errstr.str());
+        compiler->abort(rel->getParent(), errstr.str());
     }
 
     if (rel->size()==1 && ! o->getBool("pf_table") )
@@ -805,7 +879,7 @@ void PolicyCompiler_pf::PrintRule::_printDstAddr(RuleElementDst  *rel)
                << rule->getLabel()
                <<  "'  rel->front(): "
                << oref->getPointerId();
-        compiler->abort(errstr.str());
+        compiler->abort(rel->getParent(), errstr.str());
     }
 
     if (rel->size()==1 && ! o->getBool("pf_table") )
@@ -841,34 +915,12 @@ bool PolicyCompiler_pf::PrintRule::processNext()
     FWOptions  *ruleopt =rule->getOptionsObject();
     string version=compiler->fw->getStr("version");
 
-//    cerr << endl;
-//    cerr << "Rule " << rule->getPosition() << endl;
-
     tmp_queue.push_back(rule);
 
-    string rl=rule->getLabel();
-    if (rl!=current_rule_label)
-    {
-        
-        compiler->output << "# " << endl;
-        compiler->output << "# Rule  " << rl << endl;
-
-        string    comm=rule->getComment();
-        string::size_type c1,c2;
-        c1=0;
-        while ( (c2=comm.find('\n',c1))!=string::npos ) {
-            compiler->output << "# " << comm.substr(c1,c2-c1) << endl;
-            c1=c2+1;
-        }
-        compiler->output << "# " << comm.substr(c1) << endl;
-        compiler->output << "# " << endl;
-
-        current_rule_label=rl;
-    }
-
-//    cerr << "CP 1" << endl;
-
+    compiler->output << compiler->printComment(rule, current_rule_label, "#");
 
+    // string err = rule->getStr(".error_msg");
+    // if (!err.empty()) compiler->output << "# " << err << endl;
 
     RuleElementSrc *srcrel=rule->getSrc();
 //    Address        *src   =compiler->getFirstSrc(rule);  assert(src);
@@ -921,7 +973,8 @@ bool PolicyCompiler_pf::PrintRule::processNext()
         {
             // tcp service, no special flag match
 
-            if ( version == "4.x")
+//            if ( version == "4.x")
+            if (XMLTools::version_compare(version, "4.0")>=0)
             {
                 if (compiler->getCachedFwOpt()->getBool(
                         "accept_new_tcp_with_no_syn") )
@@ -945,20 +998,31 @@ bool PolicyCompiler_pf::PrintRule::processNext()
             }
         }
 
-        // in PF "modulate state", "synproxy state", "keep state" are mutually
-        // exclusive
-        // "keep state" can be used with any protocol, while "modulate state"
-        // and "synproxy state" can only be used with tcp.
+        /*
+         * in PF "modulate state", "synproxy state", "keep state" are
+         * mutually exclusive "keep state" can be used with any
+         * protocol, while "modulate state" and "synproxy state" can
+         * only be used with tcp.
+         */
 
+        bool have_state_option = false;
+
+        /*
+         * First, set explicit state tracking parameter, then add
+         * stateful tracking options. 
+         */
 	if (compiler->getCachedFwOpt()->getBool("pf_synproxy") && tcpsrv!=NULL)
+        {
 	    compiler->output << "synproxy state ";
-        else
+            have_state_option = true;
+        } else
         {
             if ((ruleopt->getBool("pf_modulate_state") || 
                  compiler->getCachedFwOpt()->getBool("pf_modulate_state")) &&
                 tcpsrv!=NULL) 
             {
                 compiler->output << "modulate state ";
+                have_state_option = true;
             } else
             {
                 /*
@@ -981,97 +1045,121 @@ bool PolicyCompiler_pf::PrintRule::processNext()
                  * interface. Adding rule option "Set 'keep state'
                  * explicitly" to cope with this.
                  */
-                if ( version != "4.x" ||
+                if (XMLTools::version_compare(version, "4.0") < 0 ||
                      compiler->getCachedFwOpt()->getBool("pf_keep_state"))
+                {
                     compiler->output << "keep state ";
+                    have_state_option = true;
+                }
             }
         }
 
-        int nopt=0;
-        if (ruleopt->getInt("pf_rule_max_state")>0)        nopt++;
-        if (ruleopt->getBool("pf_source_tracking"))        nopt+=2;
-        if (ruleopt->getInt("pf_max_src_conn")>0)          nopt++;
-        if (ruleopt->getStr("pf_max_src_conn_overload_table")!="") nopt++;
-        if (ruleopt->getInt("pf_max_src_conn_rate_num")>0) nopt++;
-        if (ruleopt->getBool("pf_sloppy_tracker"))         nopt++;
+        /*
+         * Stateful tracking options. According to the pf.conf manual,
+         * one of keep state, modulate state, or synproxy state must
+         * be specified explicitly to apply these options to a rule.
+         * Using flags need_state_option and have_state_option for that.
+         */
 
-        bool not_the_first = false;
-        if (nopt)
+        QStringList options;
+        bool need_state_option = false;
+
+        if (ruleopt->getInt("pf_rule_max_state")>0)
         {
-            if (nopt>1) compiler->output << " ( ";
-
-            if (ruleopt->getInt("pf_rule_max_state")>0)
-            {
-                compiler->output << " max "
-                                 << ruleopt->getInt("pf_rule_max_state");
-                not_the_first = true;
-            }
-
-            if (ruleopt->getBool("pf_sloppy_tracker"))
-            {
-                if (not_the_first) compiler->output << ",";
-                compiler->output << " sloppy ";
-                not_the_first = true;
-            }
-
-            if (ruleopt->getBool("pf_source_tracking"))
-            {
-                if (not_the_first) compiler->output << ",";
-
-                if (ruleopt->getInt("pf_max_src_nodes") > 0)
-                    compiler->output << " max-src-nodes "
-                                     << ruleopt->getInt("pf_max_src_nodes");
-
-                if (ruleopt->getInt("pf_max_src_states")>0)
-                    compiler->output << ", max-src-states "
-                                     << ruleopt->getInt("pf_max_src_states");
-
-                not_the_first = true;
-            }
-
-            bool check_overload_opts = false;
-            if (ruleopt->getInt("pf_max_src_conn")>0)
-            {
-                if (not_the_first) compiler->output << ",";
-
-                compiler->output << " max-src-conn "
-                                 << ruleopt->getInt("pf_max_src_conn");
-                not_the_first = true;
-                check_overload_opts = true;
-            }
-
-            if (ruleopt->getInt("pf_max_src_conn_rate_num")>0 && 
-                ruleopt->getInt("pf_max_src_conn_rate_seconds")>0)
-            {
-                if (not_the_first) compiler->output << ",";
-
-                compiler->output << " max-src-conn-rate "
-                                 << ruleopt->getInt("pf_max_src_conn_rate_num")
-                                 << "/"
-                                 << ruleopt->getInt(
-                                     "pf_max_src_conn_rate_seconds");
-                check_overload_opts = true;
-            }
-            if (check_overload_opts)
-            {
-                if (ruleopt->getStr("pf_max_src_conn_overload_table")!="")
-                    compiler->output << ", overload <"
-                                     << ruleopt->getStr(
-                                         "pf_max_src_conn_overload_table")
-                                     << ">";
-                if (ruleopt->getBool("pf_max_src_conn_flush"))
-                    compiler->output << " flush";
-                if (ruleopt->getBool("pf_max_src_conn_global"))
-                    compiler->output << " global";
-            }
-            if (nopt>1) compiler->output << " ) ";
+            options.push_back(QString("max %1").arg(ruleopt->getInt("pf_rule_max_state")));
+            need_state_option = true;
         }
+
+        if (ruleopt->getBool("pf_sloppy_tracker"))
+        {
+            options.push_back("sloppy");
+            need_state_option = true;
+        }
+
+        if (ruleopt->getBool("pf_no_sync"))
+        {
+            options.push_back("no-sync");
+            need_state_option = true;
+        }
+
+        if (ruleopt->getBool("pf_pflow"))
+        {
+            options.push_back("pflow");
+            need_state_option = true;
+        }
+
+        if (ruleopt->getBool("pf_source_tracking"))
+        {
+            if (ruleopt->getInt("pf_max_src_nodes") > 0)
+            {
+                options.push_back(QString("max-src-nodes %1").arg(
+                                      ruleopt->getInt("pf_max_src_nodes")));
+                need_state_option = true;
+            }
+
+            if (ruleopt->getInt("pf_max_src_states")>0)
+            {
+                options.push_back(QString("max-src-states %1").arg(
+                                      ruleopt->getInt("pf_max_src_states")));
+                need_state_option = true;
+            }
+        }
+
+        bool check_overload_opts = false;
+        if (ruleopt->getInt("pf_max_src_conn")>0)
+        {
+            options.push_back(QString("max-src-conn %1").arg(
+                                  ruleopt->getInt("pf_max_src_conn")));
+            check_overload_opts = true;
+            need_state_option = true;
+        }
+
+        if (ruleopt->getInt("pf_max_src_conn_rate_num")>0 && 
+            ruleopt->getInt("pf_max_src_conn_rate_seconds")>0)
+        {
+            options.push_back(QString("max-src-conn-rate %1/%2")
+                              .arg(ruleopt->getInt("pf_max_src_conn_rate_num"))
+                              .arg(ruleopt->getInt("pf_max_src_conn_rate_seconds")));
+            check_overload_opts = true;
+            need_state_option = true;
+        }
+
+        if (check_overload_opts)
+        {
+            QStringList overload_opts;
+            if (ruleopt->getStr("pf_max_src_conn_overload_table")!="")
+                overload_opts.push_back(
+                    QString("overload <%1>").arg(
+                        ruleopt->getStr("pf_max_src_conn_overload_table").c_str()));
+            if (ruleopt->getBool("pf_max_src_conn_flush"))
+                overload_opts.push_back("flush");
+            if (ruleopt->getBool("pf_max_src_conn_global"))
+                overload_opts.push_back("global");
+            if (overload_opts.size() > 0)
+                options.push_back(overload_opts.join(" "));
+        }
+
+        if (need_state_option && !have_state_option)
+        {
+            compiler->output << "keep state ";
+        }
+
+        // looks like pf.conf syntax requires '(' ')' even if there is
+        // only one option
+        if (options.size() > 0) compiler->output << "( ";
+
+        compiler->output << options.join(", ").toStdString();
+
+        if (options.size() > 0) compiler->output << " )";
+
     } else
     {
         // stateless rule
-        if ( version == "4.x")
+        if (XMLTools::version_compare(version, "4.0")>=0)
+        {
             // v4.x, stateless rule
             compiler->output << "no state ";
+        }
     }
 
     if (rule->getBool("allow_opts")) compiler->output << "allow-opts  ";
